/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
 * 					      Bologna Astronomical Observatory
 *----------------------------------------------------------------------------
 * File name 	: catalib.h
 * Author 	: Paolo Montegriffo
 * Created on	: July, 2000	
 * Language	: ANSI C
 * Description	: library function prototypes for CataPack
 * Version      : 2.0 - October 6, 2000
 * Version      : 2.1.3 - September 7, 2001
 * Version      : 2.1.18 - April 17, 2003
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 							Function prototypes
 *--------------------------------------------------------------------------*/

#ifndef CATALIB_H
#define CATALIB_H 1

#include "common.h"

BEGIN_C_DECLS

extern  char         **tokenize             (char         *line,
                                             char         *del,
                                             int          *ntok) ;
extern  void           free_list_names      (char        **names,
                                             int           n) ;
extern  char         **free_carray          (char        **array,
                                             int           n) ;
extern  int          **free_iarray          (int         **array,
                                             int           n) ;
extern  float        **free_farray          (float       **array,
                                             int           n) ;
extern  double       **free_darray          (double      **array,
                                             int           n) ;
extern  void           strip_blanks         (char         *expr) ;

extern  void           strip_newline        (char         *expr) ;

extern  int            Is_In                (unsigned int  ns,
                                             unsigned int  n) ;
extern  unsigned int   bin_switch_on        (unsigned int  ns,
                                             int           nn) ;
extern  unsigned int   bin_switch_off       (unsigned int  ns,
                                             int           nn) ;
extern  int            bin_check_n          (unsigned int  ns,
                                             int           nn) ;
extern  int            bin_get_count        (unsigned int  ns) ;

extern  char          *bin_get_string       (char         *user_buf,
                                             unsigned int  ns) ;
extern  int           *getNumbers           (char         *expr,
                                             int          *n) ;
extern  char         **getFormat            (char         *expr,
                                             int          *nf) ;
extern  int            is_integer           (char         *expr) ;

extern  int            ndecimals            (char         *expr) ;

extern  char          *iso_8601_date_format (char         *user_buf,
                                             int           strict) ;

#if HAVE_GSL

#include <gsl/gsl_math.h>
#include <gsl/gsl_sf_trig.h>
#define ang_res_pos gsl_sf_angle_restrict_pos
#define ang_res_sym gsl_sf_angle_restrict_symm
#define TWOPI (2.0*M_PI)

#else

#ifndef M_PI
#define M_PI    3.14159265358979323846
#endif   /* M_PI */
#define TWOPI (2.0*M_PI)
#define ang_res_pos my_ang_res_pos
#define ang_res_sym my_ang_res_sym
extern double my_ang_res_pos(double a) ;
extern double my_ang_res_sym(double a) ;

#endif   /* HAVE_GSL */

#ifndef RADTODEG
#define RADTODEG(a) ((a)*360.0/TWOPI)
#endif

END_C_DECLS

#endif /* !CATALIB_H */
/*--------------------------------------------------------------------------*/
