/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 							Includes
 *--------------------------------------------------------------------------*/

#if HAVE_CONFIG_H
#  include "config.h"
#endif

#include "common.h"
#include <gtk/gtk.h>

#include "cpk_gtk_util.h"
#include "cpk_gtk_cursorutil.h"
#include "cpk_buttons.h"
#include "cpk_frame.h"
#include "cpk_plot.h"
#include "ccl_main.h"
#include "ccl_buttons.h"

/*----------------------------------------------------------------------------
 * 							Function prototypes
 *--------------------------------------------------------------------------*/

static void        print_hello                 (GtkWidget     *w,
                                                gpointer       data) ;
static void        help                        (void) ;

/* Main Menu prototypes */

static void        mm_quit_program             (ccl_display   *display, 
                                                guint          action, 
                                                GtkWidget     *widget) ;

static void        mm_edit_pointer             (ccl_display   *display, 
                                                guint          action, 
                                                GtkWidget     *widget) ;
static void        mm_edit_select              (ccl_display   *display,
                                                guint          action, 
                                                GtkWidget     *widget) ;
static void        mm_edit_pan                 (ccl_display   *display, 
                                                guint          action, 
                                                GtkWidget     *widget) ;
static void        mm_edit_zoom                (ccl_display   *display, 
                                                guint          action, 
                                                GtkWidget     *widget) ;
static void        mm_edit_rotate              (ccl_display   *display, 
                                                guint          action, 
                                                GtkWidget     *widget) ;
static void        mm_edit_ruler               (ccl_display   *display, 
                                                guint          action, 
                                                GtkWidget     *widget) ;
static void        mm_edit_crosshair           (ccl_display   *display, 
                                                guint          action, 
                                                GtkWidget     *widget) ;
static void        mm_frame_new                (ccl_display   *display, 
                                                guint          action, 
                                                GtkWidget     *widget) ;
static void        mm_frame_del                (ccl_display   *display, 
                                                guint          action, 
                                                GtkWidget     *widget) ;
static void        mm_frame_prev               (ccl_display   *display, 
                                                guint          action, 
                                                GtkWidget     *widget) ;
static void        mm_frame_next               (ccl_display   *display, 
                                                guint          action, 
                                                GtkWidget     *widget) ;
static void        mm_frame_mode_single        (ccl_display   *display, 
                                                guint          action, 
                                                GtkWidget     *widget) ;
static void        mm_frame_mode_pair          (ccl_display   *display, 
                                                guint          action, 
                                                GtkWidget     *widget) ;
static void        mm_frame_mode_tile          (ccl_display   *display, 
                                                guint          action, 
                                                GtkWidget     *widget) ;
static void        mm_frame_tile_grid          (ccl_display   *display, 
                                                guint          action, 
                                                GtkWidget     *widget) ;
static void        mm_frame_tile_col           (ccl_display   *display, 
                                                guint          action, 
                                                GtkWidget     *widget) ;
static void        mm_frame_tile_row           (ccl_display   *display, 
                                                guint          action, 
                                                GtkWidget     *widget) ;


/* Buttons prototypes */

static void        ccl_edit                    (GtkWidget     *widget, 
                                                ccl_display   *display) ;
static void        ccl_frame_status            (gboolean       status) ;

static void        ccl_frame_new               (GtkWidget     *widget, 
                                                ccl_display   *display) ;
static void        ccl_frame_del               (GtkWidget     *widget, 
                                                ccl_display   *display) ;
static void        ccl_frame_prev              (GtkWidget     *widget, 
                                                ccl_display   *display) ;
static void        ccl_frame_next              (GtkWidget     *widget, 
                                                ccl_display   *display) ;
static void        ccl_frame_mode              (GtkWidget     *widget, 
                                                ccl_display   *display) ;

/*----------------------------------------------------------------------------
 * 							Private types
 *--------------------------------------------------------------------------*/



/*----------------------------------------------------------------------------
 * 							Global variables	
 *--------------------------------------------------------------------------*/

CpkButtons2 Mfile[] = {
   { "Open",           NULL, NULL,             NULL,  0, FALSE },
   { "Plot",           NULL, NULL,             NULL,  0, FALSE },
   { "Print",          NULL, NULL,             NULL,  0, FALSE },
   { "Save",           NULL, NULL,             NULL,  0, FALSE },
   { "Exit",           NULL, NULL, ccl_quit_program,  0,  TRUE }
} ;

CpkButtons2 Medit[] = {
   { "Pointer",        NULL, NULL,       ccl_edit,  1,  TRUE },
   { "Select",         NULL, NULL,       ccl_edit,  1, FALSE },
   { "Pan",            NULL, NULL,       ccl_edit,  1, FALSE },
   { "Zoom",           NULL, NULL,       ccl_edit,  1, FALSE },
   { "Rotate",         NULL, NULL,       ccl_edit,  1, FALSE },
   { "Ruler",          NULL, NULL,       ccl_edit,  1, FALSE },
   { "Crosshair",      NULL, NULL,       ccl_edit,  1, FALSE }
} ;

CpkButtons2 Mframe[] = {
   { "New",       NULL, NULL,     ccl_frame_new,  0,  TRUE },
   { "Delete",    NULL, NULL,     ccl_frame_del,  0,  TRUE },
   { "Single",    NULL, NULL,    ccl_frame_mode,  1,  TRUE },
   { "Pair",      NULL, NULL,    ccl_frame_mode,  1, FALSE },
   { "Tile",      NULL, NULL,    ccl_frame_mode,  1, FALSE },
   { "Prev",      NULL, NULL,    ccl_frame_prev,  0,  TRUE },
   { "Next",      NULL, NULL,    ccl_frame_next,  0,  TRUE }
} ;

CpkButtons2 Mzoom[] = {
   { "In",        NULL, NULL,             NULL,  0, FALSE },
   { "Out",       NULL, NULL,             NULL,  0, FALSE },
   { "Center",    NULL, NULL,             NULL,  0, FALSE },
   { "Swap X",    NULL, NULL,             NULL,  0, FALSE },
   { "Swap Y",    NULL, NULL,             NULL,  0, FALSE },
   { "Crop",      NULL, NULL,             NULL,  0, FALSE }
} ;

CpkButtons2 Mgraph[] = {
   { "Ptype", NULL, NULL, NULL,  0, FALSE },
   { "Color", NULL, NULL, NULL,  0, FALSE }
} ;

CpkButtons1 buttons[] = {
   { "File",       sizeof(Mfile )/sizeof(Mfile [0]),     Mfile },
   { "Edit",       sizeof(Medit )/sizeof(Medit [0]),     Medit },
   { "Frame",      sizeof(Mframe)/sizeof(Mframe[0]),    Mframe },
   { "Zoom",       sizeof(Mzoom )/sizeof(Mzoom [0]),     Mzoom },
   { "Graphics",   sizeof(Mgraph)/sizeof(Mgraph[0]),    Mgraph }
} ;

CpkItemFactory menu_items[] = {
{{"/_File",                      NULL,                 NULL, 0,      "<Branch>"}, NULL},
{{"/File/Open",          "<control>O",                 NULL, 0,            NULL}, &Mfile[0]},
{{"/File/Plot",          "<control>W",                 NULL, 0,            NULL}, &Mfile[1]},
{{"/File/cippa",         "<control>D",                 NULL, 0,            NULL}, NULL},
{{"/File/Print",         "<control>E",                 NULL, 0,            NULL}, &Mfile[2]},
{{"/File/_Save",         "<control>S",                 NULL, 0,            NULL}, &Mfile[3]},
{{"/File/sep1",                  NULL,                 NULL, 0,   "<Separator>"}, NULL},
{{"/File/Exit",          "<control>Q",      mm_quit_program, 1,            NULL}, &Mfile[4]},
{{"/_Edit",                      NULL,                 NULL, 0,      "<Branch>"}, NULL},
{{"/Edit/tearoff1",              NULL,                 NULL, 0,     "<Tearoff>"}, NULL},
{{"/Edit/Pointer",               NULL,      mm_edit_pointer, 0,   "<RadioItem>"}, &Medit[0]},
{{"/Edit/Select",                NULL,       mm_edit_select, 0, "/Edit/Pointer"}, &Medit[1]},
{{"/Edit/Pan",                   NULL,          mm_edit_pan, 0, "/Edit/Pointer"}, &Medit[2]},
{{"/Edit/Zoom",                  NULL,         mm_edit_zoom, 0, "/Edit/Pointer"}, &Medit[3]},
{{"/Edit/Rotate",                NULL,       mm_edit_rotate, 0, "/Edit/Pointer"}, &Medit[4]},
{{"/Edit/Ruler",                 NULL,        mm_edit_ruler, 0, "/Edit/Pointer"}, &Medit[5]},
{{"/Edit/Crosshair",             NULL,    mm_edit_crosshair, 0, "/Edit/Pointer"}, &Medit[6]},
{{"/Edit/sep1",                  NULL,                 NULL, 0,   "<Separator>"}, NULL},
{{"/F_rame",                     NULL,                 NULL, 0,      "<Branch>"}, NULL},
{{"/Frame/tearoff1",             NULL,                 NULL, 0,     "<Tearoff>"}, NULL},
{{"/Frame/New",                  NULL,         mm_frame_new, 1,           NULL }, &Mframe[0]},
{{"/Frame/Delete",               NULL,         mm_frame_del, 1,           NULL }, &Mframe[1]},
{{"/Frame/tearoff1",             NULL,                 NULL, 0,  "<Separator>" }, NULL},
{{"/Frame/Single Frame",         NULL, mm_frame_mode_single, 1,  "<RadioItem>" }, &Mframe[2]},
{{"/Frame/Pair Frames",          NULL,   mm_frame_mode_pair, 1, "/Frame/Single Frame"}, &Mframe[3]},
{{"/Frame/Tile Frames",          NULL,   mm_frame_mode_tile, 1, "/Frame/Single Frame"}, &Mframe[4]},
{{"/Frame/sep1",                 NULL,                 NULL, 0,   "<Separator>"}, NULL},
{{"/Frame/Show\134Hide Frames",  NULL,                 NULL, 0,      "<Branch>"}, NULL},
{{"/Frame/sep1",                 NULL,                 NULL, 0,   "<Separator>"}, NULL},
{{"/Frame/Prev",                 NULL,        mm_frame_prev, 1,            NULL}, &Mframe[5]},
{{"/Frame/Next",                 NULL,        mm_frame_next, 1,            NULL}, &Mframe[6]},

{{"/_Preferences",         "<shift>P",                 NULL, 0,      "<Branch>"}, NULL},
{{"/Preferences/tearoff1",       NULL,                 NULL, 0,     "<Tearoff>"}, NULL},
{{"/Preferences/Show Information Panel", NULL,         NULL, 1,   "<CheckItem>"}, NULL},
{{"/Preferences/Show Panner",    NULL,                 NULL, 1,   "<CheckItem>"}, NULL},
{{"/Preferences/Show Magnifier", NULL,                 NULL, 1,   "<CheckItem>"}, NULL},
{{"/Preferences/Show Buttons",   NULL,                 NULL, 1,   "<CheckItem>"}, NULL},
{{"/Preferences/Show Console",   NULL,                 NULL, 1,   "<CheckItem>"}, NULL},
{{"/Preferences/sep1",           NULL,                 NULL, 0,   "<Separator>"}, NULL},

{{"/Preferences/Magnifier",      NULL,                 NULL, 0,      "<Branch>"}, NULL},
{{"/Preferences/Magnifier/2x",   NULL,                 NULL, 1,   "<RadioItem>"}, NULL},
{{"/Preferences/Magnifier/4x",   NULL,                 NULL, 1, "/Preferences/Magnifier/2x"}, NULL},
{{"/Preferences/Magnifier/8x",   NULL,                 NULL, 1, "/Preferences/Magnifier/2x"}, NULL},
{{"/Preferences/Magnifier/16x",  NULL,                 NULL, 1, "/Preferences/Magnifier/2x"}, NULL},
{{"/Preferences/Magnifier/32x",  NULL,                 NULL, 1, "/Preferences/Magnifier/2x"}, NULL},
{{"/Preferences/Tile",           NULL,                 NULL, 0,      "<Branch>"}, NULL},
{{"/Preferences/Tile/tearoff1",  NULL,                 NULL, 0,     "<Tearoff>"}, NULL},
{{"/Preferences/Tile/Grid",      NULL,   mm_frame_tile_grid, 1,   "<RadioItem>"}, NULL},
{{"/Preferences/Tile/Column",    NULL,    mm_frame_tile_col, 1, "/Preferences/Tile/Grid"}, NULL},
{{"/Preferences/Tile/Row",       NULL,    mm_frame_tile_row, 1, "/Preferences/Tile/Grid"}, NULL},

{{"/Preferences/sep1",           NULL,                 NULL, 0,   "<Separator>"}, NULL},
{{"/Preferences/Coordinate Grid", NULL,                NULL, 0,   "<CheckItem>"}, NULL},
{{"/_Help",                      NULL,                 NULL, 0,  "<LastBranch>"}, NULL},
{{"/_Help/About",                NULL,                 NULL, 0,            NULL}, NULL},
{{"/Help/sep1",                  NULL,                 NULL, 0,   "<Separator>"}, NULL},
{{"/_Help/Version",              NULL,                 NULL, 0,            NULL}, NULL},
{{"/_Help/Distribution Policy",  NULL,                 NULL, 0,            NULL}, NULL}
};



guint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]) ;

GtkItemFactory *item_factory ;

extern gdouble   tunits ;


/*----------------------------------------------------------------------------
 * 							Private functions
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 							Defines
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 *							Function codes
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 							Buttons
 *--------------------------------------------------------------------------*/

GtkWidget *
ccl_buttons_new (ccl_display *display)
{
   return cpk_buttons_new (buttons, 5, display) ;
}

/*----------------------------------------------------------------------------
 * 							Main Menu Actions
 *--------------------------------------------------------------------------*/

GtkWidget *
ccl_menu_main (GtkWidget *window, ccl_display *display)
{
   GtkWidget *menu_bar ;
   menu_bar = cpk_menu_main (window, display) ;

   /* Set init values for menu radio items */
   gtk_check_menu_item_set_active  (GTK_CHECK_MENU_ITEM (Medit[0].item), TRUE) ;
   gtk_check_menu_item_set_active  (GTK_CHECK_MENU_ITEM (Mframe[4].item), TRUE) ;
   return menu_bar ;
}

void 
mm_quit_program (ccl_display *display, guint action, GtkWidget *widget)
{
   ccl_quit_program (widget, display) ;
}

/*----------------------------------------------------------------------------
 * 							Edit Menu
 *--------------------------------------------------------------------------*/

static void 
mm_edit_pointer (ccl_display *display, guint action, GtkWidget *widget)
{
   CpkFrameSet *fs = CPK_FRAME_SET (display->fs) ;
   if (!GTK_CHECK_MENU_ITEM (widget)->active)
      return ;
   cpk_buttons_radio_set_active (&Medit[0], 7, 0) ;
   display->pointer_action = fs->p_action = POINTER ;
   fs->cursor = GDK_DRAFT_SMALL ;
   cpk_frame_set_cursor (fs) ;
}

static void 
mm_edit_select (ccl_display *display, guint action, GtkWidget *widget)
{
   CpkFrameSet *fs = CPK_FRAME_SET (display->fs) ;
   if (!GTK_CHECK_MENU_ITEM (widget)->active)
      return ;
   cpk_buttons_radio_set_active (&Medit[0], 7, 1) ;
   display->pointer_action = fs->p_action = SELECT ;
   fs->cursor = CPK_SELECT_CURSOR ;
   cpk_frame_set_cursor (fs) ;
}

static void 
mm_edit_pan (ccl_display *display, guint action, GtkWidget *widget)
{
   CpkFrameSet *fs = CPK_FRAME_SET (display->fs) ;
   if (!GTK_CHECK_MENU_ITEM (widget)->active)
      return ;
   cpk_buttons_radio_set_active (&Medit[0], 7, 2) ;
   display->pointer_action = fs->p_action = PAN ;
   fs->cursor = CPK_PAN_CURSOR ;
   cpk_frame_set_cursor (fs) ;
}

static void 
mm_edit_zoom (ccl_display *display, guint action, GtkWidget *widget)
{
   CpkFrameSet *fs = CPK_FRAME_SET (display->fs) ;
   if (!GTK_CHECK_MENU_ITEM (widget)->active)
      return ;
   cpk_buttons_radio_set_active (&Medit[0], 7, 3) ;
   display->pointer_action = fs->p_action = ZOOM ;
   fs->cursor = CPK_ZOOM_CURSOR ;
   cpk_frame_set_cursor (fs) ;
}

static void 
mm_edit_rotate (ccl_display *display, guint action, GtkWidget *widget)
{
   CpkFrameSet *fs = CPK_FRAME_SET (display->fs) ;
   if (!GTK_CHECK_MENU_ITEM (widget)->active)
      return ;
   cpk_buttons_radio_set_active (&Medit[0], 7, 4) ;
   display->pointer_action = fs->p_action = ROTATE ;
   fs->cursor = CPK_ROTATE_CURSOR ;
   cpk_frame_set_cursor (fs) ;
}

static void 
mm_edit_ruler (ccl_display *display, guint action, GtkWidget *widget)
{
   CpkFrameSet *fs = CPK_FRAME_SET (display->fs) ;
   if (!GTK_CHECK_MENU_ITEM (widget)->active)
      return ;
   cpk_buttons_radio_set_active (&Medit[0], 7, 5) ;
   display->pointer_action = fs->p_action = RULER ;
   fs->cursor = CPK_RULER_CURSOR ;
   cpk_frame_set_cursor (fs) ;
}

static void 
mm_edit_crosshair (ccl_display *display, guint action, GtkWidget *widget)
{
   CpkFrameSet *fs = CPK_FRAME_SET (display->fs) ;
   if (!GTK_CHECK_MENU_ITEM (widget)->active)
      return ;
   cpk_buttons_radio_set_active (&Medit[0], 7, 6) ;
   display->pointer_action = fs->p_action = CROSSHAIR ;
   fs->cursor = GDK_CROSSHAIR ;
   cpk_frame_set_cursor (fs) ;
}

/*----------------------------------------------------------------------------
 * 							Frame Menu
 *--------------------------------------------------------------------------*/

static void 
mm_frame_new (ccl_display *display, guint action, GtkWidget *widget)
{
   ccl_frame_new (widget, display) ;
}

static void 
mm_frame_del (ccl_display *display, guint action, GtkWidget *widget)
{
   ccl_frame_del (widget, display) ;
}

static void 
mm_frame_prev (ccl_display *display, guint action, GtkWidget *widget)
{
   ccl_frame_prev (widget, display) ;
}

static void 
mm_frame_next (ccl_display *display, guint action, GtkWidget *widget)
{
   ccl_frame_next (widget, display) ;
}

static void 
mm_frame_mode_single (ccl_display *display, guint action, GtkWidget *widget)
{
   CpkFrameSet *fs ;
   if (!GTK_CHECK_MENU_ITEM (widget)->active)
      return ;
   fs = CPK_FRAME_SET (display->fs) ;
   cpk_buttons_radio_set_active (&Mframe[2], 3, 0) ;
   fs->mode = FRAME_SINGLE ;
   cpk_frame_set_reparent (fs) ;
}

static void 
mm_frame_mode_pair (ccl_display *display, guint action, GtkWidget *widget)
{
   CpkFrameSet *fs ;
   if (!GTK_CHECK_MENU_ITEM (widget)->active)
      return ;
   fs = CPK_FRAME_SET (display->fs) ;
   cpk_buttons_radio_set_active (&Mframe[2], 3, 1) ;
   fs->mode = FRAME_PAIR ;
   cpk_frame_set_reparent (fs) ;
}

static void 
mm_frame_mode_tile (ccl_display *display, guint action, GtkWidget *widget)
{
   if (!GTK_CHECK_MENU_ITEM (widget)->active)
      return ;  
   cpk_buttons_radio_set_active (&Mframe[2], 3, 2) ;
   /* if condition avoid crash on start */
   if (display->fs) {
      CpkFrameSet *fs = CPK_FRAME_SET (display->fs) ;
      fs->mode = FRAME_TILE ;
      cpk_frame_set_reparent (fs) ;
   }
}

static void 
mm_frame_tile_grid (ccl_display *display, guint action, GtkWidget *widget)
{
   CpkFrameSet *fs = CPK_FRAME_SET (display->fs) ;
   if (!GTK_CHECK_MENU_ITEM (widget)->active)
      return ;
   fs->tile_mode = TILE_GRID ;
   cpk_frame_set_reparent (fs) ;
}

static void 
mm_frame_tile_col (ccl_display *display, guint action, GtkWidget *widget)
{
   CpkFrameSet *fs = CPK_FRAME_SET (display->fs) ;
   if (!GTK_CHECK_MENU_ITEM (widget)->active)
      return ;
   fs->tile_mode = fs->last_tile_mode = TILE_COLUMNS ;
   cpk_frame_set_reparent (fs) ;
}

static void 
mm_frame_tile_row (ccl_display *display, guint action, GtkWidget *widget)
{
   CpkFrameSet *fs = CPK_FRAME_SET (display->fs) ;
   if (!GTK_CHECK_MENU_ITEM (widget)->active)
      return ;
   fs->tile_mode = fs->last_tile_mode = TILE_ROW ;
   cpk_frame_set_reparent (fs) ;
}

/*----------------------------------------------------------------------------
 * 							Buttons Actions
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 							Edit Menu
 *--------------------------------------------------------------------------*/

static void 
ccl_edit (GtkWidget *widget, ccl_display *display)
{
   guint i, j ;
   CPKDEBUG ("ccl_edit") ;

   j = NOACT ;
   for (i = POINTER; i <= CROSSHAIR; i++) 
      if (Medit[i].button == widget)
         j = i ;
   if (display->pointer_action == j || j == NOACT)
      return ;

   /* This invokes the call to the requested mode */
   gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (Medit[j].item), TRUE) ;
}

/*----------------------------------------------------------------------------
 * 							Frame Menu
 *--------------------------------------------------------------------------*/

static void
ccl_frame_status (gboolean status)
{
   gint i ;
   gint b[] = {1, 5, 6} ;
   for (i = 0; i < 3; i++) {
      Mframe[b[i]].status = status ;
      if (Mframe[b[i]].button)
         gtk_widget_set_sensitive (Mframe[b[i]].button, status) ;
      gtk_widget_set_sensitive (Mframe[b[i]].item, status) ;
   }
}

static void
ccl_frame_new (GtkWidget *widget, ccl_display *display)
{
   CpkFrameSet *fs = CPK_FRAME_SET (display->fs) ;
   if (fs->frames)
      cpk_frame_set_select (GTK_WIDGET (cpk_frame_add (fs->frames)), fs) ;
   else 
      cpk_frame_set_select (GTK_WIDGET (cpk_frame_set_initialize (fs)), fs) ;
   ccl_frame_status (TRUE) ;
}

static void
ccl_frame_del (GtkWidget *widget, ccl_display *display)
{
   CpkFrameSet *fs = CPK_FRAME_SET (display->fs) ;
   if (!fs->current)
      return ;
   if (fs->current == fs->frames)
      fs->frames = fs->frames->next ;
   if (fs->reference == fs->current)
      fs->reference = fs->frames ;
   printf("Removing frame %s\n",CPK_PLOT (fs->current->plot)->label);
   fs->current = cpk_frame_remove (fs->current) ;
   printf("New current frame %s\n",CPK_PLOT (fs->current->plot)->label);
   if (fs->current)
      cpk_frame_set_select (GTK_WIDGET (fs->current), fs) ;
   else
      ccl_frame_status (FALSE) ;
}

static void
ccl_frame_prev (GtkWidget *widget, ccl_display *display)
{
   CpkFrameSet *fs = CPK_FRAME_SET (display->fs) ;
   if (fs->current)
      cpk_frame_set_select (GTK_WIDGET (cpk_frame_prev (fs->current)), fs) ;
}

static void
ccl_frame_next (GtkWidget *widget, ccl_display *display)
{
   CpkFrameSet *fs = CPK_FRAME_SET (display->fs) ;
   if (fs->current)
      cpk_frame_set_select (GTK_WIDGET (cpk_frame_next (fs->current)), fs) ;
}

static void 
ccl_frame_mode (GtkWidget *widget, ccl_display *display)
{
   CpkFrameSet *fs = CPK_FRAME_SET (display->fs) ;
   gint i, j ;
   gint b[] = {2, 3, 4} ;
   CPKDEBUG ("ccl_frame_mode") ;

   j = -1 ;
   for (i = 0; i < 3; i++) 
      if (Mframe[b[i]].button == widget)
         j = i ;
   if (fs->mode == j || j < 0)
      return ;

   /* This invokes the call to the requested mode */
   gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (Mframe[b[j]].item), TRUE) ;
}

