/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 					      Bologna Astronomical Observatory
 *----------------------------------------------------------------------------
 * File name 	: ccl_decode.c
 * Author 	: Paolo Montegriffo
 * Created on	: May, 2003	
 * Language	: ANSI C
 * Description	: Decode catacal arguments list
 * Version      : 2.1.19 - May 10, 2003
 *--------------------------------------------------------------------------*/

#if HAVE_CONFIG_H
#  include "config.h"
#endif

#include "common.h"
#include "catalib.h"
#include "cp_messages.h"
#include "ccl_decode.h"

#include <gtk/gtk.h>
#include "ccl_init.h"

/*----------------------------------------------------------------------------
 * 							Function prototypes
 *--------------------------------------------------------------------------*/

extern void      gui_update        (void) ;

static void      usage             () ;
static void      husage            () ;
static void      HelloWorld        () ;
static char    **arg_file_name_get (int	     *argc, 
                                    char   ***argv) ;

/*----------------------------------------------------------------------------
 * 								Defines
 *--------------------------------------------------------------------------*/

#ifndef MIN
#define MIN(a,b) (((a)<=(b)) ? (a) : (b))
#endif
#define MAXLENINPUT 2048

/*----------------------------------------------------------------------------
 * 							Global variables	
 *--------------------------------------------------------------------------*/

extern    char      *version ;

int       verbose   = 6 ;

/*----------------------------------------------------------------------------
 *							Functions code
 *--------------------------------------------------------------------------*/

static void 
husage ()
{
   fprintf (stderr,"\n Type 'CataCal -h' for a brief description of command usage\n") ;
   fprintf (stderr,"\n See 'man CataCal' for a more detailed description of command usage\n") ;
   exit (1) ;
}

static void 
usage ()
{

   fprintf (stderr,"*** ATTENZIONE : LISTA NON AGGIORNATA !!!\n");

   fprintf (stderr,"\n See 'man CataCal' for a more detailed description of command usage\n") ;
   exit (1) ;
}

static void 
HelloWorld()
{
   cp_message(0,1,"\n\
\t _______________________________________________________________ \n\
\to\t\t\t\t\t\t\t\to\n\
\to  \33[1;34mCata\33[31mCal\33[0m: Catalogues Calibrator \t\t\to\n\
\to  Version:  %s\t\t\t\to\n\
\to  Author:   P. Montegriffo - Bologna Astronomical Observatory\to\n\
\to_______________________________________________________________o\n\
\n",version) ;
}

void
ccl_arg_check_help (int argc, char **argv)
{
   int i ;
   for (i = 1; i < argc; i++) {
      if (strcmp ("-h", argv[i]) == 0 || 
          strcmp ("--help", argv[i]) == 0) 
         usage () ;
   }
   return ;
}

int
ccl_arg_check_display (int *argc, char ***argv)
{
   int status = 0 ;
   int i, j, k;
   if (argc && argv) {
      for (i = 1; i < *argc; i++) {
         if (strcmp ("-nodisplay", (*argv)[i]) == 0) {
            (*argv)[i] = NULL ;
            status = 1 ;
         }
      }
      for (i = 1; i < *argc; i++) {
         for (k = i; k < *argc; k++)
            if ((*argv)[k] != NULL)
	      break ;
         if (k > i) {
            k -= i ;
            for (j = i + k; j < *argc; j++)
               (*argv)[j-k] = (*argv)[j];
            *argc -= k;
         }
      }
   }
   return status ;
}

void
ccl_arg_list_decode (int ac, char **av)
{
   int       i ;
   char    **files = (char**)NULL ;
   char      line[MAX_RECORD_LENGTH] = "\n" ;

   /* Maps input files */
   files = arg_file_name_get (&ac, &av) ;
   for (i = 0; files[i]; i++)
      ;

   HelloWorld () ;
   cp_message (1, 2, "Mapping input files\n") ;

   (void) free_carray (files, i) ;

   /* Set defaults accordling to coordinate units */

   /* Process input column strings */


   /* Decode remaining arguments list */
   for (av++; --ac > 0; av++) {
      char * str ;
      /* sprintf(command,"%s %s",command,*av) ; */
      if (*(str = *av) == '-') {
         char c ;
         c = *++str ;
         switch (c) {
	 case 'c':
	    break ;
	 case 'f':
	    break ;
         case 'e':
	    cp_error("Argument '%s' not recognized",*av);
	    husage() ;
	    break ;
	 case 'm':
	    break ;
	 case 'n':
	    cp_error ("Argument '%s' not recognized", *av) ;
	    husage () ;
	    break ;
	 case 't':
	    cp_error ("Argument '%s' not recognized",*av) ;
	    husage () ;
	    break ;
	 case 'd':
	    cp_error ("Argument '%s' not recognized",*av) ;
	    husage () ;
	    break ;
	 case 's':
	    cp_error ("Argument '%s' not recognized",*av) ;
	    husage () ;
	    break ;
	 default:
            cp_error ("Argument '%s' not recognized",*av) ;
	    husage () ;
            break ;
	 }
      }
      else {
         if (ac > 0 && *(str = *av) != '-') {
            cp_error ("Argument '%s' not recognized",*av) ;
	    husage () ;
         }
         ac++ ;
         av-- ;
      }
   }

   /* Output some infos */

   return ;
}


/*-----------------------------------------------------------------------------
 * Function  : arg_file_name_get
 * In        : standard argument list
 * Out       : NULL terminated file names array
 * Purpouse  : 
 * Notes     : file names are stripped from input argument list
 *             Intercept also varbose level
 *             
 *---------------------------------------------------------------------------*/

static char **
arg_file_name_get (int *argc, char ***argv)
{
   char **ret = (char**)NULL ;
   char   infile[MAXLENINPUT] = "\0" ;
   char   outfile[MAXLENINPUT] = "\0" ;
   char   line[MAX_RECORD_LENGTH] = "\n" ;
   int    nf ;
   if (argc && argv) {
      int i, j, k;
      for (i = 1; i < *argc;) {
         if (strcmp ("-l", (*argv)[i]) == 0) {
            (*argv)[i] = NULL ;
            if (strcmp ("-", (*argv)[i+1]) == 0) {
               i += 1 ;
               (*argv)[i] = NULL ;
               /* Read file list from STDIN */
	       cp_warning ("Reading file list from STDIN") ;
	       while (fgets (line, MAX_RECORD_LENGTH, stdin)!=NULL) {
	          strip_newline (line) ;
	          strcat (infile, line) ;
		  strcat (infile, " ") ;
	       }
            }
         }
         else if (strcmp ("-o", (*argv)[i]) == 0) {
            (*argv)[i] = NULL ;
            i += 1 ;
            if (i==*argc || !strncmp ("-", (*argv)[i], (size_t)1)) {
               cp_error ("missing output file name: exiting") ;
               exit (1) ;
            }
	    strcpy (outfile, (*argv)[i]) ;
            (*argv)[i] = NULL ;
         }
         else if (!access ((*argv)[i], F_OK)) {
            strcat (infile, (*argv)[i]) ;
	    strcat (infile, " ") ;
            (*argv)[i] = NULL ;
         }
         i += 1 ;
      }
      for (i = 1; i < *argc; i++) {
         for (k = i; k < *argc; k++)
            if ((*argv)[k] != NULL)
	      break ;
         if (k > i) {
            k -= i ;
            for (j = i + k; j < *argc; j++)
               (*argv)[j-k] = (*argv)[j];
            *argc -= k;
         }
      }
      /* Intercept verbose level */
      for (i = 1; i < *argc;) {
         if (strcmp ("-v", (*argv)[i]) == 0 ||
             strcmp ("--verbose", (*argv)[i]) == 0) {
            (*argv)[i] = NULL ;
	    if (i+1 < *argc) {
               if (is_integer ((*argv)[i+1]) == 1) {
                  i += 1 ;
	          verbose = atoi ((*argv)[i]) ;
                  (*argv)[i] = NULL ;
               }
               else verbose = 0 ;
	    }
            else
               verbose = 0 ;
         }
         i += 1 ;
      }
      for (i = 1; i < *argc; i++) {
         for (k = i; k < *argc; k++)
            if ((*argv)[k] != NULL)
	      break ;
         if (k > i) {
            k -= i ;
            for (j = i + k; j < *argc; j++)
               (*argv)[j-k] = (*argv)[j];
            *argc -= k;
         }
      }
   }
   if (strlen (infile) >= MAXLENINPUT) {
      cp_error ("input file list too long(%d characters): exiting",
      MAXLENINPUT) ;
      exit (1) ;
   }
   if (strlen (infile) == 0) {
      cp_error ("missing input file list: exiting") ;
      exit (1) ;
   }
   ret = tokenize (infile," ",&nf) ;
   ret = (char**) realloc (ret,(size_t)(nf+1)*sizeof(char*)) ;
   ret[nf] = (char)NULL ;

   return ret ;
}


