/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/

#ifndef CPK_BIBUTTON_H
#define CPK_BIBUTTON_H 1

#include "common.h"

#include <gdk/gdk.h>
#include <gtk/gtkbutton.h>

BEGIN_C_DECLS

#define TYPE_CPK_BIBUTTON            (cpk_bibutton_get_type ())
#define CPK_BIBUTTON(obj)            (GTK_CHECK_CAST ((obj), TYPE_CPK_BIBUTTON, CpkBibutton))
#define CPK_BIBUTTON_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), TYPE_CPK_BIBUTTON, CpkBibuttonClass))
#define CPK_IS_BIBUTTON(obj)         (GTK_CHECK_TYPE ((obj), TYPE_CPK_BIBUTTON))
#define CPK_IS_BIBUTTON_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), TYPE_CPK_BIBUTTON))

enum {
   LEFT_PRESSED,
   RIGHT_PRESSED,
   BIBUTTON_LAST_SIGNAL
} ;

typedef struct _CpkBibutton	  CpkBibutton ;
typedef struct _CpkBibuttonClass  CpkBibuttonClass ;

struct _CpkBibutton 
{
   GtkButton      button ;
   GtkAdjustment *adjustment;
   GtkWidget     *pixmap ;
   GtkWidget     *popup ;
   gint           w, h ;

   guint32 timer;
   guint32 pop_timer;
  
   gfloat climb_rate;
   gfloat timer_step;

   guint need_timer : 1;
   guint timer_calls : 3;

   guint in_child : 2;
   guint click_child : 2;
   guint mbutton : 2;
   guint wrap : 1;

   guint lactive : 1;
   guint ractive : 1;
   GdkWindow *event_window ;
} ;

struct _CpkBibuttonClass
{
   GtkButtonClass parent_class ;
   void (* lpressed) (CpkBibutton *button);
   void (* rpressed) (CpkBibutton *button);
} ;

extern  GtkType	       cpk_bibutton_get_type       (void) ;
extern  GtkWidget     *cpk_bibutton_new	           (GtkWidget     *parent, 
                                                    gchar        **xpm_data, 
                                                    const gchar   *lab1,
                                                    const gchar   *lab2,
                                                    GtkAdjustment *adjustment) ;
extern  void           cpk_bibutton_left_pressed   (CpkBibutton   *bibutton) ;
extern  void           cpk_bibutton_right_pressed  (CpkBibutton   *bibutton) ;

extern  void           cpk_bibutton_set_adjustment (CpkBibutton   *bibutton, 
                                                    GtkAdjustment *adjustment) ;
extern  GtkAdjustment *cpk_bibutton_get_adjustment (CpkBibutton   *bibutton) ;

END_C_DECLS

#endif /* !CPK_BIBUTTON_H */
