/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 							Includes
 *--------------------------------------------------------------------------*/

#if HAVE_CONFIG_H
#  include "config.h"
#endif

#include "common.h"

#include <gdk/gdk.h>
#include <gtk/gtkbutton.h>

#include <gtk/gtk.h>


#include "cpk_gtk_util.h"

#include "cpk_buttons.h"

/*----------------------------------------------------------------------------
 * 							Function prototypes
 *--------------------------------------------------------------------------*/

static void cpk_buttons_class_init     (CpkButtonsClass  *klass) ;
static void cpk_buttons_init           (CpkButtons       *b) ;

static void cpk_buttons_2_get          (GtkWidget        *toggle, 
                                        CpkButtons       *b) ;
static void cpk_buttons_reset          (GtkWidget        *toggle, 
                                        CpkButtons       *b) ;

/*----------------------------------------------------------------------------
 * 							Private types
 *--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
 * 							Global variables	
 *--------------------------------------------------------------------------*/

static GtkContainerClass *parent_class = NULL;

/* extern GtkItemFactoryEntry menu_items[] ; */
extern CpkItemFactory menu_items[] ;
extern guint nmenu_items ;

GtkItemFactory *item_factory ;

/*----------------------------------------------------------------------------
 * 							Private functions
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 							Defines
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 *							Function codes
 *--------------------------------------------------------------------------*/

GtkType
cpk_buttons_get_type (void)
{
  static GtkType buttons_type = 0;

  if (!buttons_type)
    {
      static const GtkTypeInfo buttons_info =
      {
        "CpkButtons",
        sizeof (CpkButtons),
        sizeof (CpkButtonsClass),
        (GtkClassInitFunc) cpk_buttons_class_init,
        (GtkObjectInitFunc) cpk_buttons_init,
        /* reserved_1 */ NULL,
        /* reserved_2 */ NULL,
        (GtkClassInitFunc) NULL,
      };

      buttons_type = gtk_type_unique (GTK_TYPE_BUTTON, &buttons_info);
    }
  return buttons_type;
}

static void
cpk_buttons_class_init (CpkButtonsClass *class)
{
   GtkObjectClass    *object_class;
   GtkWidgetClass    *widget_class;
   GtkContainerClass *container_class;

   object_class = (GtkObjectClass*) class;
   widget_class = (GtkWidgetClass*) class;
   container_class = (GtkContainerClass*) class;

   parent_class = gtk_type_class (GTK_TYPE_BUTTON);
}

static void
cpk_buttons_init (CpkButtons *b)
{
   b->box = NULL ;
   b->box2 = NULL ;
   b->lastbox2 = NULL ;
   b->buttons = NULL ;
   b->n = 0 ;
   b->nsel = 0 ;
}

GtkWidget *
cpk_buttons_new (CpkButtons1 *buttons, gint n, gpointer data)
{
   GtkWidget  *w ;
   GtkWidget  *hbox, *toggle ;
   CpkButtons *b ;
   gint       i ;

   w =  GTK_WIDGET (gtk_type_new (TYPE_CPK_BUTTONS)) ;
   b = CPK_BUTTONS (w) ;

   b->buttons = buttons ;
   b->n = n ;
   b->p = data ;
   b->box = gtk_vbox_new (FALSE, 0) ;
   gtk_container_set_border_width (GTK_CONTAINER (b->box), 0);
   hbox = gtk_hbox_new (FALSE, 0) ;
   gtk_container_set_border_width (GTK_CONTAINER (hbox), 0);
   gtk_box_pack_start (GTK_BOX (b->box), hbox, FALSE, FALSE, 0) ;

   for (i = 0; i < b->n; i++) {
      toggle = gtk_toggle_button_new_with_label (b->buttons[i].label) ;
      gtk_container_add (GTK_CONTAINER (hbox), toggle);
      gtk_signal_connect (GTK_OBJECT (toggle), "toggled",
            GTK_SIGNAL_FUNC (cpk_buttons_2_get), b) ;
      b->buttons[i].button = toggle ;
   }
   b->box2 = gtk_hbox_new (FALSE, 0) ;
   gtk_container_set_border_width (GTK_CONTAINER (b->box2), 0);
   gtk_box_pack_start (GTK_BOX (b->box), b->box2, FALSE, FALSE, 0) ;

   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (b->buttons[0].button), TRUE) ;

   gtk_widget_show_all (b->box) ;

   return w ;
}

/* Map submenues depending on primary menu button selection */
static void 
cpk_buttons_2_get (GtkWidget *toggle, CpkButtons *b)
{
   GtkWidget *hbox, *button ;
   gint       i, j ;
   struct buttons_tag *buttons2 ;

   /* If select same button just mantains it active */
   if (!GTK_TOGGLE_BUTTON (toggle)->active) {
      /* Do nothing */
      gtk_signal_disconnect_by_func (GTK_OBJECT (toggle),
            GTK_SIGNAL_FUNC (cpk_buttons_2_get), b) ;
      gtk_signal_emit_stop_by_name (GTK_OBJECT (toggle), "toggled") ;
      gtk_toggle_button_set_active (
            GTK_TOGGLE_BUTTON (toggle), TRUE) ;
      gtk_signal_connect (GTK_OBJECT (toggle), "toggled",
            GTK_SIGNAL_FUNC (cpk_buttons_2_get), b) ;
      return ;
   }

   /* Get button1 number */
   j = 0 ;
   for (i = 0; i < b->n; i++) {
      if (b->buttons[i].button == toggle)
         j = i ;
      else {
         gtk_signal_disconnect_by_func (GTK_OBJECT (b->buttons[i].button),
               GTK_SIGNAL_FUNC (cpk_buttons_2_get), b) ;
         gtk_toggle_button_set_active (
               GTK_TOGGLE_BUTTON (b->buttons[i].button), FALSE) ;
         gtk_signal_connect (GTK_OBJECT (b->buttons[i].button), "toggled",
               GTK_SIGNAL_FUNC (cpk_buttons_2_get), b) ;
      }
   }

   /* Destroy previous buttons2 */
   if (b->lastbox2) {
      for (i = 0; i < b->buttons[b->nsel].nb; i++) 
         b->buttons[b->nsel].Mbutton[i].button = NULL;
      gtk_widget_destroy (b->lastbox2) ;
   }

   /* Map submenu buttons */
   b->lastbox2 = hbox = gtk_hbox_new (FALSE, 0)  ;
   gtk_box_pack_start (GTK_BOX (b->box2), hbox, TRUE, TRUE, 0) ;

   b->nsel = j ;
   for (i = 0; i < b->buttons[j].nb; i++) {
      if (b->buttons[j].Mbutton[i].code == 0) {
         button = gtk_button_new_with_label (b->buttons[j].Mbutton[i].label) ;
         gtk_widget_set_sensitive (button, b->buttons[j].Mbutton[i].status) ;
      }
      else {
         button = gtk_toggle_button_new_with_label (
               b->buttons[j].Mbutton[i].label) ;
         gtk_toggle_button_set_active (
               GTK_TOGGLE_BUTTON (button), b->buttons[j].Mbutton[i].status) ; 
         gtk_signal_connect (GTK_OBJECT (button), "toggled",
               GTK_SIGNAL_FUNC (cpk_buttons_reset), b) ;
      }
      gtk_signal_connect (GTK_OBJECT (button), "clicked",
            GTK_SIGNAL_FUNC (b->buttons[j].Mbutton[i].callback), b->p) ;
      b->buttons[j].Mbutton[i].button = button ;
      gtk_container_add (GTK_CONTAINER (hbox), button);
   }
   gtk_widget_show_all (hbox) ;
}

/* Reset Radio Buttons groups in submenues */
static void 
cpk_buttons_reset (GtkWidget *toggle, CpkButtons *b)
{
   gint       i, j, n, group ;
   struct buttons2_tag *buttons2 ;

   CPKDEBUG5 ("cpk_buttons_reset", 4) ;

   buttons2 = b->buttons[b->nsel].Mbutton ;
   n = b->buttons[b->nsel].nb ;
   j = 0 ;
   for (i = 0; i < n; i++) 
      if (buttons2[i].button == toggle)
         j = i ;

   if (buttons2[j].code)
      buttons2[j].status = gtk_toggle_button_get_active (
            GTK_TOGGLE_BUTTON (toggle)) ;

   /* This is for swap buttons */
   if (buttons2[j].code <= 0)
      return ;

   if (!GTK_TOGGLE_BUTTON (toggle)->active) {
      /* Do nothing */
      gtk_signal_disconnect_by_func (GTK_OBJECT (toggle), 
            GTK_SIGNAL_FUNC (cpk_buttons_reset), b) ;
      gtk_signal_disconnect_by_func (GTK_OBJECT (toggle), 
            GTK_SIGNAL_FUNC (buttons2[j].callback), b->p) ;
      gtk_signal_emit_stop_by_name (GTK_OBJECT (toggle), "toggled") ;
      gtk_toggle_button_set_active (
            GTK_TOGGLE_BUTTON (toggle), TRUE) ;
      gtk_signal_connect (GTK_OBJECT (toggle), "toggled",
            GTK_SIGNAL_FUNC (cpk_buttons_reset), b) ;
      gtk_signal_connect (GTK_OBJECT (toggle), "clicked", 
            GTK_SIGNAL_FUNC (buttons2[j].callback), b->p) ;
      buttons2[j].status = TRUE ;
      return ;
   }

   group = buttons2[j].code ;
   for (i = 0; i < n; i++)
      if (buttons2[i].code == group && i != j)
         if (GTK_TOGGLE_BUTTON (buttons2[i].button)->active) {
            gtk_signal_disconnect_by_func (GTK_OBJECT (buttons2[i].button),  
                  GTK_SIGNAL_FUNC (cpk_buttons_reset), b) ;
            gtk_signal_disconnect_by_func (GTK_OBJECT (buttons2[i].button), 
                  GTK_SIGNAL_FUNC (buttons2[i].callback), b->p) ;
/***/   gtk_signal_emit_stop_by_name (GTK_OBJECT (toggle), "toggled") ;
            gtk_toggle_button_set_active (
                  GTK_TOGGLE_BUTTON (buttons2[i].button), FALSE) ;
            gtk_signal_connect (GTK_OBJECT (buttons2[i].button), "toggled",
                  GTK_SIGNAL_FUNC (cpk_buttons_reset), b) ;
            gtk_signal_connect (GTK_OBJECT (buttons2[i].button), "clicked", 
                  GTK_SIGNAL_FUNC (buttons2[i].callback), b->p) ;
            buttons2[i].status = gtk_toggle_button_get_active (
                  GTK_TOGGLE_BUTTON (buttons2[i].button)) ;
         }
}

void
cpk_buttons_radio_set_active (CpkButtons2 *button, gint n, gint j)
{
   GtkWidget *toggle = NULL ;
   gint       i ;

   CPKDEBUG5 ("cpk_buttons_radio_set_active", 5) ;

   if (j >= 0 && j < n && button[j].status)
      return ;

   /* First set buttons flag */
   for (i = 0; i < n; i++) 
      button[i].status = FALSE ;

   /* Then activate correct button: this invokes cxc_buttons_reset module */
   if (j >= 0 && j < n) {
      button[j].status = TRUE ;
      if (button[j].button)
         toggle = button[j].button ;
      if (GTK_IS_WIDGET (toggle)) 
         if (!GTK_TOGGLE_BUTTON (toggle)->active) 
            gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle), 
                  button[j].status) ;
   }
}


GtkWidget * 
cpk_menu_main (GtkWidget *window, gpointer data)
{
   GtkAccelGroup  *accel_group ;
   gint i;

  /* gtk_item_factory_parse_rc (".gtkrc") ; */

   accel_group = gtk_accel_group_new () ;

   /* This function initializes the item factory. */
   item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>", 
         accel_group) ;

   gtk_widget_set_name (
   gtk_item_factory_get_widget (item_factory, "<main>"), "cpk-itemfactory") ;

   /* This function generates the menu items. Pass the item factory,
      the number of items in the array, the array itself, and any
      callback data for the the menu items. */
   /* gtk_item_factory_create_items (item_factory, nmenu_items, menu_items, data); */
   for (i = 0; i < nmenu_items; i++) {
      gtk_item_factory_create_item (item_factory, &menu_items[i].ifactory_entry, data, 1) ;
      if (menu_items[i].button) 
         menu_items[i].button->item = gtk_item_factory_get_item (item_factory,
               menu_items[i].ifactory_entry.path) ;
   }

   /* Attach the new accelerator group to the window. */
   gtk_window_add_accel_group (GTK_WINDOW (window), accel_group) ;

   /* Finally, return the actual menu bar created by the item factory. */ 
   return gtk_item_factory_get_widget (item_factory, "<main>") ;
}
