/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/

#ifndef CPK_DRAW_H
#define CPK_DRAW_H 1

#include "common.h"

BEGIN_C_DECLS

#include <gdk/gdk.h>
#include <gtk/gtkwidget.h>

#define TYPE_CPK_DRAW                (cpk_draw_get_type ())
#define CPK_DRAW(obj)                (GTK_CHECK_CAST ((obj), TYPE_CPK_DRAW, CpkDraw))
#define CPK_DRAW_CLASS(klass)        (GTK_CHECK_CLASS_CAST ((klass), TYPE_CPK_DRAW, CpkDrawClass))
#define CPK_IS_DRAW(obj)             (GTK_CHECK_TYPE ((obj), TYPE_CPK_DRAW))
#define CPK_IS_DRAW_CLASS(klass)     (GTK_CHECK_CLASS_TYPE ((klass), TYPE_CPK_DRAW))


enum {
   MARK_CHANGED,
   DRAW_LAST_SIGNAL
} ;

typedef struct _CpkDraw	      CpkDraw ;
typedef struct _CpkDrawClass  CpkDrawClass ;

struct _CpkDraw
{
   GtkWidget  widget ;
   gint       n ;
   gint       marked ;
   gint       nx, ny ;
   gdouble    dx, dy ;
   gdouble    xcur, ycur ;
   gdouble    zoom ;
   gdouble    rot ;
} ;

struct _CpkDrawClass
{
   GtkWidgetClass parent_class ;

   /* Signals for notification/filtering of changes */
   void (* mark_changed)       (CpkDraw    *draw) ;

   /* Bindings actions */

   /* */
   gdouble (* x)                 (CpkDraw    *draw,
                                  gint        i) ;
   gdouble (* y)                 (CpkDraw    *draw,
                                  gint        i) ;
   void    (* draw_geometry)     (CpkDraw    *draw,
                                  GtkWidget  *darea,
                                  gdouble     x,
                                  gdouble     y) ;
   void    (* draw_button_press) (CpkDraw    *draw,
                                  GtkWidget  *darea,
                                  gdouble     x,
                                  gdouble     y,
                                  GdkModifierType state) ;
   void    (* draw_paint)        (CpkDraw    *draw,
                                  GdkPixmap  *pixmap,
                                  gint        moving) ;
   void    (* draw_pan)          (CpkDraw    *draw) ;
   void    (* draw_pan_border)   (CpkDraw    *draw,
                                  GtkWidget  *darea) ;
   void    (* draw_mgf)          (CpkDraw    *draw) ;

   void    (* world2draw)        (CpkDraw    *draw,
                                  gdouble     xw,
                                  gdouble     yw,
                                  gdouble    *xd,
                                  gdouble    *yd) ;
   void    (* draw2world)        (CpkDraw    *draw,
                                  gdouble     xd,
                                  gdouble     yd,
                                  gdouble    *xw,
                                  gdouble    *yw) ;
   void    (* world2pan)         (CpkDraw    *draw,
                                  gdouble     xw,
                                  gdouble     yw,
                                  gdouble    *xp,
                                  gdouble    *yp) ;
   void    (* pan2world)         (CpkDraw    *draw,
                                  gdouble     xp,
                                  gdouble     yp,
                                  gdouble    *xw,
                                  gdouble    *yw) ;
   void    (* world2mgf)         (CpkDraw    *draw,
                                  gdouble     xw,
                                  gdouble     yw,
                                  gdouble    *xm,
                                  gdouble    *ym) ;

} ;

extern GtkType     cpk_draw_get_type          (void) ;
extern void        cpk_draw_world_to_draw     (CpkDraw       *draw,
                                               gdouble        xw,
                                               gdouble        yw,
                                               gdouble       *xd,
                                               gdouble       *yd) ;

extern void        cpk_draw_draw_to_world     (CpkDraw       *draw,
                                               gdouble        xd,
                                               gdouble        yd,
                                               gdouble       *xw,
                                               gdouble       *yw) ;

extern void        cpk_draw_world_to_pan      (CpkDraw       *draw,
                                               gdouble        xw,
                                               gdouble        yw,
                                               gdouble       *xp,
                                               gdouble       *yp) ;
extern void        cpk_draw_pan_to_world      (CpkDraw       *draw,
                                               gdouble        xp,
                                               gdouble        yp,
                                               gdouble       *xw,
                                               gdouble       *yw) ;
extern void        cpk_draw_world_to_mgf      (CpkDraw       *draw,
                                               gdouble        xw,
                                               gdouble        yw,
                                               gdouble       *xm,
                                               gdouble       *ym) ;


extern void        cpk_draw_button_press      (CpkDraw         *draw,
                                               GtkWidget       *darea,
                                               gdouble          x,
                                               gdouble          y,
                                               GdkModifierType  state) ;
extern void        cpk_draw_button_release    (CpkDraw         *draw,
                                               GtkWidget       *darea,
                                               gdouble          x,
                                               gdouble          y) ;
extern void        cpk_draw_geometry          (CpkDraw         *draw,
                                               GtkWidget       *darea,
                                               gdouble          x,
                                               gdouble          y) ;
extern void        cpk_draw_paint             (CpkDraw         *draw,
                                               GdkPixmap       *pixmap,
                                               gint             moving) ;
extern void        cpk_draw_pan               (GtkWidget       *widget) ;
extern void        cpk_draw_mgf               (GtkWidget       *widget) ;
extern void        cpk_draw_pan_border        (CpkDraw         *draw,
                                               GtkWidget       *darea) ;
extern gint        cpk_draw_find_nearest      (CpkDraw         *draw,
                                               gdouble          x,
                                               gdouble          y) ;
extern void        cpk_draw_draw_mark         (CpkDraw         *draw,
                                               GtkWidget       *widget,
                                               gint             id,
                                               gboolean         emit) ;

END_C_DECLS

#endif /* !CPK_DRAW_H */
