/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/

#ifndef CPK_FRAME_H
#define CPK_FRAME_H 1

#include "common.h"
#include "cpk_gtk_cursorutil.h"

#include <gdk/gdk.h>
#include <gtk/gtkvbox.h>
#include <gtk/gtkhbox.h>

BEGIN_C_DECLS

#define TYPE_CPK_FRAME                (cpk_frame_get_type ())
#define CPK_FRAME(obj)                (GTK_CHECK_CAST ((obj), TYPE_CPK_FRAME, CpkFrame))
#define CPK_FRAME_CLASS(klass)        (GTK_CHECK_CLASS_CAST ((klass), TYPE_CPK_FRAME, CpkFrameClass))
#define CPK_IS_FRAME(obj)             (GTK_CHECK_TYPE ((obj), TYPE_CPK_FRAME))
#define CPK_IS_FRAME_CLASS(klass)     (GTK_CHECK_CLASS_TYPE ((klass), TYPE_CPK_FRAME))

#define TYPE_CPK_FRAME_SET            (cpk_frame_set_get_type ())
#define CPK_FRAME_SET(obj)            (GTK_CHECK_CAST ((obj), TYPE_CPK_FRAME_SET, CpkFrameSet))
#define CPK_FRAME_SET_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), TYPE_CPK_FRAME_SET, CpkFrameSetClass))
#define CPK_IS_FRAME_SET(obj)         (GTK_CHECK_TYPE ((obj), TYPE_CPK_FRAME_SET))
#define CPK_IS_FRAME_SET_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), TYPE_CPK_FRAME_SET))

enum {
   TILE_GRID,
   TILE_COLUMNS,
   TILE_ROW
} ;

enum mode_tag {
   FRAME_SINGLE = 0,
   FRAME_PAIR,
   FRAME_TILE,
   FRAME_BLINK,
   FRAME_MERGE, 
   FRAME_COINC
} ;

enum {
   FRAME_ADDED,
   FRAME_REMOVED,
   FRAME_FOCUS,
   FRAME_SELECT,
   FRAME_LAST_SIGNAL
} ;
enum {
   FRAME_SET_FIRST_SIGNAL,
   FRAME_SET_LAST_SIGNAL
} ;

typedef struct _CpkFrame	  CpkFrame ;
typedef struct _CpkFrameClass     CpkFrameClass ;
typedef struct _CpkFrameSet	  CpkFrameSet ;
typedef struct _CpkFrameSetClass  CpkFrameSetClass ;

struct _CpkFrame 
{
   GtkFrame    frame ;
   GtkWidget  *area ;
   GtkWidget  *plot ;
   GtkWidget  *hrule ;
   GtkWidget  *vrule ;
   gboolean    show ;
   gboolean    border ;
   CpkFrame   *prev ;
   CpkFrame   *next ;

} ;

struct _CpkFrameClass
{
   GtkFrameClass parent_class ;
   void (* frame_added)   (CpkFrame *frame);
   void (* frame_removed) (CpkFrame *frame);
   void (* frame_focus) (CpkFrame *frame);
   void (* frame_select) (CpkFrame *frame);
} ;

struct _CpkFrameSet
{
   GtkHBox        box ;
   GtkWidget     *paner_box ;
   GtkWidget     *paner ;

   /* Magnifier */
   GtkWidget     *magnifier_box ;
   GtkWidget     *magnifier ;
   guint          mgf_factor ;
   guint32        mgf_timer;

   /* Frames */
   GtkWidget     *frame_box ;
   GtkWidget     *frame_hbox ;
   GtkWidget     *frame_vbox ;
   GtkWidget     *frame_tbox ;
   guint          nframes ;         /*** serve ?*/
   guint          mode ;
   guint          tile_mode ;
   guint          last_tile_mode ;
   CpkCursorType  cursor ;
   guint          p_action ;

   CpkFrame      *frames ;
   GdkGC         *gc ;              /*** serve ?*/

   CpkFrame      *focused ;        /* frame focalizzato */
   CpkFrame      *reference ;      /* frame riferimento */
   CpkFrame      *current ;        /* frame attivo */
   CpkFrame      *old_current ;
} ;

struct _CpkFrameSetClass
{
   GtkBoxClass parent_class ;
   void (* cazcaz) (CpkFrameSet *frame_set);
} ;



extern  GtkType	       cpk_frame_get_type          (void) ;
extern  GtkWidget     *cpk_frame_new	           (void) ;
extern  void           cpk_frame_destroy           (CpkFrame      *frame) ;
extern  CpkFrame      *cpk_frame_first             (CpkFrame      *frame) ;
extern  CpkFrame      *cpk_frame_prev              (CpkFrame      *frame) ;
extern  CpkFrame      *cpk_frame_next              (CpkFrame      *frame) ;
extern  CpkFrame      *cpk_frame_last              (CpkFrame      *frame) ;
extern  CpkFrame      *cpk_frame_add               (CpkFrame      *frame) ;
extern  CpkFrame      *cpk_frame_remove            (CpkFrame      *frame) ;

extern  GtkType	       cpk_frame_set_get_type      (void) ;
extern  GtkWidget     *cpk_frame_set_new	   (void) ;
extern  void           cpk_frame_set_reparent      (CpkFrameSet   *fs) ;
extern  void           cpk_frame_set_select        (GtkWidget     *w, 
                                                    CpkFrameSet   *fs) ;
extern  CpkFrame      *cpk_frame_set_initialize    (CpkFrameSet   *fs) ;
extern  void           cpk_frame_set_cursor        (CpkFrameSet   *fs) ;

END_C_DECLS

#endif /* !CPK_FRAME_H */
