/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/

#ifndef CPK_GTK_UTIL_H
#define CPK_GTK_UTIL_H 1

#include "common.h"

BEGIN_C_DECLS

/*----------------------------------------------------------------------------
 * 							Private types
 *--------------------------------------------------------------------------*/

typedef struct color_tag {
   int r ;
   int g ;
   int b ;
} Color ;

typedef enum {
   CPK_CIRCLE,
   CPK_PLUS,
   CPK_CROSS
} cpk_symb ;

typedef struct {
   GtkWidget *entry ;
   GtkWidget *units ;
   gdouble    value ;
   gdouble    tunits ;
} uvalue ;

/*----------------------------------------------------------------------------
 * 							Global variables	
 *--------------------------------------------------------------------------*/

extern Color  red, green, blue, yellow, black, white ;
extern gdouble   tunits ;

/*----------------------------------------------------------------------------
 * 							Defines	
 *--------------------------------------------------------------------------*/

#ifndef M_PI
#define M_PI    3.14159265358979323846
#endif   /* M_PI */

#ifndef TWOPI
#define TWOPI (2.0*M_PI)
#endif

#ifndef DEGTORAD
#define DEGTORAD(a) ((a)*TWOPI/360.0)
#endif

#define MAXPATHLEN 2048

/*----------------------------------------------------------------------------
 * 							Function prototypes
 *--------------------------------------------------------------------------*/

extern    void           set_font              (gchar          *font_str) ;
extern    void           set_color             (GtkWidget      *widget,
                                                GdkGC          *gc,
                                                Color           col) ;
extern    void           set_color_sv          (GtkWidget      *widget,
                                                GdkGC          *gc,
                                                Color           col,
                                                gfloat          sat,
                                                gfloat          val) ;
extern    GdkColor      *get_color             (Color           col) ;

extern    void           cpk_set_fg_color      (GtkWidget      *widget,
                                                GdkGC          *gc,
                                                const gchar    *spec) ;
extern    void           cpk_set_bg_color      (GtkWidget      *widget,
                                                GdkGC          *gc,
                                                const gchar    *spec) ;
extern    void           cpk_set_ns_color      (GtkWidget      *widget,
                                                GdkGC          *gc,
                                                gfloat          f) ;

extern    void           draw_circle           (GdkPixmap      *pixmap,
                                                GdkGC          *gc,
                                                gdouble         x,
                                                gdouble         y,
                                                gdouble         r,
                                                gint            nv) ;
extern    void           cpk_plot_symb         (GdkPixmap      *pixmap,
                                                GdkGC          *gc,
                                                cpk_symb        type,
                                                gdouble         x,
                                                gdouble         y,
                                                gdouble         r_in,
                                                gdouble         r_out,
                                                gint            nv) ;
extern    void           cpk_plot_seg          (GdkPixmap      *pixmap,
                                                GdkGC          *gc,
                                                gdouble         x0,
                                                gdouble         y0,
                                                gdouble         x1,
                                                gdouble         y1) ;
extern    void           cpk_circle            (GdkPoint        points[],
                                                gdouble         x,
                                                gdouble         y,
                                                gdouble         r,
                                                gdouble         a0,
                                                gint            nv) ;
extern    void           cpk_poly_plus         (GdkPoint        points[],
                                                gint            x,
                                                gint            y,
                                                gint            ws,
                                                gint            wl) ;
extern    void           cpk_poly_cross        (GdkPoint        points[],
                                                gint            x,
                                                gint            y,
                                                gint            ws,
                                                gint            wl) ;

extern    GtkWidget     *cpk_warning           (GtkWidget      *window,
                                                gchar          *message) ;

extern    void           entry_changed         (GtkWidget      *entry,
                                                gchar          *text) ;
extern    void           entry_focus_out       (GtkWidget      *entry,
                                                GdkEvent       *event,
                                                gpointer        data) ;
extern    gint           cpk_confirm           (GtkWidget      *window,
                                                gchar          *title,
                                                gchar          *msg) ;
extern    GtkWidget     *make_menu_item        (gchar          *name,
                                                GtkSignalFunc   callback,
                                                gpointer        data) ;

extern    GtkWidget     *cpk_xpm_box_new       (GtkWidget      *parent,
                                                gchar         **xpm_data,
                                                const gchar    *text) ;
extern    GtkWidget     *cpk_get_units         (uvalue         *uval,
                                                gboolean        spheric) ;
extern    void           cpk_entry_with_units  (uvalue         *uval,
                                                gdouble         value, 
                                                gboolean        spheric) ;

extern    gchar         *cpk_get_file_name     (GtkWidget      *widget,
                                                char           *user_buf) ;
extern    void           cpk_get_file_entry    (GtkWidget      *entry) ;

extern    GtkWidget     *cpk_icon_button       (GtkWidget      *parent,
                                                gchar         **xpm_data,
                                                const gchar    *text,
                                                const gchar    *tip) ;
extern    GtkWidget     *cpk_console_new       (GtkWidget      *window) ;
/*----------------------------------------------------------------------------*/
END_C_DECLS

#endif /* !CXC_GTK_UTIL_H */
