/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 							Includes
 *--------------------------------------------------------------------------*/

#if HAVE_CONFIG_H
#  include "config.h"
#endif

#include "common.h"

#include <math.h>

#include <gtk/gtk.h>

#include "cxc_data.h"
#include "cxc_frames.h"
#include "cpk_gtk_util.h"
#include "cataxcorr.h"
#include "cataxcorr_p.h"
/* #include "platemodel.h" */
#include "cxc_gtk_main.h"
#include "cxc_align_set.h"

/*----------------------------------------------------------------------------
 * 							Global variables	
 *--------------------------------------------------------------------------*/

extern boolean   suspend ;
extern gdouble   sph_units ;
extern gdouble   tunits ;

/*----------------------------------------------------------------------------
 * 							Private Types
 *--------------------------------------------------------------------------*/

typedef struct tmp_tag {
   GtkWidget   *main_window ;
   GtkWidget   *dialog ;
   Sfind       *FindSet ;
   gint         mode ;
   uvalue       uval ;
   cxc_display *display ;
} Fparam ;

/*----------------------------------------------------------------------------
 * 							Function prototypes
 *--------------------------------------------------------------------------*/

static void     get_page            (GtkNotebook      *notebook, 
                                     GtkNotebookPage  *page,
                                     gint              page_num, 
                                     gpointer          data) ;
static void     f_entry_changed     (GtkWidget        *entry, 
                                     Sfind            *FindSet) ;
static void     f_entry_focus_out   (GtkWidget        *entry, 
                                     GdkEvent         *event, 
                                     gpointer          data) ;
static void     align_frames_accept (GtkWidget        *widget, 
                                     Fparam           *FindParam) ;
static void     cancel              (GtkWidget        *widget, 
                                     Sfind            *FindSet) ;
static void     close_window        (GtkWidget        *window, 
                                     gpointer          data) ;
static void     get_buttons         (GtkWidget        *dialog, 
                                     Fparam           *FindParam) ;
static void     set_mode            (GtkNotebook      *notebook, 
                                     GtkNotebookPage  *page,
                                     gint              page_num, 
                                     Fparam           *FindParam) ;

/*----------------------------------------------------------------------------
 *							Function codes
 *--------------------------------------------------------------------------*/

static void
get_page (GtkNotebook *notebook, GtkNotebookPage *page, gint page_num, 
      gpointer data)
{
   Sfind *FindSet = data ;
   FindSet->page = page_num ;
}

void
get_align_val (Sfind *FindSet, CxcData *data)
{
   g_pars   *gparam = data->gparam ;
   gdouble   s ;
   guint     i, j ;
   gdouble   tol = MAX (0.5*data->list1->param->crow_rad, 
                        5.0*data->list1->param->eps) ;

   CPKDEBUG ("get_align_val") ;
   for (i = 0; i < FindSet->nfiles; i++) {
      j = FindSet->active[i] ;
      s = (data->f_set->info[j]->spheric) ? tunits : 1.0 ;
      sprintf (FindSet->fset[i].value[0], "%g", data->list[j]->param->eps * s) ;
      sprintf (FindSet->fset[i].value[1], "%g", data->list[j]->param->min_side * s) ;
      sprintf (FindSet->fset[i].value[2], "%g", data->list[j]->param->max_side * s) ;
      sprintf (FindSet->fset[i].value[3], "%g", data->list[j]->param->crow_rad * s) ;
      if (data->list[j]->param->scale == 1.0)
         sprintf (FindSet->fset[i].value[4], "%g", data->list[j]->param->scale) ;
      else
         sprintf (FindSet->fset[i].value[4], "%g", data->list[j]->param->scale * s) ;
      sprintf (FindSet->fset[i].value[5], "%g", tol * s) ;
      sprintf (FindSet->fset[i].value[6], "%g", data->list[j]->param->max_shift * s) ;
      sprintf (FindSet->fset[i].value[7], "%g", data->list[j]->param->crow_rad * s) ;
      sprintf (FindSet->fset[i].value[8], "%g", tol * s) ;
      sprintf (FindSet->fset[i].value[9], "%g", data->list[j]->param->max_shift * s) ;
      sprintf (FindSet->fset[i].value[10], "%g", data->list[j]->param->crow_rad * s) ;
      FindSet->fset[i].sph = (data->f_set->info[j]->spheric) ? 1 : 0 ;
   }
   sprintf (FindSet->value[0], "%.1f", gparam->t0_mag) ;
   sprintf (FindSet->value[1], "%.1f", gparam->tune) ;
   sprintf (FindSet->value[2], "%.1f", gparam->t0_mag) ;
   sprintf (FindSet->value[3], "%.1f", gparam->t0_mag) ;
}

void
put_find_text (Sfind *FindSet)
{
   guint      i, j ;

   for (i = 0; i < FindSet->nfiles; i++) 
      for (j = 0; j < 11; j++) 
         if (FindSet->fset[i].entries[j])
            gtk_entry_set_text (GTK_ENTRY (FindSet->fset[i].entries[j]), 
               FindSet->fset[i].value[j]) ;
}

void
set_align_val (Sfind *FindSet, CxcData *data)
{
   FileSet  *f_set = data->f_set ;
   g_pars   *gparam = data->gparam ;
   gdouble   s ;
   guint     i, j ;
   gchar    *text ;

   switch (FindSet->page) {
      case 0 :
         gparam->find = MATCH ;
         gparam->mosaic = FALSE ;
         for (i = 0; i < FindSet->nfiles; i++) {
            j = FindSet->active[i] ;
            if (!GTK_IS_WIDGET (FindSet->fset[i].entries[0])) 
               break ;
            s = (f_set->info[j]->spheric) ? tunits : 1.0 ;
            text = gtk_entry_get_text (GTK_ENTRY (FindSet->fset[i].entries[0])) ;
            data->list[j]->param->eps = atof (text) / s ;
            text = gtk_entry_get_text (GTK_ENTRY (FindSet->fset[i].entries[1])) ;
            data->list[j]->param->min_side = atof (text) / s ;
            text = gtk_entry_get_text (GTK_ENTRY (FindSet->fset[i].entries[2])) ;
            data->list[j]->param->max_side = atof (text) / s ;
            text = gtk_entry_get_text (GTK_ENTRY (FindSet->fset[i].entries[3])) ;
            data->list[j]->param->crow_rad = atof (text) / s ;
            text = gtk_entry_get_text (GTK_ENTRY (FindSet->fset[i].entries[4])) ;
            if (atof(text) != 1.0)
               data->list[j]->param->scale = atof (text) / s ;
            else
               data->list[j]->param->scale = 1.0 ;
         }
         text = gtk_entry_get_text (GTK_ENTRY (FindSet->entries[0])) ;
         gparam->t0_mag = atof (text) ;
         text = gtk_entry_get_text (GTK_ENTRY (FindSet->entries[1])) ;
         gparam->tune = atof (text) ;
         break ;
      case 1 :
         gparam->find = SHIFT ;
         gparam->mosaic = FALSE ;

         /* ***FIXME: funziona solo se nref = 0 *** */

         for (i = 1; i < FindSet->nfiles; i++) {
            j = FindSet->active[i] ;
            if (!GTK_IS_WIDGET (FindSet->fset[i].entries[5])) 
               break ;
            s = (f_set->info[j]->spheric) ? tunits : 1.0 ;
            text = gtk_entry_get_text (GTK_ENTRY (FindSet->fset[i].entries[5])) ;
            /* data->list[j]->param->eps = atof (text) / s ; */
            gparam->min_tol0 = atof (text) / s ;
            text = gtk_entry_get_text (GTK_ENTRY (FindSet->fset[i].entries[6])) ;
            data->list[j]->param->max_shift = atof (text) / s ;
            text = gtk_entry_get_text (GTK_ENTRY (FindSet->fset[i].entries[7])) ;
            data->list[j]->param->crow_rad = atof (text) / s ;
         }
         text = gtk_entry_get_text (GTK_ENTRY (FindSet->entries[2])) ;
         gparam->t0_mag = atof (text) ;
         break ;
      case 2 :
         gparam->find = SHIFT ;
         gparam->mosaic = TRUE ;
         for (i = 1; i < FindSet->nfiles; i++) {
            j = FindSet->active[i] ;
            if (!GTK_IS_WIDGET (FindSet->fset[i].entries[8])) 
               break ;
            s = (f_set->info[j]->spheric) ? tunits : 1.0 ;
            text = gtk_entry_get_text (GTK_ENTRY (FindSet->fset[i].entries[8])) ;
            /* data->list[j]->param->eps = atof (text) / s ; */
            gparam->min_tol0 = atof (text) / s ;
            text = gtk_entry_get_text (GTK_ENTRY (FindSet->fset[i].entries[9])) ;
            data->list[j]->param->max_shift = atof (text) / s ;
            text = gtk_entry_get_text (GTK_ENTRY (FindSet->fset[i].entries[10])) ;
            data->list[j]->param->crow_rad = atof (text) / s ;
         }
         text = gtk_entry_get_text (GTK_ENTRY (FindSet->entries[3])) ;
         gparam->t0_mag = atof (text) ;
         break ;
      case 3 :
         gparam->find = NONE ;
         gparam->mosaic = FALSE ;
         break ;
   }
}

static void
f_entry_changed (GtkWidget *entry, Sfind *FindSet)
{
   gint     i, j, k ;
   gdouble  scale ;
   gchar   *text ;
   gchar    buffer[255] ;

   if (entry == NULL) 
      return ;
   if (!GTK_IS_ENTRY (entry)) 
      return ;

   text = gtk_entry_get_text (GTK_ENTRY (entry)) ;
   if (text == NULL) 
      return ;
   sprintf (buffer, "%g", atof (text)) ;
   gtk_entry_set_text (GTK_ENTRY (entry), buffer) ;

   j = -1 ;
   for (i = 0; i < (gint)FindSet->nfiles; i++)
      if (FindSet->fset[i].entries[4] == entry) 
            j = i ;
   if (j >= 0) {
      scale = atof (text) ;
      if (scale == 0.0) {
         gtk_entry_set_text (GTK_ENTRY (entry), "1.0") ;
         return ;
      }
      /* Get 1st file nonspheric and with scale 1 */
      for (i = 0; i < (gint)FindSet->nfiles; i++)
         if (i != j && !FindSet->fset[i].sph && atof(gtk_entry_get_text(
               GTK_ENTRY (FindSet->fset[i].entries[4]))) == 1)
             break ;
      /* Get 1st file nonspheric */
      if (i == (gint)FindSet->nfiles) {
         for (i = 0; i < (gint)FindSet->nfiles; i++)
            if (i != j && !FindSet->fset[i].sph) 
               break ;
      }
      /* Get 1st file */
      if (i == (gint)FindSet->nfiles) {
         for (i = 0; i < (gint)FindSet->nfiles; i++)
            if (i != j)
               break ;
      }
      scale /= atof (gtk_entry_get_text(
            GTK_ENTRY (FindSet->fset[i].entries[4]))) ;
      for (k = 0; k < 4; k++) {
         text = gtk_entry_get_text (GTK_ENTRY (FindSet->fset[i].entries[k])) ;
         sprintf (buffer, "%g", atof (text) * scale) ;
         gtk_entry_set_text (GTK_ENTRY (FindSet->fset[j].entries[k]), buffer) ;
      }
   }
}

static void
f_entry_focus_out (GtkWidget *entry, GdkEvent *event, gpointer data)
{
   f_entry_changed (entry, data) ;
}

static void
find_set_units (GtkWidget *widget, Sfind *FindSet, gdouble  s)
{
   gdouble  val ;
   gint     i, j ;
   for (i = 0; i < (gint)FindSet->nfiles; i++) {
      /* Find Set entries */
      if (FindSet->fset[i].sph) {
         for (j = 0; j < 11; j++) {
            if (FindSet->fset[i].entries[j]) {
               val = atof (gtk_entry_get_text (
                     GTK_ENTRY (FindSet->fset[i].entries[j]))) ;
               if (j != 4)
                  sprintf (FindSet->fset[i].value[j], "%g", val * s) ;
               else
                  if (val != 1.0)
                     sprintf (FindSet->fset[i].value[j], "%g", val * s) ;
            }
         }
         for (j = 0; j < 3; j++) {
            if (FindSet->fset[i].label[j]) {
               if (s > 1.0)
                  gtk_label_set_text (GTK_LABEL (FindSet->fset[i].label[j]), 
                        "[arcsec]") ;
               else
                  gtk_label_set_text (GTK_LABEL (FindSet->fset[i].label[j]), 
                        "[deg]") ;
            }
         }
      }
   }
   put_find_text (FindSet) ;
}

static void
set_units0 (GtkWidget *item, Sfind *FindSet)
{
   if (tunits == 3600.0)
      return ;
   find_set_units (item, FindSet, 3600.0) ;
   tunits = 3600.0 ;
}

static void
set_units1 (GtkWidget *item, Sfind *FindSet)
{
   if (tunits == 1.0)
      return ;
   find_set_units (item, FindSet, 1.0 / 3600.0) ;
   tunits = 1.0 ;
}

static void
units_menu (GtkWidget *box, Sfind *FindSet, cxc_display *display)
{
   GtkWidget *label, *option, *menu, *item ;
   option = gtk_option_menu_new () ;
   menu = gtk_menu_new () ;
   item = make_menu_item ("[arcsec]", GTK_SIGNAL_FUNC (set_units0), FindSet) ;
   gtk_menu_append (GTK_MENU (menu), item) ;
   item = make_menu_item ("[deg]",    GTK_SIGNAL_FUNC (set_units1), FindSet) ;
   gtk_menu_append (GTK_MENU (menu), item) ;
   if (tunits == 3600.0)
      gtk_menu_set_active (GTK_MENU (menu), 0) ;
   else
      gtk_menu_set_active (GTK_MENU (menu), 1) ;
   gtk_option_menu_set_menu (GTK_OPTION_MENU (option), menu) ;
   gtk_box_pack_end (GTK_BOX (box), option, FALSE, FALSE, 0) ;
   label = gtk_label_new ("Spherical Units:") ;
   gtk_box_pack_end (GTK_BOX (box), label, FALSE, FALSE, 0) ;
}

GtkWidget *
align_settings (Sfind *FindSet, cxc_display *display)
{
   FileSet   *f_set = CXC_DATA (display->data)->f_set ;
   GtkWidget *box, *vbox, *hbox, *notebook, *scrolled_window, *frame ;
   GtkWidget *label, *viewport, *bbox, *entry, *table ;
   guint      i, j, k ;
   guint      n ;
   gchar      lab[][10] = {"Eps:","Min Side:","Max Side:","Crow Rad:","Scale:"} ;
   gchar      lab1[][12] = {"Tol Radius:","Max Shift:","Crow Rad:"} ;

   CPKDEBUG ("align_settings") ;
   n = 2 ;
#ifdef cippalippa
   n = (code == 0) ? (guint)f_set->n : 2 ;
   active = (guint*) malloc ((size_t)n * sizeof(guint)) ;
   if (code == 0)
      for (i=0; i<n; i++)
         active[i] = i ;
   else {
      active[0] = CXC_DATA (display->data)->nref ;
      active[1] = CXC_DATA (display->data)->frm2 ;
      if ((gint)display->current != CXC_DATA (display->data)->nref)
         active[1] = display->current ;
      else {
         if (display->mode == FRAME_PAIR || display->mode == FRAME_TILE)
            for (i=0; i<(guint)f_set->n; i++)
               if ( display->show[i] && (gint)i != CXC_DATA (display->data)->nref )
                  active[1] = i ;
      }
      /***FIXME: set data->current here ??? ***//*****RIVEDERE ****/
      CXC_DATA (display->data)->frm2 = active[1] ;
   }
#endif

   /* Find Settings */

   box = gtk_vbox_new (FALSE, 0) ;

   set_font ("-*-times-medium-r-normal--18-140-*-*-*-*-*-*") ;
   frame = gtk_frame_new ("Find settings") ;
   gtk_container_set_border_width (GTK_CONTAINER (frame), 8) ;
   gtk_container_add (GTK_CONTAINER (box), frame) ;
   vbox = gtk_vbox_new (FALSE, 3) ;
   gtk_container_set_border_width (GTK_CONTAINER(vbox), 5) ;
   gtk_container_add (GTK_CONTAINER (frame), vbox) ;
   set_font ("-*-times-medium-r-normal--17-140-*-*-*-*-*-*") ;
   
   FindSet->notebook = notebook = gtk_notebook_new ();
   gtk_box_pack_start (GTK_BOX (vbox), notebook, TRUE, TRUE, 0) ;
    gtk_widget_show (notebook);

   /* Find MATCH */
   frame = gtk_frame_new ("Match settings") ;
   gtk_container_set_border_width (GTK_CONTAINER (frame), 10);
   label = gtk_label_new ("MATCH");
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook), frame, label);

   vbox = gtk_vbox_new (FALSE, 5) ;
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
   gtk_container_add (GTK_CONTAINER (frame), vbox) ;

   scrolled_window = gtk_scrolled_window_new (NULL, NULL) ;
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
         GTK_POLICY_NEVER, GTK_POLICY_ALWAYS) ;
   viewport = gtk_viewport_new (
         gtk_scrolled_window_get_hadjustment (GTK_SCROLLED_WINDOW (scrolled_window)),
         gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (scrolled_window))) ;
   gtk_viewport_set_shadow_type (GTK_VIEWPORT (viewport), GTK_SHADOW_ETCHED_OUT) ;
   gtk_widget_set_usize (viewport, 0, 90) ;
   gtk_container_add (GTK_CONTAINER (scrolled_window), viewport) ;
   gtk_box_pack_start (GTK_BOX (vbox), scrolled_window, TRUE, TRUE, 0) ;

   hbox = gtk_hbox_new (FALSE, 0) ;
   gtk_container_set_border_width (GTK_CONTAINER (hbox), 5) ;

   gtk_scrolled_window_add_with_viewport (
         GTK_SCROLLED_WINDOW (scrolled_window), hbox) ;
   table = gtk_table_new (FindSet->nfiles+1, 7, FALSE);
   gtk_table_set_col_spacings (GTK_TABLE (table), 5);
   gtk_table_set_row_spacings (GTK_TABLE (table), 5);
   gtk_box_pack_start (GTK_BOX (hbox), table, TRUE, TRUE, 0);
   label = gtk_label_new ("File:") ;
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
         GTK_FILL, GTK_SHRINK, 0, 0) ;
   label = gtk_label_new (lab[0]) ;
   gtk_table_attach (GTK_TABLE (table), label, 1, 2, 0, 1,
         GTK_FILL, GTK_SHRINK, 0, 0) ;
   label = gtk_label_new (lab[1]) ;
   gtk_table_attach (GTK_TABLE (table), label, 2, 3, 0, 1,
         GTK_FILL, GTK_SHRINK, 0, 0) ;
   label = gtk_label_new (lab[2]) ;
   gtk_table_attach (GTK_TABLE (table), label, 3, 4, 0, 1,
         GTK_FILL, GTK_SHRINK, 0, 0) ;
   label = gtk_label_new (lab[3]) ;
   gtk_table_attach (GTK_TABLE (table), label, 4, 5, 0, 1,
         GTK_FILL, GTK_SHRINK, 0, 0) ;
   label = gtk_label_new (lab[4]) ;
   gtk_table_attach (GTK_TABLE (table), label, 5, 6, 0, 1,
         GTK_FILL, GTK_SHRINK, 0, 0) ;
   label = gtk_label_new ("Units:") ;
   gtk_table_attach (GTK_TABLE (table), label, 6, 7, 0, 1,
         GTK_FILL, GTK_SHRINK, 0, 0) ;

   for (i = 0; i < FindSet->nfiles; i++) {
      k = FindSet->active[i] ;
      if (FindSet->nfiles == 2) {
         frame = gtk_frame_new (NULL) ;
         gtk_widget_set_usize (frame, 40, 0) ;
         bbox = gtk_hbox_new (FALSE, 0) ;
         gtk_container_add (GTK_CONTAINER (frame), bbox) ;
         label = gtk_label_new ((i == 0) ? "Ref:" : "Cur:") ;
         gtk_box_pack_start (GTK_BOX (bbox), label, FALSE, FALSE, 2) ;
         gtk_table_attach (GTK_TABLE (table), frame, 0, 1, i+1, i+2,
               GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0) ;
      }
      else {
         entry = gtk_entry_new () ;
         gtk_entry_set_text (GTK_ENTRY (entry), f_set->info[k]->filename) ;
         gtk_entry_set_editable (GTK_ENTRY (entry), FALSE);
         gtk_widget_set_usize (entry, 120, 0) ;
         gtk_table_attach (GTK_TABLE (table), entry, 0, 1, i+1, i+2,
               GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0) ;
      }

      for (j = 0; j < 5; j++) {
         FindSet->fset[i].entries[j] = entry = gtk_entry_new () ;;
         gtk_entry_set_max_length (GTK_ENTRY (entry), 254) ;
         gtk_entry_set_text (GTK_ENTRY (entry), FindSet->fset[i].value[j]) ;
         gtk_signal_connect (GTK_OBJECT (entry), "activate",
               GTK_SIGNAL_FUNC (f_entry_changed), (gpointer) FindSet) ;
         gtk_signal_connect (GTK_OBJECT (entry), "focus-out-event",
               GTK_SIGNAL_FUNC (f_entry_focus_out), (gpointer) FindSet) ;
         gtk_widget_set_usize (entry, 80, 0) ;
         gtk_table_attach (GTK_TABLE (table), entry, j+1, j+2, i+1, i+2,
               GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
      }
      frame = gtk_frame_new (NULL) ;
      gtk_widget_set_usize (frame, 60, 0) ;
      bbox = gtk_hbox_new (FALSE, 0) ;
      gtk_container_add (GTK_CONTAINER (frame), bbox) ;
      if (FindSet->fset[i].sph)
         FindSet->fset[i].label[0] = label = gtk_label_new ("[arcsec]") ;
      else
         FindSet->fset[i].label[0] = label = gtk_label_new ("[pixel]") ;
      gtk_box_pack_start (GTK_BOX (bbox), label, FALSE, FALSE, 2) ;
      gtk_table_attach (GTK_TABLE (table), frame, 6, 7, i+1, i+2,
               GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
   }
   hbox = gtk_hbox_new (FALSE, 5) ;
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0) ;
   label = gtk_label_new ("Tol Mag:") ;
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0) ;
   FindSet->entries[0] = entry = gtk_entry_new () ;
   gtk_entry_set_max_length (GTK_ENTRY (entry), 25) ;
   gtk_entry_set_text (GTK_ENTRY (entry), FindSet->value[0]) ;
   gtk_signal_connect (GTK_OBJECT (entry), "activate",
                 GTK_SIGNAL_FUNC (f_entry_changed), (gpointer) FindSet) ;
   gtk_signal_connect (GTK_OBJECT (entry), "focus-out-event",
                 GTK_SIGNAL_FUNC (f_entry_focus_out), (gpointer) FindSet) ;
   gtk_widget_set_usize (entry, 40, 0) ;
   gtk_box_pack_start (GTK_BOX (hbox), entry, FALSE, FALSE, 0) ;

   label = gtk_label_new ("Vertex Delta Mag:") ;
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0) ;
   FindSet->entries[1] = entry = gtk_entry_new () ;
   gtk_entry_set_max_length (GTK_ENTRY (entry), 25) ;
   gtk_entry_set_text (GTK_ENTRY (entry), FindSet->value[1]) ;
   gtk_signal_connect (GTK_OBJECT (entry), "activate",
                 GTK_SIGNAL_FUNC (f_entry_changed), (gpointer) FindSet) ;
   gtk_signal_connect (GTK_OBJECT (entry), "focus-out-event",
                 GTK_SIGNAL_FUNC (f_entry_focus_out), (gpointer) FindSet) ;
   gtk_widget_set_usize (entry, 40, 0) ;
   gtk_box_pack_start (GTK_BOX (hbox), entry, FALSE, FALSE, 0) ;
   units_menu (hbox, FindSet, display) ;

   /* Find SHIFT */
   frame = gtk_frame_new ("Shift settings") ;
   gtk_container_set_border_width (GTK_CONTAINER (frame), 10);
   label = gtk_label_new ("SHIFT");
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook), frame, label);

   vbox = gtk_vbox_new (FALSE, 5) ;
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
   gtk_container_add (GTK_CONTAINER (frame), vbox) ;
   scrolled_window = gtk_scrolled_window_new (NULL, NULL) ;
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
         GTK_POLICY_NEVER, GTK_POLICY_ALWAYS) ;
   viewport = gtk_viewport_new (
         gtk_scrolled_window_get_hadjustment (GTK_SCROLLED_WINDOW (scrolled_window)),
         gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (scrolled_window))) ;
   gtk_viewport_set_shadow_type (GTK_VIEWPORT (viewport), GTK_SHADOW_ETCHED_OUT) ;
   gtk_widget_set_usize (viewport, 0, 75) ;
   gtk_container_add (GTK_CONTAINER (scrolled_window), viewport) ;
   gtk_box_pack_start (GTK_BOX (vbox), scrolled_window, TRUE, TRUE, 0) ;

   hbox = gtk_hbox_new (FALSE, 0) ;
   gtk_container_set_border_width (GTK_CONTAINER (hbox), 5) ;

   gtk_scrolled_window_add_with_viewport (
         GTK_SCROLLED_WINDOW (scrolled_window), hbox) ;
   table = gtk_table_new (FindSet->nfiles+1, 5, FALSE);
   gtk_table_set_col_spacings (GTK_TABLE (table), 5);
   gtk_table_set_row_spacings (GTK_TABLE (table), 5);
   gtk_box_pack_start (GTK_BOX (hbox), table, TRUE, TRUE, 0);
   label = gtk_label_new ("File:") ;
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
         GTK_FILL, GTK_SHRINK, 0, 0) ;
   label = gtk_label_new (lab1[0]) ;
   gtk_table_attach (GTK_TABLE (table), label, 1, 2, 0, 1,
         GTK_FILL, GTK_SHRINK, 0, 0) ;
   label = gtk_label_new (lab1[1]) ;
   gtk_table_attach (GTK_TABLE (table), label, 2, 3, 0, 1,
         GTK_FILL, GTK_SHRINK, 0, 0) ;
   label = gtk_label_new (lab1[2]) ;
   gtk_table_attach (GTK_TABLE (table), label, 3, 4, 0, 1,
         GTK_FILL, GTK_SHRINK, 0, 0) ;
   label = gtk_label_new ("Units:") ;
   gtk_table_attach (GTK_TABLE (table), label, 4, 5, 0, 1,
         GTK_FILL, GTK_SHRINK, 0, 0) ;
   for (i = 1; i < FindSet->nfiles; i++) {
      k = FindSet->active[i] ;
      entry = gtk_entry_new () ;
      gtk_entry_set_text (GTK_ENTRY (entry), f_set->info[k]->filename) ;
      gtk_entry_set_editable (GTK_ENTRY (entry), FALSE);
      gtk_widget_set_usize (entry, 120, 0) ;
      gtk_table_attach (GTK_TABLE (table), entry, 0, 1, i, i+1,
            GTK_FILL, GTK_SHRINK, 0, 0) ;
      for (j = 0; j < 3; j++) {
         FindSet->fset[i].entries[j+5] = entry = gtk_entry_new () ;
         gtk_entry_set_max_length (GTK_ENTRY (entry), 254) ;
         gtk_entry_set_text (GTK_ENTRY (entry), FindSet->fset[i].value[j+5]) ;
         gtk_signal_connect (GTK_OBJECT (entry), "activate",
                       GTK_SIGNAL_FUNC (f_entry_changed), (gpointer) FindSet) ;
         gtk_signal_connect (GTK_OBJECT (entry), "focus-out-event",
                       GTK_SIGNAL_FUNC (f_entry_focus_out), (gpointer) FindSet) ;
         gtk_widget_set_usize (entry, 80, 0) ;
         gtk_table_attach (GTK_TABLE (table), entry, j+1, j+2, i, i+1,
               GTK_FILL, GTK_SHRINK, 0, 0);
      }
      frame = gtk_frame_new (NULL) ;
      gtk_widget_set_usize (frame, 60, 0) ;
      bbox = gtk_hbox_new (FALSE, 0) ;
      gtk_container_add (GTK_CONTAINER (frame), bbox) ;
      if (FindSet->fset[i].sph)
         FindSet->fset[i].label[1] = label = gtk_label_new ("[arcsec]") ;
      else
         FindSet->fset[i].label[1] = label = gtk_label_new ("[pixel]") ;
      gtk_box_pack_start (GTK_BOX (bbox), label, FALSE, FALSE, 2) ;
      gtk_table_attach (GTK_TABLE (table), frame, 4, 5, i, i+1,
               GTK_FILL, GTK_SHRINK, 0, 0);
   }
   hbox = gtk_hbox_new (FALSE, 5) ;
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0) ;
   label = gtk_label_new ("Tolerance Magnitude:") ;
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0) ;
   FindSet->entries[2] = entry = gtk_entry_new () ;
   gtk_entry_set_max_length (GTK_ENTRY (entry), 254) ;
   gtk_entry_set_text (GTK_ENTRY (entry), FindSet->value[2]) ;
   gtk_signal_connect (GTK_OBJECT (entry), "activate",
                 GTK_SIGNAL_FUNC (f_entry_changed), (gpointer) FindSet) ;
   gtk_signal_connect (GTK_OBJECT (entry), "focus-out-event",
                 GTK_SIGNAL_FUNC (f_entry_focus_out), (gpointer) FindSet) ;
   gtk_widget_set_usize (entry, 40, 0) ;
   gtk_box_pack_start (GTK_BOX (hbox), entry, FALSE, FALSE, 0) ;
   units_menu (hbox, FindSet, display) ;

   /* Find MOSAIC */
   frame = gtk_frame_new ("Mosaic settings") ;
   gtk_container_set_border_width (GTK_CONTAINER (frame), 10);
   label = gtk_label_new ("MOSAIC");
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook), frame, label);

   vbox = gtk_vbox_new (FALSE, 5) ;
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
   gtk_container_add (GTK_CONTAINER (frame), vbox) ;
   scrolled_window = gtk_scrolled_window_new (NULL, NULL) ;
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (scrolled_window),
         GTK_POLICY_NEVER, GTK_POLICY_ALWAYS) ;
   viewport = gtk_viewport_new (
         gtk_scrolled_window_get_hadjustment (GTK_SCROLLED_WINDOW (scrolled_window)),
         gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (scrolled_window))) ;
   gtk_viewport_set_shadow_type (GTK_VIEWPORT (viewport), GTK_SHADOW_ETCHED_OUT) ;
   gtk_widget_set_usize (viewport, 0, 75) ;
   gtk_container_add (GTK_CONTAINER (scrolled_window), viewport) ;
   gtk_box_pack_start (GTK_BOX (vbox), scrolled_window, TRUE, TRUE, 0) ;

   hbox = gtk_hbox_new (FALSE, 0) ;
   gtk_container_set_border_width (GTK_CONTAINER (hbox), 5) ;

   gtk_scrolled_window_add_with_viewport (
         GTK_SCROLLED_WINDOW (scrolled_window), hbox) ;
   table = gtk_table_new (FindSet->nfiles+1, 5, FALSE);
   gtk_table_set_col_spacings (GTK_TABLE (table), 5);
   gtk_table_set_row_spacings (GTK_TABLE (table), 5);
   gtk_box_pack_start (GTK_BOX (hbox), table, TRUE, TRUE, 0);
   label = gtk_label_new ("File:") ;
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
         GTK_FILL, GTK_SHRINK, 0, 0) ;
   label = gtk_label_new (lab1[0]) ;
   gtk_table_attach (GTK_TABLE (table), label, 1, 2, 0, 1,
         GTK_FILL, GTK_SHRINK, 0, 0) ;
   label = gtk_label_new (lab1[1]) ;
   gtk_table_attach (GTK_TABLE (table), label, 2, 3, 0, 1,
         GTK_FILL, GTK_SHRINK, 0, 0) ;
   label = gtk_label_new (lab1[2]) ;
   gtk_table_attach (GTK_TABLE (table), label, 3, 4, 0, 1,
         GTK_FILL, GTK_SHRINK, 0, 0) ;
   label = gtk_label_new ("Units:") ;
   gtk_table_attach (GTK_TABLE (table), label, 4, 5, 0, 1,
         GTK_FILL, GTK_SHRINK, 0, 0) ;
   for (i = 1; i < FindSet->nfiles; i++) {
      k = FindSet->active[i] ;
      entry = gtk_entry_new () ;
      gtk_entry_set_text (GTK_ENTRY (entry), f_set->info[k]->filename) ;
      gtk_entry_set_editable (GTK_ENTRY (entry), FALSE);
      gtk_widget_set_usize (entry, 120, 0) ;
      gtk_table_attach (GTK_TABLE (table), entry, 0, 1, i, i+1,
            GTK_FILL, GTK_SHRINK, 0, 0) ;
      for (j = 0; j < 3; j++) {
         FindSet->fset[i].entries[j+8] = entry = gtk_entry_new () ;
         gtk_entry_set_max_length (GTK_ENTRY (entry), 254) ;
         gtk_entry_set_text (GTK_ENTRY (entry), FindSet->fset[i].value[j+8]) ;
         gtk_signal_connect (GTK_OBJECT (entry), "activate",
                       GTK_SIGNAL_FUNC (f_entry_changed), (gpointer) FindSet) ;
         gtk_signal_connect (GTK_OBJECT (entry), "focus-out-event",
                       GTK_SIGNAL_FUNC (f_entry_focus_out), (gpointer) FindSet) ;
         gtk_widget_set_usize (entry, 80, 0) ;
         gtk_table_attach (GTK_TABLE (table), entry, j+1, j+2, i, i+1,
               GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
      }
      frame = gtk_frame_new (NULL) ;
      gtk_widget_set_usize (frame, 60, 0) ;
      bbox = gtk_hbox_new (FALSE, 0) ;
      gtk_container_add (GTK_CONTAINER (frame), bbox) ;
      if (FindSet->fset[i].sph)
         FindSet->fset[i].label[2] = label = gtk_label_new ("[arcsec]") ;
      else
         FindSet->fset[i].label[2] = label = gtk_label_new ("[pixel]") ;
      gtk_box_pack_start (GTK_BOX (bbox), label, FALSE, FALSE, 2) ;
      gtk_table_attach (GTK_TABLE (table), frame, 4, 5, i, i+1,
               GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
   }
   hbox = gtk_hbox_new (FALSE, 5) ;
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0) ;
   label = gtk_label_new ("Tolerance Magnitude:") ;
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0) ;
   FindSet->entries[3] = entry = gtk_entry_new () ;
   gtk_entry_set_max_length (GTK_ENTRY (entry), 254) ;
   gtk_entry_set_text (GTK_ENTRY (entry), FindSet->value[3]) ;
   gtk_signal_connect (GTK_OBJECT (entry), "activate",
                 GTK_SIGNAL_FUNC (f_entry_changed), (gpointer) FindSet) ;
   gtk_signal_connect (GTK_OBJECT (entry), "focus-out-event",
                 GTK_SIGNAL_FUNC (f_entry_focus_out), (gpointer) FindSet) ;
   gtk_widget_set_usize (entry, 40, 0) ;
   gtk_box_pack_start (GTK_BOX (hbox), entry, FALSE, FALSE, 0) ;
   units_menu (hbox, FindSet, display) ;
 
   frame = gtk_frame_new (NULL) ;
   gtk_container_set_border_width (GTK_CONTAINER (frame), 10);
   gtk_frame_set_shadow_type (GTK_FRAME (frame),GTK_SHADOW_NONE) ;
   label = gtk_label_new ("Frames already aligned");
   gtk_container_add (GTK_CONTAINER (frame), label) ;
   label = gtk_label_new ("NONE");
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook), frame, label);

   gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_TOP) ;
   gtk_signal_connect (GTK_OBJECT (notebook), "switch-page",
                       GTK_SIGNAL_FUNC (get_page), (gpointer) FindSet) ;
   
   gtk_widget_show_all (box) ;

   /* Set current page accordling to 'find' parameter */
   switch (CXC_DATA (display->data)->gparam->find)  {
      case MATCH :
         gtk_notebook_set_page (GTK_NOTEBOOK (notebook), 0) ;
         break ;
      case SHIFT :
         if (CXC_DATA (display->data)->gparam->mosaic == FALSE)
            gtk_notebook_set_page (GTK_NOTEBOOK (notebook), 1) ;
         else
            gtk_notebook_set_page (GTK_NOTEBOOK (notebook), 2) ;
         break ;
      case NONE  :
         gtk_notebook_set_page (GTK_NOTEBOOK (notebook), 3) ;
         break ;
   }

   return box ;
}

static void
align_frames_accept (GtkWidget *widget, Fparam *FindParam)
{
   CxcData   *data = CXC_DATA (FindParam->display->data) ;
   CxcMap    *current = CXC_MAP (FindParam->display->map[data->frm2]) ;
   gint       mode = FindParam->mode ;

   if (mode == 1) 
      set_align_val (FindParam->FindSet, data) ;
   else
      data->gparam->min_tol0 = FindParam->uval.value ;

   data->gparam->min_tol = data->gparam->min_tol0 ;

   data->pmodel->status = BUSY ;
   cxc_buttons_wcs_status (FindParam->display) ;

   gtk_widget_destroy (FindParam->dialog) ;

   /* Free memory */
   free (FindParam->FindSet->active) ;
   free (FindParam->FindSet->fset) ;

   /* update the GUI */
   while (gtk_events_pending ())
      gtk_main_iteration () ;

   if (mode == 0) {
      /* Manual Alignment */
      gint     find ;

      cxc_map_get_ref_wcs (current) ;

      data->list2->out_sph = data->list1->spheric ;

      find = data->gparam->find ;
      data->gparam->find = OTHER ;
      model_set_order (data->pmodel, 1) ;
      cxc_data_model_align (widget, data) ;
      data->gparam->find = find ;
   }
   else {
      /* Automatic Alignment */

      cxc_data_model_align (widget, data) ;
   }
   if (FindParam->display->we_are_exiting)
      quit_program (NULL, FindParam->display) ;
}

static void
cancel (GtkWidget *widget, Sfind *FindSet)
{
   CPKDEBUG ("align->cancel") ;
   free (FindSet->active) ;
   free (FindSet->fset) ;
   FindSet->active = NULL ;
   FindSet->fset = NULL ;
}

static void
close_window (GtkWidget *window, gpointer data)
{
   cxc_display *display = data ;
   CPKDEBUG ("align->close_window") ;
   gtk_widget_set_sensitive (display->pref_tunits_radio[0], TRUE) ;
   gtk_widget_set_sensitive (display->pref_tunits_radio[1], TRUE) ;
   if (tunits == 3600.0)
      gtk_check_menu_item_set_active  (GTK_CHECK_MENU_ITEM ( 
         display->pref_tunits_radio[0]), TRUE) ;
   else
      gtk_check_menu_item_set_active  (GTK_CHECK_MENU_ITEM ( 
         display->pref_tunits_radio[1]), TRUE) ;
   display->align_frames_dialog = NULL ;
}

static void
get_buttons (GtkWidget *dialog, Fparam *FindParam)
{
   GtkWidget *button ;
   button = gtk_button_new_with_label ("OK") ;
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
         GTK_SIGNAL_FUNC(align_frames_accept), FindParam) ;
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
         button, TRUE, TRUE, 0) ;
   GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT) ;
   gtk_widget_show(button) ;

   button = gtk_button_new_with_label ("Cancel") ;
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
         GTK_SIGNAL_FUNC(gtk_widget_destroy), (gpointer) dialog) ;
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
         button, TRUE, TRUE, 0) ;
   GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT) ;
   gtk_widget_grab_default(button) ;
   gtk_widget_show (button) ;

   button = gtk_button_new_with_label ("Help") ;
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
         button, TRUE, TRUE, 0) ;
   GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT) ;
   gtk_widget_show (button) ;
}

void
align_frames (GtkWidget *widget, cxc_display *display)
{
   GtkWidget   *dialog, *box, *vbox, *hbox, *label, *notebook, *frame ;
   GtkTooltips *ttip ;
   gint         select ;
   guint        nref = CXC_DATA (display->data)->nref ;
   gdouble      tol ;

   static Sfind   FindSet ;
   static Fparam  FindParam ;
   CPKDEBUG ("align_frames") ;

   if (GTK_IS_WIDGET (display->align_frames_dialog)) {
      /* Window is already opened */
      gtk_window_set_transient_for (GTK_WINDOW (display->align_frames_dialog), 
            GTK_WINDOW (display->window)) ;
      return ;
   }

   if (cxc_frame_select (display)) {
      cpk_warning (display->window, 
            "Please select a frame other than reference one.") ;
      return ;
   }

   if (CXC_DATA (display->data)->pmodel->status >= LOW_ALIGN)
      if (cpk_confirm (display->window, "CataXcorr: Match Frames", 
            "Frames have been already aligned!"))
         return ;

   gtk_widget_set_sensitive (display->pref_tunits_radio[0], FALSE) ;
   gtk_widget_set_sensitive (display->pref_tunits_radio[1], FALSE) ;

   /* Create a new window */
   display->align_frames_dialog = dialog = gtk_dialog_new () ;

   /* Initialize Sfind struct */

   FindSet.page = 0 ;
   FindSet.nfiles = 2 ;
   FindSet.active = (guint*) malloc ((size_t)2 * sizeof(guint)) ;
   FindSet.active[0] = CXC_DATA (display->data)->nref ;
   FindSet.active[1] = CXC_DATA (display->data)->frm2 ;
   FindSet.fset = (find_set*) calloc ((size_t)2, sizeof(find_set)) ;

   if (CXC_DATA (display->data)->gparam->mosaic)
      select = CXC_DATA (display->data)->frm2-1 ;
   else
      select = CXC_DATA (display->data)->nref ;

   get_align_val (&FindSet, CXC_DATA (display->data)) ;

   FindParam.main_window = display->window ;
   FindParam.dialog = dialog ;
   FindParam.FindSet = &FindSet ;
   FindParam.display = display ;
   FindParam.mode = 1 ;
   
   ttip = gtk_tooltips_new () ;

   gtk_window_set_title (GTK_WINDOW (dialog), "CataXcorr: Match Frames") ;
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE) ;
   gtk_signal_connect (GTK_OBJECT (dialog), "destroy", 
                       GTK_SIGNAL_FUNC (cancel), &FindSet) ;
   gtk_signal_connect (GTK_OBJECT (dialog), "destroy", 
                       GTK_SIGNAL_FUNC (close_window), (gpointer)display) ;

   /* Main container */
   box = gtk_vbox_new (FALSE, 0) ;
   gtk_container_set_border_width (GTK_CONTAINER (box), 8) ;
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), box,
                      TRUE, TRUE, 0) ;

   cxc_set_reference (box, FALSE, 5, select, display) ;

   notebook = gtk_notebook_new ();
   /* gtk_box_pack_start (GTK_BOX (box), notebook, TRUE, TRUE, 0) ; */
   gtk_container_add (GTK_CONTAINER (box), notebook) ;
   gtk_widget_show (notebook);

   frame = gtk_frame_new (NULL) ;
   gtk_container_set_border_width (GTK_CONTAINER (frame), 10);
   vbox = gtk_vbox_new (FALSE, 15) ;
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 100);
   gtk_container_add (GTK_CONTAINER (frame), vbox) ;
   set_font ("-*-times-bold-r-normal--20-*-*-*-*-*-*-*") ;
   label = gtk_label_new ("Catch current frames alignmet as seen on display.");
   gtk_container_add (GTK_CONTAINER (vbox), label) ;
   set_font ("-*-times-medium-r-normal--17-140-*-*-*-*-*-*") ;

   /* Tol Radius for *Manual Alignment* */
   hbox = gtk_hbox_new (FALSE, 2) ;   
   label = gtk_label_new ("Tol Radius:") ;
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2) ;

   tol = MAX (0.9*CXC_DATA (display->data)->list[nref]->param->crow_rad,
              5.0*CXC_DATA (display->data)->list[nref]->param->eps) ;
   cpk_entry_with_units (&FindParam.uval, tol, 
         CXC_DATA (display->data)->list[nref]->spheric) ;
   gtk_tooltips_set_tip (GTK_TOOLTIPS (ttip), FindParam.uval.entry, 
         "Minimum tolerance radius", NULL) ;
   gtk_entry_set_max_length (GTK_ENTRY (FindParam.uval.entry), 254) ;
   gtk_widget_set_usize (FindParam.uval.entry, 100, 0) ;
   gtk_box_pack_start (GTK_BOX (hbox), FindParam.uval.entry, FALSE, FALSE, 2) ;
   gtk_box_pack_start (GTK_BOX (hbox), FindParam.uval.units, FALSE, FALSE, 2) ;

   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0) ;

   label = gtk_label_new (" Manual Alignment ") ;
   gtk_label_set_pattern (GTK_LABEL (label)," ________________ ") ;
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook), frame, label);

   /* Find Settings */
   vbox = align_settings (&FindSet, display) ;
   /* gtk_container_add (GTK_CONTAINER (box), vbox) ; */
   label = gtk_label_new (" Automatic Alignment ") ;
   gtk_label_set_pattern (GTK_LABEL (label)," ___________________ ") ;
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, label);

   gtk_signal_connect (GTK_OBJECT (notebook), "switch-page",
         GTK_SIGNAL_FUNC (set_mode), &FindParam) ;


/* FIXME: AGGIUNGERE GENERAL OPTIONS HERE */

   gtk_widget_show_all (box) ;

   /* Set current page accordling to 'find' parameter */
   gtk_notebook_set_page (GTK_NOTEBOOK (notebook), 1) ;
   if (CXC_DATA (display->data)->pmodel->status)
      gtk_notebook_set_page (GTK_NOTEBOOK (FindParam.FindSet->notebook), 3) ;

   /* buttons */
   get_buttons (dialog, &FindParam) ;

   gtk_widget_show_all (dialog) ;
}

void
align_frames_all (GtkWidget *widget, cxc_display *display)
{
   GtkWidget *dialog, *box, *vbox ;
   FileSet   *f_set = CXC_DATA (display->data)->f_set ;
   size_t     nfiles = f_set->n, i ;

   static Sfind   FindSet ;
   static Fparam  FindParam ;

   if (GTK_IS_WIDGET (display->align_frames_dialog)) {
      /* Window is already opened */
      gtk_window_set_transient_for (GTK_WINDOW (display->align_frames_dialog), 
            GTK_WINDOW (display->window)) ;
      return ;
   }

   /* Create a new window */
   gtk_widget_push_visual (gdk_rgb_get_visual ()) ;
   gtk_widget_push_colormap (gdk_rgb_get_cmap ()) ;
   display->align_frames_dialog = dialog = gtk_dialog_new () ;
   gtk_widget_pop_visual () ;
   gtk_widget_pop_colormap () ;   

   /* Initialize Sfind struct */

   FindSet.page = 0 ;
   FindSet.nfiles = (guint) nfiles ;
   FindSet.active = (guint*) malloc (nfiles * sizeof(guint)) ;
   for (i = 0; i < nfiles; i++)
      FindSet.active[i] = (guint)i ;
   FindSet.fset = (find_set*) calloc (nfiles, sizeof(find_set)) ;

   get_align_val(&FindSet, CXC_DATA (display->data)) ;

   FindParam.main_window = display->window ;
   FindParam.dialog = dialog ;
   FindParam.FindSet = &FindSet ;
   FindParam.display = display ;
   FindParam.mode = 1 ;

   gtk_window_set_title (GTK_WINDOW (dialog), "CataXcorr: Match Frames") ;
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE) ;
   gtk_signal_connect (GTK_OBJECT (dialog), "destroy", 
                       GTK_SIGNAL_FUNC (cancel), &FindSet) ;
   gtk_signal_connect (GTK_OBJECT(dialog), "destroy", 
                       GTK_SIGNAL_FUNC (close_window), (gpointer)display) ;

   /* Main container */
   box = gtk_vbox_new (FALSE, 0) ;
   gtk_container_set_border_width (GTK_CONTAINER(box), 8) ;
   gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), box,
                       TRUE, TRUE, 0) ;

   /* Find Settings */
   vbox = align_settings (&FindSet, display) ;
   gtk_container_add (GTK_CONTAINER (box), vbox) ;

   gtk_widget_show_all (box) ;

   /* buttons */
   get_buttons (dialog, &FindParam) ;

   gtk_widget_show_all (dialog) ;
}

static void
set_mode (GtkNotebook *notebook, GtkNotebookPage *page, gint page_num, 
      Fparam *FindParam)
{
   FindParam->mode = page_num ;
}

