/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 							Includes
 *--------------------------------------------------------------------------*/

#if HAVE_CONFIG_H
#  include "config.h"
#endif

#include "common.h"

#include <math.h>

#include <gtk/gtk.h>

#include "cataxcorr.h"
#include "fileinfo.h"
#include "cxc_data.h"
#include "cpk_gtk_util.h"
#include "platemodel.h"
#include "cataxcorr_p.h"
#include "coordinates.h"
#include "starlist.h"
#include "model.h"
#include "cxc_gtk_main.h"
#include "cxc_match_set.h"
#include "cxc_file_set.h"
#include "cxc_mask_set.h"
#include "cxc_align_set.h"
#include "cxc_register_set.h"
#include "cxc_model_set.h"

/*----------------------------------------------------------------------------
 * 							Global variables	
 *--------------------------------------------------------------------------*/

extern boolean    suspend ;
extern gdouble    sph_units ;

/*----------------------------------------------------------------------------
 * 							Private Types
 *--------------------------------------------------------------------------*/

typedef struct {
   GtkWidget *item1[2] ;
   GtkWidget *item2[2] ;
   GtkWidget *item3[3] ;
   guint      n1 ;
   guint      n2 ;
   guint      n3 ;
} Sglob ;

typedef struct autop_tag {
   GtkWidget   *main_window ;
   GtkWidget   *dialog ;
   Sfile       *FileSet ;
   Smask       *MaskSet ;
   Sfind       *FindSet ;
   Salign      *AlignSet ;
   /* Smatch      *MatchSet ; */
   Smodel      *ModelSet ;
 /*Ssave       *SaveSet ; */
   Sglob       *GlobSet ;
   cxc_display *display ;
} Aparam ;

/*----------------------------------------------------------------------------
 * 							Function prototypes
 *--------------------------------------------------------------------------*/
void auto_exec( GtkWidget *widget, cxc_display *display ) ;

extern    GtkWidget      *global_settings     (Sglob *GlobSet, 
                                              cxc_display *display) ;
extern    void            get_global_val      (Sglob *GlobSet, CxcData *data) ;
/* extern    void            set_global_val      (Sglob *GlobSet, CxcData *data) ; */

/*----------------------------------------------------------------------------
 * 							Defines
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 *							Function codes
 *--------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
 * Function  : abort_auto_exec
 *             quit_dialog
 *             suspend_query
 * Purpouse  : Dialog box: ask permission to interrupt execution process
 * Notes     : Allocate memory for returning model
 *---------------------------------------------------------------------------*/

static void
abort_auto_exec(GtkWidget *widget, cxc_display *display)
{
   gtk_signal_disconnect_by_data (GTK_OBJECT(CXC_DATA (display->data)), 
         CXC_DATA (display->data)) ;

   /* Turn on 'suspend' locker */
   suspend = TRUE ;
   CXC_DATA (display->data)->locked = FALSE ;
   /* remove the timer */
   cxc_data_timeout_remove(CXC_DATA (display->data)->activity) ;
   /* update the GUI */
   while (gtk_events_pending ())
      gtk_main_iteration () ;
}

static void
quit_dialog(GtkWidget *widget, cxc_display *display)
{
   display->auto_exec_dialog = NULL ;
   gtk_main_quit() ;
}

static void
suspend_query(GtkWidget *window, cxc_display *display)
{
   GtkWidget *dialog, *box, *label, *button ;

   /* Create a new window */
   display->auto_exec_dialog = dialog = gtk_dialog_new( ) ;
   gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (window)) ;
   gtk_window_set_title(GTK_WINDOW(dialog), "*** Warning ***") ;
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER) ;
   gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
         (GtkSignalFunc) quit_dialog, display) ;

   box = gtk_hbox_new(FALSE, 0) ;
   gtk_container_set_border_width (GTK_CONTAINER(box), 30) ;
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), box,
                      TRUE, TRUE, 5) ;

   label = gtk_label_new("Auto Execution is already running.") ;
   gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 0) ;

   button = gtk_button_new_with_label ("Abort") ;
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
         GTK_SIGNAL_FUNC(abort_auto_exec), display) ;
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
         GTK_SIGNAL_FUNC(gtk_widget_destroy), (gpointer) dialog) ;
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
         button, TRUE, TRUE, 0) ;
   GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT) ;

   button = gtk_button_new_with_label ("Continue") ;
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
         GTK_SIGNAL_FUNC(gtk_widget_destroy), (gpointer) dialog) ;
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
         button, TRUE, TRUE, 0) ;
   GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT) ;
   gtk_widget_grab_default(button) ;

   button = gtk_button_new_with_label ("Help") ;
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
         button, TRUE, TRUE, 0) ;
   GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT) ;

   gtk_widget_show_all (dialog) ;

   /* Control */
   gtk_main() ;
}

/*-----------------------------------------------------------------------------
 * Function  : close_window
 *             cancel
 *             auto_exec_accept
 *             auto_exec_run
 *             get_buttons
 *             auto_exec
 * Purpouse  : Dialog box: 
 * Notes     : 
 *---------------------------------------------------------------------------*/

static void
close_window(GtkWidget *window, gpointer data)
{
   cxc_display *display = data ;
   display->auto_exec_dialog = NULL ;
}

static void
cancel(GtkWidget *widget, Sfind *FindSet)
{
   free (FindSet->fset) ;
}

static void
auto_exec_accept(GtkWidget *widget, Aparam *AutoParam)
{
   CxcData   *data = CXC_DATA (AutoParam->display->data) ;
   
   set_file_val(AutoParam->FileSet, data) ;

   set_mask_val(AutoParam->MaskSet, data) ;
   /* Process again ? */

   set_align_val(AutoParam->FindSet, data) ;
   set_register_val(AutoParam->AlignSet, data) ;
   /* set_match_val(AutoParam->MatchSet, data) ; */
   set_model_val(AutoParam->ModelSet, data) ;

#ifdef cippa
   set_save_val(AutoParam->SaveSet, data) ;
   set_glob_val(AutoParam->GlobSet, data) ;/*??*/
#endif
}

static void
auto_exec_run(GtkWidget *widget, Aparam *AutoParam)
{
   CxcData   *data = CXC_DATA (AutoParam->display->data) ;
#ifdef cippa
   StarList  *list1 ;
   StarList  *list2 ;
   PlateMod  *pmodel = (PlateMod*)NULL ;
   Match     *match = (Match*)NULL ;
   gint       nmatch ;
   gboolean   spheric = data->f_set->info[nrif].spheric ; /** FIXME: verificare */
   char       parfile[1024] = "\0" ;
   gint       j, status = 1 ;
#endif

   printf("**** auto_exec_run init\n");
   auto_exec_accept(widget, AutoParam) ;

   /* Connect signals for CxcData struct */

/*    gtk_signal_connect(GTK_OBJECT(data), "file-stop", 
                      GTK_SIGNAL_FUNC(), data) ; */
   gtk_signal_connect(GTK_OBJECT(data), "file-next", 
                      GTK_SIGNAL_FUNC(cxc_data_model_align), data) ;
   /* gtk_signal_connect(GTK_OBJECT(data), "star-aligned", 
                      GTK_SIGNAL_FUNC(cxc_data_base_find), data) ; */
   gtk_signal_connect(GTK_OBJECT(data), "base-found", 
                      GTK_SIGNAL_FUNC(cxc_data_model_fit), data) ;
   gtk_signal_connect(GTK_OBJECT(data), "file-stop", 
                      GTK_SIGNAL_FUNC(gtk_signal_disconnect_by_data), data) ;
/*    gtk_signal_connect(GTK_OBJECT(data), "model-fit", 
                      GTK_SIGNAL_FUNC(), data) ;
   gtk_signal_connect(GTK_OBJECT(data), "stack-added", 
                      GTK_SIGNAL_FUNC(), data) ;
   gtk_signal_connect(GTK_OBJECT(data), "", 
                      GTK_SIGNAL_FUNC(), data) ; */

   gtk_widget_destroy(AutoParam->dialog) ;
   /* update the GUI */
   while (gtk_events_pending ())
      gtk_main_iteration () ;
   /* Turn off 'suspend' locker */
   suspend = FALSE ;
   printf("**** auto_exec_run\n");
   gtk_signal_emit_by_name(GTK_OBJECT (data), "file-next") ;
   /* gtk_widget_show_all (AutoParam->main_window) ; */

#ifdef cippa
   /* Loop over files */

   list1 = list[nrif] ;
   stack = stack_init(list1, f_set->n) ;

   for (j=0; j<f_set->n; j++){
      if ( j != nrif ) {

         /* Get current star list */
         list2 = list[j] ;

         data->gparam->CRVAL[0] = list1->origin[0] ;/** ***FIXME: eliminare ? */
         data->gparam->CRVAL[1] = list1->origin[1] ;
         data->gparam->CRPIX[0] = list2->origin[0] ;
         data->gparam->CRPIX[1] = list2->origin[1] ;

         /* Initialize Plate Model */
         pmodel = model_init(list1) ;
         pmodel->crval[0] = list1->origin[0] ;
         pmodel->crval[1] = list1->origin[1] ;
         pmodel->crpix[0] = list2->origin[0] ;
         pmodel->crpix[1] = list2->origin[1] ;
         star_list_align(list1, list2, pmodel) ;

         if ( pmodel == (PlateMod*)NULL ) goto exit ;
/* **FIXME: proiettare coord */
         /* Search for matching stars */
         match = search_base(list1, list2, pmodel, &nmatch) ;

         /* Fit plate model transformation */
         if ( data->gparam->fitmodel!=LIN && nmatch!=0 )
            model_fit(list1, list2, pmodel, match, nmatch) ;

         /* Get xyrms */
         model_get_rms(list1, list2, match, nmatch, pmodel) ;

         /* Resume info */
         /* if (spheric) strcpy(line, "arcsec\0") ;
         else strcpy(line, "pixel\0") ;
         i = (spheric) ? 3600 : 1 ;
         cp_message(1,1,"Number of matching pairs is \33[1m%d\33[0m\n",nmatch);
	 cp_message(0,1,"Computed residuals from model fit in X and Y:\n") ;
	 cp_message(0,1,"   X_RMS error = %#.6g [%s]\n",
                i * ABS(pmodel->cdelt[0])*pmodel->dxdev, line) ;
	 cp_message(0,1,"   Y_RMS error = %#.6g [%s]\n", 
                i * ABS(pmodel->cdelt[1])*pmodel->dydev, line) ;
         cp_message(0,1,"Computed PIXSIZE = %#.4g [%s]\n", i * 
                    (ABS(pmodel->cdelt[0])+ABS(pmodel->cdelt[1]))/2.0, line) ; */

         /* Put coordinate in world system: spheric is that of reference frame */
         coord_rot_inv_scale(list2->data, (int)list2->nstar, pmodel) ;
         coord_rot2pix(list2->data, (int)list2->nstar, spheric, pmodel->crval) ;
         /* star_list_center(list2, &center[0],&center[1]) ;
         i = (spheric) ? 5 : 3 ;
         cp_message(0,1,"Computed plate center: X = %+#.*f; Y = %+#.*f\n", 
                    i, center[0], i, center[1]) ; */

         /* Put the current star list into the coinc structure */
         if (stack_update(&stack, list2, j, match, nmatch)) goto exit ;

         /* Save Plate Models */
	 if (data->gparam->savetrans==TRUE) {
            sprintf(parfile,"%s.plm",f_set->info[j].filename) ;
	    model_save(parfile, f_set->info[nrif].filename, pmodel, f_set->info+j) ;
	 }

         /* Save rotated star list */
         data->gparam->append = ( list1->spheric && !list2->spheric ) ;
	 if (data->gparam->fitmodel!=NONE) 
	    star_list_save(f_set->info+j, list2) ;

         /* Swap reference with frame for mosaic mode */
         if ( data->gparam->mosaic==TRUE ) 
            list1 = list2 ;

	 /* free memory */
	 free(match) ;
	 match = (Match*)NULL ;
         free(pmodel) ;
         pmodel = (PlateMod*)NULL ;
      }
   }
   /* Save Stackidence Table */
   coinc_tab_save(stack, f_set->info, data->gparam->outfile) ;

   /* If we reach here, then everything was ok */
   status = 0 ;
   exit:

   /* Free memory */
   if ( match != (Match*)NULL ) free(match) ;
   if ( pmodel != (PlateMod*)NULL ) free(pmodel) ;
#endif
   printf("**** auto_exec_run end\n");
   /* Turn off 'suspend' locker */
   /* suspend = FALSE ; */
}


static void
entry_changed0(GtkWidget *entry, gpointer data)
{
   Sfind *FindSet = data ;
   gchar *text ;
   gchar  buffer[255] ;
   if (!GTK_IS_ENTRY (entry)) 
      return ;
   text = gtk_entry_get_text(GTK_ENTRY (entry)) ;
   sprintf(buffer, "%g", atof(text)) ;
   gtk_signal_disconnect_by_func(GTK_OBJECT(entry),
         GTK_SIGNAL_FUNC(entry_changed0), (gpointer) FindSet) ;
   gtk_entry_set_text(GTK_ENTRY (entry), buffer) ;
   gtk_signal_emit_stop_by_name (GTK_OBJECT(entry), "changed") ;
   gtk_signal_connect(GTK_OBJECT(entry), "changed",
         GTK_SIGNAL_FUNC(entry_changed0), (gpointer) FindSet) ;
}

static void
get_buttons(GtkWidget *dialog, Aparam *AutoParam )
{
   GtkWidget *button ;

   button = gtk_button_new_with_label ("Run") ;
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
         GTK_SIGNAL_FUNC(auto_exec_run), AutoParam) ;

   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
         button, TRUE, TRUE, 0) ;
   GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT) ;
   gtk_widget_show(button) ;

   button = gtk_button_new_with_label ("Accept") ;
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
         GTK_SIGNAL_FUNC(auto_exec_accept), AutoParam) ;
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
         button, TRUE, TRUE, 0) ;
   GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT) ;
   gtk_widget_show(button) ;

   button = gtk_button_new_with_label ("Cancel") ;
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
         GTK_SIGNAL_FUNC(gtk_widget_destroy), (gpointer) dialog) ;
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
         button, TRUE, TRUE, 0) ;
   GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT) ;
   gtk_widget_grab_default(button) ;
   gtk_widget_show (button) ;

   button = gtk_button_new_with_label ("Help") ;
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
         button, TRUE, TRUE, 0) ;
   GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT) ;
   gtk_widget_show (button) ;
}

static void
file_set_sph(GtkWidget *widget, Aparam *AutoParam)
{
   CxcData *data = CXC_DATA (AutoParam->display->data) ;
   gint     i, n = -1 ;
   
   for(i=0; i<data->f_set->n; i++)
      if (widget == AutoParam->FileSet->set[i].spheric) 
         n = i ;
   if (n < 0) return ;
   if (gtk_toggle_button_get_active (
            GTK_TOGGLE_BUTTON (AutoParam->FileSet->set[n].spheric))) {
      sprintf(AutoParam->FindSet->fset[n].value[0], "%g", 0.000014 * sph_units) ;
      sprintf(AutoParam->FindSet->fset[n].value[1], "%g", 0.00065 * sph_units) ;
      sprintf(AutoParam->FindSet->fset[n].value[2], "%g", 0.020 * sph_units) ;
      sprintf(AutoParam->FindSet->fset[n].value[3], "%g", 1.0) ;
      sprintf(AutoParam->FindSet->fset[n].value[4], "%g", 0.000014 * sph_units) ;
      sprintf(AutoParam->FindSet->fset[n].value[5], "%g", 0.00833333 * sph_units) ;
      sprintf(AutoParam->FindSet->fset[n].value[6], "%g", 0.000014 * sph_units) ;
      sprintf(AutoParam->FindSet->fset[n].value[7], "%g", 0.00833333 * sph_units) ;
      AutoParam->FindSet->fset[n].sph = 1 ;
      sprintf(AutoParam->MaskSet->set[n].value[0], "%g", 0.00065 * sph_units) ;
   }
   else {
      sprintf(AutoParam->FindSet->fset[n].value[0], "%g", 0.1) ;
      sprintf(AutoParam->FindSet->fset[n].value[1], "%g", 10.0) ;
      sprintf(AutoParam->FindSet->fset[n].value[2], "%g", 250.0) ;
      sprintf(AutoParam->FindSet->fset[n].value[3], "%g", 1.0) ;
      sprintf(AutoParam->FindSet->fset[n].value[4], "%g", 0.1) ;
      sprintf(AutoParam->FindSet->fset[n].value[5], "%g", 50.0) ;
      sprintf(AutoParam->FindSet->fset[n].value[6], "%g", 0.1) ;
      sprintf(AutoParam->FindSet->fset[n].value[7], "%g", 50.0) ;
      AutoParam->FindSet->fset[n].sph = 0 ;
      sprintf(AutoParam->MaskSet->set[n].value[0], "%g", 10.0) ;
   }
   for(i=0; i<8; i++)
      if (AutoParam->FindSet->fset[n].entries[i])
         gtk_entry_set_text(GTK_ENTRY (AutoParam->FindSet->fset[n].entries[i]),
               AutoParam->FindSet->fset[n].value[i]) ;

   if (AutoParam->MaskSet->set[n].entries[0])
      gtk_entry_set_text(GTK_ENTRY (AutoParam->MaskSet->set[n].entries[0]),
            AutoParam->MaskSet->set[n].value[0]) ;

   /* Set value in 'Mix Rad' field of Match Settings */
   /*
    * if (n == 0) {
 *       gchar     buffer[128] ;
 *       gdouble   val ;
 *       val = AutoParam->MatchSet->values[1] ;
 *       if (gtk_toggle_button_get_active (
 *             GTK_TOGGLE_BUTTON (AutoParam->FileSet->set[0].spheric))) {
 *          AutoParam->MatchSet->s = TRUE ;
 *          if (val > 0.0) 
 *             AutoParam->MatchSet->values[1] = (0.25/3600.0) * sph_units ;
 *       }
 *       else {
 *          AutoParam->MatchSet->s = FALSE ;
 *          if (val > 0.0) 
 *             AutoParam->MatchSet->values[1] = 0.75 ;
 *       }
 *       if (val > 0.0) {
 *          sprintf(buffer, "%g",  AutoParam->MatchSet->values[1]) ;
 *          gtk_entry_set_text(GTK_ENTRY (AutoParam->MatchSet->entry), 
 *                buffer) ;
 *       }
 *    }
 */ 
}

/* Link '-fit NONE' choice to '-find NONE' setting */
static void
find_set_none(GtkWidget *widget, Aparam *AutoParam)
{
   gtk_notebook_set_page (GTK_NOTEBOOK(AutoParam->FindSet->notebook), 3) ;
}

static void
find_set_units(GtkWidget *widget, Aparam *AutoParam)
{
   CxcData *data = CXC_DATA (AutoParam->display->data) ;
   gdouble  s, val ;
   gint     i ;
   for (i=0; i<data->f_set->n; i++) {
      s = (gtk_toggle_button_get_active (
            GTK_TOGGLE_BUTTON (AutoParam->FileSet->set[i].spheric))) ? 
            ((sph_units == 3600.0) ? 3600.0 : 1.0/3600.0 ) : 1.0 ;

      /* Find Set entries */
      if (AutoParam->FindSet->fset[i].entries[0]) {
         val = atof(gtk_entry_get_text(GTK_ENTRY (AutoParam->FindSet->fset[i].entries[0]))) ;
         sprintf(AutoParam->FindSet->fset[i].value[0], "%g", val * s) ;
      }
      if (AutoParam->FindSet->fset[i].entries[1]) {
         val = atof(gtk_entry_get_text(GTK_ENTRY (AutoParam->FindSet->fset[i].entries[1]))) ;
         sprintf(AutoParam->FindSet->fset[i].value[1], "%g", val * s) ;
      }
      if (AutoParam->FindSet->fset[i].entries[2]) {
         val = atof(gtk_entry_get_text(GTK_ENTRY (AutoParam->FindSet->fset[i].entries[2]))) ;
         sprintf(AutoParam->FindSet->fset[i].value[2], "%g", val * s) ;
      }
      if (AutoParam->FindSet->fset[i].entries[3]) {
         val = atof(gtk_entry_get_text(GTK_ENTRY (AutoParam->FindSet->fset[i].entries[3]))) ;
         if (val != 1.0) 
            sprintf(AutoParam->FindSet->fset[i].value[3], "%g", val * s) ;
      }
      if (AutoParam->FindSet->fset[i].entries[4]) {
         val = atof(gtk_entry_get_text(GTK_ENTRY (AutoParam->FindSet->fset[i].entries[4]))) ;
         sprintf(AutoParam->FindSet->fset[i].value[4], "%g", val * s) ;
      }
      if (AutoParam->FindSet->fset[i].entries[5]) {
         val = atof(gtk_entry_get_text(GTK_ENTRY (AutoParam->FindSet->fset[i].entries[5]))) ;
         sprintf(AutoParam->FindSet->fset[i].value[5], "%g", val * s) ;
      }
      if (AutoParam->FindSet->fset[i].entries[6]) {
         val = atof(gtk_entry_get_text(GTK_ENTRY (AutoParam->FindSet->fset[i].entries[6]))) ;
         sprintf(AutoParam->FindSet->fset[i].value[6], "%g", val * s) ;
      }
      if (AutoParam->FindSet->fset[i].entries[7]) {
         val = atof(gtk_entry_get_text(GTK_ENTRY (AutoParam->FindSet->fset[i].entries[7]))) ;
         sprintf(AutoParam->FindSet->fset[i].value[7], "%g", val * s) ;
      }

      /* Mask crow rad */
      if (AutoParam->MaskSet->set[i].entries[0]) {
         val = atof(gtk_entry_get_text(GTK_ENTRY (AutoParam->MaskSet->set[i].entries[0]))) ;
         sprintf(AutoParam->MaskSet->set[i].value[0], "%g", val * s) ;
         gtk_entry_set_text(GTK_ENTRY (AutoParam->MaskSet->set[i].entries[0]), 
               AutoParam->MaskSet->set[i].value[0]) ;
      }
   }
   /* put_find_text(AutoParam->FindSet, CXC_DATA (AutoParam->display->data)) ; */
}

static void
match_set_units(GtkWidget *widget, Aparam *AutoParam)
{
   gdouble  s, val ;
   gchar      buffer[128] ;

   s = (gtk_toggle_button_get_active (
         GTK_TOGGLE_BUTTON (AutoParam->FileSet->set[0].spheric))) ?
         ((sph_units == 3600.0) ? 3600.0 : 1.0/3600.0 ) : 1.0 ;
   /*
    * if (AutoParam->MatchSet->entry) {
 *       val = atof(gtk_entry_get_text(GTK_ENTRY (AutoParam->MatchSet->entry))) ;
 *       if (val > 0.0) {
 *          AutoParam->MatchSet->values[1] = val * s ;
 *          sprintf(buffer, "%g", AutoParam->MatchSet->values[1]) ;
 *          gtk_entry_set_text(GTK_ENTRY (AutoParam->MatchSet->entry), buffer) ;
 *       }
 *    }
 */ 
}

void
auto_exec( GtkWidget *widget, cxc_display *display )
{
   GtkWidget *dialog, *box, *vbox, *hbox, *button ;
   GtkWidget *frame, *label, *bbox, *hbox1, *check, *entry, *table ;
   guint      i ;
   gchar      buffer[1024] ;
   FileSet   *f_set = CXC_DATA (display->data)->f_set ;
   CxcData   *data = CXC_DATA (display->data) ;
   size_t     nfiles = f_set->n ;

   static     Sfile  FilesSet ;
   static     Smask  MaskSet ;
   static     Sfind  FindSet ;
   static     Salign  AlignSet ;
   /* static     Smatch  MatchSet ; */
   static     Smodel  ModelSet ;
   static     Sglob  GlobSet ;
   static struct autop_tag  AutoParam ;


   if ( display->auto_exec_dialog ) {
      /* Window is already opened */
      gtk_window_set_transient_for (GTK_WINDOW (display->auto_exec_dialog), 
            GTK_WINDOW (display->window)) ;
      return ;
   }

   /* Suspend current execution */
   if ( CXC_DATA (display->data)->locked ) {
      suspend_query(display->window, display) ;
      printf ("LOCKED ************************************\n");
      /* gtk_signal_disconnect_by_data (GTK_OBJECT(CXC_DATA (display->data)),
         CXC_DATA (display->data)) ;*/
   /* Turn off 'suspend' locker */
    /*   suspend = TRUE ;
      CXC_DATA (display->data)->locked = FALSE ; */
       /* update the GUI */
      /* while (gtk_events_pending ())
         gtk_main_iteration () ; */
      if ( !suspend ) return ;
   }


   /* Create a new window */
   display->auto_exec_dialog = dialog = gtk_dialog_new( ) ;

   /* Initialize Sfind struct */
   FilesSet.set = (file_set*)calloc(nfiles, sizeof(file_set)) ;
   FilesSet.f_set = f_set ;
   MaskSet.set = (mask_set*)calloc(nfiles, sizeof(mask_set)) ;
   MaskSet.n = nfiles ;
   FindSet.page = 0 ;
   FindSet.nfiles = nfiles ;
   FindSet.active = (guint*) malloc (nfiles * sizeof(guint)) ;
   for (i=0; i<nfiles; i++)
      FindSet.active[i] = (guint)i ;
   FindSet.fset = (find_set*)calloc(nfiles, sizeof(find_set)) ;
   AutoParam.main_window = display->window ;
   AutoParam.dialog = dialog ;
   AutoParam.FileSet = &FilesSet ;
   AutoParam.MaskSet = &MaskSet ;
   AutoParam.FindSet = &FindSet ;
   AutoParam.AlignSet = &AlignSet ;
   /* AutoParam.MatchSet = &MatchSet ; */
   AutoParam.ModelSet = &ModelSet ;
   AutoParam.GlobSet = &GlobSet ;
   AutoParam.display = display ;

   get_file_val(&FilesSet) ;
   get_mask_val(&MaskSet, data) ;
   get_align_val(&FindSet, data) ;
   get_register_val(&AlignSet, data) ;
   /* get_match_val(&MatchSet, data) ; */
   get_model_val(&ModelSet, data) ;
   get_global_val(&GlobSet, data) ;

   gtk_window_set_title(GTK_WINDOW(dialog), "CataXcorr: Auto Execution") ;
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE) ;
   gtk_signal_connect(GTK_OBJECT(dialog), "destroy", 
                      GTK_SIGNAL_FUNC(cancel), &FindSet) ;
   gtk_signal_connect(GTK_OBJECT(dialog), "destroy", 
                      GTK_SIGNAL_FUNC(close_window), (gpointer)display) ;

   /* Main container */
   box = gtk_vbox_new(FALSE, 0) ;
   gtk_container_set_border_width (GTK_CONTAINER(box), 8) ;
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), box,
                      TRUE, TRUE, 0) ;

   /* File Settings */
   vbox = file_settings(&FilesSet, display) ;
   gtk_container_add(GTK_CONTAINER(box), vbox) ;

   /* Star Mask Settings */
   vbox = mask_settings(&MaskSet, display) ;
   gtk_container_add(GTK_CONTAINER(box), vbox) ;

   /* Find Settings */
   vbox = align_settings(&FindSet, display) ;
   gtk_container_add(GTK_CONTAINER(box), vbox) ;

   hbox1 = gtk_hbox_new(FALSE, 0) ;
   gtk_box_pack_start(GTK_BOX(box), hbox1, FALSE, FALSE, 5) ;

   /* Fine adjustment */
   vbox = register_settings(&AlignSet, display) ;
   gtk_container_add(GTK_CONTAINER(hbox1), vbox) ;

   /* Star matching settings */
   /* vbox = match_settings(&MatchSet, display) ;
   gtk_container_add(GTK_CONTAINER(hbox1), vbox) ; */

   /* Model Settings */
   vbox = model_settings(&ModelSet, display) ;
   gtk_container_add(GTK_CONTAINER(box), vbox) ;

   hbox1 = gtk_hbox_new(FALSE, 0) ;
   gtk_box_pack_start(GTK_BOX(box), hbox1, FALSE, FALSE, 5) ;

   /* Save settings */

   set_font ("-*-times-medium-r-normal--18-140-*-*-*-*-*-*") ;
   frame = gtk_frame_new("Save files") ;
   gtk_container_set_border_width (GTK_CONTAINER(frame), 8) ;
   /* gtk_container_add(GTK_CONTAINER(hbox1), frame) ; */
   gtk_box_pack_start(GTK_BOX(hbox1), frame, TRUE, TRUE, 0) ;
   set_font ("-*-times-medium-r-normal--17-140-*-*-*-*-*-*") ;
   vbox = gtk_vbox_new(FALSE, 0) ;
   gtk_container_add(GTK_CONTAINER(frame), vbox) ;
   hbox = gtk_hbox_new(FALSE, 0) ;
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5) ;

   table = gtk_table_new (4, 3, FALSE);
   gtk_table_set_col_spacings (GTK_TABLE (table), 10);
   gtk_table_set_row_spacings (GTK_TABLE (table), 2);
   gtk_box_pack_start (GTK_BOX (hbox), table, TRUE, TRUE, 5);

   bbox = gtk_hbox_new(FALSE, 5) ;
   gtk_table_attach (GTK_TABLE (table), bbox, 0, 3, 0, 1,
         GTK_FILL | GTK_EXPAND , GTK_SHRINK | GTK_FILL, 0, 0) ;
   label = gtk_label_new("Stackidence Table:") ;
   gtk_box_pack_start(GTK_BOX(bbox), label, FALSE, FALSE, 0) ;
   entry = gtk_entry_new() ;
   gtk_entry_set_text(GTK_ENTRY (entry), data->gparam->outfile) ;
   gtk_widget_set_usize(entry, 80, 0) ;
   gtk_box_pack_start(GTK_BOX(bbox), entry, TRUE, TRUE, 0) ;

   button = gtk_button_new_with_label ("Choose...") ;
   gtk_table_attach (GTK_TABLE (table), button, 3, 4, 0, 1,
         GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0) ;

   bbox = gtk_hbox_new(FALSE, 5) ;
   gtk_table_attach (GTK_TABLE (table), bbox, 0, 1, 1, 2,
         GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0) ;
   label = gtk_label_new("Save Rotated:") ;
   gtk_box_pack_start(GTK_BOX(bbox), label, TRUE, TRUE, 0) ;
   check = gtk_check_button_new ( ) ;
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check),
         TRUE) ;
   gtk_table_attach (GTK_TABLE (table), check, 1, 2, 1, 2,
         GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0) ;

   bbox = gtk_hbox_new(FALSE, 5) ;
   gtk_table_attach (GTK_TABLE (table), bbox, 2, 3, 1, 2,
         GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0) ;
   label = gtk_label_new("Extension:") ;
   gtk_box_pack_start(GTK_BOX(bbox), label, TRUE, TRUE, 0) ;
   entry = gtk_entry_new() ;
   sprintf(buffer, ".rot") ;
   gtk_entry_set_text(GTK_ENTRY (entry), buffer) ;
   gtk_widget_set_usize(entry, 40, 0) ;
   gtk_box_pack_start(GTK_BOX(bbox), entry, FALSE, FALSE, 0) ;

   


   bbox = gtk_hbox_new(FALSE, 5) ;
   gtk_table_attach (GTK_TABLE (table), bbox, 0, 1, 2, 3,
         GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0) ;
   label = gtk_label_new("Save Model:") ;
   gtk_box_pack_start(GTK_BOX(bbox), label, TRUE, TRUE, 0) ;
   check = gtk_check_button_new ( ) ;
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check),
         TRUE) ;
   gtk_table_attach (GTK_TABLE (table), check, 1, 2, 2, 3,
         GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0) ;
   bbox = gtk_hbox_new(FALSE, 5) ;
   gtk_table_attach (GTK_TABLE (table), bbox, 2, 3, 2, 3,
         GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0) ;
   label = gtk_label_new("Extension:") ;
   gtk_box_pack_start(GTK_BOX(bbox), label, TRUE, TRUE, 0) ;
   entry = gtk_entry_new() ;
   sprintf(buffer, ".plm") ;
   gtk_entry_set_text(GTK_ENTRY (entry), buffer) ;
   gtk_widget_set_usize(entry, 40, 0) ;
   gtk_box_pack_start(GTK_BOX(bbox), entry, FALSE, FALSE, 0) ;


   /* Global Settings */
   vbox = global_settings(&GlobSet, display) ;
   gtk_box_pack_start(GTK_BOX(hbox1), vbox, FALSE, FALSE, 0) ;

   /* --- */

   /* React to changes in 'spherical' toggle button */
   for (i=0; i<(guint)f_set->n; i++) 
      gtk_signal_connect (GTK_OBJECT (FilesSet.set[i].spheric), "toggled",
            GTK_SIGNAL_FUNC(file_set_sph), &AutoParam) ;
   /* React to 'fit NONE' choice */
   gtk_signal_connect (GTK_OBJECT (ModelSet.item[3]), "activate",
         GTK_SIGNAL_FUNC(find_set_none), &AutoParam) ;
   /* React to changes in 'spherical' units */
   for (i=0; i<2; i++) {
      gtk_signal_connect (GTK_OBJECT (GlobSet.item1[i]), "activate",
            GTK_SIGNAL_FUNC(find_set_units), &AutoParam) ;
      /* gtk_signal_connect (GTK_OBJECT (GlobSet.item1[i]), "activate",
            GTK_SIGNAL_FUNC(match_set_units), &AutoParam) ; */
   }


   /* --- */


   gtk_widget_show_all(box) ;


   /* buttons */
   get_buttons(dialog, &AutoParam) ;

   gtk_widget_show_all (dialog) ;
}

/*
 * Global Settings */

static void
get_units(GtkWidget *item, Sglob *GlobSet)
{
   gint i ;
   for(i=0; i<2; i++)
      if (item == GlobSet->item1[i]) 
         GlobSet->n1 = i ;
   sph_units = (GlobSet->n1 == 0) ? 1.0 : 3600.0 ;
}

static void
get_glob(GtkWidget *item, Sglob *GlobSet)
{
   gint i ;
   for(i=0; i<2; i++)
      if (item == GlobSet->item1[i]) 
         GlobSet->n1 = i ;
   for(i=0; i<2; i++)
      if (item == GlobSet->item2[i]) 
         GlobSet->n2 = i ;
   for(i=0; i<3; i++)
      if (item == GlobSet->item3[i]) 
         GlobSet->n3 = i ;
}

GtkWidget *
global_settings(Sglob *GlobSet, cxc_display *display)
{
   GtkWidget *box, *vbox, *hbox, *frame, *label, *option, *menu, *table, *item ;

   box = gtk_vbox_new(FALSE, 0) ;

   /* Global Settings */

   set_font ("-*-times-medium-r-normal--18-140-*-*-*-*-*-*") ;
   frame = gtk_frame_new("Global settings") ;
   gtk_container_set_border_width (GTK_CONTAINER(frame), 8) ;
   gtk_box_pack_start(GTK_BOX(box), frame, FALSE, FALSE, 0) ;
   /* gtk_container_add(GTK_CONTAINER(hbox1), frame) ; */
   set_font ("-*-times-medium-r-normal--17-140-*-*-*-*-*-*") ;
   vbox = gtk_vbox_new(FALSE, 0) ;
   gtk_container_add(GTK_CONTAINER(frame), vbox) ;
   hbox = gtk_hbox_new(FALSE, 0) ;
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5) ;

   table = gtk_table_new (3, 3, FALSE);
   gtk_table_set_col_spacings (GTK_TABLE (table), 2);
   gtk_table_set_row_spacings (GTK_TABLE (table), 2);
   gtk_box_pack_start (GTK_BOX (hbox), table, TRUE, TRUE, 5);

   label = gtk_label_new("Spherical Units:") ;
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
         GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0) ;
   option = gtk_option_menu_new() ;
   menu = gtk_menu_new() ;
   GlobSet->item1[0] = item = 
         make_menu_item ("Deg", GTK_SIGNAL_FUNC (get_units), GlobSet) ;
   gtk_menu_append (GTK_MENU (menu), item) ;
   GlobSet->item1[1] = item = 
         make_menu_item ("ArcSec", GTK_SIGNAL_FUNC (get_units), GlobSet) ;
   gtk_menu_append (GTK_MENU (menu), item) ;
   gtk_menu_set_active (GTK_MENU (menu), GlobSet->n1) ;
   gtk_option_menu_set_menu (GTK_OPTION_MENU (option), menu) ;
   gtk_table_attach (GTK_TABLE (table), option, 2, 3, 0, 1,
         GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0) ;


   label = gtk_label_new("WCS format:") ;
   gtk_label_set_justify(GTK_LABEL(label),  GTK_JUSTIFY_LEFT) ; /*FIXME ?? */
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
         GTK_SHRINK , GTK_SHRINK | GTK_FILL, 0, 0) ;
   option = gtk_option_menu_new() ;
   menu = gtk_menu_new() ;
   GlobSet->item2[0] = item = 
         make_menu_item ("Degrees", GTK_SIGNAL_FUNC (get_glob), GlobSet) ;
   gtk_menu_append (GTK_MENU (menu), item) ;
   GlobSet->item2[1] = item = 
         make_menu_item ("Sexagesimal", GTK_SIGNAL_FUNC (get_glob), GlobSet) ;
   gtk_menu_append (GTK_MENU (menu), item) ;
   gtk_option_menu_set_menu (GTK_OPTION_MENU (option), menu) ;
   gtk_table_attach (GTK_TABLE (table), option, 2, 3, 1, 2,
         GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0) ;

   label = gtk_label_new("WCS tangent point:") ;
   gtk_label_set_justify(GTK_LABEL(label),  GTK_JUSTIFY_LEFT) ; /*FIXME ?? */
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3,
         GTK_SHRINK , GTK_SHRINK | GTK_FILL, 0, 0) ;
   option = gtk_option_menu_new() ;
   menu = gtk_menu_new() ;
   GlobSet->item3[0] = item = 
         make_menu_item ("Frame center", GTK_SIGNAL_FUNC (get_glob), GlobSet) ;
   gtk_menu_append (GTK_MENU (menu), item) ;
   GlobSet->item3[1] = item = 
         make_menu_item ("Set CRPIX", GTK_SIGNAL_FUNC (get_glob), GlobSet) ;
   gtk_menu_append (GTK_MENU (menu), item) ;
   GlobSet->item3[2] = item = 
         make_menu_item ("Set CRVAL", GTK_SIGNAL_FUNC (get_glob), GlobSet) ;
   gtk_menu_append (GTK_MENU (menu), item) ;
   gtk_option_menu_set_menu (GTK_OPTION_MENU (option), menu) ;
   gtk_table_attach (GTK_TABLE (table), option, 2, 3, 2, 3,
         GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0) ;

   gtk_widget_show_all (box) ;

   return box ;
}

void
get_global_val (Sglob *GlobSet, CxcData *data)
{
   GlobSet->n1 = (sph_units == 1.0) ? 0 : 1 ;
   GlobSet->n2 = GlobSet->n3 = 0 ;
}
