/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 							Includes
 *--------------------------------------------------------------------------*/

#if HAVE_CONFIG_H
#  include "config.h"
#endif

#include "common.h"
#include <gtk/gtk.h>

#include "cataxcorr.h"
#include "platemodel.h"

#include "cpk_gtk_util.h"
#include "cxc_data.h"
#include "cxc_map.h"
#include "cxc_frames.h"
#include "cxc_gtk_main.h"
#include "cxc_match_set.h"
#include "cxc_align_set.h"
#include "cxc_register_set.h"
#include "cxc_model_set.h"
#include "cxc_save.h"
#include "cpk_gtk_cursorutil.h"

#include "cxc_buttons.h"

/*----------------------------------------------------------------------------
 * 							Function prototypes
 *--------------------------------------------------------------------------*/

static void        print_hello                (GtkWidget     *w,
                                               gpointer       data) ;
static void        help                       (void) ;
static void        call_quit_program          (cxc_display   *display,
                                               guint          action,
                                               GtkWidget     *item) ;
static void        set_panels                 (cxc_display   *display,
                                               guint          action,
                                               GtkWidget     *check) ;
static void        call_zoom_in               (cxc_display   *display,
                                               guint          action,
                                               GtkWidget     *item) ;
static void        call_zoom_out              (cxc_display   *display,
                                               guint          action,
                                               GtkWidget     *item) ;
static void        call_zoom_center           (cxc_display   *display,
                                               guint          action,
                                               GtkWidget     *item) ;
static void        call_swap_x                (cxc_display   *display,
                                               guint          action,
                                               GtkWidget     *item) ;
static void        call_swap_y                (cxc_display   *display,
                                               guint          action,
                                               GtkWidget     *item) ;
static void        set_zoom_value             (cxc_display   *display,
                                               guint          action,
                                               GtkWidget     *item) ;
static void        call_align_wcs             (cxc_display   *display,
                                               guint          action,
                                               GtkWidget     *item) ;
static void        set_wcs_fmt                (cxc_display   *display,
                                               guint          action,
                                               GtkWidget     *item) ;
static void        set_tunits0                (GtkWidget     *item,
                                               Sfind         *FindSet) ;
static void        set_tunits1                (GtkWidget     *item,
                                               Sfind         *FindSet) ;
static void        set_mgf_fct                (cxc_display   *display,
                                               guint          action,
                                               GtkWidget     *item) ;
static void        set_blink_rate             (cxc_display   *display,
                                               guint          action,
                                               GtkWidget     *item) ;
static void        set_frame_lock             (cxc_display   *display, 
                                               guint          action, 
                                               GtkWidget     *check) ;
static void        call_frame_next            (cxc_display   *display,
                                               guint          action,
                                               GtkWidget     *item) ;
static void        call_frame_prev            (cxc_display   *display,
                                               guint          action,
                                               GtkWidget     *item) ;
static void        set_cursor_type            (cxc_display   *display,
                                               guint          action,
                                               GtkWidget     *item) ;
static void        set_pnt                    (GtkWidget     *widget,
                                               cxc_display   *display) ;
static void        set_pan                    (GtkWidget     *widget,
                                               cxc_display   *display) ;
static void        set_zoom                   (GtkWidget     *widget,
                                               cxc_display   *display) ;
static void        set_rot                    (GtkWidget     *widget,
                                               cxc_display   *display) ;
static void        set_rul                    (GtkWidget     *widget,
                                               cxc_display   *display) ;

static void        call_reset_frame           (cxc_display   *display,
                                               guint          action,
                                               GtkWidget     *item) ;
static void        call_align_frames          (cxc_display   *display,
                                               guint          action,
                                               GtkWidget     *item) ;
static void        call_register_frames       (cxc_display   *display,
                                               guint          action,
                                               GtkWidget     *item) ;
static void        call_plate_model           (cxc_display   *display,
                                               guint          action,
                                               GtkWidget     *item) ;
static void        call_residuals             (cxc_display   *display,
                                               guint          action,
                                               GtkWidget     *item) ;
static void        call_match_stars           (cxc_display   *display,
                                               guint          action,
                                               GtkWidget     *item) ;
static void        menuitem_response          (GtkWidget     *button,
                                               guint         *show ) ;
static void        reset_frames               (GtkWidget     *button,
                                               cxc_display   *display ) ;
static void        cxc_buttons_2_get          (GtkWidget     *toggle,
                                               cxc_display   *display) ;
static void        cxc_buttons_reset          (GtkWidget     *toggle,
                                               cxc_display   *display) ;
static void        set_frame_mode             (GtkWidget     *toggle,
                                               cxc_display   *display) ;
static void        set_swap                   (GtkWidget     *toggle,
                                               cxc_display   *display) ;
static void        set_point_action           (GtkWidget     *toggle,
                                               cxc_display   *display) ;
extern void        data_content               (GtkWidget     *widget,
                                               cxc_display   *display ) ;
extern void        auto_exec                  (GtkWidget     *widget,
                                               cxc_display   *display ) ;
static void        frame_reparent_hbox        (cxc_display   *display, 
                                               guint          action, 
                                               GtkWidget     *item) ;
static void        frame_reparent_vbox        (cxc_display   *display, 
                                               guint          action, 
                                               GtkWidget     *item) ;
static void        frame_reparent_tbox        (cxc_display   *display, 
                                               guint          action, 
                                               GtkWidget     *item) ;

/*----------------------------------------------------------------------------
 * 							Private types
 *--------------------------------------------------------------------------*/

enum {
   F_PREV = FRAME_COINC + 1,
   F_NEXT
} ;

enum {
   Z_IN = 0,
   Z_OUT, 
   Z_CENTER, 
   Z_SWAPX, 
   Z_SWAPY, 
   Z_PAN, 
   Z_ZOOM, 
   Z_ROT, 
   Z_ALIGN
} ;

struct buttons_tag {
   char               *label ;
   GtkWidget          *button ;
   GtkSignalFunc       callback ;
   gint                code ;
   gboolean            status ;
} ;

/*----------------------------------------------------------------------------
 * 							Global variables	
 *--------------------------------------------------------------------------*/

struct buttons_tag Mfile[] = {
   { "Open",           NULL,     cxc_file_new,  0, FALSE },
   { "Close",          NULL,   cxc_file_close,  0, FALSE },
   { "Data Content",   NULL,     data_content,  0,  TRUE },
   { "Self Execute",   NULL,        auto_exec,  0, FALSE },
   { "Save",           NULL,         cxc_save,  0,  TRUE },
   { "Exit",           NULL,     quit_program,  0,  TRUE }
} ;

struct buttons_tag Mframe[] = {
   /* { "New",             NULL,        frame_new,  0, FALSE },
   { "Delete",          NULL,      frame_close,  0, FALSE },
   { "Reset",           NULL,      frame_reset,  0,  TRUE }, */
   { "Single",          NULL,   set_frame_mode,  1, FALSE },
   { "Pair",            NULL,   set_frame_mode,  1, FALSE },
   { "Tile",            NULL,   set_frame_mode,  1,  TRUE },
   { "Blink",           NULL,   set_frame_mode,  1, FALSE },
   { "Merge",           NULL,   set_frame_mode,  1, FALSE },
   { "Stack",           NULL,   set_frame_mode,  1, FALSE },
   { "Prev",            NULL,       frame_prev,  0,  TRUE },
   { "Next",            NULL,       frame_next,  0,  TRUE }
} ;

struct buttons_tag Mzoom[] = {
   { "In" ,             NULL,          zoom_in,  0,  TRUE },
   { "Out",             NULL,         zoom_out,  0,  TRUE },
   { "Center",          NULL,      zoom_center,  0,  TRUE },
   { "Swap X",          NULL,         set_swap, -1, FALSE },
   { "Swap Y",          NULL,         set_swap, -2, FALSE },
   { "Pan",             NULL, set_point_action,  1, FALSE },
   { "Zoom",            NULL, set_point_action,  1, FALSE },
   { "Rot",             NULL, set_point_action,  1, FALSE },
   { "Align WCS",       NULL,        align_wcs,  0, FALSE }
} ;

struct buttons_tag Mmask[] = {
   { "Crowding",        NULL,             NULL,  0,  FALSE },
   { "Magnitude",       NULL,             NULL,  0,  FALSE },
   { "Region",          NULL,             NULL,  0,  FALSE }
} ;

struct buttons_tag Mscale[] = {
   { "Cuts" ,           NULL,             NULL,  0, FALSE },
   { "Balance",         NULL,             NULL,  0, FALSE },
   { "Linear",          NULL,             NULL,  1, FALSE },
   { "Log",             NULL,             NULL,  1, FALSE },
   { "Square",          NULL,             NULL,  1, FALSE },
   { "Squared Root",    NULL,             NULL,  1, FALSE },
} ;

struct buttons_tag Mwcs[] = {
   { "Mask stars" ,     NULL,            NULL,  0, FALSE },
   { "Align Frames" ,   NULL,    align_frames,  0,  TRUE },
   { "Register Frames", NULL, register_frames,  0, FALSE },
   { "Plate Model",     NULL,     plate_model,  0, FALSE },
   { "Residuals",       NULL,   cxc_res_frame,  0, FALSE },
   { "STACK",           NULL,     match_stars,  0,  TRUE }
} ;

struct buttons_tag Mmatch[] = {
   { "Match stars" ,   NULL, match_stars,  0,  TRUE },
   { "Show matched" ,  NULL,        NULL,  0, FALSE },
   { "Show unmatched", NULL,        NULL,  0, FALSE },
   { "Residuals",      NULL,        NULL,  0, FALSE }
} ;

struct {
   char               *label ;
   gint                nb ;
   struct buttons_tag *Mbutton ;
} buttons[] = {
   { "File",       sizeof(Mfile )/sizeof(Mfile [0]),     Mfile },
   { "Frame",      sizeof(Mframe)/sizeof(Mframe[0]),    Mframe },
   { "Zoom",         sizeof(Mzoom)/sizeof(Mzoom[0]),     Mzoom },
   /* { "Mask",      sizeof(Mmask)/sizeof(Mmask[0]),     Mmask }, */
   { "Scale",      sizeof(Mscale)/sizeof(Mscale[0]),    Mscale },
   { "Analysis",       sizeof(Mwcs)/sizeof(Mwcs[0]),      Mwcs }/* ,
   { "Match Lists", sizeof(Mmatch)/sizeof(Mmatch[0]),   Mmatch } */
} ;

static GtkItemFactoryEntry menu_items[] = {
/* 0*/  { "/_File",                                  NULL,             NULL, 0, "<Branch>" },
/* 1*/  { "/File/Open",                      "<control>O",      print_hello, 0, NULL },
/* 2*/  { "/File/Close",                     "<control>W",      print_hello, 0, NULL },
/* 3*/  { "/File/_Data Content",             "<control>D",      print_hello, 0, NULL },
/* 4*/  { "/File/Self Execute",              "<control>E",      print_hello, 0, NULL },
/* 5*/  { "/File/_Save",                     "<control>S",      print_hello, 0, NULL },
/* 6*/  { "/File/sep1",                              NULL,             NULL, 0, "<Separator>" },
/* 7*/  { "/File/Exit",                      "<control>Q", call_quit_program, 1, NULL },
        { "/_Edit",                                  NULL,             NULL, 0, "<Branch>" },
/* 9*/  { "/Edit/tearoff1",                          NULL,             NULL, 0, "<Tearoff>" },
        { "/Edit/Pointer",                           NULL,  set_cursor_type, 0, "<RadioItem>" },
/*11*/  { "/Edit/Pan",                               NULL,  set_cursor_type, 0, "/Edit/Pointer" },
        { "/Edit/Zoom",                              NULL,  set_cursor_type, 0, "/Edit/Pointer" },
/*13*/  { "/Edit/Rotate",                            NULL,  set_cursor_type, 0, "/Edit/Pointer" },
        { "/Edit/Ruler",                             NULL,  set_cursor_type, 0, "/Edit/Pointer" },
/*15*/  { "/Edit/sep1",                              NULL,             NULL, 0, "<Separator>" },
        { "/F_rame",                                 NULL,             NULL, 0, "<Branch>" },
/*17*/  { "/Frame/tearoff1",                         NULL,             NULL, 0, "<Tearoff>" },
        { "/Frame/Single Frame",                     NULL, set_frame_single, 1, "<RadioItem>" },
/*19*/  { "/Frame/Pair Frames",                      NULL,   set_frame_pair, 1, "/Frame/Single Frame" },
        { "/Frame/Tile Frames",                      NULL,   set_frame_tile, 1, "/Frame/Single Frame" },
/*21*/  { "/Frame/Blink Frames",                     NULL,  set_frame_blink, 1, "/Frame/Single Frame" },
        { "/Frame/Merge Frames",                     NULL,  set_frame_merge, 1, "/Frame/Single Frame" },
/*23*/  { "/Frame/Stack Frame",                      NULL,  set_frame_coinc, 1, "/Frame/Single Frame" },
        { "/Frame/sep1",                             NULL,             NULL, 0, "<Separator>" },
/*25*/  { "/Frame/Lock Frames",                      NULL,   set_frame_lock, 1, "<CheckItem>" },
        { "/Frame/sep1",                             NULL,             NULL, 0, "<Separator>" },
/*27*/  { "/Frame/Show\134Hide Frames",              NULL,             NULL, 0, "<Branch>" },
/*28*/  { "/Frame/sep1",                             NULL,             NULL, 0, "<Separator>" },
        { "/Frame/Prev",                             NULL,  call_frame_prev, 1, NULL },
/*30*/  { "/Frame/Next",                             NULL,  call_frame_next, 1, NULL },
        { "/Frame/sep1",                             NULL,             NULL, 0, "<Separator>" },
/*32*/  { "/Frame/Get Frame...",                     NULL,             NULL, 0, NULL },
        { "/_Zoom",                                  NULL,             NULL, 0, "<Branch>" },
/*34*/  { "/Zoom/tearoff1",                          NULL,             NULL, 0, "<Tearoff>" },
        { "/Zoom/Align to Equatorial WCS",           NULL,   call_align_wcs, 1, "<CheckItem>" },
/*36*/  { "/Zoom/sep1",                              NULL,             NULL, 0, "<Separator>" },
/*37*/  { "/Zoom/Zoom In",                           NULL,     call_zoom_in, 1, NULL },
/*38*/  { "/Zoom/Zoom Out",                          NULL,    call_zoom_out, 1, NULL },
        { "/Zoom/sep1",                              NULL,             NULL, 0, "<Separator>" },
/*40*/  { "/Zoom/Center",                            NULL, call_zoom_center, 1, NULL },
        { "/Zoom/sep1",                              NULL,             NULL, 0, "<Separator>" },
/*42*/  { "/Zoom/1:16",                              NULL,   set_zoom_value, 1, "<RadioItem>" },
        { "/Zoom/1:8",                               NULL,   set_zoom_value, 1, "/Zoom/1:16" },
/*44*/  { "/Zoom/1:4",                               NULL,   set_zoom_value, 1, "/Zoom/1:16" },
        { "/Zoom/1:2",                               NULL,   set_zoom_value, 1, "/Zoom/1:16" },
/*46*/  { "/Zoom/1",                                 NULL,   set_zoom_value, 1, "/Zoom/1:16" },
        { "/Zoom/2",                                 NULL,   set_zoom_value, 1, "/Zoom/1:16" },
/*48*/  { "/Zoom/4",                                 NULL,   set_zoom_value, 1, "/Zoom/1:16" },
        { "/Zoom/8",                                 NULL,   set_zoom_value, 1, "/Zoom/1:16" },
/*50*/  { "/Zoom/16",                                NULL,   set_zoom_value, 1, "/Zoom/1:16" },
/*51*/  { "/Zoom/sep1",                              NULL,             NULL, 0, "<Separator>" },
/*52*/  { "/Zoom/Swap X",                            NULL,      call_swap_x, 1, "<CheckItem>"  },
/*53*/  { "/Zoom/Swap Y",                            NULL,      call_swap_y, 1, "<CheckItem>" },
/*54*/  { "/Zoom/sep1",                              NULL,             NULL, 0, "<Separator>" },
/*55*/  { "/Zoom/Pan Zoom Rotate to...",             NULL,             NULL, 0, NULL },
        { "/_Scale",                                 NULL,             NULL, 0, "<Branch>" },
/*57*/  { "/Scale/tearoff1",                         NULL,             NULL, 0, "<Tearoff>" },
        { "/Scale/Set User Cuts",                    NULL,             NULL, 0, NULL },
/*59*/  { "/Scale/Set Relative Calibration",         NULL,             NULL, 0, NULL },
        { "/Scale/sep1",                             NULL,             NULL, 0, "<Separator>" },
/*61*/  { "/Scale/Linear",                           NULL,             NULL, 0, "<RadioItem>" },
        { "/Scale/Log",                              NULL,             NULL, 0, "/Scale/Linear" },
/*63*/  { "/Scale/Squared",                          NULL,             NULL, 0, "/Scale/Linear" },
/*64*/  { "/Scale/Square Root",                      NULL,             NULL, 0, "/Scale/Linear" },
        { "/_Analysis",                              NULL,                 NULL, 0, "<Branch>" },
/*66*/  { "/Analysis/tearoff1",                      NULL,                 NULL, 0, "<Tearoff>" },
        { "/Analysis/Reset current frame...",        NULL,     call_reset_frame, 1, NULL },
/*68*/  { "/Analysis/sep1",                          NULL,                 NULL, 0, "<Separator>" },
        { "/Analysis/Mask Stars...",                 NULL,                 NULL, 1, NULL },
/*70*/  { "/Analysis/Align Frames...",               NULL,    call_align_frames, 1, NULL },
        { "/Analysis/Register Frames...",            NULL, call_register_frames, 1, NULL },
/*72*/  { "/Analysis/sep1",                          NULL,              NULL, 0, "<Separator>" },
/*73*/  { "/Analysis/Plate Model...",                NULL,  call_plate_model, 1, NULL },
        { "/Analysis/Show Residuals",                NULL,    call_residuals, 1, NULL },
/*75*/  { "/Analysis/sep1",                          NULL,              NULL, 0, "<Separator>" },
        { "/Analysis/STACK Control...",              NULL,  call_match_stars, 1, NULL },
/*77*/  { "/_Preferences",                     "<shift>P",           NULL, 0, "<Branch>" },
        { "/Preferences/tearoff1",                   NULL,           NULL, 0, "<Tearoff>" },
/*79*/  { "/Preferences/Show Information Panel",     NULL,     set_panels, 1, "<CheckItem>" },
        { "/Preferences/Show Panner",                NULL,     set_panels, 1, "<CheckItem>" },
/*81*/  { "/Preferences/Show Magnifier",             NULL,     set_panels, 1, "<CheckItem>" },
        { "/Preferences/Show Buttons",               NULL,     set_panels, 1, "<CheckItem>" },
/*83*/  { "/Preferences/Show Console",               NULL,     set_panels, 1, "<CheckItem>" },
        { "/Preferences/sep1",                       NULL,           NULL, 0, "<Separator>" },
/*85*/  { "/Preferences/Information Panel",          NULL,           NULL, 0, "<Branch>" },
        { "/Preferences/WCS",                        NULL,           NULL, 0, "<Branch>" },
/*87*/  { "/Preferences/WCS/Degrees",                NULL,    set_wcs_fmt, 1, "<RadioItem>" },
        { "/Preferences/WCS/Sexagesimal",            NULL,    set_wcs_fmt, 1, "/Preferences/WCS/Degrees" },
/*89*/  { "/Preferences/Spherical Units",            NULL,           NULL, 0, "<Branch>" },
        { "/Preferences/Spherical Units/Arcsec",     NULL,    set_tunits0, 1, "<RadioItem>" },
/*91*/  { "/Preferences/Spherical Units/Degrees",    NULL,    set_tunits1, 1, "/Preferences/Spherical Units/Arcsec" },
        { "/Preferences/Magnifier",                  NULL,           NULL, 0, "<Branch>" },
/*93*/  { "/Preferences/Magnifier/2x",               NULL,    set_mgf_fct, 1, "<RadioItem>" },
/*94*/  { "/Preferences/Magnifier/4x",               NULL,    set_mgf_fct, 1, "/Preferences/Magnifier/2x" },
        { "/Preferences/Magnifier/8x",               NULL,    set_mgf_fct, 1, "/Preferences/Magnifier/2x" },
/*96*/  { "/Preferences/Magnifier/16x",              NULL,    set_mgf_fct, 1, "/Preferences/Magnifier/2x" },
        { "/Preferences/Magnifier/32x",              NULL,    set_mgf_fct, 1, "/Preferences/Magnifier/2x" },
/*98*/  { "/Preferences/Tile",                       NULL,           NULL, 0, "<Branch>" },
        { "/Preferences/Tile/tearoff1",              NULL,           NULL, 0, "<Tearoff>" },
/*00*/  { "/Preferences/Tile/Grid",                  NULL, frame_reparent_tbox, 1, "<RadioItem>" },
        { "/Preferences/Tile/Column",                NULL, frame_reparent_hbox, 1, "/Preferences/Tile/Grid" },
/*02*/  { "/Preferences/Tile/Raw",                   NULL, frame_reparent_vbox, 1, "/Preferences/Tile/Grid" },
/*03*/  { "/Preferences/Blink Interval",             NULL,           NULL, 0, "<Branch>" },
        { "/Preferences/Blink Interval/tearoff1",    NULL,           NULL, 0, "<Tearoff>" },
/*05*/  { "/Preferences/Blink Interval/0.1 Seconds", NULL, set_blink_rate, 1, "<RadioItem>" },
        { "/Preferences/Blink Interval/0.2 Seconds", NULL, set_blink_rate, 1, "/Preferences/Blink Interval/0.1 Seconds" },
/*07*/  { "/Preferences/Blink Interval/0.5 Seconds", NULL, set_blink_rate, 1, "/Preferences/Blink Interval/0.1 Seconds" },
        { "/Preferences/Blink Interval/1 Second",    NULL, set_blink_rate, 1, "/Preferences/Blink Interval/0.1 Seconds" },
/*09*/  { "/Preferences/Blink Interval/2 Seconds",   NULL, set_blink_rate, 1, "/Preferences/Blink Interval/0.1 Seconds" },
        { "/Preferences/Blink Interval/4 Seconds",   NULL, set_blink_rate, 1, "/Preferences/Blink Interval/0.1 Seconds" },
/*11*/  { "/Preferences/Blink Interval/8 Seconds",   NULL, set_blink_rate, 1, "/Preferences/Blink Interval/0.1 Seconds" },
        { "/Preferences/sep1",                       NULL,           NULL, 0, "<Separator>" },
/*13*/  { "/Preferences/Coordinate Grid",            NULL,           NULL, 0, "<CheckItem>" },
/*14*/  { "/_Help",                                  NULL,           NULL, 0, "<LastBranch>" },
/*15*/  { "/_Help/About",                            NULL,           help, 0, NULL },
/*16*/  { "/Help/sep1",                              NULL,           NULL, 0, "<Separator>" },
/*17*/  { "/_Help/Version",                          NULL,           NULL, 0, NULL },
/*18*/  { "/_Help/Distribution Policy",              NULL,           NULL, 0, NULL },
/*19*/  };
/*20*/


GtkItemFactory *item_factory ;


extern gdouble   tunits ;


/*----------------------------------------------------------------------------
 * 							Private functions
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 							Defines
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 *							Function codes
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 							Buttons
 *--------------------------------------------------------------------------*/

GtkWidget *
cxc_buttons_main (GtkWidget *window, cxc_display *display)
{
   GtkWidget *hbox, *vbox, *toggle ;
   gint       i ;
   gint       n = sizeof (buttons) / sizeof (buttons[0]) ;

   vbox = gtk_vbox_new (FALSE, 0) ;
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 0);
   hbox = gtk_hbox_new (FALSE, 0) ;
   gtk_container_set_border_width (GTK_CONTAINER (hbox), 0);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0) ;
   for (i = 0; i < n; i++) {
      toggle = gtk_toggle_button_new_with_label (buttons[i].label) ;
      gtk_container_add (GTK_CONTAINER (hbox), toggle);
      gtk_signal_connect (GTK_OBJECT (toggle), "toggled",
            GTK_SIGNAL_FUNC (cxc_buttons_2_get), display) ;
      display->buttons[i] = toggle ;
   }
   display->boxbuttons2 = gtk_hbox_new (FALSE, 0) ;
   gtk_container_set_border_width (GTK_CONTAINER (display->boxbuttons2), 0);
   gtk_box_pack_start (GTK_BOX (vbox), display->boxbuttons2, FALSE, FALSE, 0) ;

   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (display->buttons[0]), TRUE) ;

   gtk_widget_show_all (vbox) ;

   return vbox ;
}

/* Map submenues depending on primary menu button selection */
static void 
cxc_buttons_2_get (GtkWidget *toggle, cxc_display *display)
{
   GtkWidget *hbox, *button ;
   gint       i, j ;
   gint       n = sizeof (buttons) / sizeof (buttons[0]) ;
   struct buttons_tag *buttons2 ;

   /* If select same button just mantains it active */
   if (!GTK_TOGGLE_BUTTON (toggle)->active) {
      /* Do nothing */
      gtk_signal_disconnect_by_func (GTK_OBJECT (toggle),
            GTK_SIGNAL_FUNC (cxc_buttons_2_get), display) ;
      gtk_signal_emit_stop_by_name (GTK_OBJECT (toggle), "toggled") ;
      gtk_toggle_button_set_active (
            GTK_TOGGLE_BUTTON (toggle), TRUE) ;
      gtk_signal_connect (GTK_OBJECT (toggle), "toggled",
            GTK_SIGNAL_FUNC (cxc_buttons_2_get), display) ;
      return ;
   }

   /* Get button1 number */
   j = 0 ;
   for (i = 0; i < n; i++) {
      if (display->buttons[i] == toggle)
         j = i ;
      else {
         gtk_signal_disconnect_by_func (GTK_OBJECT (display->buttons[i]),
               GTK_SIGNAL_FUNC (cxc_buttons_2_get), display) ;
         gtk_toggle_button_set_active (
               GTK_TOGGLE_BUTTON (display->buttons[i]), FALSE) ;
         gtk_signal_connect (GTK_OBJECT (display->buttons[i]), "toggled",
               GTK_SIGNAL_FUNC (cxc_buttons_2_get), display) ;
      }
   }

   /* Destroy previous buttons2 */
   if (display->lastboxbuttons2) {
      buttons2 = buttons[display->nsel].Mbutton ;
      n = buttons[display->nsel].nb ;
      gtk_widget_destroy (display->lastboxbuttons2) ;
      for (i = 0; i < n; i++) 
         buttons2[i].button = NULL;
   }

   /* Map submenu buttons */
   display->lastboxbuttons2 = hbox = gtk_hbox_new (FALSE, 0)  ;
   gtk_box_pack_start (GTK_BOX (display->boxbuttons2), hbox, TRUE, TRUE, 0) ;

   display->nsel = j ;
   buttons2 = buttons[j].Mbutton ;
   n = buttons[j].nb ;

   for (i = 0; i < n; i++) {
      if (buttons2[i].code == 0) {
         button = gtk_button_new_with_label (buttons2[i].label) ;
         gtk_widget_set_sensitive (button,buttons2[i].status) ;
      }
      else {
         button = gtk_toggle_button_new_with_label (buttons2[i].label) ;
         gtk_toggle_button_set_active (
               GTK_TOGGLE_BUTTON (button), buttons2[i].status) ; 
         gtk_signal_connect (GTK_OBJECT (button), "toggled",
               GTK_SIGNAL_FUNC (cxc_buttons_reset), display) ;
      }
      gtk_signal_connect (GTK_OBJECT (button), "clicked",
            GTK_SIGNAL_FUNC (buttons2[i].callback), display) ;
      buttons2[i].button = button ;
      gtk_container_add (GTK_CONTAINER (hbox), button);
   }
   gtk_widget_show_all (hbox) ;
}

/* Reset Radio Buttons groups in submenues */
static void 
cxc_buttons_reset (GtkWidget *toggle, cxc_display *display)
{
   gint       i, j, n, group ;
   struct buttons_tag *buttons2 ;

   CPKDEBUG5 ("cxc_buttons_reset", 4) ;

   buttons2 = buttons[display->nsel].Mbutton ;
   n = buttons[display->nsel].nb ;
   j = 0 ;
   for (i = 0; i < n; i++) 
      if (buttons2[i].button == toggle)
         j = i ;

   if (buttons2[j].code)
      buttons2[j].status = gtk_toggle_button_get_active (
            GTK_TOGGLE_BUTTON (toggle)) ;

   /* This is for swap buttons */
   if (buttons2[j].code <= 0)
      return ;

   if (!GTK_TOGGLE_BUTTON (toggle)->active) {
      /* Do nothing */
      gtk_signal_disconnect_by_func (GTK_OBJECT (toggle), 
            GTK_SIGNAL_FUNC (cxc_buttons_reset), display) ;
      gtk_signal_disconnect_by_func (GTK_OBJECT (toggle), 
            GTK_SIGNAL_FUNC (buttons2[j].callback), display) ;
      gtk_signal_emit_stop_by_name (GTK_OBJECT (toggle), "toggled") ;
      gtk_toggle_button_set_active (
            GTK_TOGGLE_BUTTON (toggle), TRUE) ;
      gtk_signal_connect (GTK_OBJECT (toggle), "toggled",
            GTK_SIGNAL_FUNC (cxc_buttons_reset), display) ;
      gtk_signal_connect (GTK_OBJECT (toggle), "clicked", 
            GTK_SIGNAL_FUNC (buttons2[j].callback), display) ;
      buttons2[j].status = TRUE ;
      return ;
   }

   group = buttons2[j].code ;
   for (i = 0; i < n; i++)
      if (buttons2[i].code == group && i != j)
         if (GTK_TOGGLE_BUTTON (buttons2[i].button)->active) {
            gtk_signal_disconnect_by_func (GTK_OBJECT (buttons2[i].button),  
                  GTK_SIGNAL_FUNC (cxc_buttons_reset), display) ;
            gtk_signal_disconnect_by_func (GTK_OBJECT (buttons2[i].button), 
                  GTK_SIGNAL_FUNC (buttons2[i].callback), display) ;
/***/   gtk_signal_emit_stop_by_name (GTK_OBJECT (toggle), "toggled") ;
            gtk_toggle_button_set_active (
                  GTK_TOGGLE_BUTTON (buttons2[i].button), FALSE) ;
            gtk_signal_connect (GTK_OBJECT (buttons2[i].button), "toggled",
                  GTK_SIGNAL_FUNC (cxc_buttons_reset), display) ;
            gtk_signal_connect (GTK_OBJECT (buttons2[i].button), "clicked", 
                  GTK_SIGNAL_FUNC (buttons2[i].callback), display) ;
            buttons2[i].status = gtk_toggle_button_get_active (
                  GTK_TOGGLE_BUTTON (buttons2[i].button)) ;
         }
}

void
cxc_buttons_frame_mode (cxc_display *display, enum mode_tag mode)
{
   GtkWidget *toggle = NULL ;
   guint      i ;
   enum mode_tag old_mode = display->mode ;

   CPKDEBUG5 ("cxc_buttons_frame_mode", 5) ;

   if (Mframe[mode].status)
      return ;

   /* First set buttons flag */
   for (i = FRAME_SINGLE; i <= FRAME_COINC; i++) 
      Mframe[i].status = FALSE ;

   /* Then activate correct button: this invokes cxc_buttons_reset module */
   if (mode <= FRAME_COINC) {
      Mframe[mode].status = TRUE ;
      if (Mframe[mode].button)
         toggle = Mframe[mode].button ;
      if (GTK_IS_WIDGET (toggle)) 
         if (!GTK_TOGGLE_BUTTON (toggle)->active) 
            gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle), 
                  Mframe[mode].status) ;
   }
   CPKDEBUG5 ("cxc_buttons_frame_mode_end", 5);
}

void
cxc_buttons_swap (cxc_display *display)
{
   CxcMap  *map = CXC_MAP (display->map[display->current]) ;
   gint     i ;

   CPKDEBUG ("cxc_buttons_swap") ;

   Mzoom[Z_SWAPX].status = (map->swx > 0) ? FALSE : TRUE ;
   Mzoom[Z_SWAPY].status = (map->swy > 0) ? FALSE : TRUE ;

   gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (display->swap[0]), 
         Mzoom[Z_SWAPX].status) ;
   gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (display->swap[1]), 
         Mzoom[Z_SWAPY].status) ;

   if (!GTK_IS_WIDGET (Mzoom[Z_SWAPX].button) || 
       !GTK_IS_WIDGET (Mzoom[Z_SWAPY].button))
      return ;

   for (i=Z_SWAPX; i<=Z_SWAPY; i++) {
      gtk_signal_disconnect_by_func (GTK_OBJECT (Mzoom[i].button),
            GTK_SIGNAL_FUNC (cxc_buttons_reset), display) ;
      gtk_signal_disconnect_by_func (GTK_OBJECT (Mzoom[i].button), 
            GTK_SIGNAL_FUNC (Mzoom[i].callback), display) ;
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (Mzoom[i].button), 
            Mzoom[i].status) ;
      gtk_signal_connect (GTK_OBJECT (Mzoom[i].button), "toggled",
            GTK_SIGNAL_FUNC (cxc_buttons_reset), display) ;
      gtk_signal_connect (GTK_OBJECT (Mzoom[i].button), "toggled",
            GTK_SIGNAL_FUNC (Mzoom[i].callback), display) ;
   }
}

void
cxc_buttons_pnt_action (cxc_display *display, PointerAct action)
{
   GtkWidget *toggle = NULL ;
   PointerAct old_action = display->pointer_action ;
   guint      i ;

   /* First set buttons flag */
   for (i = Z_PAN; i <= Z_ROT; i++)
      Mzoom[i].status = FALSE ;

   /* Then activate correct button: this invokes cxc_buttons_reset module */
   if (action >= PAN && action <= ROTATE ) {
      if (Mzoom[5+action-PAN].button)
         toggle = Mzoom[Z_PAN+action-PAN].button ;
      if (GTK_IS_WIDGET (toggle))
         if (!GTK_TOGGLE_BUTTON (toggle)->active) 
            gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle), TRUE) ;
      Mzoom[Z_PAN+action-PAN].status = TRUE ;
   }
   else {
      /* This deselect active button: useful for POINTER and RULER modes */
      if (old_action >= PAN && old_action <= ROTATE ) {
         if (Mzoom[Z_PAN+old_action-PAN].button)
            toggle = Mzoom[Z_PAN+old_action-PAN].button ;
         if ( GTK_IS_WIDGET (toggle)) {
            gtk_signal_disconnect_by_func (GTK_OBJECT (toggle),
                  GTK_SIGNAL_FUNC (cxc_buttons_reset), display) ;
            gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle), FALSE) ;
            gtk_signal_connect (GTK_OBJECT (toggle), "toggled",
                  GTK_SIGNAL_FUNC (cxc_buttons_reset), display) ;
         }
      }
   }
   return ;
}

/* Set status of zoom center and frame prev/next buttons */
void 
cxc_buttons_mode_status (cxc_display *display, enum mode_tag mode)
{
   CPKDEBUG5 ("cxc_buttons_mode_status",5);
   switch (mode) {
      case FRAME_COINC :
         Mzoom[Z_CENTER].status = TRUE ;
         Mframe[F_PREV].status = Mframe[F_NEXT].status = FALSE ;
         break ;
      case FRAME_MERGE : case FRAME_BLINK :
         Mzoom[Z_CENTER].status = FALSE ;
         Mframe[F_PREV].status = Mframe[F_NEXT].status = FALSE ;
         break ;
      default :
         Mzoom[Z_CENTER].status = TRUE ;
         Mframe[F_PREV].status = Mframe[F_NEXT].status = TRUE ;
         break ;
   } 

   if (Mzoom[Z_CENTER].button)
      if (GTK_IS_WIDGET (Mzoom[Z_CENTER].button))
         gtk_widget_set_sensitive (Mzoom[Z_CENTER].button, Mzoom[Z_CENTER].status) ;
   if (display->zoom_center)
      gtk_widget_set_sensitive (display->zoom_center, Mzoom[Z_CENTER].status) ;
   if (Mframe[F_PREV].button)
      if (GTK_IS_WIDGET (Mframe[F_PREV].button)) {
         gtk_widget_set_sensitive (Mframe[F_PREV].button, Mframe[F_PREV].status) ;
         gtk_widget_set_sensitive (Mframe[F_NEXT].button, Mframe[F_NEXT].status) ;
      }
   if (display->frame_prev)
      gtk_widget_set_sensitive (display->frame_prev, Mframe[F_PREV].status) ;
   if (display->frame_next)
      gtk_widget_set_sensitive (display->frame_next, Mframe[F_NEXT].status) ;

   CPKDEBUG5 ("cxc_buttons_mode_status_end",6);
}

/* Set status of buttons depending on plate model status */

void
cxc_buttons_wcs_status (cxc_display *display)
{
   CxcData *data = CXC_DATA (display->data) ;
   PlateMod *model ;
   gboolean status[4] = { FALSE, FALSE, FALSE, FALSE } ;
   gboolean align_stat = FALSE ;
   gint i, second_frame ;

   CPKDEBUG ("cxc_buttons_wcs_status") ;

   second_frame = cxc_second_frame (display) ;

   if (second_frame < display->nframes)
      model = data->model[second_frame] ;
   else
      model = NULL ;

   /* WCS buttons status*/
   if (model) {
      if (model->status  >= INIT) 
         status[0] = TRUE ;
      if (model->status  >= LOW_ALIGN) 
         status[1] = status [2] = TRUE ;
      if (model->status  == HIGH_ALIGN)
         status [2] = TRUE ;
      if (model->status  == MODEL_FITTED) {
         status [2] = FALSE ;
         status[3] = TRUE ;
      }
   }

   for (i = 0; i < 4; i++) {
      Mwcs[i+1].status = status[i] ;
      if (Mwcs[i+1].button)
         if (GTK_IS_WIDGET (Mwcs[i+1].button))
            gtk_widget_set_sensitive (Mwcs[i+1].button, status[i]) ;
      if (display->analisys[i]) /***FIXME***/
         gtk_widget_set_sensitive (display->analisys[i], status[i]) ;
   }

   /* 'Align WCS' button status */
   align_stat = status[1] ;

   if (display->current == display->nframes || display->current == data->nref)
      align_stat = TRUE ;

   Mzoom[Z_ALIGN].status = align_stat ;
   if (Mzoom[Z_ALIGN].button)
      if (GTK_IS_WIDGET (Mzoom[Z_ALIGN].button))
         gtk_widget_set_sensitive (Mzoom[Z_ALIGN].button, align_stat) ;
   if (display->align_wcs)
      gtk_widget_set_sensitive (display->align_wcs, align_stat) ;
}

/*----------------------------------------------------------------------------
 * 							Frame Menu
 *--------------------------------------------------------------------------*/

static void 
set_frame_mode ( GtkWidget *toggle, cxc_display *display)
{
   gint  i, j ;
   CPKDEBUG ("set_frame_mode") ;

   j = -1 ;
   for (i = FRAME_SINGLE; i <= FRAME_COINC; i++) 
      if (Mframe[i].button == toggle)
         j = i ;
   if (display->mode == j)
      return ;

   /* This invokes the call to the requested mode */
   if (j >= 0)
      gtk_check_menu_item_set_active  (
            GTK_CHECK_MENU_ITEM (display->frame_radio[j]), TRUE) ;

   CPKDEBUG5 ("set_frame_mode_end",6) ;
}

static void 
call_frame_prev ( cxc_display *display, guint action, GtkWidget *item)
{
   frame_prev (item, display) ;
}

static void 
call_frame_next (cxc_display *display, guint action, GtkWidget *item)
{
   frame_next (item, display) ;
}

/*----------------------------------------------------------------------------
 * 							Zoom Menu
 *--------------------------------------------------------------------------*/

static void 
call_zoom_in ( cxc_display *display, guint action, GtkWidget *item)
{
   zoom_in (item, display) ;
}

static void 
call_zoom_out ( cxc_display *display, guint action, GtkWidget *item)
{
   zoom_out (item, display) ;
}

static void 
call_zoom_center ( cxc_display *display, guint action, GtkWidget *item)
{
   zoom_center (item, display) ;
}

static void 
call_swap_x (cxc_display *display, guint action, GtkWidget *item)
{
   if (GTK_CHECK_MENU_ITEM (item)->active != Mzoom[Z_SWAPX].status) {
      swap_x (item, display) ;
      cxc_buttons_swap (display) ;
   }
}

static void 
call_swap_y (cxc_display *display, guint action, GtkWidget *item)
{
   if (GTK_CHECK_MENU_ITEM (item)->active != Mzoom[Z_SWAPY].status) {
      swap_y (item, display) ;
      cxc_buttons_swap (display) ;
   }
}

static void 
set_swap (GtkWidget *toggle, cxc_display *display)
{
   CPKDEBUG5 ("set_swap", 5) ;

   /* This invokes the call to the requested mode */
   if (Mzoom[Z_SWAPX].button == toggle) {
      swap_x (NULL, display) ;
      gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (display->swap[0]), 
            Mzoom[Z_SWAPX].status) ;
   }
   if (Mzoom[Z_SWAPY].button == toggle) {
      swap_y (NULL, display) ;
      gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (display->swap[1]), 
            Mzoom[Z_SWAPY].status) ;
   }
}

static void 
set_point_action (GtkWidget *toggle, cxc_display *display)
{
   guint  i, j ;

   j = 0 ;
   for (i = Z_PAN; i <= Z_ROT; i++) 
      if (Mzoom[i].button == toggle)
         j = i-Z_PAN+1 ;
   /* Clicking again on current mode get POINTER action */
   if (display->pointer_action == j)
      j = POINTER ;
   gtk_check_menu_item_set_active (
         GTK_CHECK_MENU_ITEM (display->zoom_radio2[j]), TRUE) ;
}

static void 
set_zoom_value (cxc_display *display, guint action, GtkWidget *item)
{
   gdouble zoom[] = {1/16.0, 1/8., 1/4., 1/2., 1.0, 2.0, 4.0, 8.0, 16.0 } ;
   gint i, j ;

   if (!GTK_CHECK_MENU_ITEM (item)->active)
      return ;

   j = -1 ;
   for (i=0; i<9; i++)
      if (item == display->zoom_radio1[i])
         j = i ;
   if (j == (gint) CXC_MAP (display->map[display->current])->izoom) 
      return ;

   if (j < 0)
      return ;

   /* Change zoom for reference map for MERGE mode */
   if ( display->mode == FRAME_MERGE ) {
      CxcMap *ref = CXC_MAP (display->map[display->current])->reference ;
      ref->zoom *= zoom[j] / CXC_MAP (display->map[display->current])->zoom ;
      for (i=0; i<9; i++)
         if (ref->zoom == zoom[i])
            ref->izoom = i ;
   }

   CXC_MAP (display->map[display->current])->izoom = j ;
   CXC_MAP (display->map[display->current])->zoom = zoom[j] ;
   cxc_map_remap (NULL, CXC_MAP (display->map[display->current])) ;
}

static void 
call_align_wcs (cxc_display *display, guint action, GtkWidget *item)
{
   align_wcs (item, display) ;
}


static void print_hello (GtkWidget *w,
                         gpointer   data)
{
  g_message ("Hello, World!\n");
}

static void 
help (void)
{
   g_print ("HELP ??? ha ha ha!\n") ;
}

static void 
call_quit_program ( cxc_display *display, guint action, GtkWidget *item)
{
   quit_program (item, display) ;
}

/* Show / hides display subframes */

static void 
set_panels (cxc_display *display, guint action, GtkWidget *check)
{
   gint i, j ;
   if (display->panels[0] == NULL) 
      return ;
   j = 0 ;
   for (i = 0; i < 5; i++)
      if (display->flag_check[i] == check)
         j = i ;

   display->settings[j] = !display->settings[j] ;

   if (display->settings[j])
      gtk_widget_show (display->panels[j]) ;
   else
      gtk_widget_hide (display->panels[j]) ;

   /* FIXME: next sequence is to force main vbox to resize it's childs */
   gtk_widget_show_all (display->panels[3]) ;
   gtk_widget_hide (display->panels[3]) ;
   if (display->settings[3])
      gtk_widget_show_all (display->panels[3]) ;
}

static void 
set_frame_lock (cxc_display *display, guint action, GtkWidget *check)
{
   display->f_lock = !display->f_lock ;
}

static void 
set_wcs_fmt (cxc_display *display, guint action, GtkWidget *item)
{
   gint i, j ;
   j = -1 ;
   for (i=0; i<2; i++)
      if (item == display->pref_wcs_radio[i])
         j = i ;
   if (j == (gint)display->wcs_fmt) 
      return ;
   display->wcs_fmt = j ;
}

static void
set_tunits0 (GtkWidget *item, Sfind *FindSet)
{
   if (tunits == 3600.0)
      return ;
   tunits = 3600.0 ;
}

static void
set_tunits1 (GtkWidget *item, Sfind *FindSet)
{
   if (tunits == 1.0)
      return ;
   tunits = 1.0 ;
}


static void 
set_mgf_fct (cxc_display *display, guint action, GtkWidget *item)
{
   guint factor[] = {2, 4, 8, 16, 32} ;
   gint  i, j ;

   if (!GTK_CHECK_MENU_ITEM (item)->active)
      return ;
   j = -1 ;
   for (i=0; i<5; i++)
      if (item == display->pref_magnifier_radio[i])
         j = i ;
   if (j < 0)
      return ;

   display->mgf_factor = factor[j] ;
}

static void 
set_blink_rate (cxc_display *display, guint action, GtkWidget *item)
{
   guint32 rate[] = { 100, 200, 500, 1000, 2000, 4000, 8000 } ;
   gint i, j ;
   j = -1 ;
   for (i=0; i<7; i++)
      if (item == display->pref_blink_radio[i])
         j = i ;
   if (j == (gint)display->blink_rate) 
      return ;
   display->blink_rate = j ;
   if (display->mode == FRAME_BLINK) {
      gtk_timeout_remove (display->timer) ;
      display->timer = 0 ;
      display->timer = gtk_timeout_add (rate[display->blink_rate], 
         blink_timer, display) ;
   }
}

static void 
set_cursor_type ( cxc_display *display, guint action, GtkWidget *item)
{
   gint i, j ;

   if (!GTK_CHECK_MENU_ITEM (item)->active)
      return ;

   j = -1 ;
   for (i = 0; i < 5; i++)
      if (item == display->zoom_radio2[i])
         j = i ;

   switch (j) {
      case (0) :
         set_pnt ( NULL, display ) ;
         break ;
      case (1) :
         set_pan ( NULL, display ) ;
         break ;
      case (2) :
         set_zoom ( NULL, display ) ;
         break ;
      case (3) :
         set_rot ( NULL, display ) ;
         break ;
      case (4) :
         set_rul ( NULL, display ) ;
         break ;
      default :
         break ;
   }
}

void
cxc_set_cursor (GtkWidget *widget, cxc_display *display)
{
   GdkWindow *window ;
   guint      i ;

   for (i = 0; i <= display->nframes; i++) {
      window = CXC_MAP (display->map[i])->draw_area->window ;
      /* window = display->frames[i]->window ; */
      if (window) 
         switch (display->pointer_action) {
            case POINTER :
               change_win_cursor (window, GDK_DRAFT_SMALL, CURSOR_MODIFIER_NONE) ;
               break ;
            case PAN :
               change_win_cursor (window, CPK_PAN_CURSOR, CURSOR_MODIFIER_NONE) ;
               break ;
            case ZOOM :
               change_win_cursor (window, CPK_ZOOM_CURSOR, CURSOR_MODIFIER_NONE) ;
               break ;
            case ROTATE :
               change_win_cursor (window, CPK_ROTATE_CURSOR, CURSOR_MODIFIER_NONE) ;
               break ;
            case RULER :
               change_win_cursor (window, GDK_PENCIL, CURSOR_MODIFIER_NONE) ;
               break ;
            default :
               break ;
         }
   }
}


static void
set_pnt (GtkWidget *widget, cxc_display *display)
{
   if (display->pointer_action == POINTER)
      return ;
   cxc_buttons_pnt_action (display, POINTER) ;
   display->pointer_action = POINTER ;
   cxc_set_cursor (widget, display) ;
}

static void
set_pan (GtkWidget *widget, cxc_display *display)
{
   if (display->pointer_action == PAN)
      return ;
   cxc_buttons_pnt_action (display, PAN) ;
   display->pointer_action = PAN ;
   cxc_set_cursor (widget, display) ;
}

static void
set_zoom (GtkWidget *widget, cxc_display *display)
{
   if (display->pointer_action == ZOOM)
      return ;
   cxc_buttons_pnt_action (display, ZOOM) ;
   display->pointer_action = ZOOM ;
   cxc_set_cursor (widget, display) ;
}

static void
set_rot (GtkWidget *widget, cxc_display *display)
{
   if (display->pointer_action == ROTATE)
      return ;
   cxc_buttons_pnt_action (display, ROTATE) ;
   display->pointer_action = ROTATE ;
   cxc_set_cursor (widget, display) ;
}

static void
set_rul (GtkWidget *widget, cxc_display *display)
{
   if (display->pointer_action == RULER)
      return ;
   cxc_buttons_pnt_action (display, RULER) ;
   display->pointer_action = RULER ;
   cxc_set_cursor (widget, display) ;
}

static void 
call_reset_frame (cxc_display *display, guint action, GtkWidget *item)
{
   frame_reset (item, display) ;
}

static void 
call_align_frames (cxc_display *display, guint action, GtkWidget *item)
{
   align_frames (item, display) ;
}

static void 
call_register_frames (cxc_display *display, guint action, GtkWidget *item)
{
   register_frames (item, display) ;
}

static void 
call_plate_model (cxc_display *display, guint action, GtkWidget *item)
{
   plate_model (item, display) ;
}

static void 
call_residuals ( cxc_display *display, guint action, GtkWidget *item)
{
   cxc_res_frame (item, display) ;
}

static void 
call_match_stars ( cxc_display *display, guint action, GtkWidget *item)
{
   match_stars (item, display) ;
}

static void
menuitem_response (GtkWidget *button, guint *show)
{
   /* *show = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)) ; */
   *show = !*show ;
}

static void
show_all (GtkWidget *button, GtkWidget *menu)
{
   GList *list = gtk_container_children (GTK_CONTAINER (menu)) ;
   list = g_list_nth (list, 3) ;
   while ((list = g_list_next (list))) 
      gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (list->data), TRUE) ;
}

static void
hide_all (GtkWidget *button, GtkWidget *menu)
{
   GList *list = gtk_container_children (GTK_CONTAINER (menu)) ;
   list = g_list_nth (list, 3) ;
   while ((list = g_list_next (list))) 
      gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (list->data), FALSE) ;
}

static void 
reset_frames (GtkWidget *button, cxc_display *display )
{
   guint i ;

   if (display->mode == FRAME_BLINK)
      return ;
   if (display->mode == FRAME_TILE) {
      for (i = 0; i < display->nframes; i++) 
         if (!display->show[i])
            gtk_widget_hide (display->frames[i]) ;
         else
            gtk_widget_show (display->frames[i]) ;
      cxc_frame_reparent (display) ;
   }
   if (!display->show[display->current])
      frame_next (button, display) ;
}

static void 
frame_reparent_hbox (cxc_display *display, guint action, GtkWidget *item)
{
   if (!display->frames[0])
      return ;
   display->tile_mode = display->last_tile_mode = 1 ;
   cxc_frame_reparent (display) ;
}

static void 
frame_reparent_vbox (cxc_display *display, guint action, GtkWidget *item)
{
   if (!display->frames[0])
      return ;
   display->tile_mode = display->last_tile_mode = 2 ;
   cxc_frame_reparent (display) ;
}

static void 
frame_reparent_tbox (cxc_display *display, guint action, GtkWidget *item)
{
   if (!display->frames[0])
      return ;
   display->tile_mode = 0 ;
   cxc_frame_reparent (display) ;
}

GtkWidget * 
cxc_menu_main (GtkWidget *window, cxc_display *display)
{
  GtkAccelGroup  *accel_group ;
  GtkWidget      *menu, *menuitem, *sep ;
  gchar           buffer[11] ;
  guint           nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]) ;
  guint           i ;

  /* gtk_item_factory_parse_rc (".gtkrc") ; */

   /* Create menu for show/hide frames option */
   menu = gtk_menu_new () ;
   menuitem = gtk_tearoff_menu_item_new () ;
   gtk_menu_append (GTK_MENU (menu), menuitem) ;
   gtk_widget_show_all (menuitem) ;

   menuitem = gtk_menu_item_new_with_label ("Show All") ;
   gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
            GTK_SIGNAL_FUNC (show_all), menu) ;
   gtk_menu_append (GTK_MENU (menu), menuitem) ;
   gtk_widget_show_all (menuitem) ;
   menuitem = gtk_menu_item_new_with_label ("Hide All") ;
   gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
            GTK_SIGNAL_FUNC (hide_all), menu) ;
   gtk_menu_append (GTK_MENU (menu), menuitem) ;
   gtk_widget_show_all (menuitem) ;

   menuitem = gtk_menu_item_new () ;
   sep = gtk_hseparator_new () ;
   gtk_container_add (GTK_CONTAINER (menuitem), sep) ;
   gtk_menu_append (GTK_MENU (menu), menuitem) ;
   gtk_widget_set_sensitive (menuitem, FALSE) ;
   gtk_widget_show_all (menuitem) ;

   for (i = 0; i < display->nframes; i++) {
      sprintf (buffer, "Frame %d", i+1) ;
      menuitem = gtk_check_menu_item_new_with_label (buffer) ;
      gtk_check_menu_item_set_active  (GTK_CHECK_MENU_ITEM (menuitem),
            (gboolean)display->show[i]);
      /* check = gtk_check_button_new_with_label (buffer) ;
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check),
            (gboolean)display->show[i]) ;
      gtk_container_add (GTK_CONTAINER (menuitem), check) ; */
      gtk_menu_append (GTK_MENU (menu), menuitem) ;
      gtk_signal_connect (GTK_OBJECT (menuitem), "toggled",
            GTK_SIGNAL_FUNC (menuitem_response), &display->show[i]) ;
      gtk_signal_connect (GTK_OBJECT (menuitem), "toggled",
            GTK_SIGNAL_FUNC (reset_frames), display) ;
      gtk_widget_show_all (menuitem) ;
   }

   accel_group = gtk_accel_group_new () ;

   /* This function initializes the item factory. */
   item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>", 
         accel_group) ;

   gtk_widget_set_name (
   gtk_item_factory_get_widget (item_factory, "<main>"), "cxc-itemfactory") ;

   /* This function generates the menu items. Pass the item factory,
      the number of items in the array, the array itself, and any
      callback data for the the menu items. */
   gtk_item_factory_create_items (item_factory, nmenu_items, menu_items, display);

   /* Attach the new accelerator group to the window. */
   gtk_window_add_accel_group (GTK_WINDOW (window), accel_group) ;

   /* CRITICAL: check offset on menu_item */

   /* Frame mode radio button */
   for (i = 0; i < 6; i++) 
      display->frame_radio[i] = 
         gtk_item_factory_get_item (item_factory, menu_items[i+18].path) ;

   display->frame_prev = 
         gtk_item_factory_get_item (item_factory, menu_items[29].path) ;
   display->frame_next = 
         gtk_item_factory_get_item (item_factory, menu_items[30].path) ;
   /* Zoom radio button */
   display->align_wcs = 
      gtk_item_factory_get_item (item_factory, menu_items[35].path) ;
   gtk_widget_set_sensitive (display->align_wcs, FALSE) ;
   display->zoom_center =
      gtk_item_factory_get_item (item_factory, menu_items[40].path) ;

   for (i = 0; i < 9; i++) 
      display->zoom_radio1[i] =
         gtk_item_factory_get_item (item_factory, menu_items[i+42].path) ;
   /*** FIXME: WARNING !!! */
   i = (guint) CXC_MAP (display->map[display->current])->izoom ;
   gtk_check_menu_item_set_active  (
        GTK_CHECK_MENU_ITEM (display->zoom_radio1[i]), TRUE) ;

   for (i = 0; i < 3; i++)
      display->swap[i] = 
            gtk_item_factory_get_item (item_factory, menu_items[i+52].path) ;

   /* Edit radio button */
   for (i = 0; i < 5; i++) 
      display->zoom_radio2[i] = 
            gtk_item_factory_get_item (item_factory, menu_items[i+10].path) ;
   /*** FIXME: WARNING !!! */
   gtk_check_menu_item_set_active  (GTK_CHECK_MENU_ITEM (
              display->zoom_radio2[display->pointer_action]), TRUE) ;

   /* Analisys buttons */
   for (i = 0; i < 2; i++)
      display->analisys[i] = 
           gtk_item_factory_get_item (item_factory, menu_items[i+70].path) ;
   for (i = 2; i < 4; i++) 
      display->analisys[i] = 
           gtk_item_factory_get_item (item_factory, menu_items[i+71].path) ;
   for (i = 1; i < 4; i++)
      gtk_widget_set_sensitive (display->analisys[i], FALSE) ;
#ifdef pippozzo
   /* Match buttons */
   for (i = 0; i < 3; i++) {
      display->match[i] = 
           gtk_item_factory_get_item (item_factory, menu_items[i+78].path) ;
      gtk_widget_set_sensitive (display->match[i], FALSE) ;
   }
#endif
   /* Show panels checkbuttons */
   for (i = 0; i < 5; i++) {
      display->flag_check[i] = 
         gtk_item_factory_get_item (item_factory, menu_items[i+79].path) ;
      gtk_check_menu_item_set_active  ( 
         GTK_CHECK_MENU_ITEM (display->flag_check[i]), display->settings[i]) ;
   }

  /* WCS radio button */
   for (i = 0; i < 2; i++) {
      display->pref_wcs_radio[i] = 
         gtk_item_factory_get_item (item_factory, menu_items[i+87].path) ;
      if (i == display->wcs_fmt)
         gtk_check_menu_item_set_active  ( 
            GTK_CHECK_MENU_ITEM (display->pref_wcs_radio[i]), TRUE) ;
   }

  /* tunits radio button */
   for (i = 0; i < 2; i++) {
      display->pref_tunits_radio[i] = 
         gtk_item_factory_get_item (item_factory, menu_items[i+90].path) ;
   }
   gtk_check_menu_item_set_active  ( 
      GTK_CHECK_MENU_ITEM (display->pref_tunits_radio[0]), TRUE) ;

   /* Magnifier radio button */
   for (i = 0; i < 5; i++) {
     display->pref_magnifier_radio[i] = 
           gtk_item_factory_get_item (item_factory, menu_items[i+93].path) ;
     if (i == 1)
        gtk_check_menu_item_set_active  ( 
           GTK_CHECK_MENU_ITEM (display->pref_magnifier_radio[i]), TRUE) ;
   }

  /* Tile mode radio button */
   for (i = 0; i < 3; i++) {
      display->pref_tile_radio[i] = 
         gtk_item_factory_get_item (item_factory, menu_items[i+100].path) ;

      if (i == display->tile_mode)
         gtk_check_menu_item_set_active  ( 
            GTK_CHECK_MENU_ITEM (display->pref_tile_radio[i]), TRUE) ;
   }

   /* Blink rate radio button */
   for (i = 0; i < 7; i++) {
      display->pref_blink_radio[i] = 
         gtk_item_factory_get_item (item_factory, menu_items[i+105].path) ;

      if (i == display->blink_rate)
         gtk_check_menu_item_set_active  ( 
            GTK_CHECK_MENU_ITEM (display->pref_blink_radio[i]), TRUE) ;
   }

   /* Set submenu for show/hide frames */
   gtk_menu_item_set_submenu ( GTK_MENU_ITEM (
      gtk_item_factory_get_item (item_factory, menu_items[27].path)), menu) ;
   gtk_widget_show (menu) ;

   /* FIXME: functions not yet realized */
   gtk_widget_set_sensitive (
         gtk_item_factory_get_item (item_factory, menu_items[1].path), FALSE) ;
   gtk_widget_set_sensitive (
         gtk_item_factory_get_item (item_factory, menu_items[2].path), FALSE) ;
   gtk_widget_set_sensitive (
         gtk_item_factory_get_item (item_factory, menu_items[4].path), FALSE) ;
   gtk_widget_set_sensitive (
         gtk_item_factory_get_item (item_factory, menu_items[14].path), FALSE) ;
   gtk_widget_set_sensitive (
         gtk_item_factory_get_item (item_factory, menu_items[32].path), FALSE) ;
   gtk_widget_set_sensitive (
         gtk_item_factory_get_item (item_factory, menu_items[55].path), FALSE) ;
   gtk_widget_set_sensitive (
         gtk_item_factory_get_item (item_factory, menu_items[58].path), FALSE) ;
   gtk_widget_set_sensitive (
         gtk_item_factory_get_item (item_factory, menu_items[59].path), FALSE) ;
   gtk_widget_set_sensitive (
         gtk_item_factory_get_item (item_factory, menu_items[61].path), FALSE) ;
   gtk_widget_set_sensitive (
         gtk_item_factory_get_item (item_factory, menu_items[62].path), FALSE) ;
   gtk_widget_set_sensitive (
         gtk_item_factory_get_item (item_factory, menu_items[63].path), FALSE) ;
   gtk_widget_set_sensitive (
         gtk_item_factory_get_item (item_factory, menu_items[64].path), FALSE) ;
   gtk_widget_set_sensitive (
         gtk_item_factory_get_item (item_factory, menu_items[69].path), FALSE) ;
   gtk_widget_set_sensitive (
         gtk_item_factory_get_item (item_factory, menu_items[85].path), FALSE) ;
   gtk_widget_set_sensitive (
         gtk_item_factory_get_item (item_factory, menu_items[113].path), FALSE) ;



   /* Finally, return the actual menu bar created by the item factory. */ 
   return gtk_item_factory_get_widget (item_factory, "<main>") ;
}
