/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
 * 							Includes
 *--------------------------------------------------------------------------*/

#if HAVE_CONFIG_H
#  include "config.h"
#endif

#include "common.h"

#include <math.h>

#include <gtk/gtk.h>

#include "cpk_gtk_util.h"
#include "cataxcorr.h"
#include "platemodel.h"
#include "cataxcorr_p.h"
#include "coordinates.h"
#include "starlist.h"
#include "model.h"
#include "catalib.h"
#include "cp_messages.h"

#include "cxc_data.h"

#include "cxc_gtk_main.h"  /* Include a sua volta cxc_map.h */
/* #include "cxc_match_set.h" */

/*----------------------------------------------------------------------------
 * 							Global variables	
 *--------------------------------------------------------------------------*/

extern boolean    suspend ;
extern GtkWidget *statusbar ;

/* Local data */

static GtkDataClass *parent_class = NULL;
static guint         data_signals[DATA_LAST_SIGNAL] = { 0 };
static const gchar  *value_index_key = "gtk-value-index";


/*----------------------------------------------------------------------------
 * 							Function prototypes
 *--------------------------------------------------------------------------*/

static void        cxc_data_class_init       (CxcDataClass   *klass) ;
static void        cxc_data_init             (CxcData        *data) ;
static StarList  **cxc_data_get_star_lists   (FileSet        *f_set) ;
static void        cxc_data_flag_star        (CxcData        *data) ;
static void        cxc_data_mask_star        (CxcData        *data,
                                              int             n) ;
static gint        cxc_data_timeout          (gpointer        data) ;

static void        get_map_center            (CxcMap         *map,
                                              StarList       *list,
                                              PlateMod       *model,
                                              gdouble         center[]) ;
static void        set_map_center            (CxcMap         *map,
                                              StarList       *list,
                                              PlateMod       *model,
                                              gdouble         center[]) ;
static void        cxc_data_recenter         (CxcData        *data, 
                                              PlateMod       *model_new,
                                              gint            frm2) ;

static void        cxc_data_stack_add        (CxcData        *data,
                                              gint            n) ;

extern void        status_msg                (GtkWidget      *status_bar,
                                              char           *msg) ;

static void        cxc_data_lock             (CxcData        *data,
                                              char           *msg) ;
static void        cxc_data_unlock           (CxcData        *data,
                                              CxcMap         *map) ;
static void        cxc_data_map_fnorm        (CxcData        *data) ;

static void        cxc_data_map_color_block  (CxcData        *data) ;

       gint        cxc_data_add              (CxcData        *data, 
                                              Finfo          *info) ;


/*** FIXME: obsoleta ***/
static void        cxc_data_prj2rot          (GtkWidget      *widget) ;

/*----------------------------------------------------------------------------
 * 							Defines
 *--------------------------------------------------------------------------*/

#define  NORMALIZE  1
#define  STACK_HAS_ONE_FILE  (bin_get_count (data->stack->nf) + bin_get_count (data->stack->nf2) == 1)

/*----------------------------------------------------------------------------
 *							Function codes
 *--------------------------------------------------------------------------*/

GtkType
cxc_data_get_type (void)
{
  static guint data_type = 0;

  if (!data_type)
    {
      static const GtkTypeInfo data_info =
      {
	"CxcData",
	sizeof (CxcData),
	sizeof (CxcDataClass),
        (GtkClassInitFunc) cxc_data_class_init,
        (GtkObjectInitFunc) cxc_data_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
        (GtkClassInitFunc) NULL,
      };

      data_type = gtk_type_unique (GTK_TYPE_DATA, &data_info);
    }

  return data_type;
}

static void
cxc_data_class_init (CxcDataClass *class)
{
  GtkObjectClass    *object_class;
  GtkWidgetClass    *widget_class;
  GtkContainerClass *container_class;

  object_class = (GtkObjectClass*) class;
  widget_class = (GtkWidgetClass*) class;
  container_class = (GtkContainerClass*) class;

  parent_class = gtk_type_class (GTK_TYPE_DATA);

  data_signals[FILE_STOP] =
        gtk_signal_new ("file-stop",
        GTK_RUN_FIRST,
        object_class->type,
        GTK_SIGNAL_OFFSET (CxcDataClass, file_stop),
        gtk_marshal_NONE__NONE,
	GTK_TYPE_NONE, 0);
  data_signals[FILE_NEXT] =
        gtk_signal_new ("file-next",
        GTK_RUN_FIRST,
        object_class->type,
        GTK_SIGNAL_OFFSET (CxcDataClass, file_next),
        gtk_marshal_NONE__NONE,
	GTK_TYPE_NONE, 0);
  data_signals[STAR_ALIGNED] =
        gtk_signal_new ("star-aligned",
        GTK_RUN_FIRST,
        object_class->type,
        GTK_SIGNAL_OFFSET (CxcDataClass, star_align),
        gtk_marshal_NONE__NONE,
	GTK_TYPE_NONE, 0);
  data_signals[BASE_FOUND] =
        gtk_signal_new ("base-found",
        GTK_RUN_FIRST,
        object_class->type,
        GTK_SIGNAL_OFFSET (CxcDataClass, base_found),
        gtk_marshal_NONE__NONE,
	GTK_TYPE_NONE, 0);
  data_signals[MODEL_FIT] =
        gtk_signal_new ("model-fit",
        GTK_RUN_FIRST,
        object_class->type,
        GTK_SIGNAL_OFFSET (CxcDataClass, model_fit),
        gtk_marshal_NONE__NONE,
	GTK_TYPE_NONE, 0);
  data_signals[STACK_ADDED] =
        gtk_signal_new ("stack-added",
        GTK_RUN_FIRST,
        object_class->type,
        GTK_SIGNAL_OFFSET (CxcDataClass, stack_added),
        gtk_marshal_NONE__NONE,
	GTK_TYPE_NONE, 0);
  data_signals[DATA_CHANGED] =
        gtk_signal_new ("data-changed",
        GTK_RUN_FIRST,
        object_class->type,
        GTK_SIGNAL_OFFSET (CxcDataClass, data_changed),
        gtk_marshal_NONE__NONE,
	GTK_TYPE_NONE, 0);

  gtk_object_class_add_signals (object_class, data_signals, DATA_LAST_SIGNAL);

  /* widget_class->event = cxc_data_events ; */

  /* object_class->finalize = cxc_data_finalize;

  widget_class->realize = cxc_data_realize;
  widget_class->unrealize = cxc_data_unrealize; */

}

static void
cxc_data_init (CxcData *data)
{
   data->f_set = NULL ;
   data->gparam = NULL ;

   data->info = NULL ;
   data->list = NULL ;
   data->model = NULL ;
   data->active = NULL ;
   data->n = 0 ;

   data->list1 = NULL ;
   data->list2 = NULL ;
   data->pmodel = NULL ;
   data->nref = 0 ;
   data->frm2 = 0 ;
   data->stack = NULL ;
   data->locked = FALSE ;
   data->activity = g_malloc (sizeof(ProgressData)) ;
   /* Timer for activity display */
   data->activity->timer = 0 ;
   return ;
}

GtkData*
cxc_data_new (FileSet *f_set, def_pars *param, g_pars *gparam)
{
   CxcData *data ;
   gint     i;
   gint     ns = 0, nl = 0 ;

   CPKDEBUG ("cxc_data_new") ;

   data = gtk_type_new (cxc_data_get_type ());

   /* file_info_remove_by_name (f_set, "cippalippa.caz") ;
   file_info_remove_by_name (f_set, "b2.tab") ;
   file_info_add (f_set, "b2.tab") ; */

   data->f_set = f_set ;
   data->gparam = gparam ;

   /* Read given files */
   for (i = 0; i < f_set->n; i++)
      if (cxc_data_add (data, f_set->info[i]))
         exit (1) ;

   data->list1 = data->list[data->nref] ;
   cxc_data_file_select (data, 1) ;

   if (gparam->find == NONE) {
      for (i = 0; i < data->n; i++) 
         if (i != data->nref) {
            star_list_align (data->list[data->nref], data->list[i], data->model[i]) ;
            data->model[i]->status = LOW_ALIGN ;
            data->model[i]->saved = FALSE ;
            data->list[i]->saved = FALSE ;
            data->list[i]->out_sph = data->list[data->nref]->out_sph ;
         }
   }

   /* Get mean map star radii in pixel and deg units */
   data->srad_pix = data->srad_deg = 0.0 ;
   for (i = 0; i < data->n; i++) {
      if (data->list[i]->spheric) {
         data->srad_deg += (data->list[i]->inv_density / 2.0) ;
         ns++ ;
      }
      else {
         data->srad_pix += (data->list[i]->inv_density / 2.0) ;
         nl++ ;
      }
   }
   if (ns)
      data->srad_deg /= (gdouble)ns ;
   if (nl)
      data->srad_pix /= (gdouble)nl ;

   /* Initialize Data Stack */
   data->stack = stack_init (data->list[0], 0, data->n) ;
   data->stack->n = 1 ;

   /* Connect signals and events */
   /* gtk_signal_connect_object (GTK_OBJECT (data), "star-aligned",
      (GtkSignalFunc)cxc_data_prj2rot, GTK_OBJECT (data)) ; */

   CPKDEBUG5 ("cxc_data_new_end", 5) ;

   return GTK_DATA (data) ;
}

gint
cxc_data_add (CxcData *data, Finfo *info)
{
   size_t n = (size_t)data->n + 1 ;

   /* Read file info */
   data->info = (Finfo**) realloc (data->info, n*sizeof(Finfo*)) ;
   data->info[n-1] = info ;

   /* Read File */
   data->list = (StarList**) realloc (data->list, n*sizeof(StarList*)) ;
   data->list[n-1] = star_list_read (data->f_set->info[n-1], NULL) ;
   if (data->list[n-1] == NULL)
      return 1 ;

   /* Init Plate Model */
   data->model = (PlateMod**) realloc (data->model, n*sizeof(PlateMod*)) ;
   data->model[n-1] = model_init (data->list[data->nref], data->list[n-1], NULL) ;
   if (data->model[n-1] == NULL)
      return 1 ;

   /* Computes projected coordinates and mean star density */
   star_list_pix2prj (data->list[n-1], data->model[n-1]->crpix) ;
   data->list[n-1]->inv_density = sqrt (star_list_density (data->list[n-1])) ;

   data->active = (gboolean*) realloc (data->active, n*sizeof(gboolean)) ;

   data->n++ ;
   return 0 ;
}

gint
cxc_data_remove_by_number (CxcData *data, int n)
{
   if (n >= data->n || n < 0) {
      cp_warning ("cxc_data_remove_by_number: Invalid file '%d'", n) ;
      return 1 ;
   }

   file_info_free (data->info[n]) ;
   star_list_destroy (data->list[n]) ;
   /******* model[n]->match ??? FIXME !!!!*****/
   free (data->model[n]) ;

   data->n-- ;
   for ( ; n < data->n; n++) {
      data->info[n] = data->info[n+1] ;
      data->list[n] = data->list[n+1] ;
      data->model[n] = data->model[n+1] ;
      data->active[n] = data->active[n+1] ;
   }
   data->info =  (Finfo**)    realloc (data->info,   n * sizeof(Finfo*)) ;
   data->list  = (StarList**) realloc (data->list,   n * sizeof(StarList*)) ;
   data->model = (PlateMod**) realloc (data->model,  n * sizeof(PlateMod*)) ;
   data->active = (gboolean*) realloc (data->active, n * sizeof(gboolean)) ;

   return 0 ;
}

static StarList **
cxc_data_get_star_lists (FileSet *f_set)
{
   StarList **list = (StarList **)NULL ;
   size_t     nl = (size_t)f_set->n ;
   gint       i, status = 1 ;

   list = (StarList **) malloc (nl*sizeof(StarList *)) ;
   if (list == (StarList **)NULL)
      goto exit ;
   for (i = 0; i < (gint)nl; i++) {
      list[i] = star_list_read (f_set->info[i], NULL) ; /*** FIXME ***/
      if (list[i] == (StarList *)NULL)
         goto exit ;

      list[i]->out_sph = list[i]->spheric ;    /***FIXMEEEE****/
   }
   status = 0 ;

   exit:
   if (status) {
      for (i = 0; i < (gint)nl; i++) 
         if (list[i] != (StarList *)NULL)
            free (list[i]) ;
      if (list != (StarList **)NULL)
         free (list) ;
   }
   return list ;
}

static void
cxc_data_flag_star (CxcData *data)
{
   guint       i ;

   for (i = 0; i < (guint)data->n; i++) {
      star_flag_crowded (data->list[i], 2.0 * data->list[i]->param->crow_rad) ;
      star_flag_mask (data->list[i], data->list[i]->param->mask_shape, 
            data->list[i]->param->mask_p) ;
   }
}

static void
cxc_data_mask_star (CxcData *data, int n)
{
   star_flag_crowded (data->list[n], /* 2.0 *  */data->list[n]->param->crow_rad) ;
   star_flag_mask (data->list[n], data->list[n]->param->mask_shape, 
         data->list[n]->param->mask_p) ;
}

gint
cxc_data_file_select (CxcData *data, guint n)
{
   if (data->model[n]->status == BUSY)
      return 1 ;
   data->frm2 = n ;
   data->list2 = data->list[data->frm2] ;
   data->pmodel = data->model[data->frm2] ;
   return 0 ;
}

void
cxc_data_model_align (GtkWidget *widget, CxcData *data)
{
   GtkWidget **maps = (GtkWidget **)data->p ;
   CxcMap     *map  = CXC_MAP (maps[data->frm2]) ;
   StarList   *list2 = data->list2 ;
   PlateMod   *model = data->pmodel ;
   gint        status = 1, frm1 ;

   CPKDEBUG ("cxc_data_model_align") ;

   cxc_data_lock (data, " Looking for frames aligment") ;
   CXC_MESSAGE (1,2,"\n") ;
   CXC_MESSAGE (1,2,"*** Frames Aligment ***\n\n") ;
   CXC_MESSAGE (0,2,"Putting '%s' on reference '%s'... ",
         data->f_set->info[data->frm2]->filename, 
         ((data->list1 == data->stack->ref) ?
         "STACK" :
         data->f_set->info[data->frm1]->filename)) ;

   frm1 = data->frm1 ;

   /***FIXME: passare direttamente la lista da flaggare al posto di frm1:
              per permettere di allineare sullo stack dei frames anziche`
              sul solo riferimento ****/
   cxc_data_mask_star (data, frm1) ;
   cxc_data_mask_star (data, data->frm2) ;

   printf("Passing list1->nstar %d\n",data->list1->nstar);
   printf("Passing list2->nstar %d\n",data->list2->nstar);
   status = star_list_align (data->list1, list2, model) ;

   /* update the GUI */
   while (gtk_events_pending ())
      gtk_main_iteration ();

   if (status == 0) {
      model->status = LOW_ALIGN ;
      model->saved = FALSE ;
      list2->saved = FALSE ;
      list2->out_sph = data->list1->out_sph ;
      /* emit 'star_align' signal */
      align_maps (CXC_MAP (maps[frm1]), map) ;
      gtk_signal_emit_by_name (GTK_OBJECT (map), "map-range") ;
      gtk_signal_emit_by_name (GTK_OBJECT (map), "map-changed") ;
   }
   else
      model->status = INIT ;

   cxc_data_unlock (data, map) ;

   CPKDEBUG5 ("cxc_data_model_align_end", 5) ;
}

void
cxc_data_register (GtkWidget *widget, CxcData *data)
{
   GtkWidget **maps = (GtkWidget **)data->p ;
   CxcMap     *map  = CXC_MAP (maps[data->frm2]) ;
   StarList   *list2 = data->list2 ;
   PlateMod   *model = data->pmodel ;
   gint        status ;

   CPKDEBUG ("cxc_data_register") ;

   CXC_MESSAGE (1,2,"\n") ;
   CXC_MESSAGE (1,2,"*** Frames Registering ***\n\n") ;
   CXC_MESSAGE (0,2,"Putting '%s' on reference '%s'\n",
         data->f_set->info[data->frm2]->filename, 
         ((data->list1 == data->stack->ref) ?
         "STACK" :
         data->f_set->info[data->frm1]->filename)) ;

   cxc_data_lock (data, " Registering frames") ;

   cxc_data_stack_remove (data, data->frm2) ;

   printf("Passing list1->nstar %d\n",data->list1->nstar);
   printf("Passing list2->nstar %d\n",data->list2->nstar);
   status = star_list_register (data->list1, list2, model) ;

   /* update the GUI */
   while (gtk_events_pending ())
      gtk_main_iteration ();

   if (status == 0) {
      model->status = HIGH_ALIGN ;
      model->saved = FALSE ;
      list2->saved = FALSE ;
      align_maps (CXC_MAP (maps[data->nref]), map) ;
      gtk_signal_emit_by_name (GTK_OBJECT (map), "map-magnitude") ;
      gtk_signal_emit_by_name (GTK_OBJECT (map), "map-changed") ;
   }
   else
      model->status = LOW_ALIGN ;
   
   cxc_data_unlock (data, map) ;

   CPKDEBUG ("cxc_data_register_end") ;
}

void
cxc_data_model_fit (GtkWidget *widget, CxcData *data)
{
   GtkWidget **maps = (GtkWidget **)data->p ;
   CxcMap     *map  = CXC_MAP (maps[data->frm2]) ;
   StarList   *list1 = data->list1 ;
   StarList   *list2 = data->list2 ;
   PlateMod   *model = data->pmodel ;
   gint        i, status, frm2 = data->frm2 ;
   gdouble     center[2] ;

   CPKDEBUG ("cxc_data_model_fit") ;

   CXC_MESSAGE (1,2,"\n") ;
   CXC_MESSAGE (1,2,"*** Fitting Plate Model ***\n\n") ;
   CXC_MESSAGE (0,2,"Putting '%s' on reference '%s'\n",
         data->f_set->info[data->frm2]->filename, 
         ((data->list1 == data->stack->ref) ?
         "STACK" :
         data->f_set->info[data->frm1]->filename)) ;

   cxc_data_lock (data, " Fitting plate model") ;

   cxc_data_stack_remove (data, data->frm2) ;

   /* Get world coordinates of display center */
   get_map_center (map, list2, model, center) ;

   /* Fit model */
   printf("Passing list1->nstar %d\n",data->list1->nstar);
   printf("Passing list2->nstar %d\n",data->list2->nstar);
   status = model_fit (list1, list2, model) ;

   if (status == 0) {

      /* Recover world coordinates of display center */
      set_map_center (map, list2, model, center) ;

      /* update the GUI */
      while (gtk_events_pending ())
         gtk_main_iteration ();

      model->status = MODEL_FITTED ;
      model->saved = FALSE ;
      list2->saved = FALSE ;

      /* emit signals */
      gtk_signal_emit_by_name (GTK_OBJECT (map), "map-range") ;
      gtk_signal_emit (GTK_OBJECT (data), data_signals[MODEL_FIT]) ;
      gtk_signal_emit_by_name (GTK_OBJECT (map), "map-changed") ;

      if (STACK_HAS_ONE_FILE) {

         /* Adjust reference center */
         if (memcmp (data->list[0]->origin, list1->origin, 
               (size_t)2*sizeof(double))) {
            star_list_pix2prj (data->list[0], list1->origin) ;
            memcpy (data->list[0]->origin, list1->origin, 
               (size_t)2*sizeof(double)) ;
            gtk_signal_emit_by_name (GTK_OBJECT (CXC_MAP (maps[0])), "map-range") ;
            align_maps (map, CXC_MAP (maps[data->nref])) ;
         }

         /* Recenter other lists */
         cxc_data_recenter (data, model, frm2) ;
      }
      else
         align_maps (CXC_MAP (maps[data->nref]), map) ;

      /* Add list to stack frame */
      cxc_data_stack_add (data, frm2) ;
   }
   else
      model->status = HIGH_ALIGN ;

   cxc_data_unlock (data, map) ;

   CPKDEBUG ("cxc_data_model_fit: end") ;
}

static void
get_map_center (CxcMap *map, StarList *list, PlateMod *model, gdouble center[])
{
   Star    s = {NULL, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, NULL} ;
   Star   *sp = &s ;

   s.xrot = map->panx ;
   s.yrot = map->pany ;
   coord_rot2prj (s.xrot, s.yrot, model, &s.xprj, &s.yprj) ;
   coord_prj2pix (&sp, 1, list->spheric, map->origin) ;
   center[0] = s.xpix ;
   center[1] = s.ypix ;
}

static void
set_map_center (CxcMap *map, StarList *list, PlateMod *model, gdouble center[])
{
   Star    s = {NULL, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, NULL} ;
   Star   *sp = &s ;

   s.xpix = center[0] ;
   s.ypix = center[1] ;
   coord_pix2prj (&sp, 1, list->spheric, model->crpix) ;
   coord_prj2rot (&sp, 1, model) ;
   map->panx = s.xrot ;
   map->pany = s.yrot ;
}

static void
cxc_data_recenter (CxcData *data, PlateMod *model_new, gint frm2)
{
   GtkWidget **maps = (GtkWidget **)data->p ;
   CxcMap     *map ;
   PlateMod   *model ;
   gint        i ;

   for (i = 0; i < data->n; i++) {
      model = data->model[i] ;
      if (memcmp (model->crval, model_new->crval, (size_t)2*sizeof(double))) {

         memcpy (model->crval, model_new->crval, (size_t)2*sizeof(double)) ;
         map =  CXC_MAP (maps[i]) ;
         if (model->match) {
            model_recenter_crpix (data->list[i], model) ;
            gtk_signal_emit_by_name (GTK_OBJECT (map), "map-range") ;
            align_maps (CXC_MAP (maps[frm2]), map) ;
            gtk_signal_emit_by_name (GTK_OBJECT (map), "map-changed") ;
         }
         else 
            model->status = INIT ;
      }
   }
}

gint
cxc_data_stack_init (GtkWidget *widget, CxcData *data)
{
   StarMatch  *match = NULL ;
   gint        i, nmatch, status = 1 ;

   CPKDEBUG ("cxc_data_stack_init") ;

   cxc_data_lock (data, " Generating files Stack") ;

   cxc_data_map_color_block (data) ;
   if (data->stack)
      stack_destroy (data->stack) ;

   i = 0 ;
   while (i < data->n && !data->active[i]) i++ ;
   data->stack = stack_init (data->list[i], i, data->n) ;
   IF_ABORT_EXIT ;

   while (++i < data->n) {
      if (data->active[i]) {
         match = star_list_match (data->stack->ref, data->list[i], 
               data->model[i], &nmatch) ;
         IF_ABORT_EXIT ;
         status = stack_add (&data->stack, data->list[i], i, match, nmatch) ;
         IF_ABORT_EXIT ;
         free (match) ;
         match = NULL ;
      }
   }
   cxc_data_map_fnorm (data) ;

   gtk_signal_emit_by_name (GTK_OBJECT (data), "stack-added") ;
   status = 0 ;

   exit :

   if (match) free (match) ;
   cxc_data_unlock (data, NULL) ;
   CPKDEBUG5 ("cxc_data_stack_init: done", 6) ;

   return status ;
}

static void
cxc_data_map_fnorm (CxcData *data)
{
   GtkWidget **maps = (GtkWidget **)data->p ;
   CxcMap     *map = CXC_MAP (maps[data->n]) ;
   gint        i ;

   map->list  = data->stack->ref ;
   map->fnorm = (NORMALIZE) ? data->stack->n : bin_get_count (data->stack->nf) +
                                               bin_get_count (data->stack->nf2) ;

   CXC_MAP_SET_RESCALING (map) ;
   for (i = 0; i < data->n; i++) {
      CXC_MAP (maps[i])->fnorm = map->fnorm ;
      if (bin_check_n (data->stack->nf, i) || 
          bin_check_n (data->stack->nf2, i-32)) {
         CXC_MAP (maps[i])->color = TRUE ;
         cxc_map_remap (NULL, CXC_MAP (maps[i])) ;
      }
      else {
         if (CXC_MAP (maps[i])->color) {
            CXC_MAP (maps[i])->color = FALSE ;
            cxc_map_remap (NULL, CXC_MAP (maps[i])) ;
         }
      }
   }
}

static void
cxc_data_map_color_block (CxcData *data)
{
   GtkWidget **maps = (GtkWidget **)data->p ;
   guint       i ;

   for (i = 0; i < data->n; i++)
      CXC_MAP (maps[i])->color = FALSE ;
}

static void
cxc_data_stack_add (CxcData *data, gint n)
{
   StarMatch *match ;
   gint       nmatch, status ;

   CPKDEBUG ("cxc_data_stack_add") ;

   if (bin_check_n (data->stack->nf, n) || 
       bin_check_n (data->stack->nf2, n-32))
      stack_remove (&data->stack, n) ;

   match = star_list_match (data->stack->ref, data->list[n], data->model[n], 
         &nmatch) ;
   status = stack_add (&data->stack, data->list[n], n, match, nmatch) ;
   free (match) ;

   cxc_data_map_fnorm (data) ;

   gtk_signal_emit_by_name (GTK_OBJECT (data), "stack-added") ;

   return ;
}

void
cxc_data_stack_remove (CxcData *data, gint n)
{
   if (!bin_check_n (data->stack->nf, n) && 
       !bin_check_n (data->stack->nf2, n-32))
      return ;
   stack_remove (&data->stack, n) ;
   cxc_data_map_fnorm (data) ;
}

/* Save Plate Models */
void
cxc_data_coinc_save (GtkWidget *widget, CxcData *data)
{ 
   CPKDEBUG ("cxc_data_coinc_save") ;
   coinc_tab_save (data->stack, data->f_set->info, data->gparam->outfile) ;
}

void
cxc_data_rot_save (GtkWidget *widget, CxcData *data)
{
   gchar     filename[1035] = "\0" ;
   gint      i ;
   StarList *out ;
   CPKDEBUG ("cxc_data_rot_save") ;

   for (i = 0; i < data->n; i++) {
      if (i != data->nref && data->model[i]->status >= LOW_ALIGN) {
         out = star_list_transform (data->list[i], data->model[i]) ;
         sprintf (filename, "%s%s", data->f_set->info[i]->filename,
               data->gparam->ext_rot) ;
         star_list_save (filename, data->f_set->info[i], out) ;
         data->list[i]->saved = out->saved ;
         star_list_destroy (out) ;
      }
   }
}

void
cxc_data_model_save (GtkWidget *widget, CxcData *data)
{ 
   gchar  filename[1035] = "\0" ;
   gint   i ;

   CPKDEBUG ("cxc_data_model_save") ;

   for (i = 0; i < data->n; i++) {
      if (i != data->nref && data->model[i]->status >= MODEL_FITTED) {
         sprintf (filename, "%s%s", data->f_set->info[i]->filename, 
               data->gparam->ext_plm) ;
         model_save (filename, data->f_set->info[data->nref]->filename, 
              data->model[i], data->f_set->info[i]) ;
      }
   }

}

static gint 
cxc_data_timeout (gpointer data)
{
   ProgressData *activity = (ProgressData *)data ;

   gtk_widget_hide (activity->icon) ;
   if (activity->icon == activity->on)
      activity->icon = activity->off ;
   else
      activity->icon = activity->on ;
   if (!activity->icon->parent) 
      gtk_box_pack_start (GTK_BOX (activity->box),
            activity->icon, FALSE, FALSE, 0) ;
   gtk_widget_show (activity->icon) ;

    /* As this is a timeout function, return TRUE so that it
     * continues to get called */
    return (TRUE);
} 

void 
cxc_data_timeout_remove (gpointer data)
{
   ProgressData *activity = (ProgressData *)data ;

   gtk_timeout_remove (activity->timer) ;
   activity->timer = 0 ;
   gtk_widget_hide (activity->icon) ;
   activity->icon = activity->off ;
   gtk_widget_show (activity->icon) ;
} 

void
status_msg (GtkWidget *status_bar, char *msg)
{
   guint context_id ;
   context_id = gtk_statusbar_get_context_id (
         GTK_STATUSBAR (status_bar), "Statusbar msg") ;
   gtk_statusbar_pop (GTK_STATUSBAR (status_bar), context_id);
   gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, msg);
}

static void
cxc_data_lock (CxcData *data, char *msg)
{
   gtk_widget_set_sensitive (data->stop, TRUE) ;
   suspend = FALSE ;
   data->locked = TRUE ;
   status_msg (statusbar, msg) ;

    /* Add the timer */
   data->activity->timer = gtk_timeout_add (250, cxc_data_timeout, 
         data->activity) ;
}

static void
cxc_data_unlock (CxcData *data, CxcMap *map)
{
   /* remove the timer */
   cxc_data_timeout_remove (data->activity) ;
   status_msg (statusbar, " ") ;
   data->locked = FALSE ;
   gtk_widget_set_sensitive (data->stop, FALSE) ;
   suspend = FALSE ;
   if (map)
      gtk_signal_emit_by_name (GTK_OBJECT (map), "map-unlock") ;
}












/* Change current file number */
/*** FIXME: inutile ed obsoleta ***/
void
cxc_data_file_next (GtkWidget *widget, CxcData *data)
{
   if (data->frm2 == data->n-1) {
      /* emit 'file-stop' signal */
      gtk_signal_emit_by_name (GTK_OBJECT (data), "file-stop") ;
      return ;
   }
   data->frm2++ ;
   if ( (data->frm2 == data->nref) && (data->frm2 == data->n-1) ) {
      /* emit 'file-stop' signal */
      gtk_signal_emit_by_name (GTK_OBJECT (data), "file-stop") ;
      return ;
   }
   else
      data->frm2++ ;
   data->list2 = data->list[data->frm2] ;
   data->pmodel = data->model[data->frm2] ;
   /* emit 'file-next' signal */
   gtk_signal_emit_by_name (GTK_OBJECT (data), "file-next") ;
}

/*** FIXME: inutile ed obsoleta ***/
static void
cxc_data_prj2rot (GtkWidget *widget)
{
   CxcData *data = CXC_DATA (widget) ;

   CPKDEBUG ("cxc_data_prj2rot") ;

   data->locked = TRUE ;

   /* Compute rotated coordinates */
   coord_prj2rot (data->list2->data, (int)data->list2->nstar, data->pmodel) ;

   data->locked = FALSE ;

   /* emit 'data changed' signal */
   gtk_signal_emit (GTK_OBJECT (data), data_signals[DATA_CHANGED]) ;
}

