/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 							Includes
 *--------------------------------------------------------------------------*/

#if HAVE_CONFIG_H
#  include "config.h"
#endif

#include "common.h"

#include <math.h>

#include <gtk/gtk.h>

#include "cataxcorr.h"

#include "cxc_gtk_main.h"

/*----------------------------------------------------------------------------
 * 							Global variables	
 *--------------------------------------------------------------------------*/

extern FileSet  *f_set ;

/*----------------------------------------------------------------------------
 * 							Private functions
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 							Function prototypes
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 							Defines
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 *							Function codes
 *--------------------------------------------------------------------------*/

static gint
delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
   printf("delete\n");
   return(FALSE) ;
}

static void
destroy(GtkWidget *widget, gpointer data)
{
   printf("destroy\n");
   gtk_main_quit() ;
}

static void
close_window(GtkWidget *window, gpointer data)
{
   cxc_display *display = data ;
   display->data_content_dialog = NULL ;
}

static void
data_content_accept(GtkWidget *widget, gpointer data)
{
   cxc_display *display = data ;
   ;
}

void
data_content( GtkWidget *widget, cxc_display *display )
{
   GtkWidget *dialog, *vbox, *button, *scrolled_win, *list, *list_item ;
   gint       i, j ;
   gchar      buffer[1024] ;
   gchar     *titles[2] = { "File", "Content" };
   gchar     *data[2] ;

   if ( display->data_content_dialog ) {
      gtk_window_set_transient_for (GTK_WINDOW (display->data_content_dialog), 
            GTK_WINDOW (display->window)) ;
      return ;
   }

   /* Create a new window */
   display->data_content_dialog = dialog = gtk_dialog_new( ) ;

   gtk_window_set_title(GTK_WINDOW(dialog), "CataXcorr: File Info") ;
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE) ;
   gtk_widget_set_usize(GTK_WIDGET (dialog), 450, 300) ;
 
   gtk_signal_connect(GTK_OBJECT(dialog), "destroy", 
                      GTK_SIGNAL_FUNC(close_window), (gpointer)display) ;

   /* Main container */
   vbox = gtk_vbox_new(FALSE, 0) ;
   gtk_container_set_border_width (GTK_CONTAINER(vbox), 8) ;
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), vbox,
                      TRUE, TRUE, 0) ;


   /* Scrolled window with files content */
   scrolled_win = gtk_scrolled_window_new(NULL, NULL) ;
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (scrolled_win),
         GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC) ;
   gtk_box_pack_start(GTK_BOX(vbox), scrolled_win, TRUE, TRUE, 0) ;

   list = gtk_clist_new_with_titles( 2, titles);
   gtk_clist_set_column_width (GTK_CLIST(list), 0, 100);
   gtk_clist_set_column_width (GTK_CLIST(list), 1, 500);
   gtk_container_add(GTK_CONTAINER(scrolled_win), list);
   /* gtk_scrolled_window_add_with_viewport (
         GTK_SCROLLED_WINDOW (scrolled_win), list) ; */

   for(i=0; i<f_set->n; i++) {
       strcpy(buffer, "") ;
      for(j=0; j<f_set->info[i]->n_cols; j++) 
         sprintf(buffer, "%s %-12s", buffer, f_set->info[i]->labels[j]) ;
/*      list_item = gtk_list_item_new_with_label (buffer) ;
      gtk_container_add (GTK_CONTAINER (list), list_item) ; */
      data[0] = f_set->info[i]->filename ;
      data[1] = buffer ;
      gtk_clist_append( (GtkCList *) list, data);
   }
   gtk_widget_show_all(list) ;

   gtk_widget_show_all(vbox) ;

  /* buttons */
   button = gtk_button_new_with_label ("Accept") ;
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
                       (GtkSignalFunc) data_content_accept, 
                       (gpointer) display) ;
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
                      button, TRUE, TRUE, 0) ;
   GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
   gtk_widget_show(button) ;

   button = gtk_button_new_with_label ("Cancel") ;
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                              (GtkSignalFunc) gtk_widget_destroy, 
                              (gpointer) dialog) ;
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
                      button, TRUE, TRUE, 0) ;
   GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT) ;
   gtk_widget_grab_default(button) ;
   gtk_widget_show (button) ;

   gtk_widget_show_all (dialog) ;


}
