/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 							Includes
 *--------------------------------------------------------------------------*/

#if HAVE_CONFIG_H
#  include "config.h"
#endif

#include "common.h"

#include <math.h>

#include <gtk/gtk.h>

#include "cxc_data.h"
#include "cpk_gtk_util.h"
#include "cataxcorr.h"
#include "coordinates.h"
#include "starlist.h"
#include "cxc_gtk_main.h"
#include "cxc_file_set.h"

/*----------------------------------------------------------------------------
 * 							Global variables	
 *--------------------------------------------------------------------------*/

extern boolean   suspend ; /***FIXME: serve?*/

/*----------------------------------------------------------------------------
 * 							Function prototypes
 *--------------------------------------------------------------------------*/

static void      star_list_remap    (CxcData *data, gint n) ;
static gint      entry_press        (GtkWidget *entry, GdkEvent *event, 
                                     Sfile *FilesSet) ;
static void      menuitem_response  (GtkWidget *menuitem, gchar *string ) ;

/*----------------------------------------------------------------------------
 *							Function codes
 *--------------------------------------------------------------------------*/

void
get_file_val(Sfile *FilesSet)
{
   FileSet  *f_set = FilesSet->f_set ;
   gint      i ;

   for (i=0; i<f_set->n; i++) {
      sprintf(FilesSet->set[i].value[0], "%s", 
            f_set->info[i]->labels[f_set->info[i]->x_pos]) ;
      sprintf(FilesSet->set[i].value[1], "%s", 
            f_set->info[i]->labels[f_set->info[i]->y_pos]) ;
      sprintf(FilesSet->set[i].value[2], "%s", 
            f_set->info[i]->labels[f_set->info[i]->m_pos]) ;
   }
}

void
set_file_val(Sfile *FilesSet, CxcData *data)
{
   FileSet  *f_set = FilesSet->f_set ;
   gint      i, j ;
   gchar    *text ;
   boolean   spheric, proj, remap ;

   for (i=0; i<f_set->n; i++) {
      proj = remap =FALSE ;
      text = gtk_entry_get_text(GTK_ENTRY (FilesSet->set[i].entries[0])) ;
      for (j=0; j<f_set->info[i]->n_cols; j++)
         if (strcmp(f_set->info[i]->labels[j], text) == 0) {
            if (f_set->info[i]->x_pos != j) remap = TRUE ;
            f_set->info[i]->x_pos = j ;
         }
      text = gtk_entry_get_text(GTK_ENTRY (FilesSet->set[i].entries[1])) ;
      for (j=0; j<f_set->info[i]->n_cols; j++)
         if (strcmp(f_set->info[i]->labels[j], text) == 0) {
            if (f_set->info[i]->y_pos != j) remap = TRUE  ;
            f_set->info[i]->y_pos = j ;
         }
      text = gtk_entry_get_text(GTK_ENTRY (FilesSet->set[i].entries[2])) ;
      for (j=0; j<f_set->info[i]->n_cols; j++)
         if (strcmp(f_set->info[i]->labels[j], text) == 0)  {
            if (f_set->info[i]->m_pos != j) remap = TRUE ;
            f_set->info[i]->m_pos = j ;
         }
      spheric = gtk_toggle_button_get_active (
            GTK_TOGGLE_BUTTON (FilesSet->set[i].spheric)) ;
      if (f_set->info[i]->spheric != spheric) 
         proj = TRUE  ;
      f_set->info[i]->spheric = data->list[i]->spheric = spheric ;

      if (remap) 
         star_list_remap(data, i) ;
      if (proj) 
         coord_pix2prj(data->list[i]->data, (int)data->list[i]->nstar,
               data->list[i]->spheric, data->model[i]->crpix) ;
   }
}

static void
star_list_remap(CxcData *data, gint n)
{
   star_list_destroy(data->list[n]) ;
   data->list[n] = star_list_read (data->f_set->info[n], NULL) ; /*** FIXME ***/
   coord_pix2prj(data->list[n]->data, (int)data->list[n]->nstar, 
         data->list[n]->spheric, data->model[n]->crpix) ;
   data->list1 = data->list[data->nref] ;
   data->list2 = data->list[data->frm2] ;
   return ;
}

static gint
entry_press(GtkWidget *entry, GdkEvent *event, Sfile *FilesSet)
{
   gint       i, j, ii, jj ;

   if (entry==NULL) 
      return FALSE ;
   if (!GTK_IS_ENTRY (entry)) 
      return FALSE ;

   ii = jj = -1 ;
   for (i=0; i<FilesSet->f_set->n; i++) 
      for (j=0; j<3; j++)
         if (FilesSet->set[i].entries[j] == entry) {
            jj = j ;
            ii = i ;
         }
   FilesSet->i = ii ;
   FilesSet->j = jj ;
   if (event->type == GDK_BUTTON_PRESS) {
      GdkEventButton *event_button = (GdkEventButton *) event ; 
      if (event_button->button > 1) {
         gtk_menu_popup (GTK_MENU (FilesSet->set[ii].menu), NULL, NULL, 
               NULL, NULL, event_button->button, event_button->time) ;
         return TRUE ;
      }
   }

   /* Tell calling code that we have not handled this event; pass it on. */
   return FALSE ;
}

static void 
menuitem_response(GtkWidget *menuitem, gchar *string )
{
   gint i, j ;
   gint n ;
   Sfile *FilesSet = (Sfile*)gtk_object_get_data(GTK_OBJECT(menuitem),"fset_par") ;

   n = FilesSet->i ;
   i = 0 ;
   for (j=0; j<FilesSet->f_set->info[n]->n_cols; j++) 
      if (strcmp(FilesSet->f_set->info[n]->labels[j], string) == 0)
            i = j ;
   gtk_entry_set_text(GTK_ENTRY (FilesSet->set[n].entries[FilesSet->j]), string) ;
}

GtkWidget *
file_settings(Sfile *FilesSet, cxc_display *display)
{
   FileSet   *f_set = FilesSet->f_set ;
   GtkWidget *box, *vbox, *check, *scrolled_window, *menu, *menuitem ;
   GtkWidget *frame, *label, *viewport, *bbox, *entry, *table ;
   guint      i, j ;
   gchar      lab[][10] = {"X:","Y:","Mag:","Spherical:"} ;

   /* Find Settings */

   box = gtk_vbox_new(FALSE, 0) ;

   set_font ("-*-times-medium-r-normal--18-140-*-*-*-*-*-*") ;
   frame = gtk_frame_new("Files settings") ;
   gtk_container_set_border_width (GTK_CONTAINER(frame), 8) ;
   gtk_container_add(GTK_CONTAINER(box), frame) ;
   vbox = gtk_vbox_new(FALSE, 0) ;
   gtk_container_set_border_width (GTK_CONTAINER(vbox), 5) ;
   gtk_container_add (GTK_CONTAINER (frame), vbox) ;
   set_font ("-*-times-medium-r-normal--17-140-*-*-*-*-*-*") ;
   
   scrolled_window = gtk_scrolled_window_new(NULL, NULL) ;
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (scrolled_window),
         GTK_POLICY_NEVER, GTK_POLICY_ALWAYS) ;
   viewport = gtk_viewport_new (
         gtk_scrolled_window_get_hadjustment (GTK_SCROLLED_WINDOW (scrolled_window)),
         gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (scrolled_window))) ;
   gtk_viewport_set_shadow_type (GTK_VIEWPORT (viewport), GTK_SHADOW_ETCHED_OUT) ;
   gtk_widget_set_usize(viewport, 0, 65) ;
   gtk_container_add (GTK_CONTAINER (scrolled_window), viewport) ;
   gtk_container_add(GTK_CONTAINER(vbox), scrolled_window) ;

   table = gtk_table_new ((guint)f_set->n, 5, FALSE);
   gtk_table_set_col_spacings (GTK_TABLE (table), 15);
   gtk_table_set_row_spacings (GTK_TABLE (table), 5);
   gtk_scrolled_window_add_with_viewport (
         GTK_SCROLLED_WINDOW (scrolled_window), table) ;

   for (i=0; i<(guint)f_set->n; i++) {
      bbox = gtk_hbox_new(FALSE, 2) ;
      label = gtk_label_new("File:") ;
      gtk_box_pack_start(GTK_BOX(bbox), label, TRUE, TRUE, 0) ;
      entry = gtk_entry_new() ;
      gtk_entry_set_text(GTK_ENTRY (entry), f_set->info[i]->filename) ;
      gtk_entry_set_editable(GTK_ENTRY (entry), FALSE);
      gtk_widget_set_usize(entry, 80, 0) ;
      gtk_box_pack_start(GTK_BOX(bbox), entry, FALSE, FALSE, 0) ;
      gtk_table_attach (GTK_TABLE (table), bbox, 0, 1, i, i+1,
            GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);

      /* Create menu for labels of current file */
      FilesSet->set[i].menu = menu = gtk_menu_new() ;
      for (j=0; j<(guint)f_set->info[i]->n_cols; j++) {
         menuitem = gtk_menu_item_new_with_label(f_set->info[i]->labels[j]) ;
         gtk_menu_append ( GTK_MENU (menu), menuitem ) ;
         gtk_object_set_data(GTK_OBJECT (menuitem), "fset_par", 
               (gpointer)FilesSet) ;
         gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
               GTK_SIGNAL_FUNC (menuitem_response), 
               (gpointer) f_set->info[i]->labels[j]);
         gtk_widget_show (menuitem);
      }

      for (j=0; j<3; j++) {
         bbox = gtk_hbox_new(FALSE, 2) ;
         label = gtk_label_new(lab[j]) ;
         gtk_box_pack_start(GTK_BOX(bbox), label, TRUE, TRUE, 0) ;
         FilesSet->set[i].entries[j] = entry = gtk_entry_new() ;;
         gtk_entry_set_max_length(GTK_ENTRY (entry), 254 ) ;
         gtk_entry_set_text(GTK_ENTRY (entry), FilesSet->set[i].value[j]) ;
         gtk_signal_connect(GTK_OBJECT(entry), "button-press-event",
               GTK_SIGNAL_FUNC(entry_press), (gpointer) FilesSet) ;
         gtk_widget_set_usize(entry, 60, 0) ;
         gtk_box_pack_start(GTK_BOX(bbox), entry, FALSE, FALSE, 0) ;
         gtk_table_attach (GTK_TABLE (table), bbox, j+1, j+2, i, i+1,
               GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
      }

      FilesSet->set[i].spheric = check = 
            gtk_check_button_new_with_label ("Spherical") ;
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check),
            (gboolean)f_set->info[i]->spheric) ;
      gtk_table_attach (GTK_TABLE (table), check, 4, 5, i, i+1,
            GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);

   }
   gtk_widget_show_all (box) ;

   return box ;
}

/*----------------------------------------------------------------------------
 * 							Private Types
 *--------------------------------------------------------------------------*/
#ifdef cippa
typedef struct tmp_tag {
   GtkWidget   *main_window ;
   GtkWidget   *dialog ;
   Sfind       *FindSet ;
   cxc_display *display ;
} Fparam ;

static void
match_frames_accept(GtkWidget *widget, Fparam *FindParam)
{
   CxcData   *data = CXC_DATA (FindParam->display->data) ;

   set_find_val(FindParam->FindSet, data) ;
   gtk_widget_destroy(FindParam->dialog) ;
   /* Connect signals for CxcData struct */
   /* ??? */
   /* update the GUI */
   while (gtk_events_pending ())
      gtk_main_iteration () ;
   /* Turn off 'suspend' locker */
   suspend = FALSE ;
   printf("**** match_frames_accept\n");
   cxc_data_model_align(widget, data) ;
   printf("**** match_frames_accept end\n");
}

static void
cancel(GtkWidget *widget, Sfind *FindSet)
{
   free (FindSet->fset) ;
}

static void
close_window(GtkWidget *window, gpointer data)
{
   cxc_display *display = data ;
   display->match_frames_dialog = NULL ;
}

static void
get_buttons(GtkWidget *dialog, Fparam *FindParam)
{
   GtkWidget *button ;
   button = gtk_button_new_with_label ("OK") ;
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
         GTK_SIGNAL_FUNC(match_frames_accept), FindParam) ;
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
         button, TRUE, TRUE, 0) ;
   GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT) ;
   gtk_widget_show(button) ;

   button = gtk_button_new_with_label ("Cancel") ;
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
         GTK_SIGNAL_FUNC(gtk_widget_destroy), (gpointer) dialog) ;
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
         button, TRUE, TRUE, 0) ;
   GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT) ;
   gtk_widget_grab_default(button) ;
   gtk_widget_show (button) ;

   button = gtk_button_new_with_label ("Help") ;
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
         button, TRUE, TRUE, 0) ;
   GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT) ;
   gtk_widget_show (button) ;
}

void
match_frames(GtkWidget *widget, cxc_display *display)
{
   FileSet   *f_set = CXC_DATA (display->data)->f_set ;
   GtkWidget *dialog, *box, *vbox ;
   size_t     nfiles = f_set->n ;

   static Sfind   FindSet ;
   static Fparam  FindParam ;

   if ( display->match_frames_dialog ) {
      /* Window is already opened */
      gtk_window_set_transient_for (GTK_WINDOW (display->match_frames_dialog), 
            GTK_WINDOW (display->window)) ;
      return ;
   }

   /* Create a new window */
   display->match_frames_dialog = dialog = gtk_dialog_new( ) ;

   /* Initialize Sfind struct */
   FindSet.page = 0 ;
   FindSet.fset = (find_set*)calloc(nfiles, sizeof(find_set)) ;
   get_find_val(&FindSet, CXC_DATA (display->data)) ;

   FindParam.main_window = display->window ;
   FindParam.dialog = dialog ;
   FindParam.FindSet = &FindSet ;
   FindParam.display = display ;

   gtk_window_set_title(GTK_WINDOW(dialog), "CataXcorr: Match Frames") ;
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE) ;
   gtk_signal_connect(GTK_OBJECT(dialog), "destroy", 
                      GTK_SIGNAL_FUNC(cancel), &FindSet) ;
   gtk_signal_connect(GTK_OBJECT(dialog), "destroy", 
                      GTK_SIGNAL_FUNC(close_window), (gpointer)display) ;

   /* Main container */
   box = gtk_vbox_new(FALSE, 0) ;
   gtk_container_set_border_width (GTK_CONTAINER(box), 8) ;
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), box,
                      TRUE, TRUE, 0) ;

   /* Find Settings */
   vbox = find_settings(&FindSet, display) ;
   gtk_container_add(GTK_CONTAINER(box), vbox) ;


/* FIXME: AGGIUNGERE GENERAL OPTIONS HERE */

   gtk_widget_show_all(box) ;

   /* buttons */
   get_buttons(dialog, &FindParam) ;

   gtk_widget_show_all (dialog) ;
}
#endif
