/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 							Includes
 *--------------------------------------------------------------------------*/

#if HAVE_CONFIG_H
#  include "config.h"
#endif

#include "common.h"

#include <math.h>

#include <gtk/gtk.h>

#include "cataxcorr.h"
#include "catalib.h"

#include "platemodel.h"
#include "starlist.h"
#include "cp_messages.h"

#include "cataxcorr_p.h"
#include "wcs.h"
#include "coordinates.h"


#include "cpk_gtk_util.h"
#include "cxc_data.h"
#include "cxc_map.h"
#include "cxc_frames.h"
#include "cxc_magnifier.h"
#include "cxc_gtk_main.h"
#include "cxc_file_set.h"
#include "cxc_match_set.h"
#include "cxc_align_set.h"
#include "cxc_register_set.h"
#include "cxc_model_set.h"
#include "cxc_save.h"
#include "cxc_buttons.h"

#include "cpk_bibutton.h"

/* #include "../pixmaps/cxc_logo.xpm" */
#include "../pixmaps/off.xpm"
#include "../pixmaps/on.xpm"
#include "../pixmaps/print.xpm"
#include "../pixmaps/contrast.xpm"
#include "../pixmaps/luminosity.xpm"
#include "../pixmaps/savelog.xpm"
#include "../pixmaps/stop.xpm"

/*----------------------------------------------------------------------------
 * 							Function prototypes
 *--------------------------------------------------------------------------*/

static GtkWidget  *get_paner_frame                (GtkWidget        *window,
                                                   cxc_display      *display) ;
static GtkWidget  *get_console                    (GtkWidget        *window,
                                                   cxc_display      *display) ;
static GtkWidget  *cxc_toolbar                    (GtkWidget        *window,
                                                   cxc_display      *display) ;
static gint        delete_event                   (GtkWidget        *widget,
                                                   GdkEvent         *event,
                                                   cxc_display      *display) ;
static void        destroy                        (GtkWidget        *widget,
                                                   gpointer          data) ;

static void        save_log                       (cxc_display      *display) ;


static void        cxc_map_align                  (GtkWidget        *widget, 
                                                   cxc_display      *display) ;
static gint        cxc_timer_lock                 (gpointer          data) ;
static void        cxc_map_unlock                 (CxcMap           *map, 
                                                   cxc_display      *display) ;
static GtkWidget  *cxc_info_frame_new             (cxc_info         *info) ;

static gint        cxc_info_frame_enter           (GtkWidget        *widget, 
                                                   GdkEventCrossing *event, 
                                                   cxc_display      *display) ;
static gint        cxc_info_frame_leave           (GtkWidget        *widget, 
                                                   GdkEventCrossing *event, 
                                                   cxc_display      *display) ;
static void        cxc_info_id_update             (GtkWidget        *widget, 
                                                   cxc_display      *display) ;

static gint        cxc_info_update                (GtkWidget        *widget,
                                                   GdkEventMotion   *event,
                                                   cxc_display      *display) ;
static void        cxc_info_print_coord           (gdouble           x,
                                                   gdouble           y,
                                                   gdouble           origin[],
                                                   gboolean          spheric,
                                                   guint             wcs_fmt,
                                                   struct prjprm    *prj,
                                                   struct celprm    *cel,
                                                   gchar            *lab_x,
                                                   gchar            *lab_y) ;
static void        sprintf_sex                    (char             *out,
                                                   gdouble           sc,
                                                   gdouble           x) ;
static void        cxc_info_popup                 (cxc_display      *display) ;

static void        cxc_toolbar_set_contrast       (cxc_display      *display) ;

static void        cxc_toolbar_set_luminosity     (cxc_display      *display) ;

static void        cxc_toolbar_luminosity_reset   (CxcMap           *map,
                                                   cxc_display      *display) ;
static void        help                           (void) ;

static void        frame_new                      (GtkWidget        *toggle,
                                                   cxc_display      *display) ;
static void        frame_close                    (GtkWidget        *toggle,
                                                   cxc_display      *display) ;
extern void        data_content                   (GtkWidget        *widget,
                                                   cxc_display      *display) ;
extern void        auto_exec                      (GtkWidget        *widget,
                                                   cxc_display      *display) ;

static void        cxc_frames_signal_connect      (cxc_display      *display) ;
static gint        frame_button_press_event       (GtkWidget        *widget, 
                                                   GdkEventButton   *event, 
                                                   cxc_display      *display) ;
static gint        frame_button_release           (GtkWidget        *widget, 
                                                   GdkEventButton   *event, 
                                                   cxc_display      *display) ;
static void
cxc_stack_draw_mark (GtkWidget *widget, cxc_display *display) ;
/*----------------------------------------------------------------------------
 * 							Private types
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 							Global variables	
 *--------------------------------------------------------------------------*/

extern int        display ;
extern    FileSet   *f_set ;
extern    def_pars  *param ;
extern    g_pars     gparam ;

cxc_display *dsp ;

struct celprm *cel_ref = NULL  ; 
struct prjprm *prj_ref = NULL  ;
struct celprm *cel_frm = NULL  ; 
struct prjprm *prj_frm = NULL  ;


GtkWidget *statusbar ;

Color  red    = {255,  0,  0} ;
Color  green  = {  0,255,  0} ;
Color  blue   = {  0,  0,255} ;
Color  yellow = {255,255,  0} ;
Color  black  = {  0,  0,  0} ;
Color  white  = {255,255,255} ;

gint grid = TRUE ;


cxc_info  *info = NULL ;


extern boolean    suspend ;
gdouble    sph_units = 1.0 ;
gdouble    tunits ;

/*----------------------------------------------------------------------------
 * 							Private functions
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 							Defines
 *--------------------------------------------------------------------------*/
#ifndef MIN
#define MIN(a,b) (((a) <= (b)) ? (a) : (b))
#endif
#ifndef MAX
#define MAX(a,b) (((a) >= (b)) ? (a) : (b))
#endif
#ifndef ABS
#define ABS(a) ((a) < 0 ? (-(a)) : (a))
#endif

#define DELAY_LOCK    (500)

/*----------------------------------------------------------------------------
 *							Function codes
 *--------------------------------------------------------------------------*/


/* Leave the program */
void
monitor()
{
   return ;
   if (dsp->mode==FRAME_COINC && dsp->current!=dsp->nframes)
      printf("******************WARNING******************\n");
   printf("Focused: %d\nCurrent: %d\nData:\nfrm2:    %d sel ? %d\nMap sel: %d \n",
         dsp->focused, dsp->current,CXC_DATA (dsp->data)->frm2,
         CXC_MAP (dsp->map[CXC_DATA (dsp->data)->frm2])->selected,
         CXC_MAP (dsp->map[dsp->current])->selected) ;
}

void
abort_exec (GtkWidget *widget, gpointer callback_data)
{
   suspend = TRUE ;
}

void
quit_program (GtkWidget *widget, cxc_display *display)
{
   if (!display->we_are_exiting)
      if (cpk_confirm (display->window, "CataXcorr: Exit", " Exit Program ? "))
         return ;

   if (CXC_DATA (display->data)->locked) {
      suspend = TRUE ;
      display->we_are_exiting = TRUE ;
      return ;
   }

   /* Distrugge tutti i children */
   if (display->data_content_dialog) 
      gtk_widget_destroy (display->data_content_dialog) ; 
   if (display->auto_exec_dialog) 
      gtk_widget_destroy (display->auto_exec_dialog) ;
   if (display->align_frames_dialog) 
      gtk_widget_destroy (display->align_frames_dialog) ;
   if (display->register_frames_dialog)
      gtk_widget_destroy (display->register_frames_dialog) ;

   if (cel_ref) free (cel_ref) ;
   if (prj_ref) free (prj_ref) ;
   if (display->info)      free (display->info) ;
   if (display->paner)     free (display->paner) ;
   if (display->frames)    free (display->frames) ;
   if (display->show)      free (display->show) ;
   if (display->blink)     free (display->blink) ;
   if (display->residuals) free (display->residuals) ;

   gtk_widget_destroy (display->window) ;
}

void
cxc_display_init (cxc_display *display)
{
   gint i ;
   display->window = NULL ;
   display->data = NULL ;
   display->we_are_exiting = FALSE ;
   display->map = NULL ;
   for (i = 0; i < 5; i++)  {
      display->flag_check[i] = NULL ;
      display->settings[i] = TRUE ;
      display->panels[i] = NULL ;
   }
   for (i = 0; i < 6; i++)
      display->buttons[i] = NULL ;
   display->boxbuttons2 = NULL ;
   display->nsel = 0 ;
   display->lastboxbuttons2 = NULL ;
   display->info = NULL ;
   display->wcs_fmt = 1 ; /* 0: degrees; 1: sexagesimal */
   display->popup = NULL ;
   display->paner = NULL ;
   display->magnifier = NULL ;
   display->mgf_factor = 4 ;
   display->mgf_timer = 0 ;
   display->frame_box = NULL ;
   display->frames = NULL ;
   display->rframe = NULL ;
   display->mode = 0 ;
   display->nframes = 0 ;
   display->user_reference = 0 ;
   display->tile_mode = 0 ;
   display->last_tile_mode = 1 ;
   display->f_lock = FALSE ;
   display->timer_lock = 0 ;
   display->show = NULL ;
   display->blink = NULL ;
   display->focused = 0 ;
   display->current = 0 ;
   display->old_current = 0 ;
   display->pointer_action = POINTER ;
   display->timer = 0 ;
   display->blink_rate = 3 ; 
   display->activity = NULL ;
   display->statusbar = NULL ;
   display->contrast = NULL ;
   display->luminosity = NULL ;
   display->stop = NULL ;
   display->console = NULL ;
   display->data_content_dialog = NULL ;
   display->auto_exec_dialog = NULL ;
   display->align_frames_dialog = NULL ;
   display->register_frames_dialog = NULL ;
   display->model_frames_dialog = NULL ;
   display->match_stars_dialog = NULL ;
   display->save_dialog = NULL ;
   display->residuals = NULL ;
}

void
cxc_display_data (cxc_display *display)
{
   display->data = cxc_data_new (f_set, param, &gparam) ;
}

void
cxc_display_map (cxc_display *display)
{
   CxcData       *data  = CXC_DATA (display->data) ;
   size_t         nfr = data->n + 1 ;
   size_t         i ;

   display->map = (GtkWidget **) malloc (nfr * sizeof(GtkWidget *)) ;
   CPKDEBUG5 ("Initialize maps", 5) ;
   for (i = 0; i < nfr; i++) {
      display->map[i] = cxc_map_new (data, (guint)i) ;
      gtk_signal_connect (GTK_OBJECT (display->map[i]), "star-changed",
            (GtkSignalFunc) cxc_info_id_update, display) ;
      gtk_signal_connect (GTK_OBJECT (display->map[i]), "star-changed",
            (GtkSignalFunc) cxc_stack_draw_mark, display) ;
      gtk_signal_connect (GTK_OBJECT (display->map[i]), "map-unlock",
            (GtkSignalFunc) cxc_map_unlock, display) ;
      gtk_signal_connect (GTK_OBJECT (display->map[i]), "map-range",
            (GtkSignalFunc) cxc_toolbar_luminosity_reset, display) ;
      gtk_signal_connect (GTK_OBJECT (display->map[i]), "map-remapped",
            (GtkSignalFunc) cxc_map_align, display) ;
   }

   for (i = 0; i < nfr; i++) {
      CXC_MAP (display->map[i])->pointer_action = &(display->pointer_action) ;
      CXC_MAP (display->map[i])->mode = &display->mode ;
      CXC_MAP (display->map[i])->reference = CXC_MAP (display->map[data->nref]) ;
   }

   data->p = display->map ;
}

void
cxc_display_new (cxc_display *display)
{

   CxcData  *data  = CXC_DATA (display->data) ;
   size_t    i ;

   dsp = display ;

   if (display->nframes > 1)
      display->mode = FRAME_TILE ;
   else
      display->mode = FRAME_SINGLE ;

   /* ***FIXME ***/
   if (data->f_set->info[0]->spheric)
      tunits = 3600.0 ;
   else
      tunits = 1.0 ;

   /* Initialize wcslib structures for reference frame */
   CPKDEBUG5 ("Initialize wcslib structures", 5) ;
   /* FIXME: da ricalcolare se si cambia riferimento o se cambia CRVAL */
   cel_ref = (struct celprm *)calloc ((size_t)1, sizeof(struct celprm)) ;
   prj_ref = (struct prjprm *)calloc ((size_t)1, sizeof(struct prjprm)) ;
   cel_ref->flag = 0 ;
   cel_ref->ref[0] = data->list[data->nref]->origin[0] ;
   cel_ref->ref[1] = data->list[data->nref]->origin[1] ;
   cel_ref->ref[2] = 999.0 ;
   cel_ref->ref[3] = 999.0 ;
   prj_ref->flag = 0 ;
   prj_ref->r0 = 0.0 ;

   display->info = (cxc_info*) malloc (sizeof(cxc_info)) ;
   display->residuals = (CxcRes*) malloc (sizeof(CxcRes)) ;
   display->residuals->dialog = NULL ; /***********************/
}

void
cxc_main_window (cxc_display *display)
{
   GtkWidget     *window, *vbox, *hbox, *box, *menu_bar, *vpaned ;
   GtkTooltips   *ttip ;

   set_font ("-*-times-medium-r-normal--17-140-*-*-*-*-*-*") ;
   ttip = gtk_tooltips_new () ;

   /* Create a new window */

   CPKDEBUG5 ("Creating the main window", 5) ;
   gtk_widget_push_visual (gdk_rgb_get_visual ()) ;
   gtk_widget_push_colormap (gdk_rgb_get_cmap ()) ;
   display->window = window = gtk_window_new (GTK_WINDOW_TOPLEVEL) ;
   gtk_widget_pop_visual () ;
   gtk_widget_pop_colormap () ;   
   gtk_window_set_title (GTK_WINDOW (window), "CataXcorr") ;
   gtk_container_set_border_width (GTK_CONTAINER (window), 0) ;
   gtk_widget_set_usize (window, 750, 800) ;
   
   gtk_window_set_policy (GTK_WINDOW (window), TRUE, TRUE, TRUE) ;

   gtk_signal_connect (GTK_OBJECT (window), "delete_event", 
                       GTK_SIGNAL_FUNC (delete_event), (gpointer)display) ;
   gtk_signal_connect (GTK_OBJECT (window), "destroy", 
                       GTK_SIGNAL_FUNC (destroy), NULL) ;

   /* Main container */
   vbox = gtk_vbox_new (FALSE, 0) ;
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 1) ;
   gtk_container_add (GTK_CONTAINER (window), vbox) ;
   gtk_widget_show (vbox) ;

   /* Create the menubar */
   CPKDEBUG5 ("Creating the menubar", 5) ;
   menu_bar = cxc_menu_main (window, display) ;
   gtk_box_pack_start (GTK_BOX (vbox), menu_bar, FALSE, FALSE, 0) ;
   gtk_widget_show (menu_bar) ;

   /* Top area */
   CPKDEBUG5 ("Creating Top area", 5) ;
   hbox = gtk_hbox_new (FALSE, 2) ;
   gtk_container_set_border_width (GTK_CONTAINER (hbox), 2) ;
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0) ;
   gtk_widget_show (hbox) ;

   /* Info Panel */
   display->panels[0] = cxc_info_frame_new (display->info) ;
   gtk_box_pack_start (GTK_BOX (hbox), display->panels[0], FALSE, FALSE, 0) ;

   /* Magnifier Panel */
   display->panels[2] = cxc_magnifier_new (window, display) ;
   gtk_box_pack_end (GTK_BOX (hbox), display->panels[2], FALSE, FALSE, 0) ;

   /* Paner Panel */
   display->panels[1] = get_paner_frame (window, display) ;
   gtk_box_pack_end (GTK_BOX (hbox), display->panels[1], FALSE, FALSE, 0) ;

   /* Buttons area */
   CPKDEBUG5 ("Creating Buttons area", 5) ;
   display->panels[3] = cxc_buttons_main (window, display) ;
   gtk_container_set_border_width (GTK_CONTAINER (display->panels[3]), 0) ;
   gtk_box_pack_start (GTK_BOX (vbox), display->panels[3], FALSE, FALSE, 0) ;
   gtk_widget_show_all (display->panels[3]) ;

   /* Central area */
   vpaned = gtk_vpaned_new () ;
   gtk_box_pack_start (GTK_BOX (vbox), vpaned, TRUE, TRUE, 0) ;
   gtk_paned_set_handle_size (GTK_PANED (vpaned), 9) ;

   /* Frames */
   CPKDEBUG5 ("Creating Frames", 5) ;
   box = gtk_vbox_new (FALSE, 0) ;
   gtk_paned_pack1 (GTK_PANED (vpaned), box, TRUE, TRUE) ;


   display->frame_box = hbox = frames_realize (window, display) ;
   gtk_box_pack_start (GTK_BOX (box), hbox, TRUE, TRUE, 0) ;

   cxc_frames_signal_connect (display) ;


   gtk_widget_show_all (vpaned) ;

   /* Toolbar */
   hbox = cxc_toolbar (window, display) ;
   gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 0) ;

   /* Console area */
   display->panels[4] = hbox = get_console (window, display) ;
   /* gtk_paned_add2 (GTK_PANED (vpaned), hbox); */
   gtk_paned_pack2 (GTK_PANED (vpaned), hbox, TRUE, TRUE) ; 

   gtk_paned_set_position (GTK_PANED (vpaned), 500) ;
   gtk_widget_show_all (vpaned) ;

   /* Show window */
   gtk_widget_show (window) ;
   gtk_widget_hide (display->frame_hbox) ;
   gtk_widget_hide (display->frame_tbox) ;
   gtk_widget_hide (display->rframe) ;
   gtk_widget_show_all (display->paner[display->current]) ;

   /* Set frame mode througt main menu check item */
   gtk_check_menu_item_set_active  (
         GTK_CHECK_MENU_ITEM (display->frame_radio[display->mode]), TRUE) ;

   if (display->nframes > 1)
      frame_select (display, 1) ;
   else
      frame_select (display, 0) ;

   return ;
}

static GtkWidget *
cxc_toolbar (GtkWidget *window, cxc_display *display)
{
   GtkWidget     *vbox, *hbox, *box, *button ;
   GtkObject     *adj ;
   CxcData       *data = CXC_DATA (display->data) ;

   /* Toolbar */
   box = gtk_vbox_new (FALSE, 0) ;
   gtk_container_set_border_width (GTK_CONTAINER (box), 1) ;

   hbox = gtk_hbox_new (FALSE, 0) ;
   gtk_container_set_border_width (GTK_CONTAINER (hbox), 0) ;
   gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 0) ;

   /* activity display */
   data->activity->box = gtk_hbox_new (FALSE, 0) ;
   gtk_box_pack_start (GTK_BOX (hbox), data->activity->box, FALSE, FALSE, 5) ;

   data->activity->on  = cpk_xpm_box_new (window, on_xpm, NULL) ;
   data->activity->off = cpk_xpm_box_new (window, off_xpm, NULL) ;
   data->activity->icon = data->activity->off ;
   gtk_widget_show (data->activity->box) ;

   gtk_box_pack_start (GTK_BOX (data->activity->box), 
         data->activity->off, FALSE, FALSE, 0) ;

   statusbar = display->statusbar = gtk_statusbar_new () ;
   gtk_box_pack_start (GTK_BOX (hbox), display->statusbar, TRUE, TRUE, 0) ;
   gtk_widget_show (statusbar) ;

   adj = gtk_adjustment_new (0.2, 0.05, 0.8, .01, .5, 1.0) ;
   display->contrast = button = 
         cpk_bibutton_new (window, contrast_xpm, "Contrast", NULL, adj) ;
   gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 1) ;
   gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE) ;
   gtk_signal_connect_object (GTK_OBJECT (CPK_BIBUTTON (button)->adjustment), 
         "value-changed", GTK_SIGNAL_FUNC (cxc_toolbar_set_contrast),
          display) ;

   adj = gtk_adjustment_new (10, 1, 50., 0.2, 1, 1.0) ;
   display->luminosity = button = 
         cpk_bibutton_new (window, luminosity_xpm, "Luminosity", NULL, adj) ;
   gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 1) ;
   gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE) ;
   gtk_signal_connect_object (GTK_OBJECT (CPK_BIBUTTON (button)->adjustment), 
         "value-changed", GTK_SIGNAL_FUNC (cxc_toolbar_set_luminosity),
          (gpointer)display) ;

   button = cpk_icon_button (window, print_xpm, NULL, "Print") ; 
   gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 2) ;
   gtk_widget_set_sensitive (button, FALSE) ;
   gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE) ;

   display->savelog = cpk_icon_button (window, savelog_xpm, NULL, "Save Log File") ; 
   gtk_box_pack_start (GTK_BOX (hbox), display->savelog, FALSE, FALSE, 2) ;
   gtk_button_set_relief (GTK_BUTTON (display->savelog), GTK_RELIEF_NONE) ;
   gtk_signal_connect_object (GTK_OBJECT (display->savelog), "clicked",
         GTK_SIGNAL_FUNC (save_log), (gpointer)display) ;

   display->stop = cpk_icon_button (window, stop_xpm, "Stop", "Stop") ;
   gtk_box_pack_start (GTK_BOX (hbox), display->stop, FALSE, FALSE, 0) ;
   gtk_widget_set_sensitive (display->stop, FALSE) ;
   gtk_button_set_relief (GTK_BUTTON (display->stop), GTK_RELIEF_NONE) ;
   gtk_signal_connect (GTK_OBJECT (display->stop), "clicked",
         GTK_SIGNAL_FUNC (abort_exec), NULL) ;
   data->stop = display->stop ;

   gtk_widget_show (box);
   return box ;
}
static GtkWidget *
get_console (GtkWidget *window, cxc_display *display)
{
   GtkWidget     *box, *table, *text, *vscrollbar ;
   GdkColormap   *cmap;
   GdkColor       color;
   GdkFont       *fixed_font;

   box = gtk_vbox_new (FALSE, 2);
   gtk_container_set_border_width (GTK_CONTAINER (box), 2);
  
   table = gtk_table_new (1, 2, FALSE);
   gtk_table_set_row_spacing (GTK_TABLE (table), 0, 2);
   gtk_table_set_col_spacing (GTK_TABLE (table), 0, 2);
   gtk_box_pack_start (GTK_BOX (box), table, TRUE, TRUE, 0);

   /* Create the GtkText widget */
   display->console = text = gtk_text_new (NULL, NULL);
   gtk_text_set_editable (GTK_TEXT (text), FALSE);
   gtk_table_attach (GTK_TABLE (table), text, 0, 1, 0, 1,
		     GTK_EXPAND | GTK_SHRINK | GTK_FILL,
		     GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);

   /* Add a vertical scrollbar to the GtkText widget */
   vscrollbar = gtk_vscrollbar_new (GTK_TEXT (text)->vadj);
   gtk_table_attach (GTK_TABLE (table), vscrollbar, 1, 2, 0, 1,
		     GTK_FILL, GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);

   /* Get the system color map and allocate the color red */
   cmap = gtk_widget_get_colormap (window) ;

   color.red = 0xffff;
   color.green = 0;
   color.blue = 0;
   if (!gdk_color_alloc (cmap, &color)) {
     g_error ("couldn't allocate color");
   }

   /* Load a fixed font */
   fixed_font = gdk_font_load ("-misc-fixed-medium-r-*-*-*-140-*-*-*-*-*-*");

   /* Realizing a widget creates a window for it,
    * ready for us to insert some text */
   /* gtk_widget_realize (text) ; */

   /* Freeze the text widget, ready for multiple updates */
   gtk_text_freeze (GTK_TEXT (text)) ;
   /* Thaw the text widget, allowing the updates to become visible */  
   gtk_text_thaw (GTK_TEXT (text)) ;
 
   gtk_widget_show_all (box);
   gdk_colormap_free_colors (cmap, &color, 1) ;

   return box ;
}

static void
save_log (cxc_display *display)
{
   FILE   *out ;
   char   *filename ;
   static char  outfile[MAXPATHLEN] = "cxc.log" ;
   char   *text ;

   gtk_widget_set_sensitive (display->savelog, FALSE) ;
   filename = cpk_get_file_name (display->console, outfile) ;
   gtk_widget_set_sensitive (display->savelog, TRUE) ;

   if (filename == NULL)
      return ;

   out = fopen (filename, "w") ;
   text = gtk_editable_get_chars (GTK_EDITABLE (display->console), 0, -1) ;
   fprintf (out, "%s", text) ;
   fclose (out) ;
   g_free ( text) ;
}
/*----------------------------------------------------------------------------
 * 							Callback Functions
 *--------------------------------------------------------------------------*/

static gint
delete_event (GtkWidget *widget, GdkEvent *event, cxc_display *display)
{
   if (CXC_DATA (display->data)->locked) {
      suspend = TRUE ;
      display->we_are_exiting = TRUE ;
      return TRUE ;
   }
   return (FALSE) ;
}

static void
destroy (GtkWidget *widget, gpointer data)
{
   if (gtk_main_level ())
      gtk_main_quit () ;
   else
      gtk_exit (0) ;
}

static void 
help (void)
{
   g_print ("HELP ??? ha ha ha!\n") ;
}

/*----------------------------------------------------------------------------
 * 							Frames 
 *--------------------------------------------------------------------------*/

void
cxc_file_new (GtkWidget *widget, cxc_display *display)
{
   ;
}

void
cxc_file_close (GtkWidget *widget, cxc_display *display)
{
   CxcMap  *map = CXC_MAP (display->map[display->current]) ;
   CxcData *data = CXC_DATA (display->data) ;
   gint     n = display->current ;

   if (cpk_confirm (display->window, "CataXcorr: Close Frame", 
         " Do you really want to kill this frame ? "))
      return ;
printf("1: n %d\n",n);
   frame_prev (NULL, display) ;
printf("2\n");

   cxc_data_stack_remove (data, n) ;
printf("3\n");

   cxc_map_destroy (map) ;
printf("4\n");

   cxc_data_remove_by_number (data, n) ;
printf("5\n");
   /* cxc_frame_destroy () ;
    */
printf("6\n");

}

static void 
cxc_frames_signal_connect (cxc_display *display)
{
   GtkWidget *map ;
   guint      i ;
   /* Note: map[nframes] correspond to stack frame */
   for (i = 0; i <= display->nframes; i++) {
      map = display->map[i] ;
      gtk_signal_connect (GTK_OBJECT (CXC_MAP (map)->draw_area),
            "button_press_event", (GtkSignalFunc)frame_button_press_event, 
            display) ;
      gtk_signal_connect (GTK_OBJECT (CXC_MAP (map)->draw_area),
            "button_release_event", (GtkSignalFunc)frame_button_release, 
            display) ;
      gtk_signal_connect (GTK_OBJECT (CXC_MAP (map)->draw_area), 
            "enter_notify_event", (GtkSignalFunc)cxc_info_frame_enter, display) ;
      gtk_signal_connect (GTK_OBJECT (CXC_MAP (map)->draw_area), 
            "enter_notify_event", (GtkSignalFunc)cxc_mgf_enter, display) ;
      gtk_signal_connect (GTK_OBJECT (CXC_MAP (map)->draw_area), 
            "motion_notify_event", (GtkSignalFunc)cxc_info_update, display) ;
      gtk_signal_connect (GTK_OBJECT (CXC_MAP (map)->draw_area), 
            "motion_notify_event", (GtkSignalFunc)cxc_mgf_update, display) ;
      gtk_signal_connect (GTK_OBJECT (CXC_MAP (map)->draw_area), 
            "leave_notify_event", (GtkSignalFunc)cxc_info_frame_leave, display) ;
   }
}

static gint
frame_button_press_event (GtkWidget *widget, GdkEventButton *event, cxc_display *display)
{
   GdkModifierType state;
   guint i ;
   gint  x, y, j ;
   g_return_val_if_fail (widget != NULL, FALSE) ;
   g_return_val_if_fail (GTK_IS_DRAWING_AREA (widget), FALSE) ;
   g_return_val_if_fail (event != NULL, FALSE) ;

   if (display->mode == FRAME_BLINK) 
      return FALSE ;
   CPKDEBUG ("frame_button_press_event") ;

   j = -1 ;
   for (i = 0; i <= display->nframes; i++)
      if (widget == CXC_MAP (display->map[i])->draw_area)
         j = (gint)i ;

   if (j < 0)
      return TRUE ; /* rframe */

   gdk_window_get_pointer (event->window, &x, &y, &state) ;
   if (state & (GDK_BUTTON1_MASK | GDK_BUTTON2_MASK)) {
      if (j == (gint)display->current)
         return TRUE ;
      frame_select (display, (guint)j) ;
   }

   if (j == display->nframes) 
      return TRUE ;   /* rframe */

   if (state & GDK_BUTTON3_MASK)
      cxc_info_popup (display) ; 

   CPKDEBUG5 ("frame_button_press_event_end",6) ;
   return TRUE ;
}

static gint
frame_button_release (GtkWidget *widget, GdkEventButton *event, cxc_display *display)
{
   g_return_val_if_fail (widget != NULL, FALSE) ;
   g_return_val_if_fail (GTK_IS_DRAWING_AREA (widget), FALSE) ;
   g_return_val_if_fail (event != NULL, FALSE) ;

   if (display->popup)
      gtk_widget_destroy (display->popup) ;
   display->popup = NULL ;
   return TRUE ;
}


/*----------------------------------------------------------------------------
 * 							Zoom
 *--------------------------------------------------------------------------*/

void
zoom_in (GtkWidget *widget, cxc_display *display)
{
   gdouble zoom[] = {1/16.0, 1/8., 1/4., 1/2., 1.0, 2.0, 4.0, 8.0, 16.0 } ;
   gint i, izoom ;

   CXC_MAP (display->map[display->current])->zoom *= 2.0 ;
   if (display->mode == FRAME_MERGE) 
      CXC_MAP (display->map[display->current])->reference->zoom *= 2.0 ;

   izoom = -1 ;
   for (i = 0; i < 9; i++)
      if (CXC_MAP (display->map[display->current])->zoom == zoom[i]) {
         izoom = i ;
         gtk_check_menu_item_set_active (
               GTK_CHECK_MENU_ITEM (display->zoom_radio1[izoom]), TRUE) ;
      }
   if (izoom < 0) 
      cxc_map_remap (NULL, CXC_MAP (display->map[display->current])) ;
   CXC_MAP (display->map[display->current])->izoom = izoom ;
}

void
zoom_out (GtkWidget *widget, cxc_display *display)
{
   gdouble zoom[] = {1/16.0, 1/8., 1/4., 1/2., 1.0, 2.0, 4.0, 8.0, 16.0 } ;
   gint i, izoom ;

   CXC_MAP (display->map[display->current])->zoom /= 2.0 ;
   if (display->mode == FRAME_MERGE) 
      CXC_MAP (display->map[display->current])->reference->zoom /= 2.0 ;

   izoom = -1 ;
   for (i = 0; i < 9; i++)
      if (CXC_MAP (display->map[display->current])->zoom == zoom[i]) {
         izoom = i ;
         gtk_check_menu_item_set_active (
               GTK_CHECK_MENU_ITEM (display->zoom_radio1[izoom]), TRUE) ;
      }
   if (izoom < 0) 
      cxc_map_remap (NULL, CXC_MAP (display->map[display->current])) ;
   CXC_MAP (display->map[display->current])->izoom = izoom ;
}

void
zoom_center (GtkWidget *widget, cxc_display *display)
{
   CxcMap *map = CXC_MAP (display->map[display->current]) ;

   map->panx = ( map->fx1 + map->fx2 ) / 2.0 ;
   map->pany = ( map->fy1 + map->fy2 ) / 2.0 ;
   cxc_map_remap (NULL, map) ;
}

void
swap_x (GtkWidget *widget, cxc_display *display)
{
   CxcMap *map = CXC_MAP (display->map[display->current]) ;
   map->swx = -map->swx ;
   cxc_map_remap (NULL, map) ;
}

void
swap_y (GtkWidget *widget, cxc_display *display)
{
   CxcMap *map = CXC_MAP (display->map[display->current]) ;
   map->swy = -map->swy ;
   cxc_map_remap (NULL, map) ;
}

void
align_to_refwcs (cxc_display *display, CxcMap *currentt)
{
   gint    nref = CXC_DATA (display->data)->nref ;
   CxcMap *current = CXC_MAP (display->map[display->current]) ;
   CxcMap *reference = CXC_MAP (display->map[nref]) ;

   if (current->model->status < LOW_ALIGN && 
       display->current != CXC_DATA (display->data)->nref && 
       display->current != display->nframes)
      return ;

   align_maps (reference, current) ;
}

void
align_wcs (GtkWidget *widget, cxc_display *display)
{
   CxcMap *current = CXC_MAP (display->map[display->current]) ;
   gint    i ;

   if (display->mode == FRAME_BLINK || display->mode == FRAME_MERGE)
      return ;

   if (current->model->status < LOW_ALIGN && 
       display->current != CXC_DATA (display->data)->nref && 
       display->current != display->nframes)
      return ;

   for (i = 0; i < (gint)display->nframes; i++) {
      if (GTK_IS_WIDGET (display->frames[i]) && i != (gint)display->current) 
         if (CXC_MAP (display->map[i])->model->status >= LOW_ALIGN || 
             i == CXC_DATA (display->data)->nref) {
            align_maps (current, CXC_MAP (display->map[i])) ;
            gtk_signal_emit_by_name (GTK_OBJECT (display->map[i]), 
                     "map-changed") ;
         }
   }
   if (display->mode != FRAME_COINC) {
      align_maps (current, CXC_MAP (display->map[display->nframes])) ;
      CXC_MAP_REMAP_LATER (display->map[display->nframes]) ;
   }
}

void
align_maps (CxcMap *current, CxcMap *map)
{
   map->zoom = current->zoom ;
   map->rot  = current->rot ;
   map->panx = current->panx ;
   map->pany = current->pany ;
   map->swx  = current->swx ;
   map->swy  = current->swy ;
}

static void
cxc_map_align (GtkWidget *widget, cxc_display *display)
{
   if (CXC_MAP (widget) != CXC_MAP (display->map[display->current]))
      return ;

   if (display->timer_lock)
      gtk_timeout_remove (display->timer_lock) ;
   if (display->f_lock) 
      display->timer_lock = gtk_timeout_add (DELAY_LOCK, cxc_timer_lock, 
            display) ;
   else
      display->timer_lock = 0 ;
}

static gint
cxc_timer_lock (gpointer data)
{
   cxc_display *display = (cxc_display*) data ;

   GDK_THREADS_ENTER () ;
   display->timer_lock = 0 ;
   align_wcs (NULL, display) ;
   GDK_THREADS_LEAVE () ;
   return FALSE ;
}

static void
cxc_map_unlock (CxcMap *map, cxc_display *display)
{
   CPKDEBUG5 ("cxc_map_unlock", 5) ;
   if (map->list != CXC_DATA (display->data)->list2)
      return ;
   cxc_buttons_wcs_status (display) ;
}

static void
cxc_toolbar_luminosity_reset (CxcMap *map, cxc_display *display)
{
   CxcMap *current = CXC_MAP (display->map[display->current]) ;
   if (current != map)
      return ;
   if (CPK_BIBUTTON (display->luminosity)->adjustment)
      gtk_adjustment_set_value (CPK_BIBUTTON (display->luminosity)->adjustment, 
            map->radius) ;
}

static void
cxc_toolbar_set_contrast (cxc_display *display)
{
   CxcMap *current = CXC_MAP (display->map[display->current]) ;

   if (display->mode == FRAME_BLINK)
      return ;

   cxc_map_range_set (current, 
         1 / CPK_BIBUTTON (display->contrast)->adjustment->value) ;
}

static void
cxc_toolbar_set_luminosity (cxc_display *display)
{
   CxcMap *current = CXC_MAP (display->map[display->current]) ;

   if (display->mode == FRAME_BLINK)
      return ;

   cxc_map_radius_set (current, 
         CPK_BIBUTTON (display->luminosity)->adjustment->value) ;
}

/*----------------------------------------------------------------------------
 * 							Info Panel
 *--------------------------------------------------------------------------*/

static gint
cxc_info_frame_enter (GtkWidget *widget, GdkEventCrossing *event, cxc_display *display)
{
   CxcMap  *map ;
   CxcData *data ;
   guint    i ;
   gint     j ;
   g_return_val_if_fail (widget != NULL, FALSE) ;
   g_return_val_if_fail (GTK_IS_DRAWING_AREA (widget), FALSE) ;
   g_return_val_if_fail (event != NULL, FALSE) ;

   CPKDEBUG5 ("cxc_info_frame_enter", 7) ;
  
   j = -1 ;
   for (i = 0; i <= display->nframes; i++)
      if (widget == CXC_MAP (display->map[i])->draw_area)
         j = (gint)i ;

   if (j < 0)
      return TRUE ;
   display->focused = j ;

   map = CXC_MAP (display->map[j]) ;
   data = CXC_DATA (display->data) ;

   if (j < display->nframes) {
      strncpy (display->info->file, data->f_set->info[j]->filename, (size_t)30) ;
      gtk_label_set_text (GTK_LABEL (display->info->labels[0]), display->info->file) ;

      sprintf (display->info->frame, "Frame%d", j+1) ;
      gtk_label_set_text (GTK_LABEL (display->info->labels[11]), display->info->frame) ;
   }
   if (map->color)
      gtk_label_set_text (GTK_LABEL (display->info->labels[14]), "NS") ;

   sprintf (display->info->zoom, "%.3f", map->zoom) ;
   gtk_label_set_text (GTK_LABEL (display->info->labels[12]), display->info->zoom) ;
   sprintf (display->info->ang, "%.3f", RADTODEG (map->rot)) ;
   gtk_label_set_text (GTK_LABEL (display->info->labels[13]), display->info->ang) ;

   if (display->mode == FRAME_BLINK)
      return TRUE ;

   if (data->f_set->info[data->nref]->spheric) {
      strcpy (display->info->xwl, "RA") ;
      strcpy (display->info->ywl, "Dec") ;
   }
   else {
      strcpy (display->info->xwl, "X") ;
      strcpy (display->info->ywl, "Y") ;
   }
   gtk_label_set_text (GTK_LABEL (display->info->labels[3]), display->info->xwl) ;
   gtk_label_set_text (GTK_LABEL (display->info->labels[5]), display->info->ywl) ;

   if (j == display->nframes)
      return TRUE ;

   if (data->f_set->info[j]->spheric) {
      strcpy (display->info->xil, "RA") ;
      strcpy (display->info->yil, "Dec") ;

      /* Initialize wcslib structures */
      cel_frm = (struct celprm *) calloc ((size_t)1, sizeof(struct celprm)) ;
      prj_frm = (struct prjprm *) calloc ((size_t)1, sizeof(struct prjprm)) ;
      cel_frm->flag = 0 ;
      cel_frm->ref[0] = map->origin[0] ;
      cel_frm->ref[1] = map->origin[1] ;
      cel_frm->ref[2] = 999.0 ;
      cel_frm->ref[3] = 999.0 ;
      prj_frm->flag = 0 ;
      prj_frm->r0 = 0.0 ;
   }
   else {
      strcpy (display->info->xil, "X") ;
      strcpy (display->info->yil, "Y") ;
   }
   gtk_label_set_text (GTK_LABEL (display->info->labels[7]), display->info->xil) ;
   gtk_label_set_text (GTK_LABEL (display->info->labels[9]), display->info->yil) ;

   return TRUE ;
}

static gint
cxc_info_frame_leave (GtkWidget *widget, GdkEventCrossing *event, cxc_display *display)
{
   CxcMap  *map ;
   CxcData *data ;
   guint    i ;
   g_return_val_if_fail (widget != NULL, FALSE) ;
   g_return_val_if_fail (GTK_IS_DRAWING_AREA (widget), FALSE) ;
   g_return_val_if_fail (event != NULL, FALSE) ;
   CPKDEBUG5 ("cxc_info_frame_leave", 7);

   map = CXC_MAP (display->map[display->focused]) ;
   data = CXC_DATA (display->data) ;

   if (cel_frm)
      free (cel_frm) ;
   if (prj_frm)
      free (prj_frm) ;
   cel_frm = NULL ;
   prj_frm = NULL ;

   for (i=0; i<15; i++)
      gtk_label_set_text (GTK_LABEL (display->info->labels[i]), "") ;
   gtk_label_set_text (GTK_LABEL (display->info->labels[11]), "Frame") ;

   return TRUE ;
}

static void
cxc_stack_draw_mark (GtkWidget *widget, cxc_display *display)
{
   CxcMap *map = CXC_MAP (widget) ;
   CxcMap *map2 ;
   Stack  *tab = NULL ;
   guint   i ;
   if (!map->s)
      return ;
   if (!map->color)
      return ;
   tab = (Stack*)map->s->p ;
   if (!tab)
      return ;
   for (i = 0; i < CXC_DATA (display->data)->n; i++) {
      map2 = CXC_MAP (display->map[i]) ; 
      if (map2 != map && GTK_WIDGET_VISIBLE (map2->draw_area->parent))
         cxc_map_draw_mark (map2->draw_area, map2, tab->data[i], FALSE) ;
   }
}

static void
cxc_info_id_update (GtkWidget *widget, cxc_display *display)
{
   CxcMap *map = CXC_MAP (widget) ;
   Stack  *tab = NULL ;
   
   CPKDEBUG5 ("cxc_info_id_update", 7) ;

   /* Set identification string */
   if (map->s)
      sprintf (display->info->id_1, "%s", map->s->id) ;
   else
      sprintf (display->info->id_1, " ") ;

   gtk_label_set_text (GTK_LABEL (display->info->labels[1]),
         display->info->id_1) ;

   /* Set NS string for FRAME_COINC mode */
   sprintf (display->info->id_2, " ")  ;
   /* if (map->s && map->map_id == CXC_DATA (display->data)->n) { */
   if (map->s && map->color) {
      tab = (Stack*)map->s->p ;
      if (tab) {
         bin_get_string (display->info->id_2, tab->ns) ;
         display->info->id_2[CXC_DATA (display->data)->n] = '\0' ;
      }
   }

   gtk_label_set_text (GTK_LABEL (display->info->labels[2]),
         display->info->id_2) ;

   gdk_draw_pixmap (display->magnifier->window,
         display->magnifier->style->fg_gc[GTK_WIDGET_STATE (display->magnifier)],
         map->pixmap_mgf, 0, 0, 0, 0, -1,-1) ;

   /***FIXME: spostare dentro cxc_map_draw_mark ***/
   cxc_mgf_id_draw (map, display) ;
}

static gint
cxc_info_update (GtkWidget *widget, GdkEventMotion *event, cxc_display *display)
{
   GdkModifierType  state;
   CxcMap          *mapr ;
   CxcMap          *mapf ;
   CxcData         *data ;
   gint             x, y ;
   g_return_val_if_fail (widget != NULL, FALSE) ;
   g_return_val_if_fail (GTK_IS_DRAWING_AREA (widget), FALSE) ;
   g_return_val_if_fail (event != NULL, FALSE) ;

   CPKDEBUG5 ("cxc_info_update", 8) ;

   if (display->mode == FRAME_BLINK)
      return TRUE ;

   if (event->is_hint)
      gdk_window_get_pointer (event->window, &x, &y, &state);
   else {
      x = event->x;
      y = event->y;
      state = event->state;
   }
   if (x < 0 || x > widget->allocation.width || 
       y < 0 || y > widget->allocation.height)
      return FALSE ;

   data = CXC_DATA (display->data) ;
   mapr = CXC_MAP (display->map[data->nref]) ;
   mapf = CXC_MAP (display->map[display->focused]) ;

   if (!GTK_WIDGET_VISIBLE (mapf->draw_area->parent)) 
      return FALSE ;

   /****FIXME *****/

   if (cel_ref->ref[0] != mapr->origin[0] ||
       cel_ref->ref[1] != mapr->origin[1]) {
      cel_ref->ref[0] = mapr->origin[0] ;
      cel_ref->ref[1] = mapr->origin[1] ;
      cel_ref->flag = 0 ;
   }
   if (cel_frm) {
      if (cel_frm->ref[0] != mapf->origin[0] || 
          cel_frm->ref[1] != mapf->origin[1]) {
         cel_frm->ref[0] = mapf->origin[0] ;
         cel_frm->ref[1] = mapf->origin[1] ;
         cel_frm->flag = 0 ;
      }
   }

   /* Set Zoom and Angle info */
   if (state & GDK_BUTTON1_MASK) {
      switch (display->pointer_action) {
         case ZOOM :
            sprintf (display->info->zoom, "%.3f", mapf->zoom) ;
            gtk_label_set_text (GTK_LABEL (display->info->labels[12]), 
                  display->info->zoom) ;
            break ;
         case ROTATE :
            sprintf (display->info->ang, "%.3f", RADTODEG (mapf->rot)) ;
            gtk_label_set_text (GTK_LABEL (display->info->labels[13]), 
                  display->info->ang) ;
            break ;
         default :
            break ;
      }
   }
   else {

      /* Set word coord info */
      if (display->focused == (guint)data->nref || 
          display->focused == display->nframes) {
         cxc_info_print_coord (mapf->xcur, mapf->ycur, 
               mapr->origin, 
               (gboolean)data->list[data->nref]->spheric, 
               display->wcs_fmt, prj_ref, cel_ref, 
               display->info->xw, display->info->yw) ;
         gtk_label_set_text (GTK_LABEL (display->info->labels[4]),
               display->info->xw) ;
         gtk_label_set_text (GTK_LABEL (display->info->labels[6]),
               display->info->yw) ;
      }
      else {
         if (data->model[display->focused]->status) {
            cxc_info_print_coord (mapf->xcur, mapf->ycur, 
                  mapr->origin, 
                  (gboolean)data->list[data->nref]->spheric, 
                  display->wcs_fmt, prj_ref, cel_ref, 
                  display->info->xw, display->info->yw) ;
            gtk_label_set_text (GTK_LABEL (display->info->labels[4]),
                  display->info->xw) ;
            gtk_label_set_text (GTK_LABEL (display->info->labels[6]),
                  display->info->yw) ;
         }
         if (display->mode == FRAME_MERGE) {
            cxc_info_print_coord (mapf->reference->xcur, mapf->reference->ycur, 
                  mapr->origin, 
                  (gboolean)data->list[data->nref]->spheric, 
                  display->wcs_fmt, prj_ref, cel_ref, 
                  display->info->xw, display->info->yw) ;
            gtk_label_set_text (GTK_LABEL (display->info->labels[4]),
                  display->info->xw) ;
            gtk_label_set_text (GTK_LABEL (display->info->labels[6]),
                  display->info->yw) ;
         }
      }

      /* Set image coord info */
      if (!(state & GDK_BUTTON3_MASK) && display->mode != FRAME_COINC) {
         double xcur = mapf->xcur, ycur = mapf->ycur ;
         gboolean sph = (display->focused == display->nframes) ?
                        (gboolean)data->list[data->nref]->spheric :
                        (gboolean)data->list[display->focused]->spheric ;

         if (display->focused != display->nframes &&
             data->model[display->focused]->status) 
            coord_rot2prj (mapf->xcur, mapf->ycur, data->model[display->focused], 
                  &xcur, &ycur) ;
         cxc_info_print_coord (xcur, ycur, mapf->origin, sph, 
               display->wcs_fmt, prj_frm, cel_frm, 
               display->info->xi, display->info->yi) ;
         gtk_label_set_text (GTK_LABEL (display->info->labels[8]),
               display->info->xi) ;
         gtk_label_set_text (GTK_LABEL (display->info->labels[10]),
               display->info->yi) ;
      }
   }
   CPKDEBUG5 ("cxc_info_update_end", 8) ;
   return TRUE ;
}

static void
cxc_info_print_coord (gdouble x, gdouble y, gdouble origin[], gboolean spheric, 
                      guint wcs_fmt, struct prjprm *prj, struct celprm *cel, 
                      gchar *lab_x, gchar *lab_y)
{
   gdouble  phi, theta, xword, yword ;

   if (spheric) {
      celrev ("TAN", x, y, prj, &phi, &theta, cel, &xword, &yword) ;
      if (wcs_fmt) {
         sprintf_sex (lab_x, 24.0/360.0, xword) ;
         sprintf_sex (lab_y, 1.0, yword) ;
      }
      else {
         sprintf (lab_x, "%.5f", xword) ;
         sprintf (lab_y, "%.5f", yword) ;
      }
   }
   else {
      /*** FIXME: rot->prj first needed !!! ***/
      xword = x + origin[0] ;
      yword = y + origin[1] ;
      sprintf (lab_x, "%.3f", xword) ;
      sprintf (lab_y, "%.3f", yword) ;
   }

}

static void
sprintf_sex (char *out, gdouble sc, gdouble x)
{
   gdouble h, m, s ;
   gchar   buffer[1024] ;
   s = (x >= 0.0) ? 1.0 : -1.0 ;
   x = ABS (x) ;
   x = x * sc ;
   h = floor (x) ;
   x = (x - h) * 60.0 ;
   m = floor (x) ;
   x = (x - m) * 60.0 ;
   sprintf (buffer, "% 02.0f:%02.0f:%06.3f", s*h, m, x) ;
   strncpy (out, buffer, (size_t)14) ;
}

static GtkWidget *
cxc_info_frame_new (cxc_info *info)
{
   GtkWidget     *box, *event_box, *hbox, *frame, *label, *table ;
   GtkTooltips   *ttip ;

   ttip = gtk_tooltips_new () ;

   /* Identity display */
   box = gtk_vbox_new (FALSE, 0) ;
   event_box = gtk_event_box_new () ;
   /* gtk_tooltips_set_tip (GTK_TOOLTIPS (ttip), event_box , 
                        "Id number of pointed stars", NULL) ; */
   gtk_box_pack_start (GTK_BOX (box), event_box, FALSE, FALSE, 0) ;

   frame = gtk_frame_new (NULL) ;
   gtk_widget_set_usize (frame, 455, -1) ;
   gtk_container_add (GTK_CONTAINER (event_box), frame) ;

   table = gtk_table_new (5, 5, FALSE) ;
   gtk_table_set_col_spacings (GTK_TABLE (table), 2) ;
   gtk_table_set_row_spacings (GTK_TABLE (table), 1) ;
   gtk_container_set_border_width (GTK_CONTAINER (frame), 2) ;
   gtk_container_add (GTK_CONTAINER (frame), table) ;

   label = gtk_label_new ("File") ;
   hbox = gtk_hbox_new (FALSE, 0) ;
   gtk_widget_set_usize (hbox, 75, 0);
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 4) ;
   gtk_table_attach (GTK_TABLE (table), hbox, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 1, 1) ;
   frame = gtk_frame_new (NULL) ;
   gtk_table_attach (GTK_TABLE (table), frame, 2, 5, 0, 1, GTK_FILL|GTK_EXPAND, GTK_FILL, 1, 1) ;
   hbox = gtk_hbox_new (FALSE, 0) ;
   info->labels[0] = label = gtk_label_new (NULL) ;
   gtk_container_add (GTK_CONTAINER (frame), hbox) ;
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2) ;


   label = gtk_label_new ("Identity") ;
   hbox = gtk_hbox_new (FALSE, 0) ;
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 4) ;
   gtk_table_attach (GTK_TABLE (table), hbox, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 1, 1) ;
   frame = gtk_frame_new (NULL) ;
   gtk_table_attach (GTK_TABLE (table), frame, 2, 3, 1, 2, GTK_FILL, GTK_FILL, 1, 1) ;
   info->labels[1] = label = gtk_label_new (NULL) ;
   hbox = gtk_hbox_new (FALSE, 0) ;
   gtk_widget_set_usize (hbox, 115, 0);
   gtk_container_add (GTK_CONTAINER (frame), hbox) ;
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 4) ;
   info->labels[14] = label = gtk_label_new (NULL) ;
   hbox = gtk_hbox_new (FALSE, 0) ;
   gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 2) ;
   gtk_table_attach (GTK_TABLE (table), hbox, 3, 4, 1, 2, GTK_FILL, GTK_FILL, 1, 1) ;
   frame = gtk_frame_new (NULL) ;
   gtk_table_attach (GTK_TABLE (table), frame, 4, 5, 1, 2, GTK_FILL, GTK_FILL, 1, 1) ;
   info->labels[2] = label = gtk_label_new (NULL) ;
   hbox = gtk_hbox_new (FALSE, 0) ;
   gtk_widget_set_usize (hbox, 115, 0);
   gtk_container_add (GTK_CONTAINER (frame), hbox) ;
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 4) ;

   label = gtk_label_new ("WCS") ;
   hbox = gtk_hbox_new (FALSE, 0) ;
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 4) ;
   gtk_table_attach (GTK_TABLE (table), hbox, 0, 1, 2, 3, GTK_FILL, GTK_FILL, 1, 1) ;
   info->labels[3] = label = gtk_label_new (" ") ;
   hbox = gtk_hbox_new (FALSE, 0) ;
   gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 2) ;
   gtk_table_attach (GTK_TABLE (table), hbox, 1, 2, 2, 3, GTK_FILL, GTK_FILL, 1, 1) ;
   frame = gtk_frame_new (NULL) ;
   gtk_table_attach (GTK_TABLE (table), frame, 2, 3, 2, 3, GTK_FILL, GTK_FILL, 1, 1) ;
   info->labels[4] = label = gtk_label_new (NULL) ;
   gtk_label_set_justify (GTK_LABEL (label),  GTK_JUSTIFY_RIGHT) ;
   gtk_container_add (GTK_CONTAINER (frame), label) ;
   info->labels[5] = label = gtk_label_new (" ") ;
   hbox = gtk_hbox_new (FALSE, 0) ;
   gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 2) ;
   gtk_table_attach (GTK_TABLE (table), hbox, 3, 4, 2, 3, GTK_FILL, GTK_FILL, 1, 1) ;
   frame = gtk_frame_new (NULL) ;
   gtk_table_attach (GTK_TABLE (table), frame, 4, 5, 2, 3, GTK_FILL, GTK_FILL, 1, 1) ;
   info->labels[6] = label = gtk_label_new (NULL) ;
   gtk_label_set_justify (GTK_LABEL (label),  GTK_JUSTIFY_RIGHT) ;
   gtk_container_add (GTK_CONTAINER (frame), label) ;

   label = gtk_label_new ("Local CS") ;
   hbox = gtk_hbox_new (FALSE, 0) ;
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 4) ;
   gtk_table_attach (GTK_TABLE (table), hbox, 0, 1, 3, 4, GTK_FILL, GTK_FILL, 1, 1) ;
   info->labels[7] = label = gtk_label_new (" ") ;
   hbox = gtk_hbox_new (FALSE, 0) ;
   gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 2) ;
   gtk_table_attach (GTK_TABLE (table), hbox, 1, 2, 3, 4, GTK_FILL, GTK_FILL, 1, 1) ;
   frame = gtk_frame_new (NULL) ;
   gtk_table_attach (GTK_TABLE (table), frame, 2, 3, 3, 4, GTK_FILL, GTK_FILL, 1, 1) ;
   info->labels[8] = label = gtk_label_new (NULL) ;
   gtk_label_set_justify (GTK_LABEL (label),  GTK_JUSTIFY_RIGHT) ;
   gtk_container_add (GTK_CONTAINER (frame), label) ;
   info->labels[9] = label = gtk_label_new (" ") ;
   hbox = gtk_hbox_new (FALSE, 0) ;
   gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 2) ;
   gtk_table_attach (GTK_TABLE (table), hbox, 3, 4, 3, 4, GTK_FILL, GTK_FILL, 1, 1) ;
   frame = gtk_frame_new (NULL) ;
   gtk_table_attach (GTK_TABLE (table), frame, 4, 5, 3, 4, GTK_FILL, GTK_FILL, 1, 1) ;
   info->labels[10] = label = gtk_label_new (NULL) ;
   gtk_label_set_justify (GTK_LABEL (label),  GTK_JUSTIFY_RIGHT) ;
   gtk_container_add (GTK_CONTAINER (frame), label) ;


   info->labels[11] = label = gtk_label_new ("Frame") ;
   hbox = gtk_hbox_new (FALSE, 0) ;
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 4) ;
   gtk_table_attach (GTK_TABLE (table), hbox, 0, 1, 4, 5, GTK_FILL, GTK_FILL, 1, 1) ;
   label = gtk_label_new ("Zoom") ;
   hbox = gtk_hbox_new (FALSE, 0) ;
   gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 2) ;
   gtk_table_attach (GTK_TABLE (table), hbox, 1, 2, 4, 5, GTK_FILL, GTK_FILL, 1, 1) ;
   frame = gtk_frame_new (NULL) ;
   gtk_table_attach (GTK_TABLE (table), frame, 2, 3, 4, 5, GTK_FILL, GTK_FILL, 1, 1) ;
   info->labels[12] = label = gtk_label_new (NULL) ;
   gtk_label_set_justify (GTK_LABEL (label),  GTK_JUSTIFY_RIGHT) ;
   gtk_container_add (GTK_CONTAINER (frame), label) ;
   label = gtk_label_new ("Ang") ;
   hbox = gtk_hbox_new (FALSE, 0) ;
   gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 2) ;
   gtk_table_attach (GTK_TABLE (table), hbox, 3, 4, 4, 5, GTK_FILL, GTK_FILL, 1, 1) ;
   frame = gtk_frame_new (NULL) ;
   gtk_table_attach (GTK_TABLE (table), frame, 4, 5, 4, 5, GTK_FILL, GTK_FILL, 1, 1) ;
   info->labels[13] = label = gtk_label_new (NULL) ;
   gtk_label_set_justify (GTK_LABEL (label),  GTK_JUSTIFY_RIGHT) ;
   gtk_container_add (GTK_CONTAINER (frame), label) ;

   if (info) 
      gtk_widget_show_all (box) ;

   return box ;
}

static void
cxc_info_popup (cxc_display *display)
{
   CxcMap    *map ;
   CxcData   *data ;
   gchar      buffer[1024] ;
   GtkWidget *label, *frame ;
   gint       x, y, scr_w, w;

   map = CXC_MAP (display->map[display->focused]) ;
   data = CXC_DATA (display->data) ;

   if (!map->s)
      return ;

   star_list_get_record (data->f_set->info[display->focused], 
         map->s->index, buffer) ;

   if (display->popup)
      gtk_widget_destroy (display->popup) ;

   display->popup = gtk_window_new (GTK_WINDOW_POPUP) ;
   gtk_window_set_transient_for (GTK_WINDOW (display->popup), 
         GTK_WINDOW (display->window)) ;

   frame = gtk_frame_new (NULL) ;
   gtk_container_add (GTK_CONTAINER (display->popup), frame) ;
   label = gtk_label_new (buffer) ;
   gtk_container_add (GTK_CONTAINER (frame), label) ;
   gtk_widget_show_all (frame) ;

   scr_w = gdk_screen_width () ;
   w = gdk_string_width (display->popup->style->font, buffer) + 5 ;
   gdk_window_get_pointer (NULL, &x, &y, NULL) ;

   if ((x + w) > scr_w)
      x -= (x + w) - scr_w ;
   else 
      if (x < 0)
         x = 0 ;   
   y += 20 ;
   gtk_widget_popup (display->popup, x, y) ;
}

/*----------------------------------------------------------------------------
 * 							Paner
 *--------------------------------------------------------------------------*/

static GtkWidget *
get_paner_frame (GtkWidget *window, cxc_display *display)
{
   GtkWidget *box, *frame ;
   guint      i ;

   box = gtk_hbox_new (FALSE, 0) ;
   for (i = 0; i <= display->nframes; i++) {
      display->paner[i] = frame = gtk_frame_new (NULL) ;
      gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN) ;
      gtk_widget_set_usize (frame, 120, 120) ;
      gtk_container_set_border_width (GTK_CONTAINER (frame), 0) ;
      gtk_container_add (GTK_CONTAINER (box), frame) ;
      gtk_container_add (GTK_CONTAINER (display->paner[i]),
            CXC_MAP (display->map[i])->paner) ;
   }
   gtk_widget_show (box) ;
   return box ;
}

/*----------------------------------------------------------------------------
 * 							WCS
 *--------------------------------------------------------------------------*/
static void
set_reference (GtkWidget *item, cxc_display *display)
{
   gchar *ic ;
   gint   i ;
   ic = (gchar*) gtk_object_get_data (GTK_OBJECT (item), "item_id");
   i = atoi (ic) ;
   if (i < display->nframes) {
      CXC_DATA (display->data)->frm1 = i ;
      CXC_DATA (display->data)->list1 = CXC_DATA (display->data)->list[i] ;
   }
   else {
      CXC_DATA (display->data)->frm1 = 0 ;
      CXC_DATA (display->data)->list1 = CXC_DATA (display->data)->stack->ref ;
   }
}

void
cxc_set_reference (GtkWidget *box, gboolean expand, guint border, gint select, 
      cxc_display *display)
{
   FileSet   *f_set = CXC_DATA (display->data)->f_set ;
   gchar      buffer[1024], buffer2[2048], *buf ;
   GtkWidget *frame, *label, *hbox, *vbox, *option, *menu, *item, *sep ;
   gint       i, w ;
   gint       frm2 = CXC_DATA (display->data)->frm2 ;
   GtkTooltips   *ttip ;

   ttip = gtk_tooltips_new () ;

   hbox = gtk_hbox_new (FALSE, 0) ;
   gtk_container_set_border_width (GTK_CONTAINER (hbox), border) ;
   gtk_container_add (GTK_CONTAINER (box), hbox) ;

   set_font ("-*-times-medium-r-normal--18-140-*-*-*-*-*-*") ;
   frame = gtk_frame_new ("Reference Frame Selection") ;
   /* gtk_container_set_border_width (GTK_CONTAINER (frame), border) ; */
   gtk_box_pack_start (GTK_BOX (hbox), frame, expand, expand, 0) ;  
   set_font ("-*-times-medium-r-normal--17-140-*-*-*-*-*-*") ;

   vbox = gtk_vbox_new (FALSE, 2) ;
   gtk_container_set_border_width (GTK_CONTAINER (vbox), border) ;
   gtk_container_add (GTK_CONTAINER (frame), vbox) ;
   sprintf (buffer, "Matching file '%s' on reference ", 
         f_set->info[CXC_DATA (display->data)->frm2]->filename) ;
   label = gtk_label_new (buffer) ;
   gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 5) ;

   hbox = gtk_hbox_new (FALSE, 0) ;
   gtk_container_add (GTK_CONTAINER (vbox), hbox) ;
   option = gtk_option_menu_new () ;
   menu = gtk_menu_new () ;
   buf = buffer ;
   for (i = 0; i < display->nframes; i++) {
      sprintf (buffer, "%d", i) ;
      item = make_menu_item (f_set->info[i]->filename, 
            GTK_SIGNAL_FUNC (set_reference), display) ;
      gtk_menu_append (GTK_MENU (menu), item) ;
      gtk_object_set_data (GTK_OBJECT (item), "item_id", 
            (gpointer) g_strdup (buf)) ;
      if ((i == CXC_DATA (display->data)->frm2) ||
          (i != CXC_DATA (display->data)->nref && 
           CXC_DATA (display->data)->model[i]->status == INIT))
         gtk_widget_set_sensitive (item, FALSE) ;
   }
   item = gtk_menu_item_new () ;
   sep = gtk_hseparator_new () ;
   gtk_widget_set_sensitive (item, FALSE) ;
   gtk_container_add (GTK_CONTAINER (item), sep) ;
   gtk_menu_append (GTK_MENU (menu), item) ;
   item = make_menu_item ("STACK", GTK_SIGNAL_FUNC (set_reference), display) ;
   sprintf (buffer, "%d", display->nframes) ;
   gtk_object_set_data (GTK_OBJECT (item), "item_id",  
            (gpointer) g_strdup (buf)) ;
   gtk_menu_append (GTK_MENU (menu), item) ;

   /* Create a Tooltip to hold file list in current stack */
   for (i = w = 0; i < display->nframes; i++) {
      sprintf (buffer, " %3d - %s", i+1, f_set->info[i]->filename) ;
      w = MAX (w, strlen (buffer)+1) ;
   }
   sprintf (buffer, "-Union_of_already_Matched_Frames-") ;
   w = MAX (w, strlen (buffer)) ;

   strcpy (buffer2, "") ;
   for (i = 0; i < (w - strlen (buffer))/2; i++) 
      strcat (buffer2, "-") ;
   strcat (buffer2, buffer) ;
   for (i = strlen (buffer2); i < w ; i++)
      strcat (buffer2, "-") ;
   w = strlen (buffer2)+1 ;
   strcat (buffer2, "\n") ;
   for (i = 0; i < display->nframes; i++) {
      if (bin_check_n (CXC_DATA (display->data)->stack->nf, i) ||
          bin_check_n (CXC_DATA (display->data)->stack->nf2, i-32)) {
         sprintf (buffer, " %3d - %s", i+1, f_set->info[i]->filename) ;
         if (strlen (buffer) + strlen (buffer2) < 2048)
            sprintf (buffer2, "%s%-*s", buffer2, w, buffer) ;
         else {
            sprintf (buffer2, "%s%-*s", buffer2, w, " ...more... ") ;
            break ;
         }
      }
   }
   gtk_tooltips_set_tip (GTK_TOOLTIPS (ttip), item , buffer2, NULL) ;

   if (select == display->nframes)
      select++ ;
   else
      while ((select == CXC_DATA (display->data)->frm2) ||
             (select != CXC_DATA (display->data)->nref && 
              CXC_DATA (display->data)->model[select]->status == INIT))
         select-- ;

   gtk_menu_set_active (GTK_MENU (menu), select) ;
   gtk_option_menu_set_menu (GTK_OPTION_MENU (option), menu) ;
   gtk_box_pack_end (GTK_BOX (hbox), option, FALSE, FALSE, 5) ;
   gtk_signal_emit_by_name (
         GTK_OBJECT (gtk_menu_get_active (GTK_MENU (menu))), "activate") ;

   gtk_widget_show_all (frame) ;
}

