/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/

#ifndef CXC_MAP_H
#define CXC_MAP_H 1

#include "common.h"

BEGIN_C_DECLS

#include <gdk/gdk.h>
#include <gtk/gtkdata.h>

#include "cataxcorr.h"
#include "platemodel.h"
#include "cpk_gtk_util.h"

#define TYPE_CXC_MAP                (cxc_map_get_type ())
#define CXC_MAP(obj)                (GTK_CHECK_CAST ((obj), TYPE_CXC_MAP, CxcMap))
#define CXC_MAP_CLASS(klass)        (GTK_CHECK_CLASS_CAST ((klass), TYPE_CXC_MAP, CxcMapClass))
#define IS_CXC_MAP(obj)             (GTK_CHECK_TYPE ((obj), TYPE_CXC_MAP))
#define IS_CXC_MAP_CLASS(klass)     (GTK_CHECK_CLASS_TYPE ((klass), TYPE_CXC_MAP))
#define CXC_MAP_REMAP_LATER(obj)    (CXC_MAP (obj)->nx = 0)
#define CXC_MAP_SET_RESCALING(obj)  (CXC_MAP (obj)->dx = 0.0)

typedef enum {
   POINTER = 0,
   PAN,
   ZOOM,
   ROTATE,
   RULER,
   NOACT
} PointerAct ;

enum {
   MAP_RANGE,
   MAP_MAGNITUDE,
   MAP_CHANGED,
   MAP_REMAPPED, 
   MAP_UNLOCK,
   STAR_CHANGED,
   MAP_LAST_SIGNAL
} ;

enum {
   MAP_NORMAL,
   MAP_FLAGGED,
   MAP_HIGHLIGHT,
   MAP_GRAY,
   MAP_SHADE,
   MAP_DISPL
} ;

typedef struct _CxcMap	CxcMap ;
typedef struct _CxcMapClass  CxcMapClass ;

struct _CxcMap
{
   GtkWidget   vbox ;
   GtkWidget  *draw_area ;
   GtkWidget  *paner ;
   GdkPixmap  *pixmap ;
   GdkPixmap  *pixmap_pan ;
   GdkPixmap  *pixmap_mgf ;
   GdkGC      *gc[6] ;
   GdkGC     **stack_gc ;
   guint       ngc ;
   guint       nx ;
   guint       ny ;
   guint       npx, npy ;
   StarList   *list ;
   PlateMod   *model ;
   gint        map_id ;
   gboolean    color ;
   gboolean    selected ;
   gint       *mode ;
   PointerAct *pointer_action ;
   gdouble     fx1,fx2, fy1, fy2, fm1, fm2 ;
   gdouble     dx, dy ;
   gboolean    xyrange ;
   guint       nbright ;
   gdouble     mcut1, mcut2 ;
   gdouble     mrange ;
   gdouble     old_mzero ;
   gdouble     zoom ;
   gint        izoom ;
   gdouble     rot ;
   gdouble     panx, pany ;
   gint        swx, swy ;
   gdouble     map_scale, pan_scale, swxx, cos_rot, sin_rot ;
   gdouble     origin[2] ;
   gdouble     radius ;
   guint       fnorm ;
   gdouble     xcur, ycur ;
   gboolean    grabbed ;
   gdouble     dragx, dragy ;
   gdouble     dragz, dragd ;
   gdouble     draga, dragr ;
   Star       *s ;
   gboolean    paner_drag ;
   gboolean    paner_active ;
   CxcMap     *reference ;
   guint32     timer ;
   guint32     timer_id ;
} ;

struct _CxcMapClass
{
   GtkVBoxClass parent_class ;
   void (* map_range)     (CxcMap *map) ;
   void (* map_magnitude) (CxcMap *map) ;
   void (* map_changed)   (CxcMap *map) ;
   void (* map_remapped)  (CxcMap *map) ;
   void (* map_unlock)    (CxcMap *map) ;
   void (* star_changed)  (CxcMap *map) ;
} ;

extern GtkType     cxc_map_get_type          (void) ;
extern GtkWidget  *cxc_map_new               (CxcData    *data,
                                              guint       n) ;

extern void        cxc_map_destroy           (CxcMap     *map) ;
extern void        cxc_map_reset             (CxcMap     *map) ;
extern gint        cxc_map_remap             (GtkWidget  *widget,
                                              CxcMap     *map) ;
extern void        cxc_map_get_ref_wcs       (CxcMap     *map) ;
extern gint        cxc_map_configure_pixmap  (GtkWidget  *widget,
                                              CxcMap     *map) ;
extern void        cxc_map_range_set         (CxcMap     *map,
                                              gfloat      val) ;
extern void        cxc_map_radius_set        (CxcMap     *map,
                                              gfloat      val) ;
extern void        world_to_draw             (CxcMap     *map,
                                              gdouble     x,
                                              gdouble     y,
                                              gdouble    *xd,
                                              gdouble    *yd) ;
extern void        draw_to_world             (CxcMap     *map,
                                              gdouble     xd,
                                              gdouble     yd,
                                              gdouble    *x,
                                              gdouble    *y) ;

extern void        cxc_map_draw_mark         (GtkWidget  *widget,
                                              CxcMap     *map,
                                              Star       *s,
                                              gboolean    emit) ;

END_C_DECLS

#endif /* !CXC_MAP_H */
