/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 							Includes
 *--------------------------------------------------------------------------*/

#if HAVE_CONFIG_H
#  include "config.h"
#endif

#include "common.h"

#include <math.h>

#include <gtk/gtk.h>

#include "cxc_data.h"
#include "cpk_gtk_util.h"
#include "cataxcorr.h"
#include "cxc_decode.h"
#include "catalib.h"
#include "cp_messages.h"
#include "coordinates.h"
#include "starlist.h"
#include "cxc_gtk_main.h"
#include "cxc_mask_set.h"

/*----------------------------------------------------------------------------
 * 							Global variables	
 *--------------------------------------------------------------------------*/

extern boolean   suspend ; /***FIXME: serve?*/
extern gdouble   sph_units ;

/*----------------------------------------------------------------------------
 * 							Function prototypes
 *--------------------------------------------------------------------------*/

static void      mask_changed       (GtkWidget *entry, Smask *MaskSet) ;
static void      mask_focus_out     (GtkWidget *entry, GdkEvent *event, 
                                     gpointer data) ;

/*----------------------------------------------------------------------------
 *							Function codes
 *--------------------------------------------------------------------------*/

void
get_mask_val(Smask *MaskSet, CxcData *data)
{
   g_pars   *gparam = data->gparam ;
   gdouble   s ;
   guint     i ;

   for (i=0; i<(guint) data->n; i++) {
      s = (data->f_set->info[i]->spheric) ? sph_units : 1.0 ;
      sprintf(MaskSet->set[i].value[0], "%g", data->list[i]->param->crow_rad * s) ;
      if (data->list[i]->param->m0 != -1.0e10)
         sprintf(MaskSet->set[i].value[1], "%.1f", data->list[i]->param->m0) ;
      else
         sprintf(MaskSet->set[i].value[1], "*") ;
      if (data->list[i]->param->m1 != 1.0e10) 
         sprintf(MaskSet->set[i].value[2], "%.1f", data->list[i]->param->m1) ;
      else
         sprintf(MaskSet->set[i].value[2], "*") ;
      sprintf(MaskSet->set[i].value[3], "%s", data->list[i]->param->mask) ;
   }
   strcpy(MaskSet->value, "") ;
   sprintf(MaskSet->value, "%g", gparam->NullValues[0]) ;
   for (i=1; i<(guint) gparam->N_NullVal; i++) 
      sprintf(MaskSet->value, "%s, %g", MaskSet->value, gparam->NullValues[i]) ;
}

void
set_mask_val(Smask *MaskSet, CxcData *data)
{
   gdouble   s ;
   gint      i ;
   gchar    *text, buffer[128] ;
   char    **tmp = (char**)NULL ;
   int       n = 0 ;

   for (i=0; i<data->f_set->n; i++) {
      s = (data->f_set->info[i]->spheric) ? sph_units : 1.0 ;
      text = gtk_entry_get_text(GTK_ENTRY (MaskSet->set[i].entries[0])) ;
      data->list[i]->param->crow_rad = atof(text) / s ;
      text = gtk_entry_get_text(GTK_ENTRY (MaskSet->set[i].entries[1])) ;
      strip_blanks(text) ;
      if (strcmp(text,"*") && strcmp(text,""))
         data->list[i]->param->m0 = atof(text) ;
      else
         data->list[i]->param->m0 = -1.0e10 ;
      text = gtk_entry_get_text(GTK_ENTRY (MaskSet->set[i].entries[2])) ;
      strip_blanks(text) ;
      if (strcmp(text,"*") && strcmp(text,""))
         data->list[i]->param->m1 = atof(text) ;
      else
         data->list[i]->param->m1 = 1.0e10 ;
      text = gtk_entry_get_text(GTK_ENTRY (MaskSet->set[i].entries[3])) ;
      if (strcmp(text, data->list[i]->param->mask)) {
         strcpy(data->list[i]->param->mask, text) ;
         if (strcmp(text, "")) {
            sprintf(buffer, "-mask%d", i+1) ;
            if (mask_decode_list(buffer, text)) {
               strcpy(data->list[i]->param->mask, "") ;
               data->list[i]->param->mask_shape = SHAPE ;
               data->list[i]->param->mask_p[0] = data->list[i]->param->mask_p[1] = 
               data->list[i]->param->mask_p[2] = data->list[i]->param->mask_p[3] = 0.0 ;
            }
         }
         else {
            data->list[i]->param->mask_shape = SHAPE ;
            data->list[i]->param->mask_p[0] = data->list[i]->param->mask_p[1] = 
            data->list[i]->param->mask_p[2] = data->list[i]->param->mask_p[3] = 0.0 ;
         }
      }
   }
   text = gtk_entry_get_text(GTK_ENTRY (MaskSet->entry)) ;
   tmp = tokenize(text,",",&n) ;
   if (n >= 5) 
      cp_warning("Cannot specify more than 5 Null Values") ;
   data->gparam->N_NullVal = MIN(5,n) ;
   for (i=0; i<5; i++)
      if (i < n) 
         data->gparam->NullValues[i] = atof(tmp[i]) ;
      else
         data->gparam->NullValues[i] = 0.0 ;
   (void)free_carray(tmp, n) ;
}

static void
mask_changed(GtkWidget *entry, Smask *MaskSet)
{
   gchar   *text ;

   if (entry==NULL) 
      return ;
   if (!GTK_IS_ENTRY (entry)) 
      return ;

   text = gtk_entry_get_text(GTK_ENTRY (entry)) ;
   if (text==NULL) 
      return ;
   strip_blanks(text)  ;
   gtk_entry_set_text(GTK_ENTRY (entry), text) ;
}

static void
mask_focus_out(GtkWidget *entry, GdkEvent *event, gpointer data)
{
   mask_changed(entry, data) ;
}

GtkWidget *
mask_settings(Smask *MaskSet, cxc_display *display)
{
   GtkWidget *box, *frame, *vbox, *scrolled_window, *viewport, *hbox, *table ;
   GtkWidget *bbox, *label, *entry ;
   FileSet   *f_set = CXC_DATA (display->data)->f_set ;
   guint      i ;

   /* Star Mask Settings */

   box = gtk_vbox_new(FALSE, 0) ;

   set_font ("-*-times-medium-r-normal--18-140-*-*-*-*-*-*") ;
   frame = gtk_frame_new("Star Mask settings") ;
   gtk_container_set_border_width (GTK_CONTAINER(frame), 8) ;
   gtk_container_add(GTK_CONTAINER(box), frame) ;
   vbox = gtk_vbox_new(FALSE, 3) ;
   gtk_container_set_border_width (GTK_CONTAINER(vbox), 5) ;
   gtk_container_add (GTK_CONTAINER (frame), vbox) ;
   set_font ("-*-times-medium-r-normal--17-140-*-*-*-*-*-*") ;

   scrolled_window = gtk_scrolled_window_new(NULL, NULL) ;
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (scrolled_window),
         GTK_POLICY_NEVER, GTK_POLICY_ALWAYS) ;
   viewport = gtk_viewport_new (
         gtk_scrolled_window_get_hadjustment (GTK_SCROLLED_WINDOW (scrolled_window)),
         gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (scrolled_window))) ;
   gtk_viewport_set_shadow_type (GTK_VIEWPORT (viewport), GTK_SHADOW_ETCHED_OUT) ;
   gtk_widget_set_usize(viewport, 0, 65) ;
   gtk_container_add (GTK_CONTAINER (scrolled_window), viewport) ;
   gtk_container_add(GTK_CONTAINER(vbox), scrolled_window) ;

   hbox = gtk_hbox_new(FALSE, 0) ;
   gtk_scrolled_window_add_with_viewport (
         GTK_SCROLLED_WINDOW (scrolled_window), hbox) ;
   table = gtk_table_new ((guint)f_set->n, 4, FALSE) ;
   gtk_table_set_col_spacings (GTK_TABLE (table), 15);
   gtk_table_set_row_spacings (GTK_TABLE (table), 5);
   gtk_box_pack_start (GTK_BOX (hbox), table, TRUE, TRUE, 0);

   for (i=0; i<(guint)f_set->n; i++) {
      bbox = gtk_hbox_new(FALSE, 2) ;
      label = gtk_label_new("File:") ;
      gtk_box_pack_start(GTK_BOX(bbox), label, TRUE, TRUE, 0) ;
      entry = gtk_entry_new() ;
      gtk_entry_set_text(GTK_ENTRY (entry), f_set->info[i]->filename) ;
      gtk_entry_set_editable(GTK_ENTRY (entry), FALSE);
      gtk_widget_set_usize(entry, 80, 0) ;
      gtk_box_pack_start(GTK_BOX(bbox), entry, FALSE, FALSE, 0) ;
      gtk_table_attach (GTK_TABLE (table), bbox, 0, 1, i, i+1,
            GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
      
      bbox = gtk_hbox_new(FALSE, 2) ;
      label = gtk_label_new("Crow Rad:") ;
      gtk_box_pack_start(GTK_BOX(bbox), label, TRUE, TRUE, 0) ;
      MaskSet->set[i].entries[0] = entry = gtk_entry_new() ;
      gtk_entry_set_text(GTK_ENTRY (entry), MaskSet->set[i].value[0]) ;
      gtk_signal_connect(GTK_OBJECT(entry), "activate",
            GTK_SIGNAL_FUNC(entry_changed), NULL) ;
      gtk_signal_connect(GTK_OBJECT(entry), "focus-out-event",
            GTK_SIGNAL_FUNC(entry_focus_out), NULL) ;
      gtk_widget_set_usize(entry, 60, 0) ;
      gtk_box_pack_start(GTK_BOX(bbox), entry, FALSE, FALSE, 0) ;
      gtk_table_attach (GTK_TABLE (table), bbox, 1, 2, i, i+1,
            GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);

      bbox = gtk_hbox_new(FALSE, 2) ;
      label = gtk_label_new("Mag Range:") ;
      gtk_box_pack_start(GTK_BOX(bbox), label, TRUE, TRUE, 0) ;
      MaskSet->set[i].entries[1] = entry = gtk_entry_new() ;
      gtk_entry_set_text(GTK_ENTRY (entry), MaskSet->set[i].value[1]) ;
      gtk_widget_set_usize(entry, 50, 0) ;
      gtk_box_pack_start(GTK_BOX(bbox), entry, FALSE, FALSE, 0) ;
      MaskSet->set[i].entries[2] = entry = gtk_entry_new() ;
      gtk_entry_set_text(GTK_ENTRY (entry), MaskSet->set[i].value[2]) ;
      gtk_widget_set_usize(entry, 50, 0) ;
      gtk_box_pack_start(GTK_BOX(bbox), entry, FALSE, FALSE, 0) ;

      gtk_table_attach (GTK_TABLE (table), bbox, 2, 3, i, i+1,
            GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);

      bbox = gtk_hbox_new(FALSE, 2) ;
      label = gtk_label_new("Mask:") ;
      gtk_box_pack_start(GTK_BOX(bbox), label, FALSE, FALSE, 0) ;
      MaskSet->set[i].entries[3] = entry = gtk_entry_new() ;
      gtk_entry_set_text(GTK_ENTRY (entry), MaskSet->set[i].value[3]) ;
      gtk_signal_connect(GTK_OBJECT(entry), "activate",
            GTK_SIGNAL_FUNC(mask_changed), (gpointer) MaskSet) ;
      gtk_signal_connect(GTK_OBJECT(entry), "focus-out-event",
            GTK_SIGNAL_FUNC(mask_focus_out), (gpointer) MaskSet) ;
      gtk_widget_set_usize(entry, 120, 0) ;
      gtk_box_pack_start(GTK_BOX(bbox), entry, TRUE, TRUE, 0) ;

      gtk_table_attach (GTK_TABLE (table), bbox, 3, 4, i, i+1,
            GTK_EXPAND | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
   }
   hbox = gtk_hbox_new(FALSE, 0) ;
   gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0) ;
   label = gtk_label_new("Magnitude Null Values") ;
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5) ;
   MaskSet->entry = entry = gtk_entry_new() ;
   gtk_entry_set_text(GTK_ENTRY (entry), MaskSet->value) ;
   gtk_signal_connect(GTK_OBJECT(entry), "activate",
         GTK_SIGNAL_FUNC(mask_changed), (gpointer) MaskSet) ;
   gtk_signal_connect(GTK_OBJECT(entry), "focus-out-event",
         GTK_SIGNAL_FUNC(mask_focus_out), (gpointer) MaskSet) ;
   gtk_widget_set_usize(entry, 200, 0) ;
   gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 0) ;
   gtk_widget_show_all (box) ;

   return box ;
}

/*----------------------------------------------------------------------------
 * 							Private Types
 *--------------------------------------------------------------------------*/
#ifdef cippa
typedef struct tmp_tag {
   GtkWidget   *main_window ;
   GtkWidget   *dialog ;
   Sfind       *FindSet ;
   cxc_display *display ;
} Fparam ;

static void
match_frames_accept(GtkWidget *widget, Fparam *FindParam)
{
   CxcData   *data = CXC_DATA (FindParam->display->data) ;

   set_find_val(FindParam->FindSet, data) ;
   gtk_widget_destroy(FindParam->dialog) ;
   /* Connect signals for CxcData struct */
   /* ??? */
   /* update the GUI */
   while (gtk_events_pending ())
      gtk_main_iteration () ;
   /* Turn off 'suspend' locker */
   suspend = FALSE ;
   printf("**** match_frames_accept\n");
   cxc_data_model_align(widget, data) ;
   printf("**** match_frames_accept end\n");
}

static void
cancel(GtkWidget *widget, Sfind *FindSet)
{
   free (FindSet->fset) ;
}

static void
close_window(GtkWidget *window, gpointer data)
{
   cxc_display *display = data ;
   display->match_frames_dialog = NULL ;
}

static void
get_buttons(GtkWidget *dialog, Fparam *FindParam)
{
   GtkWidget *button ;
   button = gtk_button_new_with_label ("OK") ;
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
         GTK_SIGNAL_FUNC(match_frames_accept), FindParam) ;
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
         button, TRUE, TRUE, 0) ;
   GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT) ;
   gtk_widget_show(button) ;

   button = gtk_button_new_with_label ("Cancel") ;
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
         GTK_SIGNAL_FUNC(gtk_widget_destroy), (gpointer) dialog) ;
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
         button, TRUE, TRUE, 0) ;
   GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT) ;
   gtk_widget_grab_default(button) ;
   gtk_widget_show (button) ;

   button = gtk_button_new_with_label ("Help") ;
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
         button, TRUE, TRUE, 0) ;
   GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT) ;
   gtk_widget_show (button) ;
}


#endif
