/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 							Includes
 *--------------------------------------------------------------------------*/

#if HAVE_CONFIG_H
#  include "config.h"
#endif

#include "common.h"

#include <math.h>

#include <gtk/gtk.h>

#include "catalib.h"
#include "platemodel.h"
#include "cxc_data.h"
#include "cpk_gtk_util.h"
#include "cxc_gtk_main.h"
#include "cxc_match_set.h"

/*----------------------------------------------------------------------------
 * 							Global variables	
 *--------------------------------------------------------------------------*/

extern boolean   suspend ; /***FIXME: serve?*/
extern gdouble   tunits ;

guint       nfiles ;
GtkWidget **ck = NULL ;

/*----------------------------------------------------------------------------
 * 							Private types
 *--------------------------------------------------------------------------*/

typedef struct tmp_tag {
   GtkWidget   *main_window ;
   GtkWidget   *dialog ;
   Smatchs     *MatchsSet ;
   gboolean    *active ;
   cxc_display *display ;
} Mparam ;

/*----------------------------------------------------------------------------
 * 							Function prototypes
 *--------------------------------------------------------------------------*/

static void       matchs_accept       (GtkWidget   *widget,
                                       Mparam      *MatchsParam) ;
static void       close_window1       (GtkWidget   *window,
                                       gpointer     data) ;
static void       get_buttons         (GtkWidget   *dialog,
                                       Mparam      *MatchsParam) ;
static void       get_matchs_val      (Smatchs     *MatchsSet,
                                       CxcData     *data) ;
static int        set_matchs_val      (Smatchs     *MatchsSet,
                                       CxcData     *data) ;
static void       set_entry_active    (GtkWidget   *check,
                                       Smatchs     *MatchsSet) ;

/*----------------------------------------------------------------------------
 *							Function codes
 *--------------------------------------------------------------------------*/

GtkWidget *
matchs_settings (Smatchs *MatchsSet, cxc_display *display)
{
   GtkWidget   *box, *vbox, *bbox, *table, *frame, *label, *spin, *check ;
   GtkWidget   *entry, *units ;
   GtkObject   *adj ;
   GtkTooltips *ttip ;

   box = gtk_vbox_new (FALSE, 0) ;

   /* Model Settings */

   ttip = gtk_tooltips_new () ;
   set_font ("-*-times-medium-r-normal--18-140-*-*-*-*-*-*") ;
   frame = gtk_frame_new ("Match Stars settings") ;
   gtk_container_set_border_width (GTK_CONTAINER (frame), 8) ;
   gtk_container_add (GTK_CONTAINER (box), frame) ;
   set_font ("-*-times-medium-r-normal--17-140-*-*-*-*-*-*") ;

   vbox = gtk_vbox_new (FALSE, 0) ;
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 5) ;
   gtk_container_add (GTK_CONTAINER (frame), vbox) ;

   table = gtk_table_new (3, 3, FALSE);
   gtk_table_set_col_spacings (GTK_TABLE (table), 7);
   gtk_table_set_row_spacings (GTK_TABLE (table), 2);
   gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 5);

   bbox = gtk_hbox_new (FALSE, 0) ;   
   label = gtk_label_new ("Tol Radius:") ;
   gtk_box_pack_end (GTK_BOX (bbox), label, FALSE, FALSE, 0) ;
   gtk_table_attach (GTK_TABLE (table), bbox, 0, 1, 0, 1,
            GTK_FILL, GTK_FILL, 0, 0) ;

   /* MatchsSet->uval[0] = cpk_entry_with_units (tol, spheric) ; */
   gtk_tooltips_set_tip (GTK_TOOLTIPS (ttip), MatchsSet->uval[0].entry, 
         "Minimum tolerance radius", NULL) ;
   gtk_entry_set_max_length (GTK_ENTRY (MatchsSet->uval[0].entry), 254) ;
   gtk_widget_set_usize (MatchsSet->uval[0].entry, 100, 0) ;
   gtk_table_attach (GTK_TABLE (table), MatchsSet->uval[0].entry, 1, 2, 0, 1,
            GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0) ;
   gtk_table_attach (GTK_TABLE (table), MatchsSet->uval[0].units, 2, 3, 0, 1, 
         GTK_FILL, GTK_FILL, 0, 0) ;

   bbox = gtk_hbox_new (FALSE, 0) ;   
   label = gtk_label_new ("Mix Radius:") ;
   gtk_box_pack_end (GTK_BOX (bbox), label, FALSE, FALSE, 0) ;
   gtk_table_attach (GTK_TABLE (table), bbox, 0, 1, 1, 2,
            GTK_FILL, GTK_FILL, 0, 0) ;

   /* MatchsSet->uval[1] = cpk_entry_with_units (tol, spheric) ; */
   gtk_tooltips_set_tip (GTK_TOOLTIPS (ttip), MatchsSet->uval[1].entry, 
         "Merging radius", NULL) ;
   gtk_entry_set_max_length (GTK_ENTRY (MatchsSet->uval[1].entry), 254) ;
   gtk_widget_set_usize (MatchsSet->uval[1].entry, 100, 0) ;
   gtk_table_attach (GTK_TABLE (table), MatchsSet->uval[1].entry, 1, 2, 1, 2,
            GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0) ;
   gtk_table_attach (GTK_TABLE (table), MatchsSet->uval[1].units, 2, 3, 1, 2, 
         GTK_FILL, GTK_FILL, 0, 0) ;

   bbox = gtk_hbox_new (FALSE, 0) ;   
   label = gtk_label_new ("Tol Mag:") ;
   gtk_box_pack_end (GTK_BOX (bbox), label, FALSE, FALSE, 0) ;
   gtk_table_attach (GTK_TABLE (table), bbox, 0, 1, 2, 3,
            GTK_FILL, GTK_FILL, 0, 0) ;
   MatchsSet->entry = entry = gtk_entry_new () ;
   gtk_tooltips_set_tip (GTK_TOOLTIPS (ttip), entry, 
         "Magnitude Tolerance", NULL) ;
   gtk_entry_set_max_length (GTK_ENTRY (entry), 254) ;
   gtk_entry_set_text (GTK_ENTRY (entry), MatchsSet->value) ;
   gtk_signal_connect (GTK_OBJECT (entry), "activate",
         GTK_SIGNAL_FUNC (entry_changed), MatchsSet->value) ;
   gtk_signal_connect (GTK_OBJECT (entry), "focus-out-event",
         GTK_SIGNAL_FUNC (entry_focus_out), MatchsSet->value) ;
   gtk_widget_set_usize (entry, 100, 0) ;
   gtk_editable_set_editable (GTK_EDITABLE (entry),MatchsSet->active) ;
   /* gtk_box_pack_start (GTK_BOX (bbox), entry, FALSE, FALSE, 0) ; */
   gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 2, 3,
            GTK_FILL, GTK_FILL, 0, 0) ;

   check = gtk_check_button_new_with_label ("Active") ;
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), 
         (gboolean)MatchsSet->active) ;
   gtk_signal_connect (GTK_OBJECT (check), "toggled",
         GTK_SIGNAL_FUNC (set_entry_active), MatchsSet) ;
   gtk_table_attach (GTK_TABLE (table), check, 2, 3, 2, 3,
            GTK_FILL, GTK_FILL, 0, 0);

   bbox = gtk_hbox_new (FALSE, 5) ;
   label = gtk_label_new ("SR Factor:") ;
   gtk_box_pack_start (GTK_BOX (bbox), label, TRUE, TRUE, 0) ;
   adj = gtk_adjustment_new (MatchsSet->values[0], 0.1, 10., .1, .5, 1.0) ;
   MatchsSet->spin[0] = spin = gtk_spin_button_new (GTK_ADJUSTMENT (adj), .1, 1) ; 
   gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spin), TRUE) ;
   gtk_tooltips_set_tip (GTK_TOOLTIPS (ttip), spin, 
         "Sigma Rejection clipping factor", NULL) ;
   gtk_box_pack_start (GTK_BOX (bbox), spin, FALSE, FALSE, 0) ;

   label = gtk_label_new ("SR Iterations:") ;
   gtk_box_pack_start (GTK_BOX (bbox), label, TRUE, TRUE, 0) ;
   adj = gtk_adjustment_new (MatchsSet->values[1], 0.0, 100.0, 1.0, 5.0, 1.0) ;
   MatchsSet->spin[1] = spin = gtk_spin_button_new (GTK_ADJUSTMENT (adj), 1.0, 0) ; 
   gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spin), TRUE) ;
   /* gtk_spin_button_set_digits (GTK_SPIN_BUTTON (spin), 0) ; */
   gtk_tooltips_set_tip (GTK_TOOLTIPS (ttip), spin, 
         "Sigma Rejection iterations", NULL) ;
   gtk_box_pack_start (GTK_BOX (bbox), spin, FALSE, FALSE, 0) ;

   gtk_box_pack_start (GTK_BOX (vbox), bbox, FALSE, FALSE, 5);

   gtk_widget_show_all (box) ;

   return box ;
}

static void
set_entry_active (GtkWidget *check, Smatchs *MatchsSet)
{
   MatchsSet->active = gtk_toggle_button_get_active (GTK_CHECK_BUTTON (check)) ;
   gtk_editable_set_editable (GTK_EDITABLE (MatchsSet->entry), 
         MatchsSet->active) ;
   if (!MatchsSet->active) {
      MatchsSet->tol_mag = atof (MatchsSet->value) ;
      sprintf(MatchsSet->value, "*") ;
   }
   else 
      sprintf(MatchsSet->value, "%g", MatchsSet->tol_mag) ;
   gtk_entry_set_text (GTK_ENTRY (MatchsSet->entry), MatchsSet->value) ;
}

static void
matchs_accept (GtkWidget *widget, Mparam *MatchsParam)
{
   CxcData   *data = CXC_DATA (MatchsParam->display->data) ;
   gint       i, n;

   if (set_matchs_val (MatchsParam->MatchsSet, data))
      return ;
   data->gparam->min_tol = data->gparam->min_tol3 ;

   for (n = i = 0; i < data->f_set->n; i++)
      if ((data->active[i] = MatchsParam->active[i]))
         n++ ;

   gtk_widget_destroy (MatchsParam->dialog) ;
   if (ck) free (ck) ;

   if (n == 0)
      return ;

   if (cxc_data_stack_init (widget, data))
      return ;

   /* Set frame mode to FRAME_COINC */
   gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM 
         (MatchsParam->display->frame_radio[FRAME_COINC]), TRUE) ;
}

static void
get_matchs_val (Smatchs *MatchsSet, CxcData *data)
{
   gboolean spheric = data->f_set->info[0]->spheric ;
   gdouble  tol ;
   gdouble  dev = 0.0 ;
   gint     i ;

   for (i = 1; i < data->f_set->n; i++) 
      if (bin_check_n (data->stack->nf, i) || 
          bin_check_n (data->stack->nf2, i-32))
         dev = MAX (dev, MAX (data->model[i]->dxdev, data->model[i]->dydev)) ;

   tol = MIN (6.0, data->gparam->sr_3+2) * dev ;
   tol = MAX (data->gparam->min_tol3, tol) ;
   cpk_entry_with_units (&MatchsSet->uval[0], tol, spheric) ;
   cpk_entry_with_units (&MatchsSet->uval[1], data->gparam->m3_rad, spheric) ;

   sprintf (MatchsSet->value, "*") ;
   MatchsSet->active = FALSE ;
   MatchsSet->tol_mag = 0.0 ;

   MatchsSet->values[0] = data->gparam->sr_3 ;
   MatchsSet->values[1] = 5 ;
}

static int
set_matchs_val (Smatchs *MatchsSet, CxcData *data)
{
   gdouble  tol_mag ;

   if (MatchsSet->active) {
      tol_mag = atof (MatchsSet->value) ;
      if (tol_mag <= 0.0) {
         (void) cpk_warning (MatchsSet->window, 
               "Set a valid 'Tol Mag' or switch off the 'Active' button!") ;
         return 1 ;
      }
   }
   else
      tol_mag = 100000 ;

   data->gparam->t3_mag = tol_mag ;
   data->gparam->min_tol3 = MatchsSet->uval[0].value ;
   data->gparam->m3_rad = MatchsSet->uval[1].value ;
   data->list2->param->m3_rad = MatchsSet->uval[1].value ; /* useless */
   data->gparam->sr_3 = gtk_spin_button_get_value_as_float (
         GTK_SPIN_BUTTON (MatchsSet->spin[0])) ;
   data->gparam->nsr_3 = gtk_spin_button_get_value_as_int (
         GTK_SPIN_BUTTON (MatchsSet->spin[1])) ;

   return 0 ;
}

static void
set_active_list (gboolean *active)
{
   *active = !*active ;
}

static void
set_active_all (GtkWidget **ck)
{
   gint i ;
   for (i = 0; i < nfiles; i++)
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ck[i]), TRUE) ;
}

static void
set_active_none (GtkWidget **ck)
{
   gint i ;
   for (i = 0; i < nfiles; i++)
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ck[i]), FALSE) ;
}

GtkWidget *
list_settings (gboolean *active, cxc_display *display)
{
   CxcData     *data = CXC_DATA (display->data) ;
   FileSet     *f_set = CXC_DATA (display->data)->f_set ;
   GtkWidget   *box, *vbox, *hbox, *table, *frame, *label, *check, *button ;
   GtkWidget   *viewport, *scrolled_window  ;
   GtkTooltips *ttip ;
   guint        i ;

   nfiles = (guint)f_set->n ;
   ck = (GtkWidget**) malloc (nfiles * sizeof(GtkWidget*)) ;

   box = gtk_vbox_new (FALSE, 0) ;

   /* Files Settings */

   set_font ("-*-times-medium-r-normal--18-140-*-*-*-*-*-*") ;
   frame = gtk_frame_new ("Star Lists Stack") ;
   gtk_container_set_border_width (GTK_CONTAINER(frame), 8) ;
   gtk_container_add (GTK_CONTAINER (box), frame) ;
   set_font ("-*-times-medium-r-normal--17-140-*-*-*-*-*-*") ;

   vbox = gtk_vbox_new (FALSE, 5) ;
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 5) ;
   gtk_container_add (GTK_CONTAINER (frame), vbox) ;

   scrolled_window = gtk_scrolled_window_new (NULL, NULL) ;
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
         GTK_POLICY_NEVER, GTK_POLICY_ALWAYS) ;
   viewport = gtk_viewport_new (
         gtk_scrolled_window_get_hadjustment (GTK_SCROLLED_WINDOW (scrolled_window)),
         gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (scrolled_window))) ;
   gtk_viewport_set_shadow_type (GTK_VIEWPORT (viewport), GTK_SHADOW_NONE) ;
   gtk_widget_set_usize (viewport, 0, 90) ;
   gtk_container_add (GTK_CONTAINER (scrolled_window), viewport) ;
   gtk_container_add (GTK_CONTAINER (vbox), scrolled_window) ;

   table = gtk_table_new (nfiles+1, 3, FALSE) ;
   gtk_table_set_col_spacings (GTK_TABLE (table), 1) ;
   gtk_table_set_row_spacings (GTK_TABLE (table), 1) ;
   gtk_scrolled_window_add_with_viewport (
         GTK_SCROLLED_WINDOW (scrolled_window), table) ;   

   label = gtk_label_new ("File:") ;
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
            GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0) ;
   label = gtk_label_new ("Include:") ;
   gtk_table_attach (GTK_TABLE (table), label, 1, 2, 0, 1,
            GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0) ;
   label = gtk_label_new ("WCS Status:") ;
   gtk_table_attach (GTK_TABLE (table), label, 2, 3, 0, 1,
            GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0) ;

   for (i = 0; i < nfiles; i++) {
      frame = gtk_frame_new (NULL) ;
      gtk_widget_set_usize (frame, 90, 0) ;
      hbox = gtk_hbox_new (FALSE, 0) ;
      gtk_container_add (GTK_CONTAINER (frame), hbox) ;
      label = gtk_label_new (f_set->info[i]->filename) ;
      gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2) ;
      gtk_table_attach (GTK_TABLE (table), frame, 0, 1, i+1, i+2,
            GTK_FILL, GTK_FILL, 0, 0) ;

      frame = gtk_frame_new (NULL) ;
      hbox = gtk_hbox_new (FALSE, 0) ;
      gtk_container_add (GTK_CONTAINER (frame), hbox) ;

      ck[i] = check = gtk_check_button_new () ;
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), active[i]) ;
      gtk_signal_connect_object (GTK_OBJECT (check), "toggled", 
            GTK_SIGNAL_FUNC (set_active_list), &active[i]) ;
      gtk_box_pack_start (GTK_BOX (hbox), check, TRUE, FALSE, 0) ;
      gtk_table_attach (GTK_TABLE (table), frame, 1, 2, i+1, i+2, 
            GTK_FILL, GTK_FILL, 0, 0) ;

      frame = gtk_frame_new (NULL) ;
      hbox = gtk_hbox_new (FALSE, 0) ;
      gtk_container_add (GTK_CONTAINER (frame), hbox) ;
      if (i == data->nref)
         label = gtk_label_new ("Reference") ;
      else {
         switch (data->model[i]->status) {
            case LOW_ALIGN :
               label = gtk_label_new ("Aligned") ;
               break ;
            case HIGH_ALIGN :
               label = gtk_label_new ("Registered") ;
               break ;
            case MODEL_FITTED :
               label = gtk_label_new ("Modeled") ;
               break ;
            default :
                  label = gtk_label_new (NULL) ;
         }
      }
      gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0) ;
      gtk_table_attach (GTK_TABLE (table), frame, 2, 3, i+1, i+2,
            GTK_FILL, GTK_FILL, 0, 0) ;
   }
   hbox = gtk_hbox_new (FALSE, 0) ;
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0) ;
   button = gtk_button_new_with_label ("Select All") ;
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked", 
         GTK_SIGNAL_FUNC (set_active_all), ck) ;
   gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0) ;
   button = gtk_button_new_with_label ("Deselect All") ;
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked", 
         GTK_SIGNAL_FUNC (set_active_none), ck) ;
   gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0) ;

   gtk_widget_show_all (box) ;

   return box ;
}

static void
close_window1 (GtkWidget *window, gpointer data)
{
   cxc_display *display = data ;
   display->match_stars_dialog = NULL ;
}

static void
get_buttons (GtkWidget *dialog, Mparam *MatchsParam)
{
   GtkWidget *button ;
   button = gtk_button_new_with_label ("OK") ;
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
         GTK_SIGNAL_FUNC (matchs_accept), MatchsParam) ;
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
         button, TRUE, TRUE, 0) ;
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT) ;
   gtk_widget_show (button) ;

   button = gtk_button_new_with_label ("Cancel") ;
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
         GTK_SIGNAL_FUNC(gtk_widget_destroy), (gpointer) dialog) ;
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
         button, TRUE, TRUE, 0) ;
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT) ;
   gtk_widget_grab_default (button) ;
   gtk_widget_show (button) ;

   button = gtk_button_new_with_label ("Help") ;
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
         button, TRUE, TRUE, 0) ;
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT) ;
   gtk_widget_show (button) ;
}

void
match_stars (GtkWidget *widget, cxc_display *display)
{
   GtkWidget *dialog, *box, *vbox ;
   CxcData   *data = CXC_DATA (display->data) ;
   size_t     nfiles, i ;
   static Smatchs MatchsSet ;
   static Mparam  MatchsParam ;

   if ( display->match_stars_dialog ) {
      /* Window is already opened */
      gtk_window_set_transient_for (GTK_WINDOW (display->match_stars_dialog), 
            GTK_WINDOW (display->window)) ;
      return ;
   }

   /* Create a new window */
   display->match_stars_dialog = dialog = gtk_dialog_new () ;

   /* Initialize Salign struct */
   get_matchs_val (&MatchsSet, CXC_DATA (display->data)) ;

   MatchsParam.main_window = display->window ;
   MatchsParam.dialog = dialog ;
   MatchsParam.MatchsSet = &MatchsSet ;
   MatchsSet.window = dialog ;
   MatchsParam.display = display ;
   nfiles = data->f_set->n ;
   MatchsParam.active = (gboolean*) malloc (nfiles * sizeof(gboolean)) ;

   for (i = 0; i < nfiles; i++) 
      if (bin_check_n (data->stack->nf, (int)i) ||
          bin_check_n (data->stack->nf2, (int)i-32))
         MatchsParam.active[i] = TRUE ;
      else
         MatchsParam.active[i] = FALSE ;

   gtk_window_set_title (GTK_WINDOW (dialog), "CataXcorr: Match Stars") ;
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE) ;
   gtk_signal_connect (GTK_OBJECT (dialog), "destroy", 
                       GTK_SIGNAL_FUNC (close_window1), (gpointer)display) ;

   /* Main container */
   box = gtk_vbox_new (FALSE, 0) ;
   gtk_container_set_border_width (GTK_CONTAINER (box), 8) ;
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), box,
                      TRUE, TRUE, 0) ;

   /* Find Settings */
   vbox = matchs_settings (&MatchsSet, display) ;
   gtk_box_pack_start (GTK_BOX (box), vbox, FALSE, FALSE, 0) ;

   vbox = list_settings (MatchsParam.active, display) ;
   gtk_box_pack_start (GTK_BOX (box), vbox, TRUE, TRUE, 0) ;

   gtk_widget_show_all (box) ;

   /* buttons */
   get_buttons (dialog, &MatchsParam) ;

   gtk_widget_show_all (dialog) ;
}
