/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 							Includes
 *--------------------------------------------------------------------------*/

#if HAVE_CONFIG_H
#  include "config.h"
#endif

#include "common.h"

#include <math.h>

#include <gtk/gtk.h>

#include "platemodel.h"
#include "cxc_data.h"
#include "cpk_gtk_util.h"
#include "cxc_gtk_main.h"
#include "cxc_frames.h"
#include "cxc_match_set.h"
#include "cxc_register_set.h"

/*----------------------------------------------------------------------------
 * 							Global variables	
 *--------------------------------------------------------------------------*/

extern boolean   suspend ; /***FIXME: serve?*/
extern gdouble   tunits ;

/*----------------------------------------------------------------------------
 * 							Function prototypes
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 *							Function codes
 *--------------------------------------------------------------------------*/

static void
get_matchs_val (Smatchs *MatchsSet, CxcData *data)
{
   gboolean spheric = data->f_set->info[0]->spheric ;
   gdouble  tol ;
   gdouble  dev = 0.0 ;

   dev = MAX (data->pmodel->dxdev, data->pmodel->dydev) ;
   tol = MIN (10.0, data->gparam->sr_1+2) * dev ;
   tol = MAX (data->gparam->min_tol1, tol) ;
   cpk_entry_with_units (&MatchsSet->uval[0], tol, spheric) ;
   /*cpk_entry_with_units (&MatchsSet->uval[1], data->gparam->m1_rad, spheric) ;*/
   /*
    * modified 22/07/13
    * original:
    * cpk_entry_with_units (&MatchsSet->uval[1], data->list2->param->m1_rad, spheric) ;
    * problem: the mix_radius m1_rad must be taken from the default value of reference frame
    * and not that of the rotating list
    */
   cpk_entry_with_units (&MatchsSet->uval[1], data->list1->param->m1_rad, spheric) ;

   if (data->gparam->t1_mag != 0.0 && data->gparam->t1_mag < 500.0) {
      sprintf (MatchsSet->value, "%g", data->gparam->t1_mag) ;
      MatchsSet->active = TRUE ;
   }
   else {
      data->gparam->t1_mag = 0.0 ;
      sprintf (MatchsSet->value, "*") ;
      MatchsSet->active = FALSE ;
   }
   MatchsSet->tol_mag = data->gparam->t1_mag ;

   MatchsSet->values[0] =  data->gparam->sr_1 ;
   MatchsSet->values[1] =  data->gparam->nsr_1 ;
}

void
get_register_val (Salign *AlignSet, CxcData *data)
{
   AlignSet->values[0] = (gdouble)data->gparam->max_iter ;
   AlignSet->values[1] = (gdouble)data->gparam->Imax_ord ;
}

static int
set_matchs_val (Smatchs *MatchsSet, CxcData *data)
{
   gdouble  tol_mag ;

   if (MatchsSet->active) {
      tol_mag = atof (MatchsSet->value) ;
      if (tol_mag <= 0.0) {
         (void) cpk_warning (MatchsSet->window, 
               "Set a valid 'Tol Mag' or switch off the 'Active' button!") ;
         return 1 ;
      }
   }
   else
      tol_mag = 500 ;

   data->gparam->t1_mag = tol_mag ;
   data->gparam->min_tol1 = MatchsSet->uval[0].value ;
   data->gparam->m1_rad = MatchsSet->uval[1].value ; /* obsoleto */
   data->list2->param->m1_rad = MatchsSet->uval[1].value ;
   data->gparam->sr_1 = gtk_spin_button_get_value_as_float (
         GTK_SPIN_BUTTON (MatchsSet->spin[0])) ;
   data->gparam->nsr_1 = gtk_spin_button_get_value_as_int (
         GTK_SPIN_BUTTON (MatchsSet->spin[1])) ;

   return 0 ;
}

void
set_register_val (Salign *AlignSet, CxcData *data)
{
   data->gparam->max_iter = gtk_spin_button_get_value_as_int (
         GTK_SPIN_BUTTON (AlignSet->spin[0])) ;
   data->gparam->Imax_ord = gtk_spin_button_get_value_as_int (
         GTK_SPIN_BUTTON (AlignSet->spin[1])) ;
   data->gparam->fitmodel = POLY ;
}

GtkWidget *
register_settings (Salign *AlignSet, cxc_display *display)
{
   GtkWidget   *box, *vbox, *hbox, *bbox, *frame, *label, *spin ;
   GtkObject   *adj ;
   GtkTooltips *ttip ;

   box = gtk_vbox_new (FALSE, 0) ;

   /* Align Settings */

   ttip = gtk_tooltips_new () ;
   set_font ("-*-times-medium-r-normal--18-140-*-*-*-*-*-*") ;
   frame = gtk_frame_new ("Register frames") ;
   gtk_container_set_border_width (GTK_CONTAINER (frame), 8) ;
   gtk_container_add (GTK_CONTAINER (box), frame) ;
   set_font ("-*-times-medium-r-normal--17-140-*-*-*-*-*-*") ;
   vbox = gtk_vbox_new (FALSE, 0) ;
   gtk_container_add (GTK_CONTAINER (frame), vbox) ;
   hbox = gtk_hbox_new (FALSE, 0) ;
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 5) ;

   bbox = gtk_hbox_new (FALSE, 5) ;
   gtk_box_pack_start (GTK_BOX (hbox), bbox, FALSE, FALSE, 5) ;
   label = gtk_label_new ("Max N. Iterations:") ;
   gtk_box_pack_start (GTK_BOX (bbox), label, TRUE, TRUE, 0) ;
   adj = gtk_adjustment_new (AlignSet->values[0], 1., 150., 1., 5., 10.) ;
   AlignSet->spin[0] = spin = gtk_spin_button_new (
         GTK_ADJUSTMENT (adj), 1., 0) ; 
   gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spin), TRUE) ;
   gtk_tooltips_set_tip (GTK_TOOLTIPS (ttip), spin, 
         "Maximum number of iterations", NULL) ;
   gtk_box_pack_start (GTK_BOX (bbox), spin, FALSE, FALSE, 0) ;

   bbox = gtk_hbox_new (FALSE, 5) ;
   gtk_box_pack_end (GTK_BOX (hbox), bbox, FALSE, FALSE, 5) ;
   label = gtk_label_new ("Max Ord:") ;
   gtk_box_pack_start (GTK_BOX (bbox), label, TRUE, TRUE, 0) ;
   adj = gtk_adjustment_new (AlignSet->values[1], 1., 10., 1., 5., 5.) ;
   AlignSet->spin[1] = spin = gtk_spin_button_new (
         GTK_ADJUSTMENT (adj), 1, 0) ; 
   gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spin), TRUE) ;
   gtk_tooltips_set_tip (GTK_TOOLTIPS (ttip), spin, 
         "Maximum transformation order", NULL) ;
   gtk_box_pack_start (GTK_BOX (bbox), spin, FALSE, FALSE, 0) ;

   gtk_widget_show_all (box) ;

   return box ;
}

/*----------------------------------------------------------------------------
 * 							Private Types
 *--------------------------------------------------------------------------*/

typedef struct tmp_tag {
   GtkWidget   *main_window ;
   GtkWidget   *dialog ;
   Salign      *AlignSet ;
   cxc_display *display ;
} Aparam ;

static void
register_frames_accept (GtkWidget *widget, Aparam *AlignParam)
{
   CxcData   *data = CXC_DATA (AlignParam->display->data) ;

   if (set_matchs_val (&AlignParam->AlignSet->MatchsSet, data))
      return ;
   set_register_val (AlignParam->AlignSet, data) ;
   data->gparam->min_tol = data->gparam->min_tol1 ;

   data->pmodel->status = BUSY ;
   cxc_buttons_wcs_status (AlignParam->display) ;
   gtk_widget_destroy (AlignParam->dialog) ;

   /* Remember user setted reference */
   if (data->list1 == data->stack->ref)
      AlignParam->display->user_reference = AlignParam->display->nframes ;
   else
      AlignParam->display->user_reference = data->frm1 ;

   /* update the GUI */
   while (gtk_events_pending ())
      gtk_main_iteration () ;

   cxc_data_register (widget, data) ;
   if (AlignParam->display->we_are_exiting)
      quit_program (NULL, AlignParam->display) ;
}

static void
close_window (GtkWidget *window, gpointer data)
{
   cxc_display *display = data ;
   display->register_frames_dialog = NULL ;
}

static void
get_buttons (GtkWidget *dialog, Aparam *AlignParam)
{
   GtkWidget *button ;
   button = gtk_button_new_with_label ("OK") ;
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
         GTK_SIGNAL_FUNC (register_frames_accept), AlignParam) ;
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
         button, TRUE, TRUE, 0) ;
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT) ;
   gtk_widget_show (button) ;

   button = gtk_button_new_with_label ("Cancel") ;
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
         GTK_SIGNAL_FUNC (gtk_widget_destroy), (gpointer) dialog) ;
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
         button, TRUE, TRUE, 0) ;
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT) ;
   gtk_widget_grab_default (button) ;
   gtk_widget_show (button) ;

   button = gtk_button_new_with_label ("Help") ;
   gtk_box_pack_start (GTK_BOX ( GTK_DIALOG (dialog)->action_area),
         button, TRUE, TRUE, 0) ;
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT) ;
   gtk_widget_show (button) ;
}

static void
get_buttons0 (GtkWidget *dialog, Aparam *AlignParam)
{
   GtkWidget *button ;
   button = gtk_button_new_with_label ("< Back") ;
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
         GTK_SIGNAL_FUNC (register_frames_accept), AlignParam) ;
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
         button, TRUE, TRUE, 0) ;
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT) ;
   gtk_widget_show (button) ;

   button = gtk_button_new_with_label ("Forward >") ;
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
         GTK_SIGNAL_FUNC (register_frames_accept), AlignParam) ;
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
         button, TRUE, TRUE, 0) ;
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT) ;
   gtk_widget_grab_default (button) ;
   gtk_widget_show (button) ;

   button = gtk_button_new_with_label ("Cancel") ;
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
         GTK_SIGNAL_FUNC (gtk_widget_destroy), (gpointer) dialog) ;
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
         button, TRUE, TRUE, 0) ;
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT) ;
   gtk_widget_show (button) ;

   button = gtk_button_new_with_label ("Help") ;
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
         button, TRUE, TRUE, 0) ;
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT) ;
   gtk_widget_show (button) ;
}

void
register_frames (GtkWidget *widget, cxc_display *display)
{
   GtkWidget *dialog, *box, *vbox ;
   static Salign  AlignSet ;
   static Aparam  AlignParam ;

   if (display->register_frames_dialog) {
      /* Window is already opened */
      gtk_window_set_transient_for (GTK_WINDOW (display->register_frames_dialog), 
            GTK_WINDOW (display->window)) ;
      return ;
   }
   if (cxc_frame_select (display)) {
      cpk_warning (display->window, 
            "Please select a frame other than reference one.") ;
      return ;
   }

   /* Create a new window */
   display->register_frames_dialog = dialog = gtk_dialog_new () ;
   gtk_window_set_title (GTK_WINDOW (dialog), "CataXcorr: Register Frames") ;
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE) ;
   gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
         GTK_SIGNAL_FUNC (close_window), (gpointer)display) ;

   /* Initialize Salign struct */
   get_matchs_val (&AlignSet.MatchsSet, CXC_DATA (display->data)) ;
   get_register_val (&AlignSet, CXC_DATA (display->data)) ;

   AlignParam.main_window = display->window ;
   AlignParam.dialog = dialog ;
   AlignParam.AlignSet = &AlignSet ;
   AlignParam.display = display ;
   AlignSet.MatchsSet.window = dialog ;

   /* Main container */
   box = gtk_vbox_new (FALSE, 0) ;
   gtk_container_set_border_width (GTK_CONTAINER (box), 8) ;
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), box, TRUE, TRUE, 0) ;

   printf("register_frames: user_reference : %d\n",display->user_reference) ;
   cxc_set_reference (box, TRUE, 8, display->user_reference, display) ;

   /* Match Settings */
   vbox = matchs_settings (&AlignSet.MatchsSet, display) ;
   gtk_container_add (GTK_CONTAINER (box), vbox) ;

   /* Register Settings */
   vbox = register_settings (&AlignSet, display) ;
   gtk_container_add (GTK_CONTAINER (box), vbox) ;

   gtk_widget_show_all (box) ;

   /* buttons */
   get_buttons (dialog, &AlignParam) ;

   gtk_widget_show_all (dialog) ;
}
