/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 							Includes
 *--------------------------------------------------------------------------*/

#if HAVE_CONFIG_H
#  include "config.h"
#endif

#include "common.h"

#include <math.h>

#include <gtk/gtk.h>

#include "platemodel.h"
#include "cxc_data.h"
#include "cpk_gtk_util.h"
#include "cxc_gtk_main.h"
#include "cxc_save.h"

/*----------------------------------------------------------------------------
 * 							Global variables	
 *--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
 * 							Private types
 *--------------------------------------------------------------------------*/

typedef struct {
   GtkWidget   *entry[3] ;
   gboolean     active[3] ;
   GtkWidget   *window ;
} Ssave ;

typedef struct tmp_tag {
   GtkWidget   *main_window ;
   GtkWidget   *dialog ;
   Ssave       *SaveSet ;
   cxc_display *display ;
} Sparam ;

/*----------------------------------------------------------------------------
 * 							Function prototypes
 *--------------------------------------------------------------------------*/

static void         set_active_list  (gboolean     *active) ;
static GtkWidget   *save_settings    (Ssave        *SaveSet,
                                      cxc_display  *display) ;
static void         get_save_val     (Ssave        *SaveSet,
                                      CxcData      *data) ;
static int          check_save_val   (Ssave        *SaveSet,
                                      CxcData      *data) ;
static void         save_accept      (GtkWidget    *widget,
                                      Sparam       *SaveParam) ;
static void         close_window1    (GtkWidget    *window,
                                      gpointer      data) ;
static void         get_buttons      (GtkWidget    *dialog,
                                      Sparam       *SaveParam) ;

/*----------------------------------------------------------------------------
 *							Function codes
 *--------------------------------------------------------------------------*/

static void
get_save_val (Ssave *SaveSet, CxcData *data)
{
   gint i ;
   SaveSet->active[0] = SaveSet->active[1] = SaveSet->active[2] = FALSE ;

   if (data->stack) 
      SaveSet->active[0] = TRUE ;
   for (i = 1; i < data->f_set->n; i++) {
      if (data->model[i]->status >= LOW_ALIGN)
         SaveSet->active[1] = TRUE ;
      if (data->model[i]->status >= MODEL_FITTED)
         SaveSet->active[2] = TRUE ;
   }
   /* ***FIXME: rivedere per il caso piu' generale */  
   data->gparam->append = (data->list1->spheric && !data->list2->spheric) ;
}

static int
check_save_val (Ssave *SaveSet, CxcData *data)
{
   gint     i ;
   gboolean check1 = FALSE, check2 = FALSE ;
   if (SaveSet->active[0] && !data->stack) {
      (void) cpk_warning (SaveSet->window,
            "Cannot save Coinc Table:\nLists haven't yet been matched!") ;
      return 1 ;
   }
   /* ***FIXME: funziona solo se nref = 0 !!! */
   for (i = 1; i < data->f_set->n; i++) {
      if (data->model[i]->status >= LOW_ALIGN)
         check1 = TRUE ;
      if (data->model[i]->status >= MODEL_FITTED)
         check2 = TRUE ;
   }
   if (SaveSet->active[1] && !check1) {
      (void) cpk_warning (SaveSet->window,
            "No Rotated List to save!") ;
      return 1 ;
   }
   if (SaveSet->active[2] && !check2) {
      (void) cpk_warning (SaveSet->window,
            "No Plate Model to save!") ;
      return 1 ;
   }
   return 0 ;
}

static void
save_accept (GtkWidget *widget, Sparam *SaveParam)
{
   CxcData   *data = CXC_DATA (SaveParam->display->data) ;
   gint       i ;

   if (check_save_val (SaveParam->SaveSet, data))
      return ;

   if (SaveParam->SaveSet->active[0]) {
      strcpy (data->gparam->outfile,  
            gtk_entry_get_text (GTK_ENTRY (SaveParam->SaveSet->entry[0]))) ;
      cxc_data_coinc_save (widget, data) ;
   }

   if (SaveParam->SaveSet->active[1]) {
      if (strcmp(data->gparam->ext_rot, gtk_entry_get_text (
            GTK_ENTRY (SaveParam->SaveSet->entry[1])))) {
         strcpy(data->gparam->ext_rot, gtk_entry_get_text (
               GTK_ENTRY (SaveParam->SaveSet->entry[1]))) ;
         for (i=1; i<data->f_set->n; i++)
            data->list[i]->saved = FALSE ;
      }
      cxc_data_rot_save (widget, data) ;
   }

   if (SaveParam->SaveSet->active[2]) {
      if (strcmp(data->gparam->ext_plm, gtk_entry_get_text (
            GTK_ENTRY (SaveParam->SaveSet->entry[2])))) {
         strcpy(data->gparam->ext_plm, gtk_entry_get_text (
               GTK_ENTRY (SaveParam->SaveSet->entry[2]))) ;
         for (i=1; i<data->f_set->n; i++)
            data->model[i]->saved = FALSE ;
      }
      cxc_data_model_save (widget, data) ;
   }

   /* Turn off 'suspend' locker */
   /* suspend = FALSE ; */

   gtk_widget_destroy (SaveParam->dialog) ;
}

static void
set_active_list (gboolean *active)
{
   *active = !*active ;
}

static void
set_opt_false (GtkWidget *item, gboolean *opt)
{
   *opt = FALSE ;
}

static void
set_opt_true (GtkWidget *item, gboolean *opt)
{
   *opt = TRUE ;
}

static GtkWidget *
save_settings (Ssave *SaveSet, cxc_display *display)
{
   CxcData     *data = CXC_DATA (display->data) ;
   GtkWidget   *box, *vbox, *hbox, *bbox, *table, *frame ;
   GtkWidget   *entry, *button, *label, *check ;
   GtkWidget   *option, *menu, *item ;

   box = gtk_vbox_new (FALSE, 0) ;

   /* Save settings */

   set_font ("-*-times-medium-r-normal--18-140-*-*-*-*-*-*") ;
   frame = gtk_frame_new ("Save settings") ;
   gtk_container_set_border_width (GTK_CONTAINER (frame), 8) ;
   gtk_container_add (GTK_CONTAINER (box), frame) ;
   set_font ("-*-times-medium-r-normal--17-140-*-*-*-*-*-*") ;

   bbox = gtk_vbox_new (FALSE, 0) ;
   gtk_container_add (GTK_CONTAINER (frame), bbox) ;
   hbox = gtk_hbox_new (FALSE, 0) ;
   gtk_box_pack_start (GTK_BOX (bbox), hbox, FALSE, FALSE, 5) ;

   table = gtk_table_new (4, 3, FALSE);
   gtk_table_set_col_spacings (GTK_TABLE (table), 1);
   gtk_table_set_row_spacings (GTK_TABLE (table), 2);
   gtk_box_pack_start (GTK_BOX (hbox), table, TRUE, TRUE, 5);

   label = gtk_label_new ("Kind:") ;
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
            GTK_FILL, GTK_FILL, 2, 0) ;
   label = gtk_label_new ("Save:") ;
   gtk_table_attach (GTK_TABLE (table), label, 1, 2, 0, 1,
            GTK_FILL, GTK_FILL, 2, 0) ;
   label = gtk_label_new ("Name:") ;
   gtk_table_attach (GTK_TABLE (table), label, 2, 3, 0, 1,
            GTK_EXPAND | GTK_FILL, GTK_FILL, 2, 0) ;

   /* Coinc Table */
   frame = gtk_frame_new (NULL) ;
   vbox = gtk_vbox_new (FALSE, 0) ;
   hbox = gtk_hbox_new (FALSE, 0) ;
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 4) ;
   gtk_container_add (GTK_CONTAINER (frame), vbox) ;
   label = gtk_label_new ("Coinc Table") ;
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 4) ;
   gtk_table_attach (GTK_TABLE (table), frame, 0, 1, 1, 2,
         GTK_FILL, GTK_FILL, 0, 0) ;

   frame = gtk_frame_new (NULL) ;
   vbox = gtk_vbox_new (FALSE, 0) ;
   hbox = gtk_hbox_new (FALSE, 0) ;
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 4) ;
   gtk_container_add (GTK_CONTAINER (frame), vbox) ;
   check = gtk_check_button_new () ;
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), SaveSet->active[0]) ;
   gtk_signal_connect_object (GTK_OBJECT (check), "toggled", 
         GTK_SIGNAL_FUNC (set_active_list), (gpointer)&SaveSet->active[0]) ;
   gtk_box_pack_start (GTK_BOX (hbox), check, TRUE, FALSE, 0) ;
   gtk_table_attach (GTK_TABLE (table), frame, 1, 2, 1, 2, 
         GTK_FILL, GTK_FILL, 0, 0) ;

   frame = gtk_frame_new (NULL) ;
   vbox = gtk_vbox_new (FALSE, 0) ;
   gtk_container_add (GTK_CONTAINER (frame), vbox) ;
   SaveSet->entry[0] = entry = gtk_entry_new () ;
   gtk_widget_set_usize (entry, 80, 0) ;
   gtk_box_pack_start (GTK_BOX (vbox), entry, FALSE, FALSE, 0) ;
   gtk_entry_set_text (GTK_ENTRY (entry), data->gparam->outfile) ;
   gtk_editable_set_position (GTK_EDITABLE (entry), -1) ;
   hbox = gtk_hbox_new (FALSE, 0) ;
   button = gtk_button_new_with_label ("  Choose...  ") ;
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
         GTK_SIGNAL_FUNC (cpk_get_file_entry), (gpointer)entry) ;
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 2) ;
   gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 2) ;
   gtk_table_attach (GTK_TABLE (table), frame, 2, 3, 1, 2, 
         GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0) ;

   /* Rotated */
   frame = gtk_frame_new (NULL) ;
   hbox = gtk_hbox_new (FALSE, 0) ;
   gtk_container_add (GTK_CONTAINER (frame), hbox) ;
   label = gtk_label_new ("Rotated Lists") ;
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 4) ;
   gtk_table_attach (GTK_TABLE (table), frame, 0, 1, 2, 3,
         GTK_FILL, GTK_FILL, 0, 0) ;

   frame = gtk_frame_new (NULL) ;
   hbox = gtk_hbox_new (FALSE, 0) ;
   gtk_container_add (GTK_CONTAINER (frame), hbox) ;
   check = gtk_check_button_new () ;
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), SaveSet->active[1]) ;
   gtk_signal_connect_object (GTK_OBJECT (check), "toggled", 
         GTK_SIGNAL_FUNC (set_active_list), (gpointer)&SaveSet->active[1]) ;
   gtk_box_pack_start (GTK_BOX (hbox), check, TRUE, FALSE, 0) ;
   gtk_table_attach (GTK_TABLE (table), frame, 1, 2, 2, 3, 
         GTK_FILL, GTK_FILL, 0, 0) ;

   frame = gtk_frame_new (NULL) ;
   hbox = gtk_hbox_new (FALSE, 2) ;
   gtk_container_add (GTK_CONTAINER (frame), hbox) ;
   label = gtk_label_new ("Extension:") ;
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2) ;
   SaveSet->entry[1] = entry = gtk_entry_new () ;
   gtk_entry_set_max_length (GTK_ENTRY (entry), 10) ;
   gtk_entry_set_text (GTK_ENTRY (entry), data->gparam->ext_rot) ;
   gtk_widget_set_usize (entry, 40, 0) ;
   gtk_box_pack_end (GTK_BOX (hbox), entry, TRUE, TRUE, 0) ;
   gtk_table_attach (GTK_TABLE (table), frame, 2, 3, 2, 3, 
         GTK_FILL, GTK_FILL, 0, 0) ;

   /* Plate Model */
   frame = gtk_frame_new (NULL) ;
   hbox = gtk_hbox_new (FALSE, 0) ;
   gtk_container_add (GTK_CONTAINER (frame), hbox) ;
   label = gtk_label_new ("Plate Models") ;
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 4) ;
   gtk_table_attach (GTK_TABLE (table), frame, 0, 1, 3, 4,
         GTK_FILL, GTK_FILL, 0, 0) ;

   frame = gtk_frame_new (NULL) ;
   hbox = gtk_hbox_new (FALSE, 0) ;
   gtk_container_add (GTK_CONTAINER (frame), hbox) ;
   check = gtk_check_button_new () ;
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), SaveSet->active[2]) ;
   gtk_signal_connect_object (GTK_OBJECT (check), "toggled", 
         GTK_SIGNAL_FUNC (set_active_list), (gpointer)&SaveSet->active[2]) ;
   gtk_box_pack_start (GTK_BOX (hbox), check, TRUE, FALSE, 0) ;
   gtk_table_attach (GTK_TABLE (table), frame, 1, 2, 3, 4, 
         GTK_FILL, GTK_FILL, 0, 0) ;

   frame = gtk_frame_new (NULL) ;
   hbox = gtk_hbox_new (FALSE, 2) ;
   gtk_container_add (GTK_CONTAINER (frame), hbox) ;
   label = gtk_label_new ("Extension:") ;
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2) ;
   SaveSet->entry[2] = entry = gtk_entry_new () ;
   gtk_entry_set_max_length (GTK_ENTRY (entry), 10) ;
   gtk_entry_set_text (GTK_ENTRY (entry), data->gparam->ext_plm) ;
   gtk_widget_set_usize (entry, 40, 0) ;
   gtk_box_pack_end (GTK_BOX (hbox), entry, TRUE, TRUE, 0) ;
   gtk_table_attach (GTK_TABLE (table), frame, 2, 3, 3, 4, 
         GTK_FILL, GTK_FILL, 0, 0) ;

   /* WCS options */
   hbox = gtk_hbox_new (FALSE, 0) ;
   gtk_box_pack_start (GTK_BOX (bbox), hbox, FALSE, FALSE, 5) ;

   frame = gtk_frame_new ("World Coordinates options") ;
   gtk_box_pack_start (GTK_BOX (hbox), frame, TRUE, TRUE, 5);

   table = gtk_table_new (3, 2, FALSE);
   gtk_table_set_col_spacings (GTK_TABLE (table), 1);
   gtk_table_set_row_spacings (GTK_TABLE (table), 2);
   /* gtk_box_pack_start (GTK_BOX (frame), table, TRUE, TRUE, 0); */
   gtk_container_add (GTK_CONTAINER (frame), table) ;

   label = gtk_label_new ("Format:") ;
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
            GTK_FILL, GTK_FILL, 2, 0) ;

   option = gtk_option_menu_new () ;
   menu = gtk_menu_new () ;
   if (data->model[data->nref]->spheric) {
      item = make_menu_item ("hh:mm:ss", GTK_SIGNAL_FUNC (set_opt_false), 
            &data->gparam->deg_fmt) ;
      gtk_menu_append (GTK_MENU (menu), item) ;
      item = make_menu_item ("degrees", GTK_SIGNAL_FUNC (set_opt_true), 
            &data->gparam->deg_fmt) ;
      gtk_menu_append (GTK_MENU (menu), item) ;
   }
   else {
      item = make_menu_item ("pixel", NULL, NULL) ;
      gtk_menu_append (GTK_MENU (menu), item) ;
   }
   gtk_option_menu_set_menu (GTK_OPTION_MENU (option), menu) ;
   gtk_table_attach (GTK_TABLE (table), option, 1, 2, 0, 1,
         GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0) ;


   label = gtk_label_new ("Action:") ;
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
            GTK_FILL, GTK_FILL, 2, 0) ;
   
   option = gtk_option_menu_new () ;
   menu = gtk_menu_new () ;
   item = make_menu_item ("Append", GTK_SIGNAL_FUNC (set_opt_true), 
         &data->gparam->append) ;
   gtk_menu_append (GTK_MENU (menu), item) ;
   item = make_menu_item ("Substitute", GTK_SIGNAL_FUNC (set_opt_false), 
         &data->gparam->append) ;
   gtk_menu_append (GTK_MENU (menu), item) ;
   gtk_menu_set_active (GTK_MENU (menu), 
         (guint)!data->gparam->append) ;
   gtk_option_menu_set_menu (GTK_OPTION_MENU (option), menu) ;
   gtk_table_attach (GTK_TABLE (table), option, 1, 2, 1, 2,
         GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0) ;

   label = gtk_label_new ("... to LCS") ;
   gtk_table_attach (GTK_TABLE (table), label, 2,3, 1, 2,
            GTK_FILL, GTK_FILL, 2, 0) ;

   gtk_widget_show_all (box) ;

   return box ;
}

static void
close_window1 (GtkWidget *window, gpointer data)
{
   cxc_display *display = data ;
   display->save_dialog = NULL ;
}

static void
get_buttons (GtkWidget *dialog, Sparam *SaveParam)
{
   GtkWidget *button ;
   button = gtk_button_new_with_label ("OK") ;
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
         GTK_SIGNAL_FUNC (save_accept), SaveParam) ;
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
         button, TRUE, TRUE, 0) ;
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT) ;
   gtk_widget_show (button) ;

   button = gtk_button_new_with_label ("Cancel") ;
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
         GTK_SIGNAL_FUNC (gtk_widget_destroy), (gpointer) dialog) ;
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
         button, TRUE, TRUE, 0) ;
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT) ;
   gtk_widget_grab_default (button) ;
   gtk_widget_show (button) ;

   button = gtk_button_new_with_label ("Help") ;
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
         button, TRUE, TRUE, 0) ;
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT) ;
   gtk_widget_show (button) ;
}

void
cxc_save (GtkWidget *widget, cxc_display *display)
{
   GtkWidget *dialog, *box, *vbox ;
   CxcData   *data = CXC_DATA (display->data) ;
   size_t     nfiles ;
   static Ssave   SaveSet ;
   static Sparam  SaveParam ;

   if (display->save_dialog) {
      /* Window is already opened */
      gtk_window_set_transient_for (GTK_WINDOW (display->save_dialog), 
            GTK_WINDOW (display->window)) ;
      return ;
   }

   /* Create a new window */
   display->save_dialog = dialog = gtk_dialog_new () ;

   /* Initialize Ssave struct */
   get_save_val (&SaveSet, CXC_DATA (display->data)) ;

   SaveParam.main_window = display->window ;
   SaveParam.dialog = dialog ;
   SaveParam.SaveSet = &SaveSet ;
   SaveSet.window = dialog ;
   SaveParam.display = display ;
   nfiles = data->f_set->n ;
   /***FIXME: cambiare 0 con nref !!! ***/

   gtk_window_set_title (GTK_WINDOW (dialog), "CataXcorr: Save Files") ;
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE) ;
   gtk_signal_connect (GTK_OBJECT (dialog), "destroy", 
                      GTK_SIGNAL_FUNC (close_window1), (gpointer)display) ;

   /* Main container */
   box = gtk_vbox_new (FALSE, 0) ;
   gtk_container_set_border_width (GTK_CONTAINER (box), 8) ;
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), box,
                      TRUE, TRUE, 0) ;
   /* head_label(box, 8, display) ; */

   /* Find Settings */
   vbox = save_settings (&SaveSet, display) ;
   gtk_container_add (GTK_CONTAINER (box), vbox) ;

   gtk_widget_show_all (box) ;

   /* buttons */
   get_buttons (dialog, &SaveParam) ;

   gtk_widget_show_all (dialog) ;
}
