/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 					      Bologna Astronomical Observatory
 *----------------------------------------------------------------------------
 * File name 	: cataxcorr.h
 * Author 	: Paolo Montegriffo
 * Created on	: September, 2001	
 * Language	: ANSI C
 * Description	: Type definitions for cataxcorr structures
 * Version      : 2.1.4 - January 24, 2002
 *--------------------------------------------------------------------------*/

#ifndef CATAXCORR_H
#define CATAXCORR_H 1

#include "common.h"
#include "fileinfo.h"

BEGIN_C_DECLS

typedef enum {
   PIXEL,
   DEG,
   UNITS
} units ;

typedef enum {
   CIRCLE,
   RECT,
   MAG,
   SHAPE
} shape ;

typedef struct 
{
   double    eps ;          /* precision in coordinate measures */
   double    crow_rad ;     /* crowding radius */
   double    min_side ;     /* minimum side length for selected triangles */
   double    max_side ;     /* maximum side length for selected triangles */
   double    scale ;
   double    max_shift ;    /* max shift for x-correlation search */
   double    m1_rad, m2_rad, m3_rad ;  /* Mix rad */ /* 1 & 2 usati per register e fit*/
   double    m0 ;
   double    m1 ;
   char      mask[128] ;
   shape     mask_shape ;
   double    mask_p[4] ;
} def_pars ;

typedef struct gparam_tag {
   int        N_NullVal ;
   double     NullValues[5] ;
   double     tune ;
   char       outfile[1024] ;
   double     t0_mag ;            /* Tolerance in magnitude for align section */
   double     t1_mag, t2_mag, t3_mag ;  /* Tolerance in magnitude comparisons */
   double     sr_1, sr_2, sr_3 ;        /* SR clipping factor */
   int        nsr_1, nsr_2, nsr_3 ;     /* N SR iterations */
   double     m1_rad, m2_rad, m3_rad ;  /* Mix rad */ /* 3 usato per stack finale */
   int        max_iter ;                /* Max iterations */
   int        max_ord ;                 /* Max transformation order */
   int        Imax_ord ;                /* Max order in registration process */
   int        feel ;                    /* feel mode: 0=RELAXED, 1=RIGID */
   double     min_tol, min_tol0, min_tol1, min_tol2, min_tol3 ;
   int        find ;
   boolean    mosaic ;
   boolean    recenter ;
   boolean    savetrans ;
   boolean    append ;
   boolean    deg_fmt ;                 /* WCS format: F=hh:mm:ss; T=deg */
   char       ext_rot[11] ;
   char       ext_plm[11] ;
   int        fitmodel ;
   double     CRVAL[2] ;
   double     CRPIX[2] ;
} g_pars ;

typedef struct Star_tag {
   char     *id ;
   int       index ;
   double    mag ;   /* star magnitude */
   double    mcal ;  /* star magnitude calibrated on reference frame */
   double    xpix ;  /* X input coordinate */
   double    ypix ;  /* Y input coordinate */
   double    xprj ;  /* X coordinate projected on reference plane */
   double    yprj ;  /* Y coordinate projected on reference plane */
   double    xrot ;  /* X coordinate distorsion corrected and cdelt scaled */
   double    yrot ;  /* Y coordinate distorsion corrected and cdelt scaled */
   int       flag ;  /* star flag */
   void     *p ;
} Star ;

typedef struct StarList_tag {
   Star    **data ;
   size_t    nstar ;
   boolean   spheric ;
   boolean   out_sph ;
   double    origin[2] ;
   boolean   proj ;
   boolean   saved ;
   double    inv_density ;
   def_pars *param ;
} StarList ;

typedef struct {
   Star         **data ;
   int            id ;             /* Colonna ID */
   unsigned int   ns ;             /* Colonna NS file 1-32  */
   unsigned int   ns2 ;            /* Colonna NS file 33-64 */
} Stack ;

typedef struct {
   Stack        **tab ;
   StarList      *ref ;
   int            nfiles ;
   unsigned int   nf ;
   unsigned int   nf2 ;
   int            n ;
} StackTab ;


typedef struct Match_tag {
   int    i ;
   int    j ;
   int    f ;
} Match ;

typedef struct StarMatch_tag {
   Star  *s1 ;
   Star  *s2 ;
} StarMatch ;

/*----------------------------------------------------------------------------
 * 								Defines
 *--------------------------------------------------------------------------*/
extern boolean no_interface ;

#define MAXPATHLEN 2048

#define NullStar     (Star*)NULL
#define NullStarList (StarList*)NULL
#define NullCoincTab (CoincTab*)NULL

#define NONE     0
#define MATCH    1
#define SHIFT    2
#define OTHER    3

#if HAVE_GTK
#include <gtk/gtk.h>
#include "cxc_init.h"

#ifdef   GCC_HAVE_VARIADIC_MACROS   /*** FIXME: settare nel configure ***/
#define  CXC_MESSAGE(...) \
   do { \
      if (no_interface == TRUE) \
         cp_message (__VA_ARGS__) ; \
      else \
         cxc_message (__VA_ARGS__) ; \
   } while (0)
#else
#define  CXC_MESSAGE cxc_message
#endif

#define  CXC_WARNING   cxc_warning
#define  CXC_ERROR     cxc_error

#else

#ifdef   GCC_HAVE_VARIADIC_MACROS
#define  CXC_MESSAGE(...)   cp_message (__VA_ARGS__)
#else
#define  CXC_MESSAGE cp_message
#endif

#define  CXC_WARNING   cp_warning
#define  CXC_ERROR     cp_error

#endif   /* HAVE_GTK */

END_C_DECLS

#endif     /* !CATAXCORR_H */
/*--------------------------------------------------------------------------*/
