/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 					      Bologna Astronomical Observatory
 *----------------------------------------------------------------------------
 * File name 	: coordinates.c
 * Author 	: Paolo Montegriffo
 * Created on	: March, 2002	
 * Language	: ANSI C
 * Description	: 
 * Version      : 2.1.5 - March 26, 2002
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 							Includes
 *--------------------------------------------------------------------------*/

#if HAVE_CONFIG_H
#  include "config.h"
#endif

#include "common.h"

#include "wcs.h"
#include "coordinates.h"

#ifndef MAX
#define MAX(a,b) (((a)>=(b)) ? (a) : (b))
#endif
#ifndef ABS
#define ABS(a) ((a) < 0 ? (-(a)) : (a))
#endif

/*----------------------------------------------------------------------------
 * 							Global variables	
 *--------------------------------------------------------------------------*/

struct celprm * cel = NULL ; 
struct prjprm * prj = NULL ;
extern double   IntNull ;

/*----------------------------------------------------------------------------
 *							Functions code
 *--------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
 * Function  : coord_pix2prj
 * In        : 
 *             
 * Out       : 
 * Purpouse  : 
 *---------------------------------------------------------------------------*/

void 
coord_pix2prj (Star **s, int nstar, boolean spheric, double origin[])
{
   int     i ;
   double  phi, theta ;
   CPKDEBUG("coord_pix2prj") ;
   if (spheric) {
      /* Initialize wcslib structures */
      cel = (struct celprm *) calloc ((size_t)1, sizeof(struct celprm)) ;
      prj = (struct prjprm *) calloc ((size_t)1, sizeof(struct prjprm)) ;
      cel->flag = 0 ;
      cel->ref[0] = origin[0] ;
      cel->ref[1] = origin[1] ;
      cel->ref[2] = 999.0 ;
      cel->ref[3] = 999.0 ;
      prj->flag = 0 ;
      prj->r0 = 0.0 ;
      for (i = 0; i < nstar; i++) 
         celfwd ("TAN", s[i]->xpix, s[i]->ypix, cel, &phi, &theta, prj, 
                 &s[i]->xprj, &s[i]->yprj) ;
      free (cel) ;
      free (prj) ;
   }
   else {
      for (i = 0; i < nstar; i++) {
         s[i]->xprj = s[i]->xpix - origin[0] ;
         s[i]->yprj = s[i]->ypix - origin[1] ;
      }
   }
   /** FIXME **/
   for (i = 0; i < nstar; i++) {
      s[i]->xrot = s[i]->xprj ;
      s[i]->yrot = s[i]->yprj ;
   }

   return ;
}

/*-----------------------------------------------------------------------------
 * Function  : coord_prj2pix
 * In        : 
 *             
 * Out       : 
 * Purpouse  : 
 *---------------------------------------------------------------------------*/

void 
coord_prj2pix (Star **s, int nstar, boolean spheric, double origin[])
{
   int     i ;
   double  phi, theta ;

   if (spheric) {
      /* Initialize wcslib structures */
      cel = (struct celprm *) calloc ((size_t)1, sizeof(struct celprm)) ;
      prj = (struct prjprm *) calloc ((size_t)1, sizeof(struct prjprm)) ;
      cel->flag = 0 ;
      cel->ref[0] = origin[0] ;
      cel->ref[1] = origin[1] ;
      cel->ref[2] = 999.0 ;
      cel->ref[3] = 999.0 ;
      prj->flag = 0 ;
      prj->r0 = 0.0 ;
      for (i = 0; i < nstar; i++) 
         celrev ("TAN", s[i]->xprj, s[i]->yprj, prj, &phi, &theta, cel, 
                 &s[i]->xpix, &s[i]->ypix) ;
      free (cel) ;
      free (prj) ;
   }
   else {
      for ( i=0; i<nstar; i++ ) {
         s[i]->xpix = s[i]->xprj + origin[0] ;
         s[i]->ypix = s[i]->yprj + origin[1] ;
      }
   }
   return ;
}

/*-----------------------------------------------------------------------------
 * Function  : coord_rot2pix
 * In        : 
 *             
 * Out       : 
 * Purpouse  : 
 * Note      : to be used once at the end of computations 
 *---------------------------------------------------------------------------*/

void 
coord_rot2pix (Star **s, int nstar, boolean spheric, double origin[])
{
   int     i ;
   double  phi, theta ;

   if (spheric) {
      /* Initialize wcslib structures */
      cel = (struct celprm *) calloc ((size_t)1, sizeof(struct celprm)) ;
      prj = (struct prjprm *) calloc ((size_t)1, sizeof(struct prjprm)) ;
      cel->flag = 0 ;
      cel->ref[0] = origin[0] ;
      cel->ref[1] = origin[1] ;
      cel->ref[2] = 999.0 ;
      cel->ref[3] = 999.0 ;
      prj->flag = 0 ;
      prj->r0 = 0.0 ;
      for (i = 0; i < nstar; i++) 
         celrev ("TAN", s[i]->xrot, s[i]->yrot, prj, &phi, &theta, cel, 
                 &s[i]->xpix, &s[i]->ypix) ;
      free (cel) ;
      free (prj) ;
   }
   else {
      for ( i=0; i<nstar; i++ ) {
         s[i]->xpix = s[i]->xrot + origin[0] ;
         s[i]->ypix = s[i]->yrot + origin[1] ;
      }
   }
   return ;
}

/*-----------------------------------------------------------------------------
 * Function  : coord_prj2rot
 * In        : 
 *             
 * Out       : 
 * Purpouse  : 
 * Note      : Calibrate mcal:
               s[i]->mcal = s[i]->mag + pmod->dmag ;
 *---------------------------------------------------------------------------*/

void 
coord_prj2rot (Star **s, int nstar, PlateMod *model)
{
   int    i, j ;
   double afunc[FN_MAXPAR], x, y, xr, yr, dx = 0.0, dy = 0.0 ;

   for (i = 0; i < nstar; i++) {

      x = s[i]->xprj ;
      y = s[i]->yprj ;
      xr = model->pc[0][0]*x + model->pc[0][1]*y ;
      yr = model->pc[1][0]*x + model->pc[1][1]*y ;

      (model->set.PlateFunc)(xr, yr, afunc, FN_MAXPAR) ;

      s[i]->xrot = s[i]->yrot = 0.0 ;
      for (j = 0; j < model->nx_coeff; j++)
         if (model->mask[j])
            s[i]->xrot += model->xfit_coeff[j]*afunc[j] ;
      for (j = 0; j < model->ny_coeff; j++)
         if (model->mask[j])
            s[i]->yrot += model->yfit_coeff[j]*afunc[j] ;

      dx = MAX (dx, ABS (s[i]->xrot-xr)) ;
      dy = MAX (dy, ABS (s[i]->yrot-yr)) ;

      s[i]->xrot *= model->cdelt[0] ;
      s[i]->yrot *= model->cdelt[1] ;

      s[i]->mcal = (s[i]->mag != IntNull) ? s[i]->mag + model->dmag : IntNull ;
   }
   model->dx = dx ;
   model->dy = dy ;
   return ;
}


#include <gsl/gsl_vector.h>
#include <gsl/gsl_multiroots.h>

int
function_f (const gsl_vector *xy, void *platemod, gsl_vector *out)
{
   PlateMod *model = (PlateMod*)platemod ;
   int       j ;
   double    afunc[FN_MAXPAR], x, y, xr, yr ;

   x = model->pc[0][0] * gsl_vector_get (xy, 0) + 
       model->pc[0][1] * gsl_vector_get (xy, 1) ;
   y = model->pc[1][0] * gsl_vector_get (xy, 0) + 
       model->pc[1][1] * gsl_vector_get (xy, 1) ;

   (model->set.PlateFunc)(x, y, afunc, FN_MAXPAR) ;
   xr = yr = 0.0 ;
   for (j = 0; j < model->nx_coeff; j++)
      if (model->mask[j])
         xr += model->xfit_coeff[j]*afunc[j] ;
   for (j = 0; j < model->ny_coeff; j++)
      if (model->mask[j])
         yr += model->yfit_coeff[j]*afunc[j] ;

   gsl_vector_set (out, 0, model->cdelt[0] * xr - model->x) ;
   gsl_vector_set (out, 1, model->cdelt[1] * yr - model->y) ;

   return GSL_SUCCESS ;
}

void
print_state (size_t iter, gsl_multiroot_fsolver *ss)
{
   printf("iter %3u  x = %.8f %.8f  out %.8f %.8f\n",iter,
         gsl_vector_get (ss->x, 0),
         gsl_vector_get (ss->x, 1),
         gsl_vector_get (ss->f, 0),
         gsl_vector_get (ss->f, 1)) ;
}

void 
coord_rot2prj (double xr, double yr, PlateMod *model, double *xp, double *yp)
{
   const gsl_multiroot_fsolver_type *T ;
   gsl_multiroot_fsolver            *ss ;
   gsl_multiroot_function            f ;
   gsl_vector                       *xy ;
   const size_t nn = 2 ;
   size_t       i, iter = 0 ;
   int          status, j ;

   model->x = xr ;
   model->y = yr ;

   f.f = &function_f ;
   f.n = nn ;
   f.params = model ;

   xy = gsl_vector_alloc (nn) ;

  /* Nota: valori iniziali cambiati da 0,0 a xr,yr nella versione 2.1.19 */
  /* Nota: valori iniziali cambiati da xr,yr a 1.0,1.0 */
   gsl_vector_set (xy, 0, 1.0) ;
   gsl_vector_set (xy, 1, 1.0) ;

   T = gsl_multiroot_fsolver_hybrids ;
   ss = gsl_multiroot_fsolver_alloc (T, 2) ;

   gsl_multiroot_fsolver_set (ss, &f, xy) ;

   /* print_state (iter, ss) ; */
   do {
      iter++ ;
      status = gsl_multiroot_fsolver_iterate (ss) ;
      /* print_state (iter, ss) ; */
      if (status)
         break ;
      status = gsl_multiroot_test_residual (ss->f, 1e-12) ;
   } while (status == GSL_CONTINUE && iter < 1000) ;

   *xp = gsl_vector_get (ss->x, 0) ;
   *yp = gsl_vector_get (ss->x, 1) ;

   gsl_multiroot_fsolver_free (ss) ;
   gsl_vector_free (xy) ;
   return   ;
}      

/*-----------------------------------------------------------------------------
 * Function  : coord_pix2prj_lin
 * In        : 
 *             
 * Out       : 
 * Purpouse  : 
 *---------------------------------------------------------------------------*/

void 
coord_pix2prj_lin (Star **s, int nstar, boolean spheric, PlateMod *model)
{
   int     i ;
   double  x, y ;
   double  phi, theta ;

   if (spheric) {
      /* Initialize wcslib structures */
      cel = (struct celprm *) calloc ((size_t)1, sizeof(struct celprm)) ;
      prj = (struct prjprm *) calloc ((size_t)1, sizeof(struct prjprm)) ;
      cel->flag = 0 ;
      cel->ref[0] = model->crpix[0] ;
      cel->ref[1] = model->crpix[1] ;
      cel->ref[2] = 999.0 ;
      cel->ref[3] = 999.0 ;
      prj->flag = 0 ;
      prj->r0 = 0.0 ;
      for (i = 0; i < nstar; i++) 
         celfwd ("TAN", s[i]->xpix, s[i]->ypix, cel, &phi, &theta, prj, 
                 &s[i]->xprj, &s[i]->yprj) ;
      free (cel) ;
      free (prj) ;
   }

   for (i = 0; i < nstar; i++) {
      if (spheric) {
         x = s[i]->xprj ;
         y = s[i]->yprj ;
      }
      else {
         x = s[i]->xpix - model->crpix[0] ;
         y = s[i]->ypix - model->crpix[1] ;
      }
      s[i]->xprj = model->pc[0][0]*x + model->pc[0][1]*y ;
      s[i]->yprj = model->pc[1][0]*x + model->pc[1][1]*y ;
   }
   return ;
}

/*-----------------------------------------------------------------------------
 * Function  : coord_prj_scale
 * In        : 
 *             
 * Out       : 
 * Purpouse  : 
 *---------------------------------------------------------------------------*/

void 
coord_prj_scale (Star **s, int nstar, PlateMod *model)
{
   int     i ;
   for (i = 0; i < nstar; i++) {
      s[i]->xrot = s[i]->xprj /= model->cdelt[0] ;
      s[i]->yrot = s[i]->yprj /= model->cdelt[1] ;
   }
   return ;
}

void 
coord_rot_inv_scale (Star **s, int nstar, PlateMod *model)
{
   int     i ;
   for (i = 0; i < nstar; i++) {
      s[i]->xrot *= model->cdelt[0] ;
      s[i]->yrot *= model->cdelt[1] ;
   }
   return ;
}
