/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 					      Bologna Astronomical Observatory
 *----------------------------------------------------------------------------
 * File name 	: fileinfo.h
 * Author 	: Paolo Montegriffo
 * Created on	: April, 2003	
 * Language	: ANSI C
 * Description	: Type definitions for info catalogues structures
 * Version      : 2.1.17 - April 10, 2003
 *--------------------------------------------------------------------------*/

#ifndef FILEINFO_H
#define FILEINFO_H 1

BEGIN_C_DECLS

typedef struct Dform_tag
{
   char type ; /* Data type descriptor */
   int  nc ;   /* Number of fields */
   int  nd ;   /* Number of decimals */
} Dform ;

typedef struct Finfo_tag 
{
   char   *filename ;     /* Nome file */
   char  **labels ;       /* Array label di identificazione colonne: maxlen 40 */
   Dform  *fmt ;          /* Data format */
   int     fl_lab ;       /* Flag identificatori: 1:si 0:no */
   int     n_cols ;       /* Numero di colonne  */
   int     n_rows ;       /* Numero di righe di dati */
   int     n_head ;       /* Numero di righe di header */
   int     m_pos ;        /* Position reference magnitude */
   int     x_pos ;        /* Position reference x coordinate */
   int     y_pos ;        /* Position reference y coordinate */
   boolean spheric ;      /* Coordinates are spheric ? */
} Finfo ;

typedef struct FileSet_tag
{
   Finfo **info ;         /* Finfo structure */
   int     n ;            /* Number of files */
} FileSet ;

/*----------------------------------------------------------------------------
 * 							Function prototypes
 *--------------------------------------------------------------------------*/

extern FileSet     *file_info                  (char     **files) ;
extern Finfo       *file_info_new              (char     *file_name) ;
extern int          file_info_add              (FileSet  *f_set,
                                                char     *file_name) ;
extern void         file_info_remove_by_name   (FileSet  *f_set,
                                                char     *file_name) ;
extern void         file_info_remove_by_number (FileSet  *f_set,
                                                int       n) ;
extern void         file_info_destroy          (FileSet  *f_set) ;
extern int          file_info_col_set          (FileSet  *f_set,
                                                char     *mlist,
                                                char     *xlist,
                                                char     *ylist) ;
extern void         file_info_free             (Finfo    *info) ;

END_C_DECLS

#endif /* !FILEINFO_H */
/*--------------------------------------------------------------------------*/
