/*----------------------------------------------------------------------------
 * CataPack
 *
 * Authors: Paolo Montegriffo <paolo.montegriffo@oabo.inaf.it>
 *
 * Copyright (C) 2001-2006 INAF - Osservatorio Astronomico di Bologna.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 					      Bologna Astronomical Observatory
 *----------------------------------------------------------------------------
 * File name 	: model.c
 * Author 	: Paolo Montegriffo
 * Created on	: March, 2002	
 * Language	: ANSI C
 * Description	: 
 * Version      : 2.1.5 - March 26, 2002
 *--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * 							Includes
 *--------------------------------------------------------------------------*/

#if HAVE_CONFIG_H
#  include "config.h"
#endif

#include "common.h"

#include "catalib.h"
#include "cp_messages.h"
#include "model.h"

#ifndef ABS
#define ABS(a) ((a) < 0 ? (-(a)) : (a))
#endif

/*----------------------------------------------------------------------------
 *							Functions code
 *--------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
 * Function  : model_save
 * In        : 
 * Out       : 
 * Purpouse  : 
 *             
 *---------------------------------------------------------------------------*/

void 
model_save(char *filename, char *refcat, PlateMod  *model, Finfo *info)
{
   char   lab[40], lab1[40] ;
   int    i ;
   FILE * out ;

   CPKDEBUG ("model_save") ;

   if (model->saved)
      return ;

   cp_message (1,2,"Saving Plate Model file '%s'\n", filename) ;

   if ( (out = fopen (filename, "w")) == (FILE*)NULL ) {
      cp_warning ("Cannot open file '%s': skipping") ;
      return ;
   }

   /* Linear transformation terms */
   fprintf (out, "%-8s= %#20.14f / %-47s\n", "CRVAL1", model->crval[0], 
         "Coordinate at reference pixel") ;
   fprintf (out, "%-8s= %#20.14f / %-47s\n", "CRVAL2", model->crval[1], 
         "Coordinate at reference pixel") ;
   fprintf (out, "%-8s= %#20.14g / %-47s\n", "CRPIX1", model->crpix[0], 
         "Reference pixel") ;
   fprintf (out, "%-8s= %#20.14g / %-47s\n", "CRPIX2", model->crpix[1], 
         "Reference pixel") ;
   sprintf (lab, "'%s'", "linear") ;
   if (model->spheric) sprintf (lab, "'%s'", "RA---TAN") ;
   fprintf (out, "%-8s= %-20s / %-47s\n", "CTYPE1", lab, 
            "Coordinate/algorithm type") ;
   if (model->spheric) sprintf (lab, "'%s'", "DEC--TAN") ;
   fprintf (out, "%-8s= %-20s / %-47s\n", "CTYPE2", lab, 
            "Coordinate/algorithm type") ;
   if (model->spheric) sprintf (lab, "'%s'", "deg") ;
   else sprintf (lab, "'%s'", "pixel") ;
   fprintf (out, "%-8s= %-20s / %-47s\n", "CUNIT1", lab, 
            "Units of coordinate values") ;
   fprintf (out, "%-8s= %-20s / %-47s\n", "CUNIT2", lab, 
            "Units of coordinate values") ;
   fprintf (out, "%-8s= %#20.14g / %-47s\n", "CDELT1", model->cdelt[0], 
         "Coordinate increment per pixel") ;
   fprintf (out, "%-8s= %#20.14g / %-47s\n", "CDELT2", model->cdelt[1], 
         "Coordinate increment per pixel") ;
   fprintf (out, "%-8s= %#20.14g / %-47s\n", "PC1_1", model->pc[0][0], 
         "Pixel transformation matrix") ;
   fprintf (out, "%-8s= %#20.14g / %-47s\n", "PC1_2", model->pc[0][1], " ") ;
   fprintf (out, "%-8s= %#20.14g / %-47s\n", "PC2_1", model->pc[1][0], " ") ;
   fprintf (out, "%-8s= %#20.14g / %-47s\n", "PC2_2", model->pc[1][1], " ") ;
   fprintf (out, "%-8s= %#20.14g / %-47s\n", "CROTA2", model->crota2, 
         "Rotation from stated coordinate type") ;

   sprintf (lab, "'%s'", info->labels[info->x_pos]) ;
   fprintf (out, "%-8s= %-20s / %-47s\n", "X_LABEL", lab, 
         "X-axis label identification") ;
   sprintf (lab, "'%s'", info->labels[info->y_pos]) ;
   fprintf (out, "%-8s= %-20s / %-47s\n", "Y_LABEL", lab, 
         "Y-axis label identification") ;
   sprintf(lab, "'%s'", info->labels[info->m_pos]) ;
   fprintf (out, "%-8s= %-20s / %-47s\n", "M_LABEL", lab, 
         "Magnitude label identification") ;
   fprintf (out, "%-8s= %-#20f / %-47s\n", "MAGZERO", model->dmag, 
         "Magnitude zero point") ;

   strncpy (lab, refcat, (size_t)38) ;
   sprintf (lab1, "'%s'", lab) ;
   fprintf (out, "%-8s= %-20s / %-47s\n", "REFCAT", lab1, 
         "Name of reference file") ;
   strncpy (lab, info->filename, (size_t)38) ;
   sprintf (lab1, "'%s'", lab) ;
   fprintf (out, "%-8s= %-20s / %-47s\n", "PARENT", lab1, 
         "Name of parent file") ;
   fprintf (out, "%-8s= %-20s / %-47s\n", "DATE", 
            iso_8601_date_format (NULL, FALSE), 
            "Date Model file was generated") ;
   fprintf (out, "%-8s  %-70s\n", "COMMENT", 
            "Generated by CataXcorr - Version " VERSION) ;

   /* Plate model */
   switch (model->fitmodel) {
      case LIN:       sprintf (lab, "'LIN'") ;       break ;
      case SAO:       sprintf (lab, "'SAO'") ;       break ;
      case POLY:      sprintf (lab, "'POLY'") ;      break ;
      case POLYRAD:   sprintf (lab, "'POLYRAD'") ;   break ;
      case LEGENDRE:  sprintf (lab, "'LEGENDRE'") ;  break ;
      case NUM_MODEL: sprintf (lab, "''") ;          break ;
      default: break ;
   }
   fprintf (out, "%-8s= %-20s / %-47s\n", "PLTMODEL", lab, 
         "Plate Model Function") ;
   fprintf (out, "#%-8s= %-20d / %-46s\n", "PLTORDER", model->order, 
         "Plate Model Order") ;
   fprintf (out, "%-8s= %-20d / %-47s\n", "NMATCH", model->nmatch, 
         "Number of matched points") ;
   fprintf (out, "%-8s= %#20.14g / %-47s\n", "X_RMS", 
         ABS (model->cdelt[0]) * model->dxdev, 
         "X-axis model Root Mean Square deviation") ;
   fprintf (out, "%-8s= %#20.14g / %-47s\n", "Y_RMS", 
         ABS (model->cdelt[1]) * model->dydev, 
         "Y-axis model Root Mean Square deviation") ;
   fprintf (out, "%-8s= %#20.14g / %-47s\n", "X_CHISQ", model->xchisq, 
         "X-axis model reduced Chi-square") ;
   fprintf (out, "%-8s= %#20.14g / %-47s\n", "Y_CHISQ", model->ychisq, 
         "Y-axis model reduced Chi-square") ;
   fprintf (out, "%-8s= %-20d / %-47s\n", "NCOEFF", model->nx_coeff, 
         "Number of fitting coefficents") ;

   for (i = 0; i < model->nx_coeff; i++) {
      sprintf (lab, "CO1_%d", i+1) ;
      fprintf (out, "%-8s= %#20.14g / %-47s\n", lab, model->xfit_coeff[i], 
              (i == 0 ? "X plate model coefficents" : " ")) ;
   }
   for (i = 0; i < model->ny_coeff; i++) {
      sprintf (lab, "CO2_%d", i+1) ;
      fprintf (out, "%-8s= %#20.14g / %-47s\n", lab, model->yfit_coeff[i], 
              (i == 0 ? "Y plate model coefficents" : " ")) ;
   }

   fclose (out) ;
   model->saved = TRUE ;
   return ;
}


