/*----------------------------------------------------------------------------
 * 					      Bologna Astronomical Observatory
 *----------------------------------------------------------------------------
 * File name 	: starlist.h
 * Author 	: Paolo Montegriffo
 * Created on	: February, 2002	
 * Language	: ANSI C
 * Description	: Type definitions for stalist and coinctab structures
 * Version      : 2.1.4 - February 14, 2002
 *--------------------------------------------------------------------------*/

#ifndef STARLIST_H
#define STARLIST_H 1

#include "common.h"

#ifdef debug
#define STARCHECK(a, b) if (verbose > 4) star_check(a, b)
#else
#define STARCHECK(a, b) 
#endif

BEGIN_C_DECLS

extern Star     *star_vec_dup         (Star       *in, 
                                       size_t      n) ;
extern Star    **star_ptr_dup         (Star      **in, 
                                       size_t      n) ;
extern Star    **star_destroy         (Star      **in, 
                                       size_t      n) ;
extern StarList *star_list_new        (size_t      n_rows) ;

extern StarList *star_list_destroy    (StarList    *in) ;

extern StarList *star_list_free       (StarList    *in) ;

extern StarList *star_list_dup        (StarList    *in) ;

extern StarList *star_list_read       (Finfo       *f_info,
                                       def_pars    *param) ;

extern void      star_list_save       (char        *filename, 
                                       Finfo       *f_info, 
                                       StarList    *list ) ;
extern int       star_list_center     (StarList    *list, 
                                       double      *xcenter, 
                                       double      *ycenter) ;
extern StarList *star_list_get_coinc  (StackTab    *coinc) ;

extern void      star_list_get_record (Finfo       *f_info, 
                                       int          n, 
                                       char        *buffer) ;
extern StackTab *stack_init           (StarList    *list, 
                                       int          n, 
                                       int          nfiles) ;
extern void      stack_destroy        (StackTab    *coinc) ;

extern int       stack_add            (StackTab   **coinc, 
                                       StarList    *list, 
                                       int          nfile, 
                                       StarMatch   *match, 
                                       int          nmatch) ;
extern int       stack_remove         (StackTab   **coinc, 
                                       int          nfile) ;
extern void      coinc_tab_save       (StackTab    *coinc, 
                                       Finfo      **f_info, 
                                       char        *outfile ) ;
extern void      star_flag_crowded    (StarList    *list, 
                                       double       tol) ;
extern void      star_flag_mask       (StarList    *list, 
                                       shape        mask_shape, 
                                       double       mask_p[]) ;
extern void      star_flag_crow_r     (StarList    *list, 
                                       double       tol) ;
extern void      star_check           (Star       **s, 
                                       size_t       n) ;
extern void      star_list_pix2prj    (StarList    *list, 
                                       double       origin[]) ;
extern StarList *star_list_sample     (StarList    *sl, 
                                       boolean      mag) ;
extern double    star_list_density    (StarList    *list) ;

END_C_DECLS

#endif     /* !STARLIST_H */
/*--------------------------------------------------------------------------*/
